@@ -77,10 +77,11 @@ const char* Configuration::getValue(const String& sect, const String& key, const
     return s ? s->c_str() : defvalue;
 }
 
-int Configuration::getIntValue(const String& sect, const String& key, int defvalue) const
+int Configuration::getIntValue(const String& sect, const String& key, int defvalue,
+    int minvalue, int maxvalue, bool clamp) const
 {
     const NamedString *s = getKey(sect,key);
-    return s ? s->toInteger(defvalue) : defvalue;
+    return s ? s->toInteger(defvalue,0,minvalue,maxvalue,clamp) : defvalue;
 }
 
 int Configuration::getIntValue(const String& sect, const String& key, const TokenDict* tokens, int defvalue) const
@@ -112,6 +113,13 @@ void Configuration::clearSection(const char* sect)
 	m_sections.clear();
 }
 
+// Make sure a section with a given name exists, create it if required
+NamedList* Configuration::createSection(const String& sect)
+{
+    ObjList* o = makeSectHolder(sect);
+    return o ? static_cast<NamedList*>(o->get()) : 0;
+}
+
 void Configuration::clearKey(const String& sect, const String& key)
 {
     NamedList *l = getSection(sect);
