@@ -110,6 +110,7 @@ static TranslatorCaps s_stereoCaps[] = {
 };
 
 static Mutex s_dataMutex(true,"DataEndpoint");
+static Mutex s_consSrcMutex(false,"DataConsumer::Source");
 
 class ThreadedSourcePrivate : public Thread
 {
@@ -143,39 +144,44 @@ class SimpleTranslator : public DataTranslator
 {
 public:
     SimpleTranslator(const DataFormat& sFormat, const DataFormat& dFormat)
-	: DataTranslator(sFormat,dFormat) { }
+	: DataTranslator(sFormat,dFormat), m_valid(false) {
+	    if (!getTransSource())
+		return;
+	    int nchan = m_format.numChannels();
+	    if (nchan != getTransSource()->getFormat().numChannels())
+		return;
+	    m_valid = true;
+	    m_sFmt = m_format;
+	    m_dFmt = getTransSource()->getFormat();
+	    if (nchan != 1) {
+		// get rid of the channel prefix
+		m_sFmt >> "*";
+		m_dFmt >> "*";
+	    }
+	}
     virtual unsigned long Consume(const DataBlock& data, unsigned long tStamp, unsigned long flags)
 	{
 	    if (!ref())
 		return 0;
 	    unsigned long len = 0;
-	    while (getTransSource()) {
-		int nchan = m_format.numChannels();
-		if (nchan != getTransSource()->getFormat().numChannels())
-		    break;
-		String sFmt = m_format;
-		String dFmt = getTransSource()->getFormat();
-		if (nchan != 1) {
-		    // get rid of the channel prefix
-		    sFmt >> "*";
-		    dFmt >> "*";
+	    if (m_valid && getTransSource() && m_buffer.convert(data,m_sFmt,m_dFmt)) {
+		if (tStamp == invalidStamp()) {
+		    unsigned int delta = data.length();
+		    if (delta > m_buffer.length())
+			delta = m_buffer.length();
+		    tStamp = m_timestamp + delta;
 		}
-		DataBlock oblock;
-		if (oblock.convert(data, sFmt, dFmt)) {
-		    if (tStamp == invalidStamp()) {
-			unsigned int delta = data.length();
-			if (delta > oblock.length())
-			    delta = oblock.length();
-			tStamp = m_timestamp + delta;
-		    }
-		    m_timestamp = tStamp;
-		    len = getTransSource()->Forward(oblock, tStamp, flags);
-		}
-		break;
+		m_timestamp = tStamp;
+		len = getTransSource()->Forward(m_buffer,tStamp,flags);
 	    }
 	    deref();
 	    return len;
 	}
+private:
+    bool m_valid;
+    String m_sFmt;
+    String m_dFmt;
+    DataBlock m_buffer;
 };
 
 // slin basic mono resampler
@@ -299,8 +305,8 @@ public:
 class SimpleFactory : public TranslatorFactory
 {
 public:
-    SimpleFactory(const TranslatorCaps* caps)
-	: m_caps(caps)
+    SimpleFactory(const TranslatorCaps* caps, const char* name)
+	: TranslatorFactory(name), m_caps(caps)
 	{ }
     virtual DataTranslator* create(const DataFormat& sFormat, const DataFormat& dFormat)
 	{ return converts(sFormat,dFormat) ? new SimpleTranslator(sFormat,dFormat) : 0; }
@@ -312,6 +318,9 @@ private:
 
 class ResampFactory : public TranslatorFactory
 {
+public:
+    ResampFactory() : TranslatorFactory("resample")
+	{ }
     virtual DataTranslator* create(const DataFormat& sFormat, const DataFormat& dFormat)
 	{ return converts(sFormat,dFormat) ? new ResampTranslator(sFormat,dFormat) : 0; }
     virtual const TranslatorCaps* getCapabilities() const
@@ -320,6 +329,9 @@ class ResampFactory : public TranslatorFactory
 
 class StereoFactory : public TranslatorFactory
 {
+public:
+    StereoFactory() : TranslatorFactory("stereo")
+	{ }
     virtual DataTranslator* create(const DataFormat& sFormat, const DataFormat& dFormat)
 	{ return converts(sFormat,dFormat) ? new StereoTranslator(sFormat,dFormat) : 0; }
     virtual const TranslatorCaps* getCapabilities() const
@@ -342,9 +354,12 @@ public:
 	{ return m_length; }
     virtual const FormatInfo* intermediate() const;
     virtual bool intermediate(const FormatInfo* info) const;
+    virtual const char* name() const
+	{ return m_name.c_str(); }
 private:
     TranslatorFactory* m_factory1;
     TranslatorFactory* m_factory2;
+    String m_name;
     DataFormat m_format;
     unsigned int m_length;
     const TranslatorCaps* m_capabilities;
@@ -452,7 +467,7 @@ void DataConsumer::destroyed()
 
 void* DataConsumer::getObject(const String& name) const
 {
-    if (name == "DataConsumer")
+    if (name == YSTRING("DataConsumer"))
 	return const_cast<DataConsumer*>(this);
     return DataNode::getObject(name);
 }
@@ -502,9 +517,9 @@ bool DataConsumer::synchronize(DataSource* source)
     DDebug(DebugInfo,"Offsetting consumer %s timestamps by " FMT64 " [%p]",
 	(override ? "override" : "regular"),dt,this);
     if (override)
-	m_overrideTsDelta = dt;
+	m_overrideTsDelta = (long int)dt;
     else
-	m_regularTsDelta = dt;
+	m_regularTsDelta = (long int)dt;
     return true;
 }
 
@@ -541,7 +556,7 @@ unsigned long DataSource::Forward(const DataBlock& data, unsigned long tStamp, u
 	tStamp = m_nextStamp;
     // still no timestamp known - wild guess based on this packet size
     if (tStamp == invalidStamp()) {
-	DDebug(DebugNote,"Unknow timestamp - assuming %lu + %lu [%p]",
+	DDebug(DebugNote,"Unknown timestamp - assuming %lu + %lu [%p]",
 	    m_timestamp,nSamp,this);
 	tStamp = m_timestamp + nSamp;
     }
@@ -585,10 +600,22 @@ bool DataSource::attach(DataConsumer* consumer, bool override)
     Lock mylock(this);
     DataSource*& src = override ? consumer->m_override : consumer->m_source;
     if (src != this) {
+	s_consSrcMutex.lock();
 	RefPointer<DataSource> src2(src);
-	if (src2)
+	if (src2 && (this != src2)) {
+	    s_consSrcMutex.unlock();
 	    src2->detach(consumer);
+	    s_consSrcMutex.lock();
+	    if (src) {
+		Debug(DebugGoOn,"DataSource %p raced us attaching consumer %p [%p]",
+		    src,consumer,this);
+		s_consSrcMutex.unlock();
+		consumer->deref();
+		return false;
+	    }
+	}
 	src = this;
+	s_consSrcMutex.unlock();
     }
     consumer->synchronize(this);
     m_consumers.append(consumer);
@@ -618,10 +645,12 @@ bool DataSource::detachInternal(DataConsumer* consumer)
 	return false;
     DataConsumer *temp = static_cast<DataConsumer *>(m_consumers.remove(consumer,false));
     if (temp) {
+	s_consSrcMutex.lock();
 	if (temp->m_source == this)
 	    temp->m_source = 0;
 	if (temp->m_override == this)
 	    temp->m_override = 0;
+	s_consSrcMutex.unlock();
 	temp->deref();
 	return true;
     }
@@ -631,6 +660,7 @@ bool DataSource::detachInternal(DataConsumer* consumer)
 
 void DataSource::destroyed()
 {
+    m_translator = 0;
     clear();
     DataNode::destroyed();
 }
@@ -662,7 +692,7 @@ void DataSource::synchronize(unsigned long tStamp)
 
 void* DataSource::getObject(const String& name) const
 {
-    if (name == "DataSource")
+    if (name == YSTRING("DataSource"))
 	return const_cast<DataSource*>(this);
     return DataNode::getObject(name);
 }
@@ -695,7 +725,7 @@ void DataEndpoint::destroyed()
 
 void* DataEndpoint::getObject(const String& name) const
 {
-    if (name == "DataEndpoint")
+    if (name == YSTRING("DataEndpoint"))
 	return const_cast<DataEndpoint*>(this);
     return RefObject::getObject(name);
 }
@@ -869,8 +899,23 @@ void DataEndpoint::setConsumer(DataConsumer* consumer)
     m_consumer = consumer;
     if (source && temp)
 	DataTranslator::detachChain(source,temp);
-    if (temp)
+    if (temp) {
+	s_consSrcMutex.lock();
+	RefPointer<DataSource> src = temp->getConnSource();
+	s_consSrcMutex.unlock();
+	if (src) {
+	    src->detach(temp);
+	    src = 0;
+	}
+	s_consSrcMutex.lock();
+	src = temp->getOverSource();
+	s_consSrcMutex.unlock();
+	if (src) {
+	    src->detach(temp);
+	    src = 0;
+	}
 	temp->attached(false);
+    }
     if (consumer)
 	consumer->attached(true);
     lock.drop();
@@ -1107,7 +1152,7 @@ DataTranslator::~DataTranslator()
 
 void* DataTranslator::getObject(const String& name) const
 {
-    if (name == "DataTranslator")
+    if (name == YSTRING("DataTranslator"))
 	return const_cast<DataTranslator*>(this);
     return DataConsumer::getObject(name);
 }
@@ -1143,9 +1188,9 @@ Mutex DataTranslator::s_mutex(true,"DataTranslator");
 ObjList DataTranslator::s_factories;
 unsigned int DataTranslator::s_maxChain = 3;
 static ObjList s_compose;
-static SimpleFactory s_sFactory(s_simpleCaps);
-static SimpleFactory s_sFactory16k(s_simpleCaps16k);
-static SimpleFactory s_sFactory32k(s_simpleCaps32k);
+static SimpleFactory s_sFactory(s_simpleCaps,"g711");
+static SimpleFactory s_sFactory16k(s_simpleCaps16k,"g711wb");
+static SimpleFactory s_sFactory32k(s_simpleCaps32k,"g711uwb");
 // FIXME
 static ResampFactory s_rFactory;
 static StereoFactory s_stereoFactory;
@@ -1182,9 +1227,11 @@ void DataTranslator::compose()
 
 void DataTranslator::compose(TranslatorFactory* factory)
 {
-    XDebug(DebugInfo,"Composing TranslatorFactory %p=(%u,'%s')",
-	factory,factory->length(),factory->intermediate() ? factory->intermediate()->name : "");
     const TranslatorCaps* caps = factory->getCapabilities();
+    XDebug(DebugInfo,"Composing TranslatorFactory '%s' (%u,'%s'->'%s'->'%s')",
+	factory->name(),factory->length(),caps ? caps->src->name : "",
+	factory->intermediate() ? factory->intermediate()->name : "",
+	caps ? caps->dest->name : "");
     if ((!caps) || (factory->length() >= s_maxChain))
 	return;
     Lock lock(s_mutex);
@@ -1195,27 +1242,42 @@ void DataTranslator::compose(TranslatorFactory* factory)
 	if (f2 == factory)
 	    continue;
 	// don't try to build a too long chain
-	if ((factory->length() + f2->length()) > s_maxChain)
+	if ((factory->length() + f2->length()) > s_maxChain) {
+	    XDebug(DebugAll,"Chain would be too long");
 	    continue;
+	}
+	const FormatInfo* fi;
 	// and avoid loops
-	if (factory->intermediate(f2->intermediate()) ||
-	    f2->intermediate(factory->intermediate()))
+	if (factory->intermediate(fi = f2->intermediate()) ||
+	    f2->intermediate(fi = factory->intermediate())) {
+	    XDebug(DebugAll,"Would directly loop through format '%s'",fi->name);
 	    continue;
-	XDebug(DebugInfo,"Composing %p with %p=(%u,'%s')",
-	    factory,f2,f2->length(),f2->intermediate() ? f2->intermediate()->name : "");
+	}
 	const TranslatorCaps* c2 = f2->getCapabilities();
+	XDebug(DebugInfo,"Composing '%s' with '%s' (%u,'%s'->'%s'->'%s')",
+	    factory->name(),f2->name(),f2->length(),c2 ? c2->src->name : "",
+	    f2->intermediate() ? f2->intermediate()->name : "",
+	    c2 ? c2->dest->name : "");
 	for (; c2 && c2->src && c2->dest; c2++) {
-	    if (factory->intermediate(c2->src) || factory->intermediate(c2->dest))
+	    if (!(c2->src->converter || c2->dest->converter))
+		continue;
+	    if (factory->intermediate(fi = c2->src) || factory->intermediate(fi = c2->dest)) {
+		XDebug(DebugAll,"Factory '%s' already has intermediate '%s'",factory->name(),fi->name);
 		break;
+	    }
 	    for (const TranslatorCaps* c = caps; c->src && c->dest; c++) {
-		if (f2->intermediate(c->src) || f2->intermediate(c->dest)) {
-		    c2 = 0;
+		if (!(c->src->converter || c->dest->converter))
+		    continue;
+		if (f2->intermediate(fi = c->src) || f2->intermediate(fi = c->dest)) {
+		    XDebug(DebugAll,"Factory '%s' already has intermediate '%s'",f2->name(),fi->name);
 		    break;
 		}
 		if ((c->src == c2->dest) && c->src->converter) {
-		    if (canConvert(c2->src,c->dest))
+		    if (canConvert(c2->src,c->dest)) {
+			XDebug(DebugAll,"Can already convert '%s' to '%s'",c2->src->name,c->dest->name);
 			continue;
-		    DDebug(DebugInfo,"Building chain (%s,...)%s%s -> (%s) -> %s%s(%s,...)",
+		    }
+		    DDebug(DebugInfo,"Building chain (%s)%s%s -> (%s) -> %s%s(%s)",
 			c2->src->name,
 			f2->intermediate() ? " -> " : "",
 			f2->intermediate() ? f2->intermediate()->name : "",
@@ -1224,13 +1286,14 @@ void DataTranslator::compose(TranslatorFactory* factory)
 			factory->intermediate() ? " -> " : "",
 			c->dest->name);
 		    new ChainedFactory(f2,factory,c->src);
-		    c2 = 0;
 		    break;
 		}
 		if ((c2->src == c->dest) && c2->src->converter) {
-		    if (canConvert(c->src,c2->dest))
+		    if (canConvert(c->src,c2->dest)) {
+			XDebug(DebugAll,"Can already convert '%s' to '%s'",c->src->name,c2->dest->name);
 			continue;
-		    DDebug(DebugInfo,"Building chain (%s,...)%s%s -> (%s) -> %s%s(%s,...)",
+		    }
+		    DDebug(DebugInfo,"Building chain (%s)%s%s -> (%s) -> %s%s(%s)",
 			c->src->name,
 			factory->intermediate() ? " -> " : "",
 			factory->intermediate() ? factory->intermediate()->name : "",
@@ -1239,12 +1302,9 @@ void DataTranslator::compose(TranslatorFactory* factory)
 			f2->intermediate() ? " -> " : "",
 			c2->dest->name);
 		    new ChainedFactory(factory,f2,c->dest);
-		    c2 = 0;
 		    break;
 		}
 	    }
-	    if (!c2)
-		break;
 	}
     }
 }
@@ -1530,11 +1590,15 @@ bool DataTranslator::detachChain(DataSource* source, DataConsumer* consumer)
     if (!source || !consumer)
 	return false;
 
-    DataSource *tsource = consumer->getConnSource();
+    s_consSrcMutex.lock();
+    RefPointer<DataSource> tsource = consumer->getConnSource();
+    s_consSrcMutex.unlock();
     if (tsource) {
 	if (source->detach(consumer))
 	    return true;
-	DataTranslator *trans = tsource->getTranslator();
+	tsource->lock();
+	RefPointer<DataTranslator> trans = tsource->getTranslator();
+	tsource->unlock();
 	if (trans && detachChain(source,trans))
 	    return true;
 	Debug(DebugWarn,"DataTranslator failed to detach chain [%p] -> [%p]",source,consumer);
@@ -1583,15 +1647,17 @@ bool TranslatorFactory::converts(const DataFormat& sFormat, const DataFormat& dF
 
 
 ChainedFactory::ChainedFactory(TranslatorFactory* factory1, TranslatorFactory* factory2, const FormatInfo* info)
-    : m_factory1(factory1), m_factory2(factory2), m_format(info),
+    : TranslatorFactory("chained"),
+      m_factory1(factory1), m_factory2(factory2), m_format(info),
       m_length(factory1->length()+factory2->length()), m_capabilities(0)
 {
-    XDebug(DebugInfo,"ChainedFactory::ChainedFactory(%p=(%d,'%s'),%p=(%d,'%s'),'%s') len=%u [%p]",
-	factory1,factory1->length(),factory1->intermediate() ? factory1->intermediate()->name : "",
-	factory2,factory2->length(),factory2->intermediate() ? factory2->intermediate()->name : "",
+    XDebug(DebugInfo,"ChainedFactory::ChainedFactory('%s' (%d,'%s'), '%s' (%d,'%s'),'%s') len=%u [%p]",
+	factory1->name(),factory1->length(),factory1->intermediate() ? factory1->intermediate()->name : "",
+	factory2->name(),factory2->length(),factory2->intermediate() ? factory2->intermediate()->name : "",
 	info->name,m_length,this);
+    m_name << factory1->name() << "(" << info->name << ")" << factory2->name();
     if (!info->converter)
-	Debug(DebugMild,"Building chain factory using non-converter format '%s'",info->name);
+	Debug(DebugMild,"Building chain factory '%s' using non-converter format",m_name.c_str());
     const TranslatorCaps* cap1 = factory1->getCapabilities();
     const TranslatorCaps* cap2 = factory2->getCapabilities();
     int c1 = 0;
@@ -1639,8 +1705,8 @@ ChainedFactory::ChainedFactory(TranslatorFactory* factory1, TranslatorFactory* f
 
 ChainedFactory::~ChainedFactory()
 {
-    XDebug(DebugInfo,"ChainedFactory::~ChainedFactory() %p,%p,'%s' [%p]",
-	m_factory1,m_factory2,m_format.c_str(),this);
+    XDebug(DebugInfo,"ChainedFactory::~ChainedFactory() '%s' [%p]",
+	m_name.c_str(),this);
     delete[] const_cast<TranslatorCaps*>(m_capabilities);
     m_capabilities = 0;
 }
@@ -1690,7 +1756,8 @@ DataTranslator* ChainedFactory::create(const DataFormat& sFormat, const DataForm
 	trans1->deref();
     }
     else
-	trans->destruct();
+	// trans may be a chain itself so clear from first translator
+	trans->getFirstTranslator()->destruct();
     return trans2;
 }
 
