@@ -44,7 +44,7 @@
 __declspec(dllimport) BOOL WINAPI SHGetSpecialFolderPathA(HWND,LPSTR,INT,BOOL);
 #endif
 
-#else
+#else // _WINDOWS
 
 #include "yatepaths.h"
 #include <dirent.h>
@@ -54,9 +54,19 @@ __declspec(dllimport) BOOL WINAPI SHGetSpecialFolderPathA(HWND,LPSTR,INT,BOOL);
 typedef void* HMODULE;
 #define PATH_SEP "/"
 #ifndef CFG_DIR
+#ifdef HAVE_MACOSX_SUPPORT
+#define CFG_DIR "Yate"
+#else
 #define CFG_DIR ".yate"
 #endif
+#endif
 
+static int s_childsig = 0;
+
+#endif // _WINDOWS
+
+#ifdef HAVE_MACOSX_SUPPORT
+#include "MacOSXUtils.h"
 #endif
 
 #include <unistd.h>
@@ -118,6 +128,15 @@ using namespace TelEngine;
 #define CFG_SUFFIX ".conf"
 #endif
 
+// Initialization events capture by default
+#ifndef CAPTURE_EVENTS
+#define CAPTURE_EVENTS true
+#endif
+
+// Maximum number of engine.stop messages we allow
+#ifndef MAX_STOP
+#define MAX_STOP 5
+#endif
 
 // Supervisor control constants
 
@@ -138,6 +157,10 @@ using namespace TelEngine;
 #define HOST_NAME_MAX 255
 #endif
 
+#ifndef PATH_MAX
+#define PATH_MAX 270
+#endif
+
 static u_int64_t s_nextinit = 0;
 static u_int64_t s_restarts = 0;
 static bool s_makeworker = true;
@@ -194,6 +217,7 @@ ObjList Engine::s_extramod;
 NamedList Engine::s_params("");
 
 Engine::RunMode Engine::s_mode = Engine::Stopped;
+Engine::CallAccept Engine::s_accept = Engine::Accept;
 Engine* Engine::s_self = 0;
 int Engine::s_haltcode = -1;
 int EnginePrivate::count = 0;
@@ -205,6 +229,19 @@ static bool s_dynplugin = false;
 static Engine::PluginMode s_loadMode = Engine::LoadFail;
 static int s_maxworkers = 10;
 static bool s_debug = true;
+static bool s_capture = CAPTURE_EVENTS;
+static int s_maxevents = 10;
+static Mutex s_eventsMutex(false,"EventsList");
+static ObjList s_events;
+static String s_startMsg;
+
+const TokenDict Engine::s_callAccept[] = {
+    {"accept",      Engine::Accept},
+    {"partial",     Engine::Partial},
+    {"congestion",  Engine::Congestion},
+    {"reject",      Engine::Reject},
+    {0,0}
+};
 
 #ifdef RLIMIT_CORE
 static bool s_coredump = false;
@@ -234,6 +271,9 @@ void EngineCheck::setChecker(EngineCheck* ptr)
     s_engineCheck = ptr;
 }
 
+
+namespace { // anonymous
+
 class SLib : public String
 {
 public:
@@ -241,9 +281,10 @@ public:
     static SLib* load(const char* file, bool local, bool nounload);
     bool unload(bool doNow);
 private:
-    SLib(HMODULE handle, const char* file, bool nounload);
+    SLib(HMODULE handle, const char* file, bool nounload, unsigned int count);
     HMODULE m_handle;
     bool m_nounload;
+    unsigned int m_count;
 };
 
 class EngineSuperHandler : public MessageHandler
@@ -269,6 +310,41 @@ public:
     virtual bool received(Message &msg);
 };
 
+class EngineEventList : public ObjList
+{
+public:
+    inline EngineEventList(const char* name)
+	: m_name(name)
+	{ }
+
+    virtual const String& toString() const
+	{ return m_name; }
+private:
+    String m_name;
+};
+
+class EngineEventHandler : public MessageHandler
+{
+public:
+    EngineEventHandler() : MessageHandler("module.update",0) { }
+    virtual bool received(Message &msg);
+};
+
+class RefList : public RefObject
+{
+public:
+    inline RefList()
+	{ }
+    virtual void* getObject(const String& name) const
+	{ return (name == YSTRING("ObjList")) ? (void*)&m_list : RefObject::getObject(name); }
+    inline ObjList& list()
+	{ return m_list; }
+private:
+    ObjList m_list;
+};
+
+}; // anonymous namespace
+
 
 // helper function to initialize user application data dir
 static void initUsrPath(String& path, const char* newPath = 0)
@@ -281,6 +357,12 @@ static void initUsrPath(String& path, const char* newPath = 0)
 	char szPath[MAX_PATH];
 	if (SHGetSpecialFolderPathA(NULL,szPath,CSIDL_APPDATA,TRUE))
 	    path = szPath;
+#elif defined(HAVE_MACOSX_SUPPORT)
+	MacOSXUtils::applicationSupportPath(path);
+	if (path.null()) {
+	    Debug(DebugMild,"Could not get system user path on MacOS X, setting it to $(HOME)");
+	    path = ::getenv("HOME");
+	}
 #else
 	path = ::getenv("HOME");
 #endif
@@ -307,24 +389,84 @@ bool EngineStatusHandler::received(Message &msg)
 	return false;
     msg.retValue() << "name=engine,type=system";
     msg.retValue() << ",version=" << YATE_VERSION;
+    msg.retValue() << ",nodename=" << Engine::nodeName();
     msg.retValue() << ";plugins=" << plugins.count();
     msg.retValue() << ",inuse=" << Engine::self()->usedPlugins();
     msg.retValue() << ",handlers=" << Engine::self()->handlerCount();
     msg.retValue() << ",messages=" << Engine::self()->messageCount();
     msg.retValue() << ",supervised=" << (s_super_handle >= 0);
     msg.retValue() << ",runattempt=" << s_run_attempt;
+#ifndef _WINDOWS
+    msg.retValue() << ",lastsignal=" << s_childsig;
+#endif
     msg.retValue() << ",threads=" << Thread::count();
     msg.retValue() << ",workers=" << EnginePrivate::count;
     msg.retValue() << ",mutexes=" << Mutex::count();
     msg.retValue() << ",locks=" << Mutex::locks();
     msg.retValue() << ",semaphores=" << Semaphore::count();
     msg.retValue() << ",waiting=" << Semaphore::locks();
+    msg.retValue() << ",acceptcalls=" << lookup(Engine::accept(),Engine::getCallAcceptStates());
+    if (msg.getBoolValue("details",true)) {
+	NamedIterator iter(Engine::runParams());
+	char sep = ';';
+	while (const NamedString* p = iter.get()) {
+	    if (p->name().find("path") < 0)
+		continue;
+	    msg.retValue() << sep << p->name() << "=" << *p;
+	    sep = ',';
+	}
+    }
     msg.retValue() << "\r\n";
     return false;
 }
 
-static char s_cmdsOpt[] = "  module {{load|reload} modulefile|unload modulename|list}\r\n";
-static char s_cmdsMsg[] = "Controls the modules loaded in the Telephony Engine\r\n";
+bool EngineEventHandler::received(Message &msg)
+{
+    if (Engine::nodeName() && !msg.getParam(YSTRING("nodename")))
+	msg.addParam("nodename",Engine::nodeName());
+    const String* type = msg.getParam(YSTRING("from"));
+    if (TelEngine::null(type))
+	return false;
+    bool full = true;
+    const String* text = msg.getParam(YSTRING("fulltext"));
+    if (TelEngine::null(text)) {
+	text = msg.getParam(YSTRING("text"));
+	if (TelEngine::null(text))
+	    return false;
+	full = false;
+    }
+    String* ev = 0;
+    int level = msg.getBoolValue(YSTRING("operational"),true) ? DebugNote : DebugWarn;
+    level = msg.getIntValue(YSTRING("level"),level);
+    if (full)
+	ev = new CapturedEvent(level,*text);
+    else {
+	// build a full text with timestamp and sender
+	char tstamp[24];
+	Debugger::formatTime(tstamp);
+	ev = new CapturedEvent(level,tstamp);
+	*ev << "<" << *type << "> " << *text;
+	msg.setParam("fulltext",*ev);
+    }
+    Lock mylock(s_eventsMutex);
+    ObjList* list = static_cast<EngineEventList*>(s_events[*type]);
+    if (!list) {
+	list = new EngineEventList(*type);
+	s_events.append(list);
+    }
+    while ((s_maxevents > 0) && (int)list->count() >= s_maxevents)
+	list->remove();
+    list->append(ev);
+    return false;
+}
+
+static const char s_cmdsOptNoUnload[] = "  module {load modulefile|list}\r\n";
+static const char s_cmdsOpt[] = "  module {{load|reload} modulefile|unload modulename|list}\r\n";
+static const char s_cmdsMsg[] = "Controls the modules loaded in the Telephony Engine\r\n";
+static const char s_evtsOpt[] = "  events [clear] [type]\r\n";
+static const char s_evtsMsg[] = "Show or clear events or alarms collected since the engine startup\r\n";
+static const char s_logvOpt[] = "  logview\r\n";
+static const char s_logvMsg[] = "Show log of engine startup and initialization process\r\n";
 
 // get the base name of a module file
 static String moduleBase(const String& fname)
@@ -426,27 +568,48 @@ void completeModule(String& ret, const String& part, ObjList& mods, bool reload,
 // perform command line completion
 void EngineCommand::doCompletion(Message &msg, const String& partLine, const String& partWord)
 {
-    if (partLine.null() || (partLine == "help"))
+    if (partLine.null() || (partLine == YSTRING("help"))) {
 	completeOne(msg.retValue(),"module",partWord);
-    else if (partLine == "status")
+	completeOne(msg.retValue(),"events",partWord);
+	completeOne(msg.retValue(),"logview",partWord);
+    }
+    else if (partLine == YSTRING("status"))
 	completeOne(msg.retValue(),"engine",partWord);
-    else if (partLine == "module") {
+    else if (partLine == YSTRING("module")) {
 	completeOne(msg.retValue(),"load",partWord);
-	completeOne(msg.retValue(),"unload",partWord);
-	completeOne(msg.retValue(),"reload",partWord);
+	if (!s_nounload) {
+	    completeOne(msg.retValue(),"unload",partWord);
+	    completeOne(msg.retValue(),"reload",partWord);
+	}
 	completeOne(msg.retValue(),"list",partWord);
     }
-    else if (partLine == "module load")
+    else if (partLine == YSTRING("module load"))
 	completeModule(msg.retValue(),partWord,Engine::self()->m_libs,false);
-    else if (partLine == "module reload")
+    else if (partLine == YSTRING("module reload"))
 	completeModule(msg.retValue(),partWord,Engine::self()->m_libs,true);
-    else if (partLine == "module unload") {
+    else if (partLine == YSTRING("module unload")) {
 	for (ObjList* l = Engine::self()->m_libs.skipNull();l;l = l->skipNext()) {
 	    SLib* s = static_cast<SLib*>(l->get());
 	    if (s->unload(false))
 		completeOne(msg.retValue(),*s,partWord);
 	}
     }
+    else if (partLine == YSTRING("reload")) {
+	for (ObjList* l = plugins.skipNull(); l; l = l->skipNext()) {
+	    const Plugin* p = static_cast<const Plugin*>(l->get());
+	    completeOne(msg.retValue(),p->name(),partWord);
+	}
+    }
+    else if (partLine == YSTRING("events") || partLine == YSTRING("events clear")) {
+	Lock mylock(s_eventsMutex);
+	for (ObjList* l = s_events.skipNull(); l; l = l->skipNext()) {
+	    const EngineEventList* e = static_cast<const EngineEventList*>(l->get());
+	    completeOne(msg.retValue(),e->toString(),partWord);
+	}
+	completeOne(msg.retValue(),"log",partWord);
+	if (partLine == YSTRING("events"))
+	    completeOne(msg.retValue(),"clear",partWord);
+    }
 }
 
 bool EngineCommand::received(Message &msg)
@@ -456,8 +619,30 @@ bool EngineCommand::received(Message &msg)
 	doCompletion(msg,msg.getValue("partline"),msg.getValue("partword"));
 	return false;
     }
-    if (!line.startSkip("module"))
+    if (!line.startSkip("module")) {
+	if (line.startSkip("events") || (line == "logview" && (line.clear(),true))) {
+	    bool clear = line.startSkip("clear");
+	    line.startSkip("log");
+	    if (clear) {
+		Engine::clearEvents(line);
+		return true;
+	    }
+	    RefList* list = 0;
+	    int cnt = 0;
+	    for (const ObjList* l = Engine::events(line); l; l = l->skipNext()) {
+		const CapturedEvent* ev = static_cast<const CapturedEvent*>(l->get());
+		if (!list)
+		    list = new RefList();
+		list->list().append(new CapturedEvent(*ev));
+		cnt++;
+	    }
+	    msg.userData(list);
+	    TelEngine::destruct(list);
+	    (msg.retValue() = "Events: ") << cnt << "\r\n";
+	    return true;
+	}
 	return false;
+    }
 
     bool ok = false;
     int sep = line.find(' ');
@@ -520,14 +705,18 @@ bool EngineCommand::received(Message &msg)
 
 bool EngineHelp::received(Message &msg)
 {
+    const char* opts = (s_nounload ? s_cmdsOptNoUnload : s_cmdsOpt);
     String line = msg.getValue("line");
     if (line.null()) {
-	msg.retValue() << s_cmdsOpt;
+	msg.retValue() << opts << s_evtsOpt << s_logvOpt;
 	return false;
     }
-    if (line != "module")
-	return false;
-    msg.retValue() << s_cmdsOpt << s_cmdsMsg;
+    if (line == YSTRING("module"))
+	msg.retValue() << opts << s_cmdsMsg;
+    else if (line == YSTRING("events"))
+	msg.retValue() << s_evtsOpt << s_evtsMsg;
+    else if (line == YSTRING("logview"))
+	msg.retValue() << s_logvOpt << s_logvMsg;
     return true;
 }
 
@@ -545,7 +734,7 @@ void EnginePrivate::run()
 static bool logFileOpen()
 {
     if (s_logfile) {
-	int flags = O_WRONLY|O_CREAT;
+	int flags = O_WRONLY|O_CREAT|O_LARGEFILE;
 	if (s_logtruncate) {
 	    s_logtruncate = false;
 	    flags |= O_TRUNC;
@@ -566,12 +755,17 @@ static bool logFileOpen()
     }
     return false;
 }
-
-static int engineRun()
+static int engineRun(EngineLoop loop = 0)
 {
     time_t t = ::time(0);
-    Output("Yate (%u) is starting %s",::getpid(),::ctime(&t));
-    int retcode = Engine::self()->run();
+    s_startMsg << "Yate (" << ::getpid() << ") is starting " << ::ctime(&t);
+    s_startMsg.trimSpaces();
+    Output("%s",s_startMsg.c_str());
+    int retcode = Engine::self()->engineInit();
+    if (!retcode)
+	retcode = (loop ? loop() : Engine::self()->run());
+    if (!retcode)
+	retcode = Engine::self()->engineCleanup();
     t = ::time(0);
     Output("Yate (%u) is stopping %s",::getpid(),::ctime(&t));
     return retcode;
@@ -718,6 +912,7 @@ static int supervise(void)
     ::signal(SIGABRT,superhandler);
     ::signal(SIGUSR1,superhandler);
     ::signal(SIGUSR2,superhandler);
+    ::signal(SIGALRM,SIG_IGN);
     int retcode = 0;
     while (s_runagain) {
 	int wdogfd[2];
@@ -778,14 +973,17 @@ static int supervise(void)
 		// Child exited for some reason
 		if (WIFEXITED(status)) {
 		    retcode = WEXITSTATUS(status);
+		    ::fprintf(stderr,"Supervisor: child %d exited with code %d\n",s_childpid,retcode);
 		    if (retcode <= 127)
 			s_runagain = false;
 		    else
 			retcode &= 127;
+		    s_childsig = 0;
 		}
 		else if (WIFSIGNALED(status)) {
 		    retcode = WTERMSIG(status);
 		    ::fprintf(stderr,"Supervisor: child %d died on signal %d\n",s_childpid,retcode);
+		    s_childsig = retcode;
 		}
 		s_childpid = -1;
 		break;
@@ -818,8 +1016,13 @@ static int supervise(void)
 	if (s_childpid > 0) {
 	    // Child failed to proof sanity. Kill it - no need to be gentle.
 	    ::fprintf(stderr,"Supervisor: killing unresponsive child %d\n",s_childpid);
+#ifdef RLIMIT_CORE
+	    // If -Da or -C were specified try to get a corefile
+	    if (s_sigabrt || s_coredump) {
+#else
 	    // If -Da was specified try to get a corefile
 	    if (s_sigabrt) {
+#endif
 		::kill(s_childpid,SIGABRT);
 		::usleep(500000);
 	    }
@@ -833,6 +1036,7 @@ static int supervise(void)
 		    break;
 	    }
 	    s_childpid = -1;
+	    s_childsig = SIGCHLD;
 	}
 	if (s_logrotator) {
 	    copystream(2,logfd[0]);
@@ -855,19 +1059,21 @@ static int supervise(void)
 #endif /* _WINDOWS */
 
 
-SLib::SLib(HMODULE handle, const char* file, bool nounload)
-    : String(moduleBase(file)), m_handle(handle), m_nounload(nounload)
+SLib::SLib(HMODULE handle, const char* file, bool nounload, unsigned int count)
+    : String(moduleBase(file)),
+      m_handle(handle), m_nounload(nounload), m_count(count)
 {
-    DDebug(DebugAll,"SLib::SLib(%p,'%s',%s) [%p]",
-	handle,file,String::boolText(nounload),this);
+    DDebug(DebugAll,"SLib::SLib(%p,'%s',%s,%u) [%p]",
+	handle,file,String::boolText(nounload),count,this);
     checkPoint();
 }
 
 SLib::~SLib()
 {
 #ifdef DEBUG
-    Debugger debug("SLib::~SLib()"," '%s' [%p]",c_str(),this);
+    Debugger debug("SLib::~SLib()"," '%s' %u [%p]",c_str(),m_count,this);
 #endif
+    unsigned int count = plugins.count();
     if (s_nounload || m_nounload) {
 #ifdef _WINDOWS
 	typedef void (WINAPI *pFini)(HINSTANCE,DWORD,LPVOID);
@@ -883,22 +1089,31 @@ SLib::~SLib()
 	    fini();
 #endif
 	if (fini || m_nounload) {
+	    count -= plugins.count();
+	    if (count != m_count)
+		Debug(DebugGoOn,"Finalizing '%s' removed %u out of %u plugins",
+		    c_str(),count,m_count);
 	    checkPoint();
 	    return;
 	}
-	Debug(DebugWarn,"Could not finalize, will dlclose(%p)",m_handle);
+	Debug(DebugWarn,"Could not finalize '%s', will dlclose(%p)",c_str(),m_handle);
     }
     int err = dlclose(m_handle);
     if (err)
-	Debug(DebugGoOn,"Error %d on dlclose(%p)",err,m_handle);
+	Debug(DebugGoOn,"Error %d on dlclose(%p) of '%s'",err,m_handle,c_str());
     else if (s_keepclosing) {
 	int tries;
 	for (tries=0; tries<10; tries++)
 	    if (dlclose(m_handle))
 		break;
 	if (tries)
-	    Debug(DebugGoOn,"Made %d attempts to dlclose(%p)",tries,m_handle);
+	    Debug(DebugGoOn,"Made %d attempts to dlclose(%p) '%s'",
+		tries,m_handle,c_str());
     }
+    count -= plugins.count();
+    if (count != m_count)
+	Debug(DebugGoOn,"Unloading '%s' removed %u out of %u plugins",
+	    c_str(),count,m_count);
     checkPoint();
 }
 
@@ -910,14 +1125,15 @@ SLib* SLib::load(const char* file, bool local, bool nounload)
     if (!local)
 	flags |= RTLD_GLOBAL;
 #endif
+    unsigned int count = plugins.count();
     HMODULE handle = ::dlopen(file,flags);
     if (handle)
-	return new SLib(handle,file,nounload);
+	return new SLib(handle,file,nounload,plugins.count() - count);
 #ifdef _WINDOWS
     Debug(DebugWarn,"LoadLibrary error %u in '%s'",::GetLastError(),file);
 #else
     Debug(DebugWarn,"%s",dlerror());
-#endif    
+#endif
     return 0;
 }
 
@@ -940,17 +1156,18 @@ Engine::Engine()
 Engine::~Engine()
 {
 #ifdef DEBUG
-    Debugger debug("Engine::~Engine()"," [%p]",this);
+    Debugger debug("Engine::~Engine()"," libs=%u plugins=%u [%p]",
+	m_libs.count(),plugins.count(),this);
 #endif
     assert(this == s_self);
     m_dispatcher.clear();
-    plugins.clear();
     m_libs.clear();
+    s_events.clear();
     s_mode = Stopped;
     s_self = 0;
 }
 
-int Engine::run()
+int Engine::engineInit()
 {
 #ifdef _WINDOWS
     // In Windows we must initialize the socket library very early because even trivial
@@ -963,9 +1180,15 @@ int Engine::run()
     }
 #else
     ::signal(SIGPIPE,SIG_IGN);
+    ::signal(SIGALRM,SIG_IGN);
 #endif
+    CapturedEvent::capturing(s_capture);
     s_cfg = configFile(s_cfgfile);
     s_cfg.load();
+    s_capture = s_cfg.getBoolValue("general","startevents",s_capture);
+    CapturedEvent::capturing(s_capture);
+    if (s_capture && s_startMsg)
+	CapturedEvent::append(-1,s_startMsg);
 #ifdef _WINDOWS
     int winTimerRes = s_cfg.getIntValue("general","wintimer");
     if ((winTimerRes > 0) && (winTimerRes < 100)) {
@@ -989,6 +1212,7 @@ int Engine::run()
 	    Debug(DebugWarn,"Timer resolution not set: %s",err.c_str());
     }
 #endif
+    Thread::idleMsec(s_cfg.getIntValue("general","idlemsec",(clientMode() ? 2 * Thread::idleMsec() : 0)));
     SysUsage::init();
 
     s_runid = Time::secNow();
@@ -1003,6 +1227,7 @@ int Engine::run()
     if (modPath)
 	s_modpath = modPath;
     s_maxworkers = s_cfg.getIntValue("general","maxworkers",s_maxworkers);
+    s_maxevents = s_cfg.getIntValue("general","maxevents",s_maxevents);
     s_restarts = s_cfg.getIntValue("general","restarts");
     m_dispatcher.warnTime(1000*(u_int64_t)s_cfg.getIntValue("general","warntime"));
     extraPath(clientMode() ? "client" : "server");
@@ -1024,13 +1249,33 @@ int Engine::run()
     s_params.addParam("clientmode",String::boolText(clientMode()));
     s_params.addParam("supervised",String::boolText(s_super_handle >= 0));
     s_params.addParam("runattempt",String(s_run_attempt));
+#ifndef _WINDOWS
+    s_params.addParam("lastsignal",String(s_childsig));
+#endif
     s_params.addParam("maxworkers",String(s_maxworkers));
+    s_params.addParam("maxevents",String(s_maxevents));
+#ifdef _WINDOWS
+    {
+	char buf[PATH_MAX];
+	DWORD ret = ::GetCurrentDirectoryA(PATH_MAX,buf);
+	if (ret && (ret < PATH_MAX))
+	    s_params.addParam("workpath",buf);
+    }
+#elif defined (HAVE_GETCWD)
+    {
+	char buf[PATH_MAX];
+	if (::getcwd(buf,PATH_MAX))
+	    s_params.addParam("workpath",buf);
+    }
+#endif
     DDebug(DebugAll,"Engine::run()");
     install(new EngineStatusHandler);
+    install(new EngineEventHandler);
     install(new EngineCommand);
     install(new EngineHelp);
     loadPlugins();
     Debug(DebugAll,"Loaded %d plugins",plugins.count());
+    internalStatisticsStart();
     if (s_super_handle >= 0) {
 	install(new EngineSuperHandler);
 	if (s_restarts)
@@ -1045,10 +1290,8 @@ int Engine::run()
     ::signal(SIGINT,sighandler);
     ::signal(SIGTERM,sighandler);
     Debug(DebugAll,"Engine dispatching start message");
-    if (dispatch("engine.start"))
-	Debug(DebugGoOn,"Message engine.start was unexpectedly handled!");
+    dispatch("engine.start",true);
     setStatus(SERVICE_RUNNING);
-    long corr = 0;
 #ifndef _WINDOWS
     ::signal(SIGHUP,sighandler);
     ::signal(SIGQUIT,sighandler);
@@ -1056,9 +1299,25 @@ int Engine::run()
     ::signal(SIGUSR1,sighandler);
     ::signal(SIGUSR2,sighandler);
 #endif
+    if (s_startMsg) {
+	Message* m = new Message("module.update");
+	m->addParam("fulltext",s_startMsg);
+	if (nodeName())
+	    m->addParam("nodename",nodeName());
+	enqueue(m);
+    }
     Output("Yate%s engine is initialized and starting up%s%s",
 	clientMode() ? " client" : "",s_node.null() ? "" : " on " ,s_node.safe());
-    while (s_haltcode == -1) {
+
+    return 0;
+}
+
+int Engine::run()
+{
+    // engine loop
+    long corr = 0;
+    int stops = MAX_STOP;
+    while (s_haltcode == -1 || ((--stops >= 0) && dispatch("engine.stop",true))) {
 	if (s_cmds) {
 	    Output("Executing initial commands");
 	    for (ObjList* c = s_cmds->skipNull(); c; c=c->skipNext()) {
@@ -1097,6 +1356,11 @@ int Engine::run()
 		}
 	    }
 	}
+	else if (s_capture) {
+	    // end capturing startup messages
+	    s_capture = false;
+	    CapturedEvent::capturing(false);
+	}
 
 	// Create worker thread if we didn't hear about any of them in a while
 	if (s_makeworker && (EnginePrivate::count < s_maxworkers)) {
@@ -1124,24 +1388,33 @@ int Engine::run()
 	    t += 1000000;
 	XDebug(DebugAll,"Sleeping for %ld",t);
 	Thread::usleep(t);
-	Message *m = new Message("engine.timer");
+	Message* m = new Message("engine.timer",0,true);
 	m->addParam("time",String((int)m->msgTime().sec()));
-	// Try to fine tune the ticker
-	t = (long)(m->msgTime().usec() % 1000000);
-	if (t > 500000)
-	    corr -= (1000000-t)/10;
-	else
-	    corr += t/10;
-	XDebug(DebugAll,"Adjustment at %ld, corr %ld",t,corr);
+	if (nodeName())
+	    m->addParam("nodename",nodeName());
+	if (s_haltcode == -1) {
+	    // Try to fine tune the ticker unless exiting
+	    t = (long)(m->msgTime().usec() % 1000000);
+	    if (t > 500000)
+		corr -= (1000000-t)/10;
+	    else
+		corr += t/10;
+	    XDebug(DebugAll,"Adjustment at %ld, corr %ld",t,corr);
+	}
 	enqueue(m);
 	Thread::yield();
     }
     s_haltcode &= 0xff;
+    return 0;
+}
+
+int Engine::engineCleanup()
+{
     Output("Yate engine is shutting down with code %d",s_haltcode);
+    CapturedEvent::capturing(false);
     setStatus(SERVICE_STOP_PENDING);
     ::signal(SIGINT,SIG_DFL);
-    if (dispatch("engine.halt"))
-	Debug(DebugGoOn,"Message engine.halt was unexpectedly handled!");
+    dispatch("engine.halt",true);
     checkPoint();
     Thread::msleep(200);
     m_dispatcher.dequeue();
@@ -1157,7 +1430,11 @@ int Engine::run()
     ::signal(SIGQUIT,SIG_DFL);
 #endif
     delete this;
-    Debug(DebugAll,"Exiting with %d locked mutexes",Mutex::locks());
+    int mux = Mutex::locks();
+    unsigned int cnt = plugins.count();
+    plugins.clear();
+    if (mux || cnt)
+	Debug(DebugGoOn,"Exiting with %d locked mutexes and %u plugins loaded!",mux,cnt);
 #ifdef _WINDOWS
     ::WSACleanup();
 #endif
@@ -1165,6 +1442,12 @@ int Engine::run()
     return s_haltcode;
 }
 
+void Engine::internalStatisticsStart()
+{
+    // This is here so runtime analyzers can start or reset statistics
+    //  after the cruft of module load + global objects initialization
+}
+
 Engine* Engine::self()
 {
     if (!s_self)
@@ -1265,6 +1548,17 @@ bool Engine::loadPluginDir(const String& relPath)
 	    path += PATH_SEP;
 	path += relPath;
     }
+    if (path.endsWith(s_modsuffix)) {
+	int sep = path.rfind(PATH_SEP[0]);
+	if (sep >= 0)
+	    sep++;
+	else
+	    sep = 0;
+	String name = path.substr(sep);
+	if (loadPlugin(path,s_cfg.getBoolValue("localsym",name,s_localsymbol),
+	    s_cfg.getBoolValue("nounload",name)))
+	    return true;
+    }
     if (path.endsWith(PATH_SEP))
 	path = path.substr(0,path.length()-1);
 #ifdef _WINDOWS
@@ -1313,8 +1607,11 @@ void Engine::loadPlugins()
         unsigned int len = l->length();
         for (unsigned int i=0; i<len; i++) {
             NamedString *n = l->getParam(i);
-            if (n && n->toBoolean())
-                loadPlugin(n->name());
+            if (n && n->toBoolean()) {
+        	String path(n->name());
+        	s_params.replaceParams(path);
+                loadPlugin(path);
+            }
 	    if (exiting())
 		break;
 	}
@@ -1331,8 +1628,11 @@ void Engine::loadPlugins()
 	    if (exiting())
 		return;
             NamedString *n = l->getParam(i);
-            if (n && n->toBoolean())
-                loadPlugin(n->name());
+            if (n && n->toBoolean()) {
+        	String path(n->name());
+        	s_params.replaceParams(path);
+                loadPlugin(path);
+	    }
 	}
     }
 }
@@ -1342,8 +1642,7 @@ void Engine::initPlugins()
     if (exiting())
 	return;
     Output("Initializing plugins");
-    if (dispatch("engine.init"))
-	Debug(DebugGoOn,"Message engine.init was unexpectedly handled!");
+    dispatch("engine.init",true);
     ObjList *l = plugins.skipNull();
     for (; l; l = l->skipNext()) {
 	Plugin *p = static_cast<Plugin *>(l->get());
@@ -1407,6 +1706,28 @@ void Engine::init()
     s_init = true;
 }
 
+bool Engine::init(const String& name)
+{
+    if (exiting() || !s_self)
+	return false;
+    if (name.null() || name == "*" || name == "all") {
+	s_init = true;
+	return true;
+    }
+    Output("Initializing plugin '%s'",name.c_str());
+    Message msg("engine.init",0,true);
+    msg.addParam("plugin",name);
+    if (nodeName())
+	msg.addParam("nodename",nodeName());
+    bool ok = s_self->m_dispatcher.dispatch(msg);
+    Plugin* p = static_cast<Plugin*>(plugins[name]);
+    if (p) {
+	p->initialize();
+	ok = true;
+    }
+    return ok;
+}
+
 bool Engine::install(MessageHandler* handler)
 {
     return s_self ? s_self->m_dispatcher.install(handler) : false;
@@ -1432,11 +1753,13 @@ bool Engine::dispatch(Message& msg)
     return s_self ? s_self->m_dispatcher.dispatch(msg) : false;
 }
 
-bool Engine::dispatch(const char* name)
+bool Engine::dispatch(const char* name, bool broadcast)
 {
     if (!(s_self && name && *name))
 	return false;
-    Message msg(name);
+    Message msg(name,0,broadcast);
+    if (nodeName())
+	msg.addParam("nodename",nodeName());
     return s_self->m_dispatcher.dispatch(msg);
 }
 
@@ -1445,6 +1768,25 @@ unsigned int Engine::runId()
     return s_runid;
 }
 
+const ObjList* Engine::events(const String& type)
+{
+    if (type.null())
+	return CapturedEvent::events().skipNull();
+    Lock mylock(s_eventsMutex);
+    const ObjList* list = static_cast<const EngineEventList*>(s_events[type]);
+    return list ? list->skipNull() : 0;
+}
+
+void Engine::clearEvents(const String& type)
+{
+    Lock mylock(s_eventsMutex);
+    if (type.null())
+	CapturedEvent::eventsRw().clear();
+    else
+	s_events.remove(type);
+}
+
+
 static void usage(bool client, FILE* f)
 {
     ::fprintf(f,
@@ -1474,6 +1816,7 @@ static void usage(bool client, FILE* f)
 "   -D[options]    Special debugging options\n"
 "     a            Abort if bugs are encountered\n"
 "     m            Attempt to debug mutex deadlocks\n"
+"     d            Disable locking debugging and safety features\n"
 #ifdef RTLD_GLOBAL
 "     l            Try to keep module symbols local\n"
 #endif
@@ -1487,6 +1830,7 @@ static void usage(bool client, FILE* f)
 "     t            Timestamp debugging messages relative to program start\n"
 "     e            Timestamp debugging messages based on EPOCH (1-1-1970 GMT)\n"
 "     f            Timestamp debugging in GMT format YYYYMMDDhhmmss.uuuuuu\n"
+"     z            Timestamp debugging in local timezone YYYYMMDDhhmmss.uuuuuu\n"
     ,client ? "" :
 #ifdef _WINDOWS
 "   --service      Run as Windows service\n"
@@ -1521,7 +1865,7 @@ static void version()
     ::fprintf(stdout,"Yate " YATE_VERSION " " YATE_STATUS YATE_RELEASE "\n");
 }
 
-int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bool fail)
+int Engine::main(int argc, const char** argv, const char** env, RunMode mode, EngineLoop loop, bool fail)
 {
 #ifdef _WINDOWS
     int service = 0;
@@ -1553,6 +1897,9 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
     for (i=1;i<argc;i++) {
 	const char *pc = argv[i];
 	if (inopt && (pc[0] == '-') && pc[1]) {
+	    // skip over Mac OS X process serial number
+	    if (!::strncmp(pc,"-psn_",5))
+		continue;
 	    while (pc && *++pc) {
 		switch (*pc) {
 		    case '-':
@@ -1729,6 +2076,9 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 					Lockable::wait(lockWait);
 				    }
 				    break;
+				case 'd':
+				    Lockable::disableSafety();
+				    break;
 #ifdef RTLD_GLOBAL
 				case 'l':
 				    s_localsymbol = true;
@@ -1761,6 +2111,9 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 				case 'f':
 				    tstamp = Debugger::Textual;
 				    break;
+				case 'z':
+				    tstamp = Debugger::TextLocal;
+				    break;
 				default:
 				    initUsrPath(s_usrpath);
 				    badopt(client,*pc,argv[i]);
@@ -1970,7 +2323,7 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
     }
     else
 #endif
-	retcode = engineRun();
+	retcode = engineRun(loop);
 
     return retcode;
 }
