@@ -35,7 +35,7 @@ typedef HANDLE HSEMAPHORE;
 
 #ifdef MUTEX_HACK
 extern "C" {
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__)
 extern int pthread_mutexattr_settype(pthread_mutexattr_t *__attr, int __kind);
 #define PTHREAD_MUTEX_RECURSIVE_NP PTHREAD_MUTEX_RECURSIVE
 #else
@@ -131,6 +131,7 @@ HMUTEX GlobalMutex::s_mutex;
 static GlobalMutex s_global;
 static unsigned long s_maxwait = 0;
 static bool s_unsafe = MUTEX_STATIC_UNSAFE;
+static bool s_safety = true;
 
 volatile int MutexPrivate::s_count = 0;
 volatile int MutexPrivate::s_locks = 0;
@@ -217,7 +218,8 @@ MutexPrivate::~MutexPrivate()
     if (m_locked) {
 	warn = true;
 	m_locked--;
-	s_locks--;
+	if (s_safety)
+	    s_locks--;
 #ifdef _WINDOWS
 	::ReleaseMutex(m_mutex);
 #else
@@ -248,13 +250,15 @@ bool MutexPrivate::lock(long maxwait)
 	maxwait = (long)s_maxwait;
 	warn = true;
     }
-    GlobalMutex::lock();
-    ref();
-    m_waiting++;
+    if (s_safety)
+	GlobalMutex::lock();
     Thread* thr = Thread::current();
     if (thr)
 	thr->m_locking = true;
-    GlobalMutex::unlock();
+    if (s_safety) {
+	m_waiting++;
+	GlobalMutex::unlock();
+    }
 #ifdef _WINDOWS
     DWORD ms = 0;
     if (maxwait < 0)
@@ -295,12 +299,15 @@ bool MutexPrivate::lock(long maxwait)
 #endif // HAVE_TIMEDLOCK
     }
 #endif // _WINDOWS
-    GlobalMutex::lock();
+    if (s_safety) {
+	GlobalMutex::lock();
+	m_waiting--;
+    }
     if (thr)
 	thr->m_locking = false;
-    m_waiting--;
     if (rval) {
-	s_locks++;
+	if (s_safety)
+	    s_locks++;
 	m_locked++;
 	if (thr) {
 	    thr->m_locks++;
@@ -309,9 +316,8 @@ bool MutexPrivate::lock(long maxwait)
 	else
 	    m_owner = 0;
     }
-    else
-	deref();
-    GlobalMutex::unlock();
+    if (s_safety)
+	GlobalMutex::unlock();
     if (warn && !rval)
 	Debug(DebugFail,"Thread '%s' could not lock mutex '%s' owned by '%s' waited by %u others for %lu usec!",
 	    Thread::currentName(),m_name,m_owner,m_waiting,maxwait);
@@ -322,7 +328,8 @@ bool MutexPrivate::unlock()
 {
     bool ok = false;
     // Hope we don't hit a bug related to the debug mutex!
-    GlobalMutex::lock();
+    if (s_safety)
+	GlobalMutex::lock();
     if (m_locked) {
 	Thread* thr = Thread::current();
 	if (thr)
@@ -334,25 +341,27 @@ bool MutexPrivate::unlock()
 		    m_name,tname,m_owner,this);
 	    m_owner = 0;
 	}
-	int locks = --s_locks;
-	if (locks < 0) {
-	    // this is very very bad - abort right now
-	    abortOnBug(true);
-	    s_locks = 0;
-	    Debug(DebugFail,"MutexPrivate::locks() is %d [%p]",locks,this);
+	if (s_safety) {
+	    int locks = --s_locks;
+	    if (locks < 0) {
+		// this is very very bad - abort right now
+		abortOnBug(true);
+		s_locks = 0;
+		Debug(DebugFail,"MutexPrivate::locks() is %d [%p]",locks,this);
+	    }
 	}
-	if (!s_unsafe)
 #ifdef _WINDOWS
-	    ::ReleaseMutex(m_mutex);
+	ok = s_unsafe || ::ReleaseMutex(m_mutex);
 #else
-	    ::pthread_mutex_unlock(&m_mutex);
+	ok = s_unsafe || !::pthread_mutex_unlock(&m_mutex);
 #endif
-	deref();
-	ok = true;
+	if (!ok)
+	    Debug(DebugFail,"Failed to unlock mutex '%s' [%p]",m_name,this);
     }
     else
 	Debug(DebugFail,"MutexPrivate::unlock called on unlocked '%s' [%p]",m_name,this);
-    GlobalMutex::unlock();
+    if (s_safety)
+	GlobalMutex::unlock();
     return ok;
 }
 
@@ -395,14 +404,16 @@ bool SemaphorePrivate::lock(long maxwait)
 	maxwait = (long)s_maxwait;
 	warn = true;
     }
-    GlobalMutex::lock();
-    ref();
-    s_locks++;
-    m_waiting++;
+    if (s_safety)
+	GlobalMutex::lock();
     Thread* thr = Thread::current();
     if (thr)
 	thr->m_locking = true;
-    GlobalMutex::unlock();
+    if (s_safety) {
+	s_locks++;
+	m_waiting++;
+	GlobalMutex::unlock();
+    }
 #ifdef _WINDOWS
     DWORD ms = 0;
     if (maxwait < 0)
@@ -443,19 +454,21 @@ bool SemaphorePrivate::lock(long maxwait)
 #endif // HAVE_TIMEDWAIT
     }
 #endif // _WINDOWS
-    GlobalMutex::lock();
-    m_waiting--;
-    int locks = --s_locks;
-    if (locks < 0) {
-	// this is very very bad - abort right now
-	abortOnBug(true);
-	s_locks = 0;
-	Debug(DebugFail,"SemaphorePrivate::locks() is %d [%p]",locks,this);
+    if (s_safety) {
+	GlobalMutex::lock();
+	int locks = --s_locks;
+	if (locks < 0) {
+	    // this is very very bad - abort right now
+	    abortOnBug(true);
+	    s_locks = 0;
+	    Debug(DebugFail,"SemaphorePrivate::locks() is %d [%p]",locks,this);
+	}
+	m_waiting--;
     }
     if (thr)
 	thr->m_locking = false;
-    deref();
-    GlobalMutex::unlock();
+    if (s_safety)
+	GlobalMutex::unlock();
     if (warn && !rval)
 	Debug(DebugFail,"Thread '%s' could not lock semaphore '%s' waited by %u others for %lu usec!",
 	    Thread::currentName(),m_name,m_waiting,maxwait);
@@ -464,9 +477,9 @@ bool SemaphorePrivate::lock(long maxwait)
 
 bool SemaphorePrivate::unlock()
 {
-    GlobalMutex::lock();
-    ref();
     if (!s_unsafe) {
+	if (s_safety)
+	    GlobalMutex::lock();
 #ifdef _WINDOWS
 	::ReleaseSemaphore(m_semaphore,1,NULL);
 #else
@@ -474,9 +487,9 @@ bool SemaphorePrivate::unlock()
 	if (!::sem_getvalue(&m_semaphore,&val) && (val < (int)m_maxcount))
 	    ::sem_post(&m_semaphore);
 #endif
+	if (s_safety)
+	    GlobalMutex::unlock();
     }
-    deref();
-    GlobalMutex::unlock();
     return true;
 }
 
@@ -508,6 +521,11 @@ void Lockable::startUsingNow()
     s_unsafe = false;
 }
 
+void Lockable::disableSafety()
+{
+    s_safety = false;
+}
+
 void Lockable::wait(unsigned long maxwait)
 {
     s_maxwait = maxwait;
@@ -518,6 +536,7 @@ unsigned long Lockable::wait()
     return s_maxwait;
 }
 
+
 Mutex::Mutex(bool recursive, const char* name)
     : m_private(0)
 {
@@ -601,6 +620,31 @@ bool Mutex::efficientTimedLock()
 }
 
 
+MutexPool::MutexPool(unsigned int len, bool recursive, const char* name)
+    : m_name(0), m_data(0), m_length(len ? len : 1)
+{
+    if (TelEngine::null(name))
+	name = "Pool";
+    m_name = new String[m_length];
+    for (unsigned int i = 0; i < m_length; i++)
+	m_name[i] << name << "::" << (i + 1);
+    m_data = new Mutex*[m_length];
+    for (unsigned int i = 0; i < m_length; i++)
+	m_data[i] = new Mutex(recursive,m_name[i]);
+}
+
+MutexPool::~MutexPool()
+{
+    if (m_data) {
+	for (unsigned int i = 0; i < m_length; i++)
+	    delete m_data[i];
+	delete[] m_data;
+    }
+    if (m_name)
+	delete[] m_name;
+}
+
+
 Semaphore::Semaphore(unsigned int maxcount, const char* name)
     : m_private(0)
 {
