@@ -33,12 +33,15 @@
 
 #include "yateclass.h"
 
-#ifdef HAVE_SCTP_NETINET
-#include <netinet/sctp.h>
-#endif
-
 #include <string.h>
 
+#ifdef HAVE_POLL
+#include <poll.h>
+#ifndef POLLRDHUP
+#define POLLRDHUP 0
+#endif
+#endif
+
 #undef HAS_AF_UNIX
 
 #ifndef _WINDOWS
@@ -125,13 +128,12 @@ SocketAddr::~SocketAddr()
 
 void SocketAddr::clear()
 {
-    m_host.clear();
     m_length = 0;
-    if (m_address) {
-	void* tmp = m_address;
-	m_address = 0;
+    m_host.clear();
+    void* tmp = m_address;
+    m_address = 0;
+    if (tmp)
 	::free(tmp);
-    }
 }
 
 bool SocketAddr::assign(int family)
@@ -165,7 +167,6 @@ void SocketAddr::assign(const struct sockaddr* addr, socklen_t len)
 {
     if (addr == m_address)
 	return;
-    clear();
     if (addr && !len) {
 	switch (addr->sa_family) {
 	    case AF_INET:
@@ -183,6 +184,9 @@ void SocketAddr::assign(const struct sockaddr* addr, socklen_t len)
 #endif
 	}
     }
+    if (addr && m_address && (len == m_length) && !::memcmp(addr,m_address,len))
+	return;
+    clear();
     if (addr && (len >= (socklen_t)sizeof(struct sockaddr))) {
 	void* tmp = ::malloc(len);
 	::memcpy(tmp,addr,len);
@@ -271,6 +275,9 @@ bool SocketAddr::host(const String& name)
 
 void SocketAddr::stringify()
 {
+    m_host.clear();
+    if (!(m_length && m_address))
+	return;
     switch (family()) {
 	case AF_INET:
 #ifdef HAVE_NTOP
@@ -383,7 +390,7 @@ SocketFilter::~SocketFilter()
 
 void* SocketFilter::getObject(const String& name) const
 {
-    if (name == "SocketFilter")
+    if (name == YSTRING("SocketFilter"))
 	return const_cast<SocketFilter*>(this);
     return GenObject::getObject(name);
 }
@@ -407,6 +414,11 @@ bool Stream::canRetry() const
     return false;
 }
 
+bool Stream::inProgress() const
+{
+    return false;
+}
+
 bool Stream::setBlocking(bool block)
 {
     return false;
@@ -852,7 +864,7 @@ bool File::createPipe(File& reader, File& writer)
     return false;
 }
 
-// Retrive the file's modification time (the file must be already opened)
+// Retrieve the file's modification time (the file must be already opened)
 bool File::getFileTime(unsigned int& secEpoch)
 {
 #ifdef _WINDOWS
@@ -1123,6 +1135,7 @@ bool File::listDirectory(const char* path, ObjList* dirs, ObjList* files, int* e
 	*error = code;
     ::FindClose(hFind);
 #else
+    errno = 0;
     DIR* dir = ::opendir(path);
     if (!dir) {
 	if (!errno)
@@ -1309,6 +1322,23 @@ bool Socket::canRetry() const
 #endif
 }
 
+// Check if the last error code indicates a non blocking operation in progress
+bool Socket::inProgress() const
+{
+    if (!valid())
+	return false;
+#ifdef _WINDOWS
+    return m_error == WSAEWOULDBLOCK;
+#else
+    return m_error == EINPROGRESS;
+#endif
+}
+
+bool Socket::canSelect() const
+{
+    return canSelect(m_handle);
+}
+
 bool Socket::bind(struct sockaddr* addr, socklen_t addrlen)
 {
     return checkError(::bind(m_handle,addr,addrlen));
@@ -1349,26 +1379,16 @@ SOCKET Socket::acceptHandle(struct sockaddr* addr, socklen_t* addrlen)
     return res;
 }
 
-Socket* Socket::peelOff(unsigned int assoc)
+// Update socket error from socket options
+bool Socket::updateError()
 {
-    SOCKET sock = peelOffHandle(assoc);
-    return (sock == invalidHandle()) ? 0 : new Socket(sock);
-}
-
-SOCKET Socket::peelOffHandle(unsigned int assoc)
-{
-#ifdef SCTP_SOCKOPT_PEELOFF
-    sctp_peeloff_arg_t buffer;
-    buffer.associd = assoc;
-    buffer.sd = invalidHandle();
-    socklen_t length = sizeof(buffer);
-    if (!getOption(SOL_SCTP, SCTP_SOCKOPT_PEELOFF, &buffer, &length))
-	return invalidHandle();
-    return buffer.sd;
-#else
-    Debug(DebugMild,"Socket::peelOffHandle() not supported on this platform");
-    return invalidHandle();
-#endif
+    int error = 0;
+    socklen_t len = sizeof(error);
+    if (getOption(SOL_SOCKET,SO_ERROR,&error,&len)) {
+	m_error = error;
+	return true;
+    }
+    return false;
 }
 
 bool Socket::connect(struct sockaddr* addr, socklen_t addrlen)
@@ -1513,15 +1533,26 @@ int Socket::readData(void* buffer, int length)
 #endif
 }
 
+bool Socket::efficientSelect()
+{
+#if defined(_WINDOWS) || defined(HAVE_POLL)
+    return true;
+#else
+    return false;
+#endif
+}
+
 bool Socket::canSelect(SOCKET handle)
 {
     if (handle == invalidHandle())
 	return false;
 #ifdef FD_SETSIZE
 #ifndef _WINDOWS
+#ifndef HAVE_POLL
     if (handle >= (SOCKET)FD_SETSIZE)
 	return false;
 #endif
+#endif
 #endif
     return true;
 }
@@ -1531,6 +1562,33 @@ bool Socket::select(bool* readok, bool* writeok, bool* except, struct timeval* t
     SOCKET tmp = m_handle;
     if (!valid())
 	return false;
+
+#ifdef HAVE_POLL
+    struct pollfd fds;
+    fds.fd = tmp;
+    fds.events = 0;
+    fds.revents = 0;
+    if (readok)
+	fds.events |= POLLIN;
+    if (writeok)
+	fds.events |= POLLOUT;
+    if (except)
+	fds.events |= POLLRDHUP;
+    int tout = -1;
+    if (timeout)
+	tout = (timeout->tv_sec * 1000) + (timeout->tv_usec / 1000);
+    if (checkError(::poll(&fds,1,tout),true)) {
+	if (readok)
+	    *readok = (fds.revents & POLLIN) != 0;
+	if (writeok)
+	    *writeok = (fds.revents & POLLOUT) != 0;
+	if (except)
+	    *except = (fds.revents & (POLLRDHUP|POLLERR|POLLHUP|POLLNVAL)) != 0;
+	return true;
+    }
+
+#else // HAVE_POLL
+
 #ifdef FD_SETSIZE
 #ifndef _WINDOWS
     static bool localFail = true;
@@ -1572,6 +1630,8 @@ bool Socket::select(bool* readok, bool* writeok, bool* except, struct timeval* t
 	    *except = (FD_ISSET(tmp,efds) != 0);
 	return true;
     }
+#endif // HAVE_POLL
+
     if (tmp != m_handle) {
 	if (except)
 	    *except = true;
@@ -1724,4 +1784,9 @@ void Socket::timerTick(const Time& when)
     }
 }
 
+
+SctpSocket::~SctpSocket()
+{
+}
+
 /* vi: set ts=8 sw=4 sts=4 noet: */
