@@ -22,7 +22,6 @@
 
 #include "yateclass.h"
 
-#include <limits.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
@@ -417,7 +416,8 @@ String String::substr(int offs, int len) const
     return String(c_str()+offs,len);
 }
 
-int String::toInteger(int defvalue, int base) const
+int String::toInteger(int defvalue, int base, int minvalue, int maxvalue,
+    bool clamp) const
 {
     if (!m_string)
 	return defvalue;
@@ -425,7 +425,11 @@ int String::toInteger(int defvalue, int base) const
     int val = strtoi(m_string,&eptr,base);
     if (!eptr || *eptr)
 	return defvalue;
-    return val;
+    if (val >= minvalue && val <= maxvalue)
+	return val;
+    if (clamp)
+	return (val < minvalue) ? minvalue : maxvalue;
+    return defvalue;
 }
 
 int String::toInteger(const TokenDict* tokens, int defvalue, int base) const
@@ -440,6 +444,27 @@ int String::toInteger(const TokenDict* tokens, int defvalue, int base) const
     return toInteger(defvalue,base);
 }
 
+long int String::toLong(long int defvalue, int base, long int minvalue, long int maxvalue,
+    bool clamp) const
+{
+    if (!m_string)
+	return defvalue;
+    char *eptr = 0;
+
+    errno = 0;
+    long int val = ::strtol(m_string,&eptr,base);
+    // on overflow/underflow mark the entire string as unreadable
+    if ((errno == ERANGE) && eptr)
+	eptr = m_string;
+    if (!eptr || *eptr)
+	return defvalue;
+    if (val >= minvalue && val <= maxvalue)
+	return val;
+    if (clamp)
+	return (val < minvalue) ? minvalue : maxvalue;
+    return defvalue;
+}
+
 double String::toDouble(double defvalue) const
 {
     if (!m_string)
@@ -558,12 +583,15 @@ String& String::operator+=(const char* value)
 	value = 0;
     if (value) {
 	if (m_string) {
-	    int len = ::strlen(value)+length();
+	    int olen = length();
+	    int len = ::strlen(value)+olen;
 	    char *tmp1 = m_string;
 	    char *tmp2 = (char *) ::malloc(len+1);
 	    if (tmp2) {
-		::strcpy(tmp2,m_string);
-		::strcat(tmp2,value);
+		::strncpy(tmp2,m_string,olen);
+		tmp2[olen] = 0;
+		::strncat(tmp2,value,len-olen);
+		tmp2[len] = 0;
 		m_string = tmp2;
 		::free(tmp1);
 	    }
@@ -711,11 +739,42 @@ String& String::append(const char* value, const char* separator, bool force)
 
 String& String::append(const ObjList* list, const char* separator, bool force)
 {
-    for (; list; list = list->next()) {
-	const GenObject* obj = list->get();
-	if (obj)
-	    append(obj->toString().c_str(),separator,force);
+    if (!list)
+	return *this;
+    int olen = length();
+    int sepLen = 0;
+    if (!TelEngine::null(separator))
+	sepLen = ::strlen(separator);
+    int len = 0;
+    for (ObjList* o = list->skipNull(); o; o = o->skipNext()) {
+	const String& src = o->get()->toString();
+	if (sepLen && (len || olen) && (src.length() || force))
+	    len += sepLen;
+	len += src.length();
     }
+    if (!len)
+	return *this;
+    char* oldStr = m_string;
+    char* newStr = (char*)::malloc(olen + len + 1);
+    if (!newStr) {
+	Debug("String",DebugFail,"malloc(%d) returned NULL!",olen + len + 1);
+	return *this;
+    }
+    if (m_string)
+	::memcpy(newStr,m_string,olen);
+    for (list = list->skipNull(); list; list = list->skipNext()) {
+	const String& src = list->get()->toString();
+	if (sepLen && olen && (src.length() || force)) {
+	    ::memcpy(newStr + olen,separator,sepLen);
+	    olen += sepLen;
+	}
+	::memcpy(newStr + olen,src.c_str(),src.length());
+	olen += src.length();
+    }
+    newStr[olen] = 0;
+    m_string = newStr;
+    ::free(oldStr);
+    changed();
     return *this;
 }
 
@@ -723,10 +782,8 @@ String& String::append(double value, unsigned int decimals)
 {
     if (decimals > 12)
 	decimals = 12;
-    char fmt[8];
-    ::sprintf(fmt,"%%0.%uf",decimals);
     char buf[80];
-    ::sprintf(buf,fmt,value);
+    ::sprintf(buf,"%0.*f",decimals,value);
     return operator+=(buf);
 }
 
@@ -838,7 +895,53 @@ bool String::endsWith(const char* what, bool wordBreak, bool caseInsensitive) co
     return (::strncmp(m_string+m_length-l,what,l) == 0);
 }
 
-bool String::matches(Regexp& rexp)
+String& String::extractTo(const char* sep, String& str)
+{
+    int pos = find(sep);
+    if (pos >= 0) {
+	str = substr(0,pos);
+	assign(m_string+pos+::strlen(sep));
+    }
+    else {
+	str = *this;
+	clear();
+    }
+    return *this;
+}
+
+String& String::extractTo(const char* sep, bool& store)
+{
+    String str;
+    extractTo(sep,str);
+    store = str.toBoolean(store);
+    return *this;
+}
+
+String& String::extractTo(const char* sep, int& store, int base)
+{
+    String str;
+    extractTo(sep,str);
+    store = str.toInteger(store,base);
+    return *this;
+}
+
+String& String::extractTo(const char* sep, int& store, const TokenDict* tokens, int base)
+{
+    String str;
+    extractTo(sep,str);
+    store = str.toInteger(tokens,store,base);
+    return *this;
+}
+
+String& String::extractTo(const char* sep, double& store)
+{
+    String str;
+    extractTo(sep,str);
+    store = str.toDouble(store);
+    return *this;
+}
+
+bool String::matches(const Regexp& rexp)
 {
     if (m_matches)
 	clearMatches();
@@ -1051,8 +1154,9 @@ unsigned int String::hash(const char* value)
 	return 0;
 
     unsigned int h = 0;
+    // sdbm hash algorithm, hash(i) = hash(i-1) * 65599 + str[i]
     while (unsigned char c = (unsigned char) *value++)
-	h = (h << 1) + c;
+	h = (h << 6) + (h << 16) - h + c;
     return h;
 }
 
@@ -1238,7 +1342,7 @@ int String::fixUtf8(const char* replace, unsigned int maxSeq, bool overlong)
 
 void* String::getObject(const String& name) const
 {
-    if (name == "String")
+    if (name == YSTRING("String"))
 	return const_cast<String*>(this);
     return GenObject::getObject(name);
 }
@@ -1275,7 +1379,7 @@ Regexp::~Regexp()
     cleanup();
 }
 
-bool Regexp::matches(const char* value, StringMatchPrivate* matchlist)
+bool Regexp::matches(const char* value, StringMatchPrivate* matchlist) const
 {
     XDebug(DebugInfo,"Regexp::matches(\"%s\",%p)",value,matchlist);
     if (!value)
@@ -1289,8 +1393,7 @@ bool Regexp::matches(const char* value, StringMatchPrivate* matchlist)
 
 bool Regexp::matches(const char* value) const
 {
-    // this cast is wrong but so handy...
-    return const_cast<Regexp*>(this)->matches(value,0);
+    return matches(value,0);
 }
 
 void Regexp::changed()
@@ -1299,7 +1402,7 @@ void Regexp::changed()
     String::changed();
 }
 
-bool Regexp::compile()
+bool Regexp::compile() const
 {
     XDebug(DebugInfo,"Regexp::compile()");
     if (c_str() && !m_regexp) {
@@ -1363,7 +1466,7 @@ const String& NamedString::toString() const
 
 void* NamedString::getObject(const String& name) const
 {
-    if (name == "NamedString")
+    if (name == YSTRING("NamedString"))
 	return (void*)this;
     return String::getObject(name);
 }
@@ -1388,7 +1491,7 @@ void NamedPointer::userData(GenObject* data)
     m_data = data;
 }
 
-// Retrive the pointer carried by this object and release ownership
+// Retrieve the pointer carried by this object and release ownership
 GenObject* NamedPointer::takeData()
 {
     GenObject* tmp = m_data;
@@ -1398,7 +1501,7 @@ GenObject* NamedPointer::takeData()
 
 void* NamedPointer::getObject(const String& name) const
 {
-    if (name == "NamedPointer")
+    if (name == YSTRING("NamedPointer"))
 	return (void*)this;
     void* p = NamedString::getObject(name);
     if (p)
