@@ -30,22 +30,11 @@
 
 
 #ifdef _WINDOWS
-extern "C" {
-
-static u_int32_t s_randSeed = (u_int32_t)(TelEngine::Time::now() & 0xffffffff);
-
-long int random()
-{ return (s_randSeed = (s_randSeed + 1) * 0x8088405) % RAND_MAX; }
-
-void srandom(unsigned int seed)
-{ s_randSeed = seed % RAND_MAX; }
-
-}
 
 #ifndef HAVE_GMTIME_S
 #include <errno.h>
 
-int _gmtime_s(struct tm* _tm, const time_t* time)
+static int _gmtime_s(struct tm* _tm, const time_t* time)
 {
     static TelEngine::Mutex m(false,"_gmtime_s");
     struct tm* tmp;
@@ -66,9 +55,30 @@ int _gmtime_s(struct tm* _tm, const time_t* time)
     return 0;
 }
 
+static int _localtime_s(struct tm* _tm, const time_t* time)
+{
+    static TelEngine::Mutex m(false,"_localtime_s");
+    struct tm* tmp;
+    if (!_tm)
+	return EINVAL;
+    _tm->tm_isdst = _tm->tm_yday = _tm->tm_wday = _tm->tm_year = _tm->tm_mon = _tm->tm_mday =
+	_tm->tm_hour = _tm->tm_min = _tm->tm_sec = -1;
+    if (!time)
+	return EINVAL;
+    m.lock();
+    tmp = localtime(time);
+    if (!tmp) {
+	m.unlock();
+	return EINVAL;
+    }
+    *_tm = *tmp;
+    m.unlock();
+    return 0;
+}
+
 #endif
 
-#else
+#else // !_WINDOWS
 #include <sys/resource.h>
 #endif
 
@@ -76,10 +86,16 @@ int _gmtime_s(struct tm* _tm, const time_t* time)
 namespace TelEngine {
 
 #define DebugMin DebugFail
+#define DebugVis DebugConf
 #define DebugMax DebugAll
 
 #define OUT_BUFFER_SIZE 8192
 
+// RefObject mutex pool array size
+#ifndef REFOBJECT_MUTEX_COUNT
+#define REFOBJECT_MUTEX_COUNT 47
+#endif
+
 static int s_debug = DebugWarn;
 static int s_indent = 0;
 static bool s_debugging = true;
@@ -90,9 +106,9 @@ static Debugger::Formatting s_fmtstamp = Debugger::None;
 
 static const char* const s_colors[11] = {
     "\033[5;41;1;33m\033[K",// DebugFail - blinking yellow on red
-    "\033[41;1;33m\033[K",  // Unnamed   - yellow on red
+    "\033[44;1;37m\033[K",  // DebugTest - white on blue
     "\033[41;1;37m\033[K",  // DebugGoOn - white on red
-    "\033[41;37m\033[K",    // Unnamed   - gray on red
+    "\033[41;37m\033[K",    // DebugConf - gray on red
     "\033[40;31m\033[K",    // DebugStub - red on black
     "\033[40;1;31m\033[K",  // DebugWarn - light red on black
     "\033[40;1;33m\033[K",  // DebugMild - yellow on black
@@ -104,9 +120,9 @@ static const char* const s_colors[11] = {
 
 static const char* const s_levels[11] = {
     "FAIL",
-    "FAIL",
-    "GOON",
+    "TEST",
     "GOON",
+    "CONF",
     "STUB",
     "WARN",
     "MILD",
@@ -146,6 +162,9 @@ static Mutex out_mux(false,"DebugOutput");
 static Mutex ind_mux(false,"DebugIndent");
 static Thread* s_thr = 0;
 
+bool CapturedEvent::s_capturing = false;
+ObjList CapturedEvent::s_events;
+
 static bool reentered()
 {
     if (!s_thr)
@@ -162,12 +181,16 @@ static void common_output(int level,char* buf)
     int n = ::strlen(buf);
     if (n && (buf[n-1] == '\n'))
 	    n--;
-    buf[n] = '\n';
-    buf[n+1] = '\0';
     // serialize the output strings
     out_mux.lock();
     // TODO: detect reentrant calls from foreign threads and main thread
     s_thr = Thread::current();
+    if (CapturedEvent::capturing()) {
+	buf[n] = '\0';
+	CapturedEvent::append(level,buf);
+    }
+    buf[n] = '\n';
+    buf[n+1] = '\0';
     if (s_output)
 	s_output(buf,level);
     if (s_intout)
@@ -181,29 +204,7 @@ static void dbg_output(int level,const char* prefix, const char* format, va_list
     if (!(s_output || s_intout))
 	return;
     char buf[OUT_BUFFER_SIZE];
-    unsigned int n = 0;
-    if (s_fmtstamp != Debugger::None) {
-	u_int64_t t = Time::now();
-	if (s_fmtstamp == Debugger::Relative)
-	    t -= s_timestamp;
-	unsigned int s = (unsigned int)(t / 1000000);
-	unsigned int u = (unsigned int)(t % 1000000);
-	if (s_fmtstamp == Debugger::Textual) {
-	    time_t sec = (time_t)s;
-	    struct tm tmp;
-#ifdef _WINDOWS
-	    _gmtime_s(&tmp,&sec);
-#else
-	    gmtime_r(&sec,&tmp);
-#endif
-	    ::sprintf(buf,"%04d%02d%02d%02d%02d%02d.%06u ",
-		tmp.tm_year+1900,tmp.tm_mon+1,tmp.tm_mday,
-		tmp.tm_hour,tmp.tm_min,tmp.tm_sec,u);
-	}
-	else
-	    ::sprintf(buf,"%07u.%06u ",s,u);
-	n = ::strlen(buf);
-    }
+    unsigned int n = Debugger::formatTime(buf,s_fmtstamp);
     unsigned int l = s_indent*2;
     if (l >= sizeof(buf)-n)
 	l = sizeof(buf)-n-1;
@@ -324,7 +325,7 @@ bool abortOnBug(bool doAbort)
     bool tmp = s_abort;
     s_abort = doAbort;
     return tmp;
-}  
+}
 
 int debugLevel()
 {
@@ -333,8 +334,8 @@ int debugLevel()
 
 int debugLevel(int level)
 {
-    if (level < DebugMin)
-	level = DebugMin;
+    if (level < DebugVis)
+	level = DebugVis;
     if (level > DebugMax)
 	level = DebugMax;
     return (s_debug = level);
@@ -356,8 +357,8 @@ const char* debugColor(int level)
 
 int DebugEnabler::debugLevel(int level)
 {
-    if (level < DebugMin)
-	level = DebugMin;
+    if (level < DebugVis)
+	level = DebugVis;
     if (level > DebugMax)
 	level = DebugMax;
     m_chain = 0;
@@ -460,6 +461,11 @@ void Debugger::enableOutput(bool enable, bool colorize)
 	setOutput(dbg_colorize_func);
 }
 
+Debugger::Formatting Debugger::getFormatting()
+{
+    return s_fmtstamp;
+}
+
 void Debugger::setFormatting(Formatting format)
 {
     // start stamp will be rounded to full second
@@ -467,6 +473,42 @@ void Debugger::setFormatting(Formatting format)
     s_fmtstamp = format;
 }
 
+unsigned int Debugger::formatTime(char* buf, Formatting format)
+{
+    if (!buf)
+	return 0;
+    if (None != format) {
+	u_int64_t t = Time::now();
+	if (Relative == format)
+	    t -= s_timestamp;
+	unsigned int s = (unsigned int)(t / 1000000);
+	unsigned int u = (unsigned int)(t % 1000000);
+	if (Textual == format || TextLocal == format) {
+	    time_t sec = (time_t)s;
+	    struct tm tmp;
+	    if (TextLocal == format)
+#ifdef _WINDOWS
+		_localtime_s(&tmp,&sec);
+#else
+		localtime_r(&sec,&tmp);
+#endif
+	    else
+#ifdef _WINDOWS
+		_gmtime_s(&tmp,&sec);
+#else
+		gmtime_r(&sec,&tmp);
+#endif
+	    ::sprintf(buf,"%04d%02d%02d%02d%02d%02d.%06u ",
+		tmp.tm_year+1900,tmp.tm_mon+1,tmp.tm_mday,
+		tmp.tm_hour,tmp.tm_min,tmp.tm_sec,u);
+	}
+	else
+	    ::sprintf(buf,"%07u.%06u ",s,u);
+	return ::strlen(buf);
+    }
+    buf[0] = '\0';
+    return 0;
+}
 
 
 u_int64_t Time::now()
@@ -524,7 +566,7 @@ void Time::toTimeval(struct timeval* tv, u_int64_t usec)
 unsigned int Time::toEpoch(int year, unsigned int month, unsigned int day,
 	unsigned int hour, unsigned int minute, unsigned int sec, int offset)
 {
-    Debug(DebugAll,"Time::toEpoch(%d,%u,%u,%u,%u,%u,%d)",
+    DDebug(DebugAll,"Time::toEpoch(%d,%u,%u,%u,%u,%u,%d)",
 	year,month,day,hour,minute,sec,offset);
     if (year < 1970)
 	return (unsigned int)-1;
@@ -593,11 +635,34 @@ bool Time::toDateTime(unsigned int epochTimeSec, int& year, unsigned int& month,
     minute = t.tm_min;
     sec = t.tm_sec;
 #endif
-    Debug(DebugAll,"Time::toDateTime(%u,%d,%u,%u,%u,%u,%u)",
+    DDebug(DebugAll,"Time::toDateTime(%u,%d,%u,%u,%u,%u,%u)",
 	epochTimeSec,year,month,day,hour,minute,sec);
     return true;
 }
 
+static Random s_random;
+static Mutex s_randomMutex(false,"Random");
+
+u_int32_t Random::next()
+{
+    return (m_random = (m_random + 1) * 0x8088405);
+}
+
+long int Random::random()
+{
+    s_randomMutex.lock();
+    long int ret = s_random.next() % RAND_MAX;
+    s_randomMutex.unlock();
+    return ret;
+}
+
+void Random::srandom(unsigned int seed)
+{
+    s_randomMutex.lock();
+    s_random.set(seed % RAND_MAX);
+    s_randomMutex.unlock();
+}
+
 
 bool GenObject::alive() const
 {
@@ -610,7 +675,17 @@ void GenObject::destruct()
 }
 
 
-static Mutex s_refmutex(false,"RefObject");
+#ifndef ATOMIC_OPS
+static MutexPool s_refMutex(REFOBJECT_MUTEX_COUNT,false,"RefObject");
+#endif
+
+RefObject::RefObject()
+    : m_refcount(1), m_mutex(0)
+{
+#ifndef ATOMIC_OPS
+    m_mutex = s_refMutex.mutex(this);
+#endif
+}
 
 RefObject::~RefObject()
 {
@@ -618,6 +693,12 @@ RefObject::~RefObject()
 	Debug(DebugFail,"RefObject [%p] destroyed with count=%d",this,m_refcount);
 }
 
+void* RefObject::getObject(const String& name) const
+{
+    if (name == YSTRING("RefObject"))
+	return (void*)this;
+    return GenObject::getObject(name);
+}
 bool RefObject::alive() const
 {
     return m_refcount > 0;
@@ -628,32 +709,48 @@ void RefObject::destruct()
     deref();
 }
 
-bool RefObject::refInternal()
+bool RefObject::ref()
 {
+#ifdef ATOMIC_OPS
+#ifdef _WINDOWS
+    if (InterlockedIncrement((LONG*)&m_refcount) > 1)
+	return true;
+    InterlockedDecrement((LONG*)&m_refcount);
+#else
+    if (__sync_add_and_fetch(&m_refcount,1) > 1)
+	return true;
+    __sync_sub_and_fetch(&m_refcount,1);
+#endif
+#else
+    Lock lock(m_mutex);
     if (m_refcount > 0) {
 	++m_refcount;
 	return true;
     }
+#endif
     return false;
 }
 
-bool RefObject::ref()
-{
-    Lock lock(s_refmutex);
-    return refInternal();
-}
-
 bool RefObject::deref()
 {
-    bool zeroCall = false;
-    s_refmutex.lock();
+#ifdef ATOMIC_OPS
+#ifdef _WINDOWS
+    int i = InterlockedDecrement((LONG*)&m_refcount) + 1;
+    if (i <= 0)
+	InterlockedIncrement((LONG*)&m_refcount);
+#else
+    int i = __sync_fetch_and_sub(&m_refcount,1);
+    if (i <= 0)
+	__sync_fetch_and_add(&m_refcount,1);
+#endif
+#else
+    m_mutex->lock();
     int i = m_refcount;
     if (i > 0)
 	--m_refcount;
+    m_mutex->unlock();
+#endif
     if (i == 1)
-	zeroCall = zeroRefsTest();
-    s_refmutex.unlock();
-    if (zeroCall)
 	zeroRefs();
     else if (i <= 0)
 	Debug(DebugFail,"RefObject::deref() called with count=%d [%p]",i,this);
@@ -666,28 +763,41 @@ void RefObject::zeroRefs()
     delete this;
 }
 
-bool RefObject::zeroRefsTest()
-{
-    return true;
-}
-
 bool RefObject::resurrect()
 {
-    s_refmutex.lock();
+#ifdef ATOMIC_OPS
+#ifdef _WINDOWS
+    if (InterlockedIncrement((LONG*)&m_refcount) == 1)
+	return true;
+    InterlockedDecrement((LONG*)&m_refcount);
+    return false;
+#else
+    if (__sync_add_and_fetch(&m_refcount,1) == 1)
+	return true;
+    __sync_sub_and_fetch(&m_refcount,1);
+    return false;
+#endif
+#else
+    m_mutex->lock();
     bool ret = (0 == m_refcount);
     if (ret)
 	m_refcount = 1;
-    s_refmutex.unlock();
+    m_mutex->unlock();
     return ret;
+#endif
 }
 
 void RefObject::destroyed()
 {
 }
 
-Mutex& RefObject::refMutex()
+bool RefObject::efficientIncDec()
 {
-    return s_refmutex;
+#ifdef ATOMIC_OPS
+    return true;
+#else
+    return false;
+#endif
 }
 
 void RefPointerBase::assign(RefObject* oldptr, RefObject* newptr, void* pointer)
