@@ -51,6 +51,8 @@ static int pthread_attr_setinheritsched(pthread_attr_t *,int) { return 0; }
 #ifndef THREAD_IDLE_MSEC
 #define THREAD_IDLE_MSEC 5
 #endif
+#define THREAD_IDLE_MIN  1
+#define THREAD_IDLE_MAX 20
 
 namespace TelEngine {
 
@@ -87,7 +89,7 @@ public:
 
 using namespace TelEngine;
 
-#define SOFT_KILLS 5
+#define SOFT_WAITS 3
 #define HARD_KILLS 5
 #define KILL_WAIT 32
 
@@ -132,6 +134,7 @@ static TokenDict s_prio[] = {
     { 0, 0 }
 };
 
+static unsigned long s_idleMs = 1000 * THREAD_IDLE_MSEC;
 static ObjList s_threads;
 static Mutex s_tmutex(true,"Thread");
 
@@ -425,13 +428,46 @@ void ThreadPrivate::killall()
     Debugger debug("ThreadPrivate::killall()");
     ThreadPrivate *t;
     bool sledgehammer = false;
-    int c = 1;
     s_tmutex.lock();
-    ObjList *l = &s_threads;
+    ThreadPrivate* crt = ThreadPrivate::current();
+    int c = s_threads.count();
+    if (crt)
+	Debug(DebugNote,"Thread '%s' is soft cancelling other %d running threads",crt->m_name,c-1);
+    else
+	Debug(DebugNote,"Soft cancelling %d running threads",c);
+    ObjList* l = &s_threads;
     while (l && (t = static_cast<ThreadPrivate *>(l->get())) != 0)
     {
+	if (t != crt) {
+	    Debug(DebugInfo,"Stopping ThreadPrivate '%s' [%p]",t->m_name,t);
+	    t->cancel(false);
+	}
+	l = l->next();
+    }
+    for (int w = 0; w < SOFT_WAITS; w++) {
+	s_tmutex.unlock();
+	Thread::idle();
+	s_tmutex.lock();
+	c = s_threads.count();
+	// ignore the current thread if we have one
+	if (crt && c)
+	    c--;
+	if (!c) {
+	    s_tmutex.unlock();
+	    return;
+	}
+    }
+    Debug(DebugMild,"Hard cancelling %d remaining threads",c);
+    l = &s_threads;
+    c = 1;
+    while (l && (t = static_cast<ThreadPrivate *>(l->get())) != 0)
+    {
+	if (t == crt) {
+	    l = l->next();
+	    continue;
+	}
 	Debug(DebugInfo,"Trying to kill ThreadPrivate '%s' [%p], attempt %d",t->m_name,t,c);
-	bool ok = t->cancel(c > SOFT_KILLS);
+	bool ok = t->cancel(true);
 	if (ok) {
 	    int d = 0;
 	    // delay a little (exponentially) so threads have a chance to clean up
@@ -463,7 +499,7 @@ void ThreadPrivate::killall()
 		continue;
 	    }
 	    Thread::msleep(1);
-	    if (++c >= (SOFT_KILLS+HARD_KILLS)) {
+	    if (++c >= HARD_KILLS) {
 		Debug(DebugGoOn,"Could not kill %p, will use sledgehammer later.",t);
 		sledgehammer = true;
 		t->m_thread = 0;
@@ -599,8 +635,7 @@ void Thread::cleanup()
 
 void Thread::killall()
 {
-    if (!ThreadPrivate::current())
-	ThreadPrivate::killall();
+    ThreadPrivate::killall();
 }
 
 void Thread::exit()
@@ -659,7 +694,7 @@ void Thread::idle(bool exitCheck)
 	Debug(DebugMild,"Thread '%s' idling with %d mutex locks held [%p]",
 	    t->name(),t->locks(),t);
 #endif
-    msleep(THREAD_IDLE_MSEC,exitCheck);
+    msleep(s_idleMs,exitCheck);
 }
 
 void Thread::sleep(unsigned int sec, bool exitCheck)
@@ -702,12 +737,23 @@ void Thread::usleep(unsigned long usec, bool exitCheck)
 
 unsigned long Thread::idleUsec()
 {
-    return THREAD_IDLE_MSEC * 1000;
+    return s_idleMs * 1000;
 }
 
 unsigned long Thread::idleMsec()
 {
-    return THREAD_IDLE_MSEC;
+    return s_idleMs;
+}
+
+void Thread::idleMsec(unsigned long msec)
+{
+    if (msec == 0)
+	msec = THREAD_IDLE_MSEC;
+    else if (msec < THREAD_IDLE_MIN)
+	msec = THREAD_IDLE_MIN;
+    else if (msec > THREAD_IDLE_MAX)
+	msec = THREAD_IDLE_MAX;
+    s_idleMs = msec;
 }
 
 Thread::Priority Thread::priority(const char* name, Thread::Priority defvalue)
