@@ -30,9 +30,29 @@
 
 using namespace TelEngine;
 
+// Local call number to set when rejecting calls with missing call token
+#define IAX2_CALLTOKEN_REJ_CALLNO 1
+// Local call number to set when sending call token message
+#define IAX2_CALLTOKEN_CALLNO 1
+// Minimum value for local call numbers
+#define IAX2_MIN_CALLNO 2
+
+// Build an MD5 digest from secret, address, integer value and engine run id
+// MD5(addr.host() + secret + addr.port() + t)
+static void buildSecretDigest(String& buf, const String& secret, unsigned int t,
+    const SocketAddr& addr)
+{
+    String tmp;
+    tmp << addr.host() << secret << addr.port() << t;
+    MD5 md5(tmp);
+    buf << md5.hexDigest();
+}
+
+
 IAXEngine::IAXEngine(const char* iface, int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
 	u_int16_t authTimeout, u_int16_t transTimeout, u_int16_t maxFullFrameDataLen,
-	u_int32_t format, u_int32_t capab, u_int32_t trunkSendInterval, bool authRequired)
+	u_int32_t format, u_int32_t capab, u_int32_t trunkSendInterval, bool authRequired,
+	NamedList* params)
     : Mutex(true,"IAXEngine"),
     m_lastGetEvIndex(0),
     m_authRequired(authRequired),
@@ -43,7 +63,12 @@ IAXEngine::IAXEngine(const char* iface, int port, u_int16_t transListCount, u_in
     m_retransInterval(retransInterval),
     m_authTimeout(authTimeout),
     m_transTimeout(transTimeout),
+    m_callToken(false),
+    m_callTokenAge(10),
+    m_showCallTokenFailures(false),
+    m_printMsg(true),
     m_format(format),
+    m_formatVideo(0),
     m_capability(capab),
     m_mutexTrunk(true,"IAXEngine::Trunk"),
     m_trunkSendInterval(trunkSendInterval)
@@ -64,14 +89,45 @@ IAXEngine::IAXEngine(const char* iface, int port, u_int16_t transListCount, u_in
     m_transListCount = transListCount;
     for(i = 0; i <= IAX2_MAX_CALLNO; i++)
 	m_lUsedCallNo[i] = false;
+    if (params)
+	m_callTokenSecret = params->getValue("calltoken_secret");
+    if (!m_callTokenSecret)
+	for (i = 0; i < 3; i++)
+	    m_callTokenSecret << (int)(Random::random() ^ Time::now());
     m_socket.create(AF_INET,SOCK_DGRAM);
     SocketAddr addr(AF_INET);
     addr.host(iface);
     addr.port(port);
     m_socket.setBlocking(false);
-    if (!m_socket.bind(addr))
-	Debug(this,DebugWarn,"Failed to bind socket to %s:%d",c_safe(iface),port);
-    m_startLocalCallNo = 1 + (u_int16_t)(random() % IAX2_MAX_CALLNO);
+    bool ok = m_socket.bind(addr);
+    if (!ok) {
+	bool force = !params || params->getBoolValue("force_bind",true);
+	String tmp;
+	Thread::errorString(tmp,m_socket.error());
+	Debug(this,DebugWarn,"Failed to bind socket on '%s:%d'%s. %d: '%s'",
+	    c_safe(iface),port,force ? " - trying a random port" : "",
+	    m_socket.error(),tmp.c_str());
+	if (force) {
+	    addr.port(0);
+	    ok = m_socket.bind(addr);
+	    if (!ok)
+		Debug(this,DebugWarn,"Failed to bind on any port");
+	    else {
+		ok = m_socket.getSockName(addr);
+		if (!ok)
+		    Debug(this,DebugWarn,"Failed to retrieve bound address");
+	    }
+	}
+    }
+    if (ok)
+	Debug(this,DebugInfo,"Bound on '%s:%d'",addr.host().c_str(),addr.port());
+    m_startLocalCallNo = 1 + (u_int16_t)(Random::random() % IAX2_MAX_CALLNO);
+    if (m_startLocalCallNo < IAX2_MIN_CALLNO)
+	m_startLocalCallNo = IAX2_MIN_CALLNO;
+    if (params)
+	initialize(*params);
+    else
+	initialize(NamedList::empty());
 }
 
 IAXEngine::~IAXEngine()
@@ -90,12 +146,29 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
     Lock lock(this);
     // Transaction exists for this frame?
     // Incomplete transactions. They MUST receive a full frame
-    if (frame->fullFrame()) {
+    IAXFullFrame* fullFrame = frame->fullFrame();
+    if (fullFrame) {
 	l = m_incompleteTransList.skipNull();
 	for (; l; l = l->next()) {
 	    tr = static_cast<IAXTransaction*>(l->get());
-	    if (!(tr && tr->localCallNo() == frame->fullFrame()->destCallNo() && addr == tr->remoteAddr()))
+	    if (!(tr && tr->localCallNo() == fullFrame->destCallNo() && addr == tr->remoteAddr()))
 		continue;
+	    // Incomplete outgoing receiving call token
+	    if (fullFrame->type() == IAXFrame::IAX &&
+		fullFrame->subclass() == IAXControl::CallToken) {
+		RefPointer<IAXTransaction> t = tr;
+		lock.drop();
+		if (!t)
+		    return 0;
+		fullFrame->updateIEList(true);
+		IAXIEList* list = fullFrame->ieList();
+		DataBlock db;
+		if (list)
+		    list->getBinary(IAXInfoElement::CALLTOKEN,db);
+		t->processCallToken(db);
+		t = 0;
+		return 0;
+	    }
 	    // Complete transaction
 	    if (tr->processFrame(frame)) {
 		tr->m_rCallNo = frame->sourceCallNo();
@@ -115,7 +188,7 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
 	if (!(tr && tr->remoteCallNo() == frame->sourceCallNo()))
 	    continue;
 	// Mini frame
-	if (!frame->fullFrame()) {
+	if (!fullFrame) {
 	    if (addr == tr->remoteAddr()) {
 		// keep transaction referenced but unlock the engine
 		RefPointer<IAXTransaction> t = tr;
@@ -126,7 +199,7 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
 	}
 	// Full frame
 	// Has a local number assigned? If not, test socket
-	if ((frame->fullFrame())->destCallNo() || addr == tr->remoteAddr()) {
+	if (fullFrame->destCallNo() || addr == tr->remoteAddr()) {
 	    // keep transaction referenced but unlock the engine
 	    RefPointer<IAXTransaction> t = tr;
 	    lock.drop();
@@ -135,10 +208,12 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
     }
     // Frame doesn't belong to an existing transaction
     // Test if it is a full frame with an IAX control message that needs a new transaction
-    if (!frame->fullFrame() || frame->type() != IAXFrame::IAX)
+    if (!fullFrame || frame->type() != IAXFrame::IAX)
 	return 0;
-    switch (frame->fullFrame()->subclass()) {
+    switch (fullFrame->subclass()) {
 	case IAXControl::New:
+	     if (!checkCallToken(addr,*fullFrame))
+		return 0;
 	case IAXControl::RegReq:
 	case IAXControl::RegRel:
 	case IAXControl::Poke:
@@ -149,14 +224,14 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
 	case IAXControl::FwDownl:
 	default:
 #ifdef DEBUG
-	    if (frame->fullFrame()) {
-	        if (frame->fullFrame()->destCallNo() == 0)
+	    if (fullFrame) {
+	        if (fullFrame->destCallNo() == 0)
 		    Debug(this,DebugAll,"Unsupported incoming transaction Frame(%u,%u). Source call no: %u",
-			frame->type(),frame->fullFrame()->subclass(),frame->fullFrame()->sourceCallNo());
+			frame->type(),fullFrame->subclass(),fullFrame->sourceCallNo());
 #ifdef XDEBUG
 		else
 		    Debug(this,DebugAll,"Unmatched Frame(%u,%u) for (%u,%u)",
-			frame->type(),frame->fullFrame()->subclass(),frame->fullFrame()->destCallNo(),frame->fullFrame()->sourceCallNo());
+			frame->type(),fullFrame->subclass(),fullFrame->destCallNo(),fullFrame->sourceCallNo());
 #endif
 	    }
 #endif
@@ -167,7 +242,7 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, IAXFrame* frame)
     if (!lcn)
 	return 0;
     // Create and add transaction
-    tr = IAXTransaction::factoryIn(this,(IAXFullFrame*)frame->fullFrame(),lcn,addr);
+    tr = IAXTransaction::factoryIn(this,fullFrame,lcn,addr);
     if (tr)
 	m_transList[frame->sourceCallNo() % m_transListCount]->append(tr);
     else
@@ -180,7 +255,7 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, const unsigned char*
     IAXFrame* frame = IAXFrame::parse(buf,len,this,&addr);
     if (!frame)
 	return 0;
-    if (frame->fullFrame() && debugAt(DebugAll)) {
+    if (m_printMsg && frame->fullFrame() && debugAt(DebugInfo)) {
 	String s;
 	SocketAddr local;
 	m_socket.getSockName(local);
@@ -212,6 +287,20 @@ bool IAXEngine::process()
     return ok;
 }
 
+// (Re)Initialize the engine
+void IAXEngine::initialize(const NamedList& params)
+{
+    m_callToken = params.getBoolValue("calltoken_in");
+    int callTokenAge = params.getIntValue("calltoken_age",10);
+    if (callTokenAge > 1 && callTokenAge < 25)
+	m_callTokenAge = callTokenAge;
+    else
+	m_callTokenAge = 10;
+    m_showCallTokenFailures = params.getBoolValue("calltoken_printfailure");
+    m_rejectMissingCallToken = params.getBoolValue("calltoken_rejectmissing",true);
+    m_printMsg = params.getBoolValue("printmsg",true);
+}
+
 void IAXEngine::readSocket(SocketAddr& addr)
 {
     unsigned char buf[1500];
@@ -219,19 +308,23 @@ void IAXEngine::readSocket(SocketAddr& addr)
     while (1) {
 	int len = m_socket.recvFrom(buf,sizeof(buf),addr);
 	if (len == Socket::socketError()) {
-	    if (!m_socket.canRetry())
+	    if (!m_socket.canRetry()) {
+		String tmp;
+		Thread::errorString(tmp,m_socket.error());
 		Debug(this,DebugWarn,"Socket read error: %s (%d)",
-		    ::strerror(m_socket.error()),m_socket.error());
-	    Thread::msleep(1,true);
+		    tmp.c_str(),m_socket.error());
+	    }
+	    Thread::idle(true);
 	    continue;
 	}
 	addFrame(addr,buf,len);
     }
 }
 
-bool IAXEngine::writeSocket(const void* buf, int len, const SocketAddr& addr, IAXFullFrame* frame)
+bool IAXEngine::writeSocket(const void* buf, int len, const SocketAddr& addr,
+    IAXFullFrame* frame, unsigned int* sent)
 {
-    if (frame && debugAt(DebugAll)) {
+    if (m_printMsg && frame && debugAt(DebugInfo)) {
 	String s;
 	SocketAddr local;
 	m_socket.getSockName(local);
@@ -240,16 +333,24 @@ bool IAXEngine::writeSocket(const void* buf, int len, const SocketAddr& addr, IA
     }
     len = m_socket.sendTo(buf,len,addr);
     if (len == Socket::socketError()) {
-	if (!m_socket.canRetry())
+	if (!m_socket.canRetry()) {
+	    String tmp;
+	    Thread::errorString(tmp,m_socket.error());
 	    Debug(this,DebugWarn,"Socket write error: %s (%d)",
-		::strerror(m_socket.error()),m_socket.error());
+		tmp.c_str(),m_socket.error());
+	}
 #ifdef DEBUG
-	else
+	else {
+	    String tmp;
+	    Thread::errorString(tmp,m_socket.error());
 	    Debug(this,DebugMild,"Socket temporary unavailable: %s (%d)",
-		::strerror(m_socket.error()),m_socket.error());
+		tmp.c_str(),m_socket.error());
+	}
 #endif
 	return false;
     }
+    if (sent)
+	*sent = (unsigned int)len;
     return true;
 }
 
@@ -385,13 +486,13 @@ u_int16_t IAXEngine::generateCallNo()
 
     m_startLocalCallNo++;
     if (m_startLocalCallNo > IAX2_MAX_CALLNO)
-	m_startLocalCallNo = 1;
+	m_startLocalCallNo = IAX2_MIN_CALLNO;
     for (i = m_startLocalCallNo; i <= IAX2_MAX_CALLNO; i++)
 	if (!m_lUsedCallNo[i]) {
 	    m_lUsedCallNo[i] = true;
 	    return i;
 	}
-    for (i = 1; i < m_startLocalCallNo; i++)
+    for (i = IAX2_MIN_CALLNO; i < m_startLocalCallNo; i++)
 	if (!m_lUsedCallNo[i]) {
 	    m_lUsedCallNo[i] = true;
 	    return i;
@@ -422,60 +523,115 @@ IAXTransaction* IAXEngine::startLocalTransaction(IAXTransaction::Type type, cons
     return tr;
 }
 
-bool IAXEngine::acceptFormatAndCapability(IAXTransaction* trans)
+// Check call token on incoming call requests.
+bool IAXEngine::checkCallToken(const SocketAddr& addr, IAXFullFrame& frame)
+{
+    XDebug(this,DebugAll,"IAXEngine::checkCallToken('%s:%d') calltoken=%u",
+	addr.host().c_str(),addr.port(),m_callToken);
+    if (!m_callToken)
+	return true;
+    frame.updateIEList(true);
+    IAXIEList* list = frame.ieList();
+    IAXInfoElementBinary* ct = 0;
+    if (list)
+	ct = static_cast<IAXInfoElementBinary*>(list->getIE(IAXInfoElement::CALLTOKEN));
+    // No call token support
+    if (!ct) {
+	if (m_showCallTokenFailures)
+	    Debug(this,DebugNote,
+		"Missing required %s parameter in call request %u from '%s:%d'",
+		IAXInfoElement::ieText(IAXInfoElement::CALLTOKEN),frame.sourceCallNo(),
+		addr.host().c_str(),addr.port());
+	if (m_rejectMissingCallToken) {
+	    IAXIEList* ies = new IAXIEList;
+	    ies->appendString(IAXInfoElement::CAUSE,"CALLTOKEN support required");
+	    IAXFullFrame* rsp = new IAXFullFrame(IAXFrame::IAX,IAXControl::Reject,
+		IAX2_CALLTOKEN_REJ_CALLNO,frame.sourceCallNo(),0,1,2,
+		ies,maxFullFrameDataLen());
+	    writeSocket(addr,rsp);
+	    TelEngine::destruct(rsp);
+	}
+	return false;
+    }
+    // Request with call token
+    if (ct->data().length()) {
+	String tmp((char*)ct->data().data(),ct->data().length());
+	int age = addrSecretAge(tmp,m_callTokenSecret,addr);
+	XDebug(this,DebugAll,"Call request %u from '%s:%d' with call token age=%d",
+	    frame.sourceCallNo(),addr.host().c_str(),addr.port(),age);
+	if (age >= 0 && age <= m_callTokenAge)
+	    return true;
+	if (m_showCallTokenFailures)
+	    Debug(this,DebugNote,
+		"Ignoring call request %u from '%s:%d' with %s call token age=%d",
+		frame.sourceCallNo(),addr.host().c_str(),addr.port(),
+		(age > 0) ? "old" : "invalid",age);
+	return false;
+    }
+    // Request with empty call token: send one
+    String tmp;
+    buildAddrSecret(tmp,m_callTokenSecret,addr);
+    IAXIEList* ies = new IAXIEList;
+    ies->appendBinary(IAXInfoElement::CALLTOKEN,(unsigned char*)tmp.c_str(),tmp.length());
+    IAXFullFrame* rsp = new IAXFullFrame(IAXFrame::IAX,IAXControl::CallToken,
+	IAX2_CALLTOKEN_CALLNO,frame.sourceCallNo(),0,1,1,ies,maxFullFrameDataLen());
+    writeSocket(addr,rsp);
+    TelEngine::destruct(rsp);
+    return false;
+}
+
+bool IAXEngine::acceptFormatAndCapability(IAXTransaction* trans, unsigned int* caps, int type)
 {
     if (!trans)
 	return false;
-    u_int32_t format = trans->format();
-    u_int32_t capability = m_capability & trans->capability();
-#ifdef XDEBUG
-    if (!trans->outgoing()) {
-	String rec;
-	IAXFormat::formatList(rec,m_capability);
-	Debug(this,DebugAll,"acceptFormatAndCapability. Local: Format: %u(%s). Capabilities: '%s'.",
-	    m_format,IAXFormat::audioText(m_format),rec.c_str());
-	IAXFormat::formatList(rec,trans->capability());
-	Debug(this,DebugAll,"acceptFormatAndCapability. Received: Format: %u(%s). Capabilities: '%s'.",
-	    format,IAXFormat::audioText(format),rec.c_str());
+    u_int32_t transCapsNonType = IAXFormat::clear(trans->m_capability,type);
+    IAXFormat* fmt = trans->getFormat(type);
+    if (!fmt) {
+	DDebug(this,DebugStub,"acceptFormatAndCapability() No media %s in transaction",
+	    IAXFormat::typeName(type));
+	trans->m_capability = transCapsNonType;
+	return false;
     }
-#endif
+    u_int32_t transCapsType = IAXFormat::mask(trans->m_capability,type);
+    u_int32_t capability = transCapsType & m_capability;
+    if (caps)
+	capability &= IAXFormat::mask(*caps,type);
+    trans->m_capability = transCapsNonType | capability;
+    XDebug(this,DebugAll,
+	"acceptFormatAndCapability trans(%u,%u) type=%s caps(trans/our/param/result)=%u/%u/%u/%u",
+	trans->localCallNo(),trans->remoteCallNo(),
+	fmt->typeName(),transCapsType,IAXFormat::mask(m_capability,type),
+	caps ? IAXFormat::mask(*caps,type) : 0,capability);
     // Valid capability ?
     if (!capability) {
-#ifdef DEBUG
-	if (!trans->outgoing())
-	    Debug(this,DebugNote,"acceptFormatAndCapability. No capabilities received.");
-#endif
+	// Warn if we should have media
+	if (type == IAXFormat::Audio || 0 != (trans->outgoing() ? fmt->in() : fmt->out()))
+	    Debug(this,DebugNote,"Transaction(%u,%u) no common format(s) for media '%s' [%p]",
+		trans->localCallNo(),trans->remoteCallNo(),fmt->typeName(),trans);
+	// capability is 0, use it to set format
+	if (trans->outgoing())
+	    fmt->set(&capability,&capability,0);
+	else
+	    fmt->set(&capability,0,&capability);
 	return false;
     }
-    for (;;) {
-	// Received format is valid ?
-	if (0 != (format & capability) && IAXFormat::audioText(format))
-	    break;
-	// Local format is valid ?
-	format = m_format;
-	if (0 != (m_format & capability) && IAXFormat::audioText(format))
-	    break;
-	// No valid format: choose one from capability
-	format = 0;
-	u_int32_t i = 0;
-	for (; IAXFormat::audioData[i].value; i++)
-	    if (0 != (capability & IAXFormat::audioData[i].value))
-		break;
-	if (IAXFormat::audioData[i].value) {
-	    format = IAXFormat::audioData[i].value;
-	    break;
-	}
-	DDebug(this,DebugNote,"acceptFormatAndCapability. Unable to choose a format.");
-	return false;
+    u_int32_t format = fmt->format();
+    // Received format is valid ?
+    if (0 == (format & capability)) {
+	format = (type == IAXFormat::Audio) ? m_format : 0;
+	format = IAXFormat::pickFormat(capability,format);
     }
-    DDebug(this,DebugAll,"acceptFormatAndCapability. Format %u: '%s'.",
-	format,IAXFormat::audioText(format));
-    trans->m_format = format;
-    trans->m_capability = capability;
-    trans->m_formatIn = format;
-    if (!trans->outgoing())
-	trans->m_formatOut = format;
-    return true;
+    if (format) {
+	fmt->set(&format,&format,&format);
+	Debug(this,DebugAll,"Transaction(%u,%u) set format %u (%s) for media '%s' [%p]",
+	    trans->localCallNo(),trans->remoteCallNo(),format,fmt->formatName(),
+	    fmt->typeName(),trans);
+    }
+    else
+	Debug(this,DebugNote,
+	    "Transaction(%u,%u) failed to choose a common format for media '%s' [%p]",
+	    trans->localCallNo(),trans->remoteCallNo(),fmt->typeName(),trans);
+    return format != 0;
 }
 
 void IAXEngine::defaultEventHandler(IAXEvent* event)
@@ -540,27 +696,52 @@ bool IAXEngine::isMD5ChallengeCorrect(const String& md5data, const String& chall
     return md5data == md5.hexDigest();
 }
 
+// Build a time signed secret used to authenticate an IP address
+void IAXEngine::buildAddrSecret(String& buf, const String& secret, const SocketAddr& addr)
+{
+    unsigned int t = Time::secNow();
+    buildSecretDigest(buf,secret,t,addr);
+    buf << "." << t;
+}
+
+// Decode a secret built using buildAddrSecret()
+int IAXEngine::addrSecretAge(const String& buf, const String& secret, const SocketAddr& addr)
+{
+    int pos = buf.find('.');
+    if (pos < 1)
+	return -1;
+    int t = buf.substr(pos + 1).toInteger();
+    String tmp;
+    buildSecretDigest(tmp,secret,t,addr);
+    return (tmp == buf.substr(0,pos)) ? ((int)Time::secNow() - t) : -1;
+}
+
 /*
 * IAXEvent
 */
 IAXEvent::IAXEvent(Type type, bool local, bool final, IAXTransaction* transaction, u_int8_t frameType, u_int32_t subclass)
-    : m_type(type), m_frameType(frameType), m_subClass(subclass), m_local(local), m_final(final), m_transaction(0)
-
+    : m_type(type), m_frameType(frameType), m_subClass(subclass),
+    m_local(local), m_final(final), m_transaction(0), m_ieList(0)
 {
     if (transaction && transaction->ref())
 	m_transaction = transaction;
+    m_ieList = new IAXIEList;
 }
 
-IAXEvent::IAXEvent(Type type, bool local, bool final, IAXTransaction* transaction, const IAXFullFrame* frame)
-    : m_type(type), m_frameType(0), m_subClass(0), m_local(local), m_final(final), m_transaction(0), m_ieList(frame)
-
+IAXEvent::IAXEvent(Type type, bool local, bool final, IAXTransaction* transaction, IAXFullFrame* frame)
+    : m_type(type), m_frameType(0), m_subClass(0), m_local(local),
+    m_final(final), m_transaction(0), m_ieList(0)
 {
     if (transaction && transaction->ref())
 	m_transaction = transaction;
     if (frame) {
 	m_frameType = frame->type();
 	m_subClass = frame->subclass();
+	frame->updateIEList(true);
+	m_ieList = frame->removeIEList(false);
     }
+    if (!m_ieList)
+	m_ieList = new IAXIEList;
 }
 
 IAXEvent::~IAXEvent()
@@ -571,6 +752,8 @@ IAXEvent::~IAXEvent()
 	m_transaction->eventTerminated(this);
 	m_transaction->deref();
     }
+    if (m_ieList)
+	delete m_ieList;
 }
 
 /* vi: set ts=8 sw=4 sts=4 noet: */
