@@ -34,6 +34,17 @@ String IAXTransaction::s_iax_modNoUsername("Username is missing");
 
 unsigned char IAXTransaction::m_maxInFrames = 100;
 
+
+// Print statistics
+void IAXMediaData::print(String& buf)
+{
+    Lock lck(this);
+    buf << "PS=" << m_sent << ",OS=" << m_sentBytes;
+    buf << ",PR=" << m_recv << ",OR=" << m_recvBytes;
+    buf << ",PL=" << m_ooPackets << ",OL=" << m_ooBytes;
+}
+
+
 IAXTransaction::IAXTransaction(IAXEngine* engine, IAXFullFrame* frame, u_int16_t lcallno, 
 	const SocketAddr& addr, void* data)
     : Mutex(true,"IAXTransaction"),
@@ -51,10 +62,7 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, IAXFullFrame* frame, u_int16_t
     m_engine(engine),
     m_userdata(data),
     m_lastFullFrameOut(0),
-    m_lastMiniFrameOut(0xFFFF),
-    m_lastMiniFrameIn(0),
     m_lastAck(0xFFFF),
-    m_mutexInMedia(true,"IAXTransaction::InMedia"),
     m_pendingEvent(0),
     m_currentEvent(0),
     m_retransCount(5),
@@ -66,14 +74,12 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, IAXFullFrame* frame, u_int16_t
     m_inDroppedFrames(0),
     m_authmethod(IAXAuthMethod::MD5),
     m_expire(60),
-    m_format(0),
-    m_formatIn(0),
-    m_formatOut(0),
-    m_capability(0),
+    m_format(IAXFormat::Audio), m_formatVideo(IAXFormat::Video),
+    m_capability(0), m_callToken(false),
     m_trunkFrame(0)
 {
-    XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u) incoming [%p]",
-	localCallNo(),remoteCallNo(),this);
+    Debug(m_engine,DebugAll,"Transaction(%u,%u) incoming type=%u remote=%s:%d [%p]",
+	localCallNo(),remoteCallNo(),m_type,m_addr.host().c_str(),m_addr.port(),this);
     // Setup transaction
     m_retransCount = engine->retransCount();
     m_retransInterval = engine->retransInterval();
@@ -92,8 +98,8 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, IAXFullFrame* frame, u_int16_t
 	    m_type = Poke;
 	    break;
 	default:
-	    XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u) incoming [%p]. Unsupported type: %u",
-		localCallNo(),remoteCallNo(),this,frame->subclass());
+	    Debug(m_engine,DebugNote,"Transaction(%u,%u) incoming with unsupported type %u [%p]",
+		localCallNo(),remoteCallNo(),frame->subclass(),this);
 	    return;
     }
     // Append frame to incoming list
@@ -119,10 +125,7 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, Type type, u_int16_t lcallno,
     m_engine(engine),
     m_userdata(data),
     m_lastFullFrameOut(0),
-    m_lastMiniFrameOut(0xFFFF),
-    m_lastMiniFrameIn(0),
     m_lastAck(0xFFFF),
-    m_mutexInMedia(true,"IAXTransaction::InMedia"),
     m_pendingEvent(0),
     m_currentEvent(0),
     m_retransCount(5),
@@ -134,14 +137,12 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, Type type, u_int16_t lcallno,
     m_inDroppedFrames(0),
     m_authmethod(IAXAuthMethod::MD5),
     m_expire(60),
-    m_format(0),
-    m_formatIn(0),
-    m_formatOut(0),
-    m_capability(0),
+    m_format(IAXFormat::Audio), m_formatVideo(IAXFormat::Video),
+    m_capability(0), m_callToken(false),
     m_trunkFrame(0)
 {
-    XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u) outgoing. [%p]",
-	localCallNo(),remoteCallNo(),this);
+    Debug(m_engine,DebugAll,"Transaction(%u,%u) outgoing type=%u remote=%s:%d [%p]",
+	localCallNo(),remoteCallNo(),m_type,m_addr.host().c_str(),m_addr.port(),this);
     // Init data members
     if (!m_addr.port()) {
 	XDebug(m_engine,DebugAll,
@@ -153,56 +154,45 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, Type type, u_int16_t lcallno,
     m_retransInterval = engine->retransInterval();
     m_timeToNextPing = m_timeStamp + m_pingInterval;
     init(ieList);
-    ieList.clear();
     IAXControl::Type frametype;
+    IAXIEList* ies = new IAXIEList;
     // Create IE list to send
     switch (type) {
 	case New:
-	    ieList.insertVersion();
-	    ieList.appendString(IAXInfoElement::USERNAME,m_username);
-	    ieList.appendString(IAXInfoElement::CALLING_NUMBER,m_callingNo);
-	    ieList.appendString(IAXInfoElement::CALLING_NAME,m_callingName);
-	    ieList.appendString(IAXInfoElement::CALLED_NUMBER,m_calledNo);
-	    ieList.appendString(IAXInfoElement::CALLED_CONTEXT,m_calledContext);
-	    ieList.appendNumeric(IAXInfoElement::FORMAT,m_format,4);
-	    ieList.appendNumeric(IAXInfoElement::CAPABILITY,m_capability,4);
+	    ies->insertVersion();
+	    ies->appendString(IAXInfoElement::USERNAME,m_username);
+	    ies->appendString(IAXInfoElement::CALLING_NUMBER,m_callingNo);
+	    ies->appendString(IAXInfoElement::CALLING_NAME,m_callingName);
+	    ies->appendString(IAXInfoElement::CALLED_NUMBER,m_calledNo);
+	    ies->appendString(IAXInfoElement::CALLED_CONTEXT,m_calledContext);
+	    ies->appendNumeric(IAXInfoElement::FORMAT,m_format.format() | m_formatVideo.format(),4);
+	    ies->appendNumeric(IAXInfoElement::CAPABILITY,m_capability,4);
+	    if (m_callToken)
+		ies->appendBinary(IAXInfoElement::CALLTOKEN,0,0);
 	    frametype = IAXControl::New;
 	    break;
 	case RegReq:
 	case RegRel:
-	    ieList.appendString(IAXInfoElement::USERNAME,m_username);
-	    ieList.appendNumeric(IAXInfoElement::REFRESH,m_expire,2);
+	    ies->appendString(IAXInfoElement::USERNAME,m_username);
+	    ies->appendNumeric(IAXInfoElement::REFRESH,m_expire,2);
 	    frametype = (type == RegReq ? IAXControl::RegReq : IAXControl::RegRel);
 	    break;
 	case Poke:
 	    frametype = IAXControl::Poke;
 	    break;
 	default:
-	    XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u) outgoing. Unsupported type: %u. [%p]",
+	    Debug(m_engine,DebugStub,"Transaction(%u,%u) outgoing with unsupported type %u [%p]",
 		localCallNo(),remoteCallNo(),m_type,this);
+	    delete ies;
 	    m_type = Incorrect;
 	    return;
     }
-    DataBlock d;
-    ieList.toBuffer(d);
-    if (d.length() > (unsigned int)m_engine->maxFullFrameDataLen()) {
-	XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u). Buffer too long (%u > %u). [%p]",
-		localCallNo(),remoteCallNo(),d.length(),(unsigned int)m_engine->maxFullFrameDataLen(),this);
-	d.clear();
-    }
-    postFrame(IAXFrame::IAX,frametype,(void*)(d.data()),d.length());
+    postFrameIes(IAXFrame::IAX,frametype,ies);
     changeState(NewLocalInvite);
 }
 
 IAXTransaction::~IAXTransaction()
 {
-#ifdef XDEBUG
-    print();
-#endif
-    if (m_trunkFrame)
-	m_trunkFrame->deref();
-    if (state() != Terminating && state() != Terminated)
-	sendReject("Server shutdown");
     XDebug(m_engine,DebugAll,"IAXTransaction::~IAXTransaction(%u,%u). [%p]",
 	localCallNo(),remoteCallNo(),this);
 }
@@ -227,6 +217,26 @@ IAXTransaction* IAXTransaction::factoryOut(IAXEngine* engine, Type type, u_int16
     return 0;
 }
 
+// Retrieve the media of a given type
+IAXFormat* IAXTransaction::getFormat(int type)
+{
+    if (type == IAXFormat::Audio)
+	return &m_format;
+    if (type == IAXFormat::Video)
+	return &m_formatVideo;
+    return 0;
+}
+
+// Retrieve the media data for a given type
+IAXMediaData* IAXTransaction::getData(int type)
+{
+    if (type == IAXFormat::Audio)
+	return &m_dataAudio;
+    if (type == IAXFormat::Video)
+	return &m_dataVideo;
+    return 0;
+}
+
 IAXTransaction* IAXTransaction::processFrame(IAXFrame* frame)
 {
     if (!frame)
@@ -245,8 +255,16 @@ IAXTransaction* IAXTransaction::processFrame(IAXFrame* frame)
 	    return 0;
     }
     // Mini frame
-    if (!frame->fullFrame())
-	return processMedia(frame->data(),frame->timeStamp());
+    if (!frame->fullFrame()) {
+	int t = 0;
+	if (frame->type() == IAXFrame::Voice)
+	    t = IAXFormat::Audio;
+	else if (frame->type() == IAXFrame::Video)
+	    t = IAXFormat::Video;
+	else
+	    return 0;
+	return processMedia(frame->data(),frame->timeStamp(),t,false,frame->mark());
+    }
     Lock lock(this);
     m_inTotalFramesCount++;
     // Frame is VNAK ?
@@ -254,7 +272,8 @@ IAXTransaction* IAXTransaction::processFrame(IAXFrame* frame)
 	return retransmitOnVNAK(frame->fullFrame()->iSeqNo());
     // Do we have enough space to keep this frame ?
     if (m_inFrames.count() == m_maxInFrames) {
-	Debug(DebugWarn,"Transaction(%u,%u). processFrame. Buffer overrun! (MAX=%u)",localCallNo(),remoteCallNo(),m_maxInFrames);
+	Debug(m_engine,DebugWarn,"Transaction(%u,%u). processFrame. Buffer overrun! (MAX=%u)",
+	    localCallNo(),remoteCallNo(),m_maxInFrames);
 	m_inDroppedFrames++;
 	return 0;
     }
@@ -262,13 +281,17 @@ IAXTransaction* IAXTransaction::processFrame(IAXFrame* frame)
     if (!fAck && !isFrameAcceptable(frame->fullFrame()))
 	return 0;
     incrementSeqNo(frame->fullFrame(),true);
-    // Voice full frame: process voice data & format
-    if (frame->type() == IAXFrame::Voice && type() == New) {
-	if (!processVoiceFrame(frame->fullFrame()))
+    // Video/Voice full frame: process data & format
+    if (type() == New && (frame->type() == IAXFrame::Voice ||
+	frame->type() == IAXFrame::Video)) {
+	int t = IAXFormat::Audio;
+	if (frame->type() == IAXFrame::Video)
+	    t = IAXFormat::Video;
+	if (!processMediaFrame(frame->fullFrame(),t))
 	    return 0;
 	// Frame accepted: process voice data
 	lock.drop();
-	return processMedia(frame->data(),frame->timeStamp(),true);
+	return processMedia(frame->data(),frame->timeStamp(),t,true,frame->mark());
     }
     // Process incoming Ping
     if (frame->type() == IAXFrame::IAX && frame->fullFrame()->subclass() == IAXControl::Ping) {
@@ -286,67 +309,182 @@ IAXTransaction* IAXTransaction::processFrame(IAXFrame* frame)
     return this;
 }
 
-IAXTransaction* IAXTransaction::processMedia(DataBlock& data, u_int32_t tStamp, bool voice)
+IAXTransaction* IAXTransaction::processMedia(DataBlock& data, u_int32_t tStamp, int type,
+    bool full, bool mark)
 {
-    Lock lock(&m_mutexInMedia);
-    if (!(voice || (tStamp & 0xffff0000))) {
-	// Miniframe timestamp
-	int16_t delta = (int16_t)(tStamp - m_lastMiniFrameIn);
-	if (delta < 0)
+    if (state() == Terminated || state() == Terminating)
+	return 0;
+    IAXMediaData* d = getData(type);
+    IAXFormat* fmt = getFormat(type);
+    if (!(d && fmt)) {
+	Debug(m_engine,DebugStub,
+	    "IAXTransaction::processMedia() no media data for type '%s'",
+	    IAXFormat::typeName(type));
+	return 0;
+    }
+    Lock lck(d);
+    if (!fmt->in()) {
+	if (d->m_showInNoFmt) {
+	    Debug(m_engine,DebugInfo,
+		"Transaction(%u,%u) received %s data without format [%p]",
+		localCallNo(),remoteCallNo(),fmt->typeName(),this);
+	    d->m_showInNoFmt = false;
+	}
+	return 0;
+    }
+    d->m_showInNoFmt = true;
+    d->m_recv++;
+    d->m_recvBytes += data.length();
+    if (!full) {
+	// Miniframe or video meta frame timestamp
+	// Voice: timestamp is lowest 16 bits
+	// Video: timestamp is lowest 15 bits
+	u_int32_t mask = 0xffff;
+	if (type == IAXFormat::Video)
+	    mask = 0x7fff;
+	tStamp &= mask;
+	// Interval between received timestamp and last one:
+	// Negative: wraparound if less then half mask
+	int delta = (int)tStamp - (int)(d->m_lastIn & mask);
+	if (delta < 0 && ((u_int32_t)-delta) < (mask / 2)) {
+	    d->m_ooPackets++;
+	    d->m_ooBytes += data.length();
+	    DDebug(m_engine,DebugNote,
+		"Transaction(%u,%u) dropping %u %s mini data mark=%u ts=%u last=%u [%p]",
+		localCallNo(),remoteCallNo(),data.length(),fmt->typeName(),
+		mark,tStamp,d->m_lastIn & mask,this);
 	    return 0;
-	// add upper bits from last frame
-	tStamp |= m_lastMiniFrameIn & 0xffff0000;
-	// check if timestamp wrapped around by a miniframe, adjust if so
-	if ((tStamp & 0xffff) < (m_lastMiniFrameIn & 0xffff)) {
-	    DDebug(m_engine,DebugAll,"Timestamp wraparound, ts=%u last=%u [%p]",tStamp & 0xffff,m_lastMiniFrameIn,this);
-	    tStamp += 0x10000;
+	}
+	// Add upper bits from last frame, adjust timestamp if wrapped around
+	tStamp |= d->m_lastIn & ~mask;
+	if (delta < 0) {
+	    DDebug(m_engine,DebugInfo,
+		"Transaction(%u,%u) timestamp wraparound media=%s ts=%u last=%u [%p]",
+		localCallNo(),remoteCallNo(),fmt->typeName(),
+		tStamp & mask,d->m_lastIn & mask,this);
+	    tStamp += mask + 1;
 	}
     }
-    int32_t interval = (int32_t)tStamp - m_lastMiniFrameIn;
-    if (interval <= 0)
+    bool forward = false;
+    if (type != IAXFormat::Video)
+	forward = (tStamp > d->m_lastIn);
+    else
+	forward = (tStamp >= d->m_lastIn);
+    if (forward) {
+	d->m_lastIn = tStamp; // New frame is newer then the last one
+	XDebug(m_engine,DebugAll,
+	    "Transaction(%u,%u) forwarding %u %s data mark=%u ts=%u [%p]",
+	    localCallNo(),remoteCallNo(),data.length(),fmt->typeName(),
+	    mark,tStamp,this);
+	m_engine->processMedia(this,data,tStamp,type,mark);
 	return 0;
-    if (voice || interval < 32767) {
-	m_lastMiniFrameIn = tStamp; // New frame is newer then the last one
-	m_engine->processMedia(this,data,tStamp);
     }
+    d->m_ooPackets++;
+    d->m_ooBytes += data.length();
+    DDebug(m_engine,DebugNote,
+	"Transaction(%u,%u) dropping %u %s data full=%u mark=%u ts=%u last=%u [%p]",
+	localCallNo(),remoteCallNo(),data.length(),fmt->typeName(),
+	full,mark,tStamp,d->m_lastIn,this);
     return 0;
 }
 
-IAXTransaction* IAXTransaction::sendMedia(const DataBlock& data, u_int32_t format)
+unsigned int IAXTransaction::sendMedia(const DataBlock& data, u_int32_t format,
+    int type, bool mark)
 {
     if (!data.length())
 	return 0;
+    if (state() == Terminated || state() == Terminating)
+	return 0;
+    IAXFormat* fmt = getFormat(type);
+    IAXMediaData* d = getData(type);
+    if (!(fmt && d)) {
+	Debug(m_engine,DebugStub,
+	    "IAXTransaction::sendMedia() no media desc for type '%s'",
+	    IAXFormat::typeName(type));
+	return 0;
+    }
+    d->m_sent++;
+    d->m_sentBytes += data.length();
     u_int32_t ts = (u_int32_t)timeStamp();
-    // Avoid to send the same timestamp twice
-    if ((u_int16_t)ts == m_lastMiniFrameOut)
+    // Avoid sending the same timestamp twice for non video
+    if (type != IAXFormat::Video && d->m_lastOut && ts == d->m_lastOut)
 	ts++;
-    // Format changed or timestamp wrapped around? Send Voice full frame
-    if ((u_int16_t)ts < m_lastMiniFrameOut || m_formatOut != format ) {
-	if (m_formatOut != format) {
-	    DDebug(m_engine,DebugNote,"Outgoing format changed (New: %u, Old: %u). Send VOICE. [%p]",
-		format,m_formatOut,this);
-	    m_formatOut = format;
+    // Format changed or timestamp wrapped around
+    // Send a full frame
+    bool fullFrame = (fmt->out() != format) || !d->m_lastOut;
+    if (!fullFrame) {
+	// Voice: timestamp is lowest 16 bits
+	// Video: timestamp is lowest 15 bits
+	u_int32_t mask = 0xffff;
+	if (type == IAXFormat::Video)
+	    mask = 0x7fff;
+	// Timestamp wraparound if mini timestamp is less then last one or
+	// we had a media gap greater then mask
+	fullFrame = ((ts & mask) < (d->m_lastOut & mask)) || ((ts - d->m_lastOut) > mask);
+    }
+    if (fullFrame) {
+	if (fmt->out() != format) {
+	    Debug(m_engine,DebugNote,
+		"Transaction(%u,%u). Outgoing %s format changed %u --> %u [%p]",
+		localCallNo(),remoteCallNo(),fmt->typeName(),fmt->out(),format,this);
+	    fmt->set(0,0,&format);
 	}
 #ifdef DEBUG
 	else
-	    Debug(m_engine,DebugNote,"Transaction(%u,%u) Time to send VOICE: ts=%u last=%u [%p]",
-		localCallNo(),remoteCallNo(),ts,m_lastMiniFrameOut,this);
+	    Debug(m_engine,DebugInfo,
+		"Transaction(%u,%u). Sending full frame for media '%s': ts=%u last=%u [%p]",
+		localCallNo(),remoteCallNo(),fmt->typeName(),ts,d->m_lastOut,this);
 #endif
-	m_lastMiniFrameOut = (u_int16_t)ts;
-	postFrame(IAXFrame::Voice,m_formatOut,data.data(),data.length(),ts,true);
-	return this;
     }
-    // Send mini frame
-    m_lastMiniFrameOut = (u_int16_t)ts;
-    if (m_trunkFrame)
-	m_trunkFrame->add(localCallNo(),data,m_lastMiniFrameOut);
-    else {
-	unsigned char b[4] = {localCallNo() >> 8,localCallNo() & 0xff,m_lastMiniFrameOut >> 8,m_lastMiniFrameOut & 0xff};
-	DataBlock buf(b,4);
-	buf += data;
-	m_engine->writeSocket(buf.data(),buf.length(),remoteAddr());
+    d->m_lastOut = ts;
+    unsigned int sent = 0;
+    if (type == IAXFormat::Audio) {
+	if (fullFrame) {
+	    // Send trunked frame before full frame to keep the media order
+	    if (m_trunkFrame)
+		m_engine->sendTrunkFrame(m_trunkFrame);
+	    postFrame(IAXFrame::Voice,fmt->out(),data.data(),data.length(),ts,true);
+	    sent = data.length();
+	}
+	else if (m_trunkFrame) {
+	    m_trunkFrame->add(localCallNo(),data,ts);
+	    sent = data.length();
+	}
+	else {
+	    DataBlock buf;
+	    IAXFrame::buildMiniFrame(buf,localCallNo(),ts,data.data(),data.length());
+	    m_engine->writeSocket(buf.data(),buf.length(),remoteAddr(),0,&sent);
+	    // Decrease with mini frame header
+	    if (sent > 4)
+		sent -= 4;
+	    else
+		sent = 0;
+	}
     }
-    return this;
+    else if (type == IAXFormat::Video) {
+	if (fullFrame) {
+	    postFrame(IAXFrame::Video,fmt->out(),data.data(),data.length(),ts,true,mark);
+	    sent = data.length();
+	}
+	else {
+	    DataBlock buf;
+	    IAXFrame::buildVideoMetaFrame(buf,localCallNo(),ts,mark,data.data(),data.length());
+	    m_engine->writeSocket(buf.data(),buf.length(),remoteAddr(),0,&sent);
+	    // Decrease with mini frame header
+	    if (sent > 6)
+		sent -= 6;
+	    else
+		sent = 0;
+	}
+    }
+    else
+	Debug(m_engine,DebugStub,
+	    "IAXTransaction::sendMedia() not implemented for type '%s'",fmt->typeName());
+    DDebug(m_engine,sent == data.length() ? DebugAll : DebugNote,
+	"Transaction(%u,%u) sent %u/%u media=%s mark=%u ts=%u [%p]",
+	localCallNo(),remoteCallNo(),sent,data.length(),
+	fmt->typeName(),mark,ts,this);
+    return sent;
 }
 
 IAXEvent* IAXTransaction::getEvent(u_int64_t time)
@@ -467,7 +605,7 @@ IAXEvent* IAXTransaction::getEvent(u_int64_t time)
     return 0;
 }
 
-bool IAXTransaction::sendAccept()
+bool IAXTransaction::sendAccept(unsigned int* expires)
 {
     Lock lock(this);
     if (!((type() == New && (state() == NewRemoteInvite || state() == NewRemoteInvite_RepRecv)) ||
@@ -475,19 +613,22 @@ bool IAXTransaction::sendAccept()
 	((type() == RegReq || type() == RegRel) && state() == NewRemoteInvite_RepRecv)))
 	return false;
     if (type() == New) {
-	unsigned char d[12] = {IAXInfoElement::FORMAT,4,m_format >> 24,m_format >> 16,m_format >> 8,m_format,
-		IAXInfoElement::CAPABILITY,4,m_capability >> 24,m_capability >> 16,m_capability >> 8,m_capability};
-	postFrame(IAXFrame::IAX,IAXControl::Accept,d,sizeof(d),0,true);
+	IAXIEList* ies = new IAXIEList;
+	ies->appendNumeric(IAXInfoElement::FORMAT,m_format.format() | m_formatVideo.format(),4);
+	ies->appendNumeric(IAXInfoElement::CAPABILITY,m_capability,4);
+	postFrameIes(IAXFrame::IAX,IAXControl::Accept,ies,0,true);
 	changeState(Connected);
     }
     else {
-	IAXIEList ieList;
-	ieList.appendString(IAXInfoElement::USERNAME,m_username);
-	ieList.appendNumeric(IAXInfoElement::REFRESH,m_expire,2);
-	ieList.appendIE(IAXInfoElementBinary::packIP(remoteAddr()));
-	DataBlock data;
-	ieList.toBuffer(data);
-	postFrame(IAXFrame::IAX,IAXControl::RegAck,data.data(),data.length(),0,true);
+	IAXIEList* ies = new IAXIEList;
+	ies->appendString(IAXInfoElement::USERNAME,m_username);
+	if (type() == RegReq) {
+	    if (expires)
+		m_expire = *expires;
+	    ies->appendNumeric(IAXInfoElement::REFRESH,m_expire,2);
+	}
+	ies->appendIE(IAXInfoElementBinary::packIP(remoteAddr()));
+	postFrameIes(IAXFrame::IAX,IAXControl::RegAck,ies,0,true);
 	changeState(Terminating);
 	m_localReqEnd = true;
     }
@@ -496,27 +637,15 @@ bool IAXTransaction::sendAccept()
 
 bool IAXTransaction::sendHangup(const char* cause, u_int8_t code)
 {
-    String s(cause);
-    unsigned char d[3];
-    DataBlock data,aux;
-
     Lock lock(this);
     if (type() != New || state() == Terminated || state() == Terminating)
 	return false;
-    if (cause) {
-	d[0] = IAXInfoElement::CAUSE;
-	d[1] = s.length();
-	data.assign(d,2);
-	data.append(s);
-    }
-    if (code) {
-	d[0] = IAXInfoElement::CAUSECODE;
-	d[1] = 1;
-	d[2] = code;
-	aux.assign(d,3);
-	data += aux;
-    }
-    postFrame(IAXFrame::IAX,IAXControl::Hangup,data.data(),data.length(),0,true);
+    IAXIEList* ies = new IAXIEList;
+    if (!TelEngine::null(cause))
+	ies->appendString(IAXInfoElement::CAUSE,cause);
+    if (code)
+	ies->appendNumeric(IAXInfoElement::CAUSECODE,code,1);
+    postFrameIes(IAXFrame::IAX,IAXControl::Hangup,ies,0,true);
     changeState(Terminating);
     m_localReqEnd = true;
     Debug(m_engine,DebugAll,"Transaction(%u,%u). Hangup call. Cause: '%s'",localCallNo(),remoteCallNo(),cause);
@@ -525,10 +654,6 @@ bool IAXTransaction::sendHangup(const char* cause, u_int8_t code)
 
 bool IAXTransaction::sendReject(const char* cause, u_int8_t code)
 {
-    String s(cause);
-    unsigned char d[3];
-    DataBlock data,aux;
-
     Lock lock(this);
     if (state() == Terminated || state() == Terminating)
 	return false;
@@ -545,20 +670,12 @@ bool IAXTransaction::sendReject(const char* cause, u_int8_t code)
 	default:
 	    return false;
     }
-    if (cause) {
-	d[0] = IAXInfoElement::CAUSE;
-	d[1] = s.length();
-	data.assign(d,2);
-	data.append(s);
-    }
-    if (code) {
-	d[0] = IAXInfoElement::CAUSECODE;
-	d[1] = 1;
-	d[2] = code;
-	aux.assign(d,3);
-	data += aux;
-    }
-    postFrame(IAXFrame::IAX,frametype,data.data(),data.length(),0,true);
+    IAXIEList* ies = new IAXIEList;
+    if (!TelEngine::null(cause))
+	ies->appendString(IAXInfoElement::CAUSE,cause);
+    if (code)
+	ies->appendNumeric(IAXInfoElement::CAUSECODE,code,1);
+    postFrameIes(IAXFrame::IAX,frametype,ies,0,true);
     Debug(m_engine,DebugAll,"Transaction(%u,%u). Reject. Cause: '%s'",localCallNo(),remoteCallNo(),cause);
     changeState(Terminating);
     m_localReqEnd = true;
@@ -572,29 +689,31 @@ bool IAXTransaction::sendAuth()
 	return false;
     switch (m_authmethod) {
 	case IAXAuthMethod::MD5:
-	    m_challenge = (int)random();
+	    m_challenge = (int)Random::random();
 	    break;
 	case IAXAuthMethod::RSA:
 	case IAXAuthMethod::Text:
 	default:
 	    return false;
     }
-    IAXIEList ieList;
-    ieList.appendString(IAXInfoElement::USERNAME,m_username);
-    ieList.appendNumeric(IAXInfoElement::AUTHMETHODS,m_authmethod,2);
-    ieList.appendString(IAXInfoElement::CHALLENGE,m_challenge);
-    DataBlock data;
-    ieList.toBuffer(data);
+    IAXControl::Type t = IAXControl::Unsupport;
     switch (type()) {
 	case New:
-	    postFrame(IAXFrame::IAX,IAXControl::AuthReq,data.data(),data.length(),0,false);
+	    t = IAXControl::AuthReq;
 	    break;
 	case RegReq:
 	case RegRel:
-	    postFrame(IAXFrame::IAX,IAXControl::RegAuth,data.data(),data.length(),0,false);
+	    t = IAXControl::RegAuth;
 	    break;
 	default: ;
     }
+    if (t != IAXControl::Unsupport) {
+	IAXIEList* ies = new IAXIEList;
+	ies->appendString(IAXInfoElement::USERNAME,m_username);
+	ies->appendNumeric(IAXInfoElement::AUTHMETHODS,m_authmethod,2);
+	ies->appendString(IAXInfoElement::CHALLENGE,m_challenge);
+	postFrameIes(IAXFrame::IAX,t,ies);
+    }
     changeState(NewRemoteInvite_AuthSent);
     return true;
 }
@@ -605,7 +724,7 @@ bool IAXTransaction::sendAuthReply(const String& response)
     if (state() != NewLocalInvite_AuthRecv)
 	return false;
     m_authdata = response;
-    IAXIEList ieList;
+    IAXIEList* ies = new IAXIEList;
     IAXControl::Type subclass;
     switch (type()) {
 	case New:
@@ -613,21 +732,23 @@ bool IAXTransaction::sendAuthReply(const String& response)
 	    break;
 	case RegReq:
 	    subclass = IAXControl::RegReq;
-	    ieList.appendString(IAXInfoElement::USERNAME,m_username);
+	    ies->appendString(IAXInfoElement::USERNAME,m_username);
+	    ies->appendNumeric(IAXInfoElement::REFRESH,m_expire,2);
 	    break;
 	case RegRel:
 	    subclass = IAXControl::RegRel;
-	    ieList.appendString(IAXInfoElement::USERNAME,m_username);
+	    ies->appendString(IAXInfoElement::USERNAME,m_username);
 	    break;
 	default:
+	    delete ies;
 	    return false;
     }
-    if (m_authmethod != IAXAuthMethod::MD5)
+    if (m_authmethod != IAXAuthMethod::MD5) {
+	delete ies;
 	return false;
-    ieList.appendString(IAXInfoElement::MD5_RESULT,response);
-    DataBlock data;
-    ieList.toBuffer(data);
-    postFrame(IAXFrame::IAX,subclass,data.data(),data.length(),0,false);
+    }
+    ies->appendString(IAXInfoElement::MD5_RESULT,response);
+    postFrameIes(IAXFrame::IAX,subclass,ies);
     changeState(NewLocalInvite_RepSent);
     return true;
 }
@@ -662,8 +783,11 @@ bool IAXTransaction::abortReg()
     if (!(type() == RegReq || type() == RegRel) ||
 	state() == Terminating || state() == Terminated)
 	return false;
+    lock();
     m_userdata = 0;
-    sendReject();
+    m_outFrames.clear();
+    unlock();
+    sendReject("Aborted");
     return true;
 }
 
@@ -678,28 +802,91 @@ bool IAXTransaction::enableTrunking(IAXMetaTrunkFrame* trunkFrame)
     return true;
 }
 
-void IAXTransaction::print()
+// Process a received call token
+void IAXTransaction::processCallToken(const DataBlock& callToken)
 {
-    static SocketAddr addr;
-    ObjList* l;
-    String frames;
-    frames << "\r\nOutgoing frames: " << m_outFrames.count();
-    for(l = m_outFrames.skipNull(); l; l = l->skipNext()) {
-	IAXFrameOut* frame = static_cast<IAXFrameOut*>(l->get());
-	frame->toString(frames,addr,remoteAddr(),false);
+    Lock lock(this);
+    IAXFrameOut* frame = 0;
+    if (state() == NewLocalInvite && m_callToken) {
+	ObjList* o = m_outFrames.skipNull();
+	frame = o ? static_cast<IAXFrameOut*>(o->get()) : 0;
+	if (frame && frame->type() != IAXFrame::IAX && frame->subclass() != IAXControl::New)
+	    frame = 0;
     }
-    frames << "\r\nIncoming frames: " << m_inFrames.count();
-    for(l = m_inFrames.skipNull(); l; l = l->skipNext()) {
-	IAXFrameOut* frame = static_cast<IAXFrameOut*>(l->get());
-	frame->toString(frames,addr,remoteAddr(),true);
+    m_callToken = false;
+    if (!frame) {
+	Debug(m_engine,DebugNote,
+	    "Transaction(%u,%u). Received call token in invalid state [%p]",
+	    localCallNo(),remoteCallNo(),this);
+	return;
     }
-    Debug(m_engine,DebugInfo,"Transaction(%u,%u). Remote address: %s:%u. Type: %u. State: %u. Timestamp: " FMT64U " [%p]%s",
-	localCallNo(),remoteCallNo(),remoteAddr().host().c_str(),remoteAddr().port(),
-	type(),state(),(u_int64_t)timeStamp(),this,frames.c_str());
+    frame->updateIEList(false);
+    IAXIEList* ies = frame->ieList();
+    if (!ies) {
+	Debug(m_engine,DebugNote,
+	    "Transaction(%u,%u). No IE list in first frame [%p]",
+	    localCallNo(),remoteCallNo(),this);
+	return;
+    }
+    IAXInfoElementBinary* ct = static_cast<IAXInfoElementBinary*>(ies->getIE(IAXInfoElement::CALLTOKEN));
+    if (ct)
+	ct->setData(callToken.data(),callToken.length());
+    else
+	ies->appendBinary(IAXInfoElement::CALLTOKEN,(unsigned char*)callToken.data(),callToken.length());
+    frame->updateBuffer(m_engine->maxFullFrameDataLen());
+    sendFrame(frame);
+}
+
+void IAXTransaction::print(bool printStats, bool printFrames, const char* location)
+{
+    if (m_engine && !m_engine->debugAt(DebugAll))
+	return;
+    String stats;
+    if (printStats && m_type == New) {
+	stats << " audio: ";
+	m_dataAudio.print(stats);
+	stats << " video: ";
+	m_dataVideo.print(stats);
+    }
+    String buf;
+    if (printFrames) {
+	SocketAddr addr;
+	ObjList* l;
+	buf << "\r\nOutgoing frames: " << m_outFrames.count();
+	for(l = m_outFrames.skipNull(); l; l = l->skipNext()) {
+	    IAXFrameOut* frame = static_cast<IAXFrameOut*>(l->get());
+	    frame->toString(buf,addr,remoteAddr(),false);
+	}
+	buf << "\r\nIncoming frames: " << m_inFrames.count();
+	for(l = m_inFrames.skipNull(); l; l = l->skipNext()) {
+	    IAXFullFrame* frame = static_cast<IAXFullFrame*>(l->get());
+	    frame->toString(buf,addr,remoteAddr(),true);
+	}
+    }
+    Debug(m_engine,DebugAll,
+	"Transaction(%u,%u) %s remote=%s:%u type=%u state=%u timestamp=" FMT64U "%s [%p]%s%s%s",
+	localCallNo(),remoteCallNo(),location,remoteAddr().host().c_str(),remoteAddr().port(),
+	type(),state(),(u_int64_t)timeStamp(),stats.safe(),this,
+	buf ? "\r\n-----" : "",buf.safe(),buf ? "\r\n-----" : "");
+}
+
+// Cleanup
+void IAXTransaction::destroyed()
+{
+#ifndef DEBUG
+    print(false,false,"destroyed");
+#else
+    print(true,true,"destroyed");
+#endif
+    TelEngine::destruct(m_trunkFrame);
+    if (state() != Terminating && state() != Terminated)
+	sendReject("Server shutdown");
+    RefObject::destroyed();
 }
 
 void IAXTransaction::init(IAXIEList& ieList)
 {
+    u_int32_t fmt = 0;
     switch (type()) {
 	case New:
 	    ieList.getString(IAXInfoElement::USERNAME,m_username);
@@ -707,12 +894,14 @@ void IAXTransaction::init(IAXIEList& ieList)
 	    ieList.getString(IAXInfoElement::CALLING_NAME,m_callingName);
 	    ieList.getString(IAXInfoElement::CALLED_NUMBER,m_calledNo);
 	    ieList.getString(IAXInfoElement::CALLED_CONTEXT,m_calledContext);
-	    ieList.getNumeric(IAXInfoElement::FORMAT,m_format);
+	    ieList.getNumeric(IAXInfoElement::FORMAT,fmt);
 	    ieList.getNumeric(IAXInfoElement::CAPABILITY,m_capability);
+	    m_capability &= m_engine->capability();
+	    fmt &= m_capability;
+	    m_format.set(&fmt,&fmt,&fmt);
+	    m_formatVideo.set(&fmt,&fmt,&fmt);
 	    if (outgoing())
-		m_formatOut = m_format;
-	    else
-		m_formatIn = m_format;
+		m_callToken = (0 != ieList.getIE(IAXInfoElement::CALLTOKEN));
 	    break;
 	case RegReq:
 	    ieList.getString(IAXInfoElement::CALLED_NUMBER,m_calledNo);
@@ -785,7 +974,7 @@ bool IAXTransaction::changeState(State newState)
     return true;
 }
 
-IAXEvent* IAXTransaction::terminate(u_int8_t evType, bool local, const IAXFullFrame* frame, bool createIEList)
+IAXEvent* IAXTransaction::terminate(u_int8_t evType, bool local, IAXFullFrame* frame, bool createIEList)
 {
     IAXEvent* ev;
     if (createIEList)
@@ -802,7 +991,7 @@ IAXEvent* IAXTransaction::terminate(u_int8_t evType, bool local, const IAXFullFr
     return ev;
 }
 
-IAXEvent* IAXTransaction::waitForTerminate(u_int8_t evType, bool local, const IAXFullFrame* frame)
+IAXEvent* IAXTransaction::waitForTerminate(u_int8_t evType, bool local, IAXFullFrame* frame)
 {
     IAXEvent* ev = new IAXEvent((IAXEvent::Type)evType,local,true,this,frame);
     Debug(m_engine,DebugAll,"Transaction(%u,%u). Terminating. Event: %u, Frame(%u,%u)",
@@ -812,28 +1001,30 @@ IAXEvent* IAXTransaction::waitForTerminate(u_int8_t evType, bool local, const IA
     return ev;
 }
 
-void IAXTransaction::postFrame(IAXFrame::Type type, u_int32_t subclass, void* data, u_int16_t len, u_int32_t tStamp, bool ackOnly)
+void IAXTransaction::postFrame(IAXFrame::Type type, u_int32_t subclass, void* data,
+    u_int16_t len, u_int32_t tStamp, bool ackOnly, bool mark)
 {
     Lock lock(this);
     if (state() == Terminated)
 	return;
-    if (!tStamp) {
-	tStamp = (u_int32_t)timeStamp();
-	if (m_lastFullFrameOut) {
-	    // adjust timestamp to be different from the last sent
-	    int32_t delta = tStamp - m_lastFullFrameOut;
-	    if (delta <= 0)
-		tStamp = m_lastFullFrameOut + 1;
-	}
-	m_lastFullFrameOut = tStamp;
-    }
+    adjustTStamp(tStamp);
     IAXFrameOut* frame = new IAXFrameOut(type,subclass,m_lCallNo,m_rCallNo,m_oSeqNo,m_iSeqNo,tStamp,
-			 (unsigned char*)data,len,m_retransCount,m_retransInterval,ackOnly);
-    DDebug(m_engine,DebugAll,"Transaction(%u,%u) posting Frame(%u,%u) oseq=%u iseq=%u stamp=%u [%p]",
-	localCallNo(),remoteCallNo(),type,subclass,m_oSeqNo,m_iSeqNo,tStamp,this);
-    incrementSeqNo(frame,false);
-    m_outFrames.append(frame);
-    sendFrame(frame);
+			 (unsigned char*)data,len,m_retransCount,m_retransInterval,ackOnly,mark);
+    postFrame(frame);
+}
+
+// Constructs an IAXFrameOut frame, send it to remote peer and put it in the transmission list
+void IAXTransaction::postFrameIes(IAXFrame::Type type, u_int32_t subclass, IAXIEList* ies,
+    u_int32_t tStamp, bool ackOnly)
+{
+    Lock lock(this);
+    if (state() == Terminated)
+	return;
+    adjustTStamp(tStamp);
+    IAXFrameOut* frame = new IAXFrameOut(type,subclass,m_lCallNo,m_rCallNo,m_oSeqNo,
+	m_iSeqNo,tStamp,ies,m_engine->maxFullFrameDataLen(),m_retransCount,
+	m_retransInterval,ackOnly);
+    postFrame(frame);
 }
 
 bool IAXTransaction::sendFrame(IAXFrameOut* frame, bool vnak)
@@ -851,7 +1042,7 @@ bool IAXTransaction::sendFrame(IAXFrameOut* frame, bool vnak)
     return b;
 }
 
-IAXEvent* IAXTransaction::createEvent(u_int8_t evType, bool local, const IAXFullFrame* frame, State newState)
+IAXEvent* IAXTransaction::createEvent(u_int8_t evType, bool local, IAXFullFrame* frame, State newState)
 {
     IAXEvent* ev;
     changeState(newState);
@@ -992,12 +1183,13 @@ IAXEvent* IAXTransaction::processAccept(IAXEvent* event)
     if (event->type() != IAXEvent::Accept)
 	return event;
     Debug(m_engine,DebugAll,"Transaction(%u,%u). Accept received",localCallNo(),remoteCallNo());
-    // We might have a format received with a Voice frame
-    if (m_formatIn)
-	return event;
-    m_format = 0;
-    event->getList().getNumeric(IAXInfoElement::FORMAT,m_format);
-    if (m_engine->acceptFormatAndCapability(this))
+    u_int32_t fmt = 0;
+    event->getList().getNumeric(IAXInfoElement::FORMAT,fmt);
+    m_format.set(&fmt,0,0);
+    m_formatVideo.set(&fmt,0,0);
+    m_engine->acceptFormatAndCapability(this,0,IAXFormat::Audio);
+    m_engine->acceptFormatAndCapability(this,0,IAXFormat::Video);
+    if (m_format.format() || m_formatVideo.format())
 	return event;
     delete event;
     return internalReject(s_iax_modNoMediaFormat);
@@ -1254,7 +1446,7 @@ bool IAXTransaction::sendConnected(IAXFullFrame::ControlType subclass, IAXFrame:
 {
     if (state() != Connected)
 	return false;
-    postFrame(frametype,subclass,0,0,0,true);
+    postFrameIes(frametype,subclass,0,0,true);
     return true;
 }
 
@@ -1288,8 +1480,10 @@ void IAXTransaction::sendVNAK()
 
 void IAXTransaction::sendUnsupport(u_int32_t subclass)
 {
-    unsigned char d[3] = {IAXInfoElement::IAX_UNKNOWN,1,IAXFrame::packSubclass(subclass)};
-    postFrame(IAXFrame::IAX,IAXControl::Unsupport,d,sizeof(d),0,true);
+    IAXIEList* ies = new IAXIEList;
+    u_int8_t val = IAXFrame::packSubclass(subclass);
+    ies->appendNumeric(IAXInfoElement::IAX_UNKNOWN,val,1);
+    postFrameIes(IAXFrame::IAX,IAXControl::Unsupport,ies,0,true);
 }
 
 IAXEvent* IAXTransaction::processInternalOutgoingRequest(IAXFrameOut* frame, bool& delFrame)
@@ -1325,7 +1519,7 @@ IAXEvent* IAXTransaction::processInternalIncomingRequest(const IAXFullFrame* fra
     return 0;
 }
 
-IAXEvent* IAXTransaction::processMidCallControl(const IAXFullFrame* frame, bool& delFrame)
+IAXEvent* IAXTransaction::processMidCallControl(IAXFullFrame* frame, bool& delFrame)
 {
     delFrame = true;
     switch (frame->subclass()) {
@@ -1355,7 +1549,7 @@ IAXEvent* IAXTransaction::processMidCallControl(const IAXFullFrame* frame, bool&
     return 0;
 }
 
-IAXEvent* IAXTransaction::processMidCallIAXControl(const IAXFullFrame* frame, bool& delFrame)
+IAXEvent* IAXTransaction::processMidCallIAXControl(IAXFullFrame* frame, bool& delFrame)
 {
     delFrame = true;
     switch (frame->subclass()) {
@@ -1405,7 +1599,7 @@ IAXEvent* IAXTransaction::processMidCallIAXControl(const IAXFullFrame* frame, bo
     return 0;
 }
 
-IAXEvent* IAXTransaction::remoteRejectCall(const IAXFullFrame* frame, bool& delFrame)
+IAXEvent* IAXTransaction::remoteRejectCall(IAXFullFrame* frame, bool& delFrame)
 {
     delFrame = true;
     switch (type()) {
@@ -1435,33 +1629,60 @@ IAXEvent* IAXTransaction::getEventTerminating(u_int64_t time)
     return 0;
 }
 
-IAXTransaction* IAXTransaction::processVoiceFrame(const IAXFullFrame* frame)
+IAXTransaction* IAXTransaction::processMediaFrame(const IAXFullFrame* frame, int type)
 {
-    // Process format 
-    DDebug(m_engine,DebugAll,"Transaction(%u,%u). Received Voice Frame(%u,%u) iseq=%u oseq=%u stamp=%u [%p]",
-	localCallNo(),remoteCallNo(),frame->type(),frame->subclass(),
+    DDebug(m_engine,DebugAll,
+	"Transaction(%u,%u). Received %s (%u,%u) iseq=%u oseq=%u stamp=%u [%p]",
+	localCallNo(),remoteCallNo(),IAXFrame::typeText(frame->type()),
+	frame->type(),frame->subclass(),
 	frame->iSeqNo(),frame->oSeqNo(),frame->timeStamp(),this);
     sendAck(frame);
+    IAXFormat* fmt = getFormat(type);
+    if (!fmt)
+	return this;
+    if (!frame->subclass())
+	return this;
+    // Check the format
+    u_int32_t recvFmt = IAXFormat::mask(frame->subclass(),type);
+    if (recvFmt == fmt->in())
+        return this;
+    if (!recvFmt) {
+	String tmp;
+	IAXFormat::formatList(tmp,frame->subclass());
+	Debug(m_engine,DebugInfo,
+	    "IAXTransaction(%u,%u). Received %s frame with invalid format=%s (0x%x) [%p]",
+	    localCallNo(),remoteCallNo(),IAXFrame::typeText(frame->type()),
+	    tmp.c_str(),frame->subclass(),this);
+	return this;
+    }
+    if (!IAXFormat::formatName(recvFmt)) {
+	Debug(m_engine,DebugNote,
+	    "IAXTransaction(%u,%u). Received %s frame with unknown format=0x%x [%p]",
+	    localCallNo(),remoteCallNo(),IAXFrame::typeText(frame->type()),recvFmt,this);
+	m_pendingEvent = internalReject(s_iax_modNoMediaFormat);
+	return 0;
+    }
     // We might have an incoming media format received with an Accept frame
-    if (m_formatIn) {
-	if (frame->subclass() && frame->subclass() != m_formatIn) {
-	    // Format changed.
-	    if (m_engine->voiceFormatChanged(this,frame->fullFrame()->subclass()))
-		m_formatIn = frame->fullFrame()->subclass();
-	    else {
-		DDebug(m_engine,DebugAll,"IAXTransaction(%u,%u). Process Voice Frame. Media format (%u) change rejected!",
-		    localCallNo(),remoteCallNo(),m_format);
-		m_pendingEvent = internalReject(s_iax_modNoMediaFormat);
-		return 0;
-	    }
+    if (fmt->in()) {
+	// Format changed.
+	if (m_engine->mediaFormatChanged(this,type,recvFmt)) {
+	    Debug(m_engine,DebugNote,
+		"Transaction(%u,%u). Incoming %s format changed %u --> %u [%p]",
+		localCallNo(),remoteCallNo(),fmt->typeName(),fmt->in(),recvFmt,this);
+	    fmt->set(0,&recvFmt,0);
+	}
+	else {
+	    DDebug(m_engine,DebugAll,
+		"IAXTransaction(%u,%u). Format change rejected media=%s current=%u [%p]",
+		localCallNo(),remoteCallNo(),fmt->typeName(),fmt->format(),this);
+	    m_pendingEvent = internalReject(s_iax_modNoMediaFormat);
+	    return 0;
         }
     }
     else {
-	m_format = frame->subclass();
-	if (!m_engine->acceptFormatAndCapability(this)) {
-	    m_pendingEvent = internalReject(s_iax_modNoMediaFormat);
+	fmt->set(&recvFmt,0,0);
+	if (!m_engine->acceptFormatAndCapability(this,0,type))
 	    return 0;
-	}
     }
     return this;
 }
@@ -1508,4 +1729,31 @@ void IAXTransaction::eventTerminated(IAXEvent* event)
     }
 }
 
+void IAXTransaction::adjustTStamp(u_int32_t& tStamp)
+{
+    if (tStamp)
+	return;
+    tStamp = (u_int32_t)timeStamp();
+    if (m_lastFullFrameOut) {
+	// adjust timestamp to be different from the last sent
+	int32_t delta = tStamp - m_lastFullFrameOut;
+	if (delta <= 0)
+	    tStamp = m_lastFullFrameOut + 1;
+    }
+    m_lastFullFrameOut = tStamp;
+}
+
+void IAXTransaction::postFrame(IAXFrameOut* frame)
+{
+    if (!frame)
+	return;
+    DDebug(m_engine,DebugAll,
+	"Transaction(%u,%u) posting Frame(%u,%u) oseq=%u iseq=%u stamp=%u [%p]",
+	localCallNo(),remoteCallNo(),frame->type(),frame->subclass(),
+	m_oSeqNo,m_iSeqNo,frame->timeStamp(),this);
+    incrementSeqNo(frame,false);
+    m_outFrames.append(frame);
+    sendFrame(frame);
+}
+
 /* vi: set ts=8 sw=4 sts=4 noet: */
