@@ -51,6 +51,7 @@ class IAXInfoElement;
 class IAXInfoElementString;
 class IAXInfoElementNumeric;
 class IAXInfoElementBinary;
+class IAXFrame;
 class IAXFullFrame;
 class IAXFrameOut;
 class IAXEvent;
@@ -123,6 +124,7 @@ public:
         RR_DELAY = 0x31,         // W
         RR_DROPPED = 0x32,       // DW
         RR_OOO = 0x33,           // DW
+        CALLTOKEN = 0X36,        // BIN
     };
 
     /**
@@ -307,6 +309,14 @@ public:
     inline DataBlock& data()
         { return m_data; }
 
+    /**
+     * Set the data
+     * @param buf Source buffer to construct this IE
+     * @param len Buffer length
+     */
+    inline void setData(void* buf, unsigned len)
+        { m_data.assign(buf,len); }
+
     /**
      * Constructs a buffer containing this Information Element
      * @param buf Destination buffer
@@ -348,22 +358,19 @@ public:
     /**
      * Constructor
      */
-    inline IAXIEList() : m_invalidIEList(false)
-	{}
+    IAXIEList();
 
     /**
      * Constructor. Construct the list from an IAXFullFrame object
      * @param frame Source object
      * @param incoming True if it is an incoming frame
      */
-    inline IAXIEList(const IAXFullFrame* frame, bool incoming = true) : m_invalidIEList(false)
-	{ createFromFrame(frame,incoming); }
+    IAXIEList(const IAXFullFrame* frame, bool incoming = true);
 
     /**
      * Destructor
      */
-    inline ~IAXIEList()
-	{}
+    ~IAXIEList();
 
     /**
      * Get the invalid IE list flag
@@ -378,6 +385,13 @@ public:
     inline void clear()
 	{ m_list.clear(); }
 
+    /**
+     * Check if the list is empty
+     * @return True if the list is empty
+     */
+    inline bool empty()
+	{ return 0 == m_list.skipNull(); }
+
     /**
      * Insert a VERSION Information Element in the list if not already done
      */
@@ -535,65 +549,193 @@ class YIAX_API IAXFormat
 {
 public:
     /**
-     * Audio format enumeration types
+     * Format enumeration types
      */
-    enum Audio {
+    enum Formats {
         G723_1 = (1 <<  0),
         GSM    = (1 <<  1),
         ULAW   = (1 <<  2),
         ALAW   = (1 <<  3),
-        MP3    = (1 <<  4),
+        G726   = (1 <<  4),
         ADPCM  = (1 <<  5),
         SLIN   = (1 <<  6),
         LPC10  = (1 <<  7),
-        G729A  = (1 <<  8),
+        G729   = (1 <<  8),
         SPEEX  = (1 <<  9),
         ILBC   = (1 << 10),
+        G726AAL2 = (1 << 11),
+        G722   = (1 << 12),
+        AMR    = (1 << 13),
+        AudioMask = G723_1 | GSM | ULAW | ALAW | G726 | ADPCM | SLIN | LPC10 | G729 | SPEEX |
+            ILBC | G726AAL2 | G722 | AMR,
+        JPEG   = (1 << 16),
+        PNG    = (1 << 17),
+        ImageMask = JPEG | PNG,
+        H261   = (1 << 18),
+        H263   = (1 << 19),
+        H263p  = (1 << 20),
+        H264   = (1 << 21),
+        VideoMask = H261 | H263 | H263p | H264,
     };
 
     /**
-     * Video format enumeration types
+     * Media type enumeration
      */
-    enum Video {
-        JPEG   = (1 << 16),
-        PNG    = (1 << 17),
-        H261   = (1 << 18),
-        H263   = (1 << 19),
+    enum Media {
+        Audio = 0,
+        Video,
+        Image,
+        TypeCount
     };
 
+    /**
+     * Constructor. Build an audio format
+     * @param type Media type
+    */
+    inline IAXFormat(int type = Audio)
+	: m_type(type), m_format(0), m_formatIn(0), m_formatOut(0)
+	{}
+
+    /**
+     * Get the media type
+     * @return Media type
+    */
+    inline int type() const
+	{ return m_type; }
+
+    /**
+     * Get the format
+     * @return The format
+    */
+    inline u_int32_t format() const
+	{ return m_format; }
+
+    /**
+     * Get the incoming format
+     * @return The incoming format
+    */
+    inline u_int32_t in() const
+	{ return m_formatIn; }
+
+    /**
+     * Get the outgoing format
+     * @return The outgoing format
+    */
+    inline u_int32_t out() const
+	{ return m_formatOut; }
+
+    /**
+     * Get the text associated with the format
+     * @return Format name
+    */
+    inline const char* formatName() const
+	{ return formatName(m_format); }
+
+    /**
+     * Get the text associated with the media type
+     * @return Media name
+    */
+    inline const char* typeName() const
+	{ return typeName(m_type); }
+
+    /**
+     * Set format
+     * @param fmt Optional pointer to format to set
+     * @param fmtIn Optional pointer to incoming format to set
+     * @param fmtOut Optional pointer to outgoing format to set
+    */
+    void set(u_int32_t* fmt, u_int32_t* fmtIn, u_int32_t* fmtOut);
+
     /**
      * Create a string list from formats
      * @param dest The destination
      * @param formats The formats
+     * @param dict Optional dictionary to use, 0 to use s_formats
      * @param sep The separator to use
     */
-    static void formatList(String& dest, u_int32_t formats, char sep = ',');
+    static void formatList(String& dest, u_int32_t formats, const TokenDict* dict = 0,
+	const char* sep = ",");
 
     /**
-     * Get the text associated with an audio format
-     * @param audio The desired format
+     * Pick a format from a list of capabilities
+     * @param formats Capabilities list
+     * @param format Optional format to pick
+     * @return IAX format, 0 if not found
+    */
+    static u_int32_t pickFormat(u_int32_t formats, u_int32_t format = 0);
+
+    /**
+     * Encode a formats list
+     * @param formats Formats list
+     * @param dict Dictionary to use
+     * @param sep Formats list separator
+     * @return Encoded formats
+    */
+    static u_int32_t encode(const String& formats, const TokenDict* dict, char sep = ',');
+
+    /**
+     * Mask formats by type
+     * @param value Input format(s)
+     * @param type Media type to retrieve
+     * @return Media format(s) from input
+    */
+    static inline u_int32_t mask(u_int32_t value, int type) {
+	    if (type == Audio)
+		return value & AudioMask;
+	    if (type == Video)
+		return value & VideoMask;
+	    if (type == Image)
+		return value & ImageMask;
+	    return 0;
+	}
+
+    /**
+     * Clear formats by type
+     * @param value Input format(s)
+     * @param type Media type to clear
+     * @return Cleared format(s) from input
+    */
+    static inline u_int32_t clear(u_int32_t value, int type) {
+	    if (type == Audio)
+		return value & ~AudioMask;
+	    if (type == Video)
+		return value & ~VideoMask;
+	    if (type == Image)
+		return value & ~ImageMask;
+	    return value;
+	}
+
+    /**
+     * Get the text associated with a format
+     * @param fmt The desired format
      * @return A pointer to the text associated with the format or 0 if the format doesn't exist
     */
-    static inline const char* audioText(u_int32_t audio)
-	{ return lookup(audio,audioData); }
+    static inline const char* formatName(u_int32_t fmt)
+	{ return lookup(fmt,s_formats); }
 
     /**
-     * Get the text associated with a video format
-     * @param video The desired format
-     * @return A pointer to the text associated with the format or 0 if the format doesn't exist
+     * Get the text associated with a media type
+     * @param type The media type
+     * @return A pointer to the text associated with the media type
     */
-    static inline const char* videoText(u_int32_t video)
-	{ return lookup(video,videoData); }
+    static inline const char* typeName(int type)
+	{ return lookup(type,s_types); }
 
     /**
-     * Keep the texts associated with the audio formats
+     * Keep the texts associated with the formats
     */
-    static TokenDict audioData[];
+    static const TokenDict s_formats[];
 
     /**
-     * Keep the texts associated with the video formats
-    */
-    static TokenDict videoData[];
+     * Keep the texts associated with type
+     */
+    static const TokenDict s_types[];
+
+protected:
+    int m_type;
+    u_int32_t m_format;
+    u_int32_t m_formatIn;
+    u_int32_t m_formatOut;
 };
 
 /**
@@ -644,6 +786,7 @@ public:
         Provision = 0x23,
         FwDownl   = 0x24,
         FwData    = 0x25,
+        CallToken = 0x28,
     };
 
     /**
@@ -689,9 +832,10 @@ public:
      * @param retrans Retransmission flag
      * @param buf IE buffer
      * @param len IE buffer length
+     * @param mark Mark flag
      */
     IAXFrame(Type type, u_int16_t sCallNo, u_int32_t tStamp, bool retrans,
-	     const unsigned char* buf, unsigned int len);
+	     const unsigned char* buf, unsigned int len, bool mark = false);
 
     /**
      * Destructor
@@ -733,11 +877,18 @@ public:
     inline u_int32_t timeStamp() const
 	{ return m_tStamp; }
 
+    /**
+     * Get the mark flag
+     * @return The mark flag
+     */
+    inline bool mark() const
+	{ return m_mark; }
+
     /**
      * Get a pointer to this frame if it is a full frame
      * @return A pointer to this frame if it is a full frame or 0
      */
-    virtual const IAXFullFrame* fullFrame() const;
+    virtual IAXFullFrame* fullFrame();
 
     /**
      * Parse a received buffer and returns a IAXFrame pointer if valid
@@ -749,6 +900,29 @@ public:
      */
     static IAXFrame* parse(const unsigned char* buf, unsigned int len, IAXEngine* engine = 0, const SocketAddr* addr = 0);
 
+    /**
+     * Build a miniframe buffer
+     * @param dest Destination buffer
+     * @param sCallNo Source call number
+     * @param tStamp Frame timestamp
+     * @param data Data
+     * @param len Data length
+     */
+    static void buildMiniFrame(DataBlock& dest, u_int16_t sCallNo, u_int32_t tStamp,
+	void* data, unsigned int len);
+
+    /**
+     * Build a video meta frame buffer
+     * @param dest Destination buffer
+     * @param sCallNo Source call number
+     * @param tStamp Frame timestamp
+     * @param mark Frame mark
+     * @param data Data
+     * @param len Data length
+     */
+    static void buildVideoMetaFrame(DataBlock& dest, u_int16_t sCallNo, u_int32_t tStamp,
+	bool mark, void* data, unsigned int len);
+
     /**
      * Pack a subclass value according to IAX protocol
      * @param value Value to pack
@@ -787,6 +961,7 @@ private:
     Type m_type;		// Frame type
     u_int16_t m_sCallNo;	// Source call number
     u_int32_t m_tStamp;		// Frame timestamp
+    bool m_mark;		// Mark flag
 };
 
 /**
@@ -829,11 +1004,12 @@ public:
      * @param retrans Retransmission flag
      * @param buf IE buffer
      * @param len IE buffer length
+     * @param mark Mark flag
      */
     IAXFullFrame(Type type, u_int32_t subclass, u_int16_t sCallNo, u_int16_t dCallNo,
 		 unsigned char oSeqNo, unsigned char iSeqNo,
 		 u_int32_t tStamp, bool retrans,
-		 const unsigned char* buf, unsigned int len);
+		 const unsigned char* buf, unsigned int len, bool mark = false);
 
     /**
      * Constructor. Constructs an outgoing full frame
@@ -846,11 +1022,29 @@ public:
      * @param tStamp Frame timestamp
      * @param buf IE buffer
      * @param len IE buffer length
+     * @param mark Mark flag
      */
     IAXFullFrame(Type type, u_int32_t subclass, u_int16_t sCallNo, u_int16_t dCallNo,
 		 unsigned char oSeqNo, unsigned char iSeqNo,
 		 u_int32_t tStamp,
-		 const unsigned char* buf = 0, unsigned int len = 0);
+		 const unsigned char* buf = 0, unsigned int len = 0, bool mark = false);
+
+    /**
+     * Constructor. Constructs an outgoing full frame
+     * @param type Frame type
+     * @param subclass Frame subclass
+     * @param sCallNo Source (remote) call number
+     * @param dCallNo Destination (local) call number
+     * @param oSeqNo Outgoing sequence number
+     * @param iSeqNo Incoming (expected) sequence number
+     * @param tStamp Frame timestamp
+     * @param ieList List of frame IEs
+     * @param maxlen Max frame data length
+     * @param mark Mark flag
+     */
+    IAXFullFrame(Type type, u_int32_t subclass, u_int16_t sCallNo, u_int16_t dCallNo,
+		 unsigned char oSeqNo, unsigned char iSeqNo,
+		 u_int32_t tStamp, IAXIEList* ieList, u_int16_t maxlen, bool mark = false);
 
     /**
      * Destructor
@@ -889,7 +1083,34 @@ public:
      * Get a pointer to this frame if it is a full frame
      * @return A pointer to this frame
      */
-    virtual const IAXFullFrame* fullFrame() const;
+    virtual IAXFullFrame* fullFrame();
+
+    /**
+     * Rebuild frame buffer from the list of IEs
+     * @param maxlen Max frame data length
+     */
+    void updateBuffer(u_int16_t maxlen);
+
+    /**
+     * Retrieve the IE list
+     * @return IAXIEList pointer or NULL
+     */
+    inline IAXIEList* ieList()
+	{ return m_ieList; }
+
+    /**
+     * Update IE list from buffer if not already done
+     * @param incoming True if this is an incoming frame
+     * @return True if the list is valid
+     */
+    bool updateIEList(bool incoming);
+
+    /**
+     * Remove the IE list
+     * @param delObj True to delete it
+     * @return IAXIEList pointer or NULL if requested to delete it or already NULL
+     */
+    IAXIEList* removeIEList(bool delObj = true);
 
     /**
      * Fill a string with this frame
@@ -899,7 +1120,7 @@ public:
      * @param incoming True if it is an incoming frame
      */
     void toString(String& dest, const SocketAddr& local, const SocketAddr& remote,
-	bool incoming) const;
+	bool incoming);
 
     /**
      * Get the string associated with the given IAX control type
@@ -909,12 +1130,21 @@ public:
     static inline const char* controlTypeText(int type)
 	{ return lookup(type,s_controlTypes,0); }
 
+protected:
+    /**
+     * Destroyed notification. Clear data
+     */
+    virtual void destroyed();
+
 private:
+    // Build frame buffer header
+    void setDataHeader();
     static TokenDict s_controlTypes[]; // Keep the association between control types and their names
     u_int16_t m_dCallNo;	// Destination call number
     unsigned char m_oSeqNo;	// Out sequence number
     unsigned char m_iSeqNo;	// In sequence number
     u_int32_t m_subclass;	// Subclass
+    IAXIEList* m_ieList;        // List of IEs
 };
 
 /**
@@ -938,11 +1168,37 @@ public:
      * @param retransCount Retransmission counter
      * @param retransInterval Time interval to the next retransmission
      * @param ackOnly Acknoledge only flag. If true, the frame only expects an ACK
+     * @param mark Mark flag
      */
     inline IAXFrameOut(Type type, u_int32_t subclass, u_int16_t sCallNo, u_int16_t dCallNo,
                        unsigned char oSeqNo, unsigned char iSeqNo, u_int32_t tStamp, const unsigned char* buf, unsigned int len,
-                       u_int16_t retransCount, u_int32_t retransInterval, bool ackOnly)
-        : IAXFullFrame(type,subclass,sCallNo,dCallNo,oSeqNo,iSeqNo,tStamp,buf,len),
+                       u_int16_t retransCount, u_int32_t retransInterval, bool ackOnly, bool mark = false)
+        : IAXFullFrame(type,subclass,sCallNo,dCallNo,oSeqNo,iSeqNo,tStamp,buf,len,mark),
+          m_ack(false), m_ackOnly(ackOnly), m_retransCount(retransCount), m_retransTimeInterval(retransInterval),
+	  m_nextTransTime(Time::msecNow() + m_retransTimeInterval)
+	{}
+
+    /**
+     * Constructor. Constructs an outgoing full frame
+     * @param type Frame type
+     * @param subclass Frame subclass
+     * @param sCallNo Source (remote) call number
+     * @param dCallNo Destination (local) call number
+     * @param oSeqNo Outgoing sequence number
+     * @param iSeqNo Incoming (expected) sequence number
+     * @param tStamp Frame timestamp
+     * @param ieList List of frame IEs
+     * @param maxlen Max frame data length
+     * @param retransCount Retransmission counter
+     * @param retransInterval Time interval to the next retransmission
+     * @param ackOnly Acknoledge only flag. If true, the frame only expects an ACK
+     * @param mark Mark flag
+     */
+    inline IAXFrameOut(Type type, u_int32_t subclass, u_int16_t sCallNo, u_int16_t dCallNo,
+                       unsigned char oSeqNo, unsigned char iSeqNo, u_int32_t tStamp,
+		       IAXIEList* ieList, u_int16_t maxlen,
+                       u_int16_t retransCount, u_int32_t retransInterval, bool ackOnly, bool mark = false)
+        : IAXFullFrame(type,subclass,sCallNo,dCallNo,oSeqNo,iSeqNo,tStamp,ieList,maxlen,mark),
           m_ack(false), m_ackOnly(ackOnly), m_retransCount(retransCount), m_retransTimeInterval(retransInterval),
 	  m_nextTransTime(Time::msecNow() + m_retransTimeInterval)
 	{}
@@ -1065,7 +1321,7 @@ public:
      * @param tStamp Frame timestamp
      * @return The result of the write operation
      */
-    bool send(u_int32_t tStamp);
+    bool send(u_int32_t tStamp = Time::msecNow());
 
 private:
     u_int8_t* m_data;		// Data buffer
@@ -1075,6 +1331,43 @@ private:
     SocketAddr m_addr;		// Remote peer address
 };
 
+/**
+ * This class holds data used by transaction to sync media.
+ * The mutex is not reentrant
+ * @short IAX2 transaction media data
+ */
+class YIAX_API IAXMediaData : public Mutex
+{
+    friend class IAXTransaction;
+public:
+    /**
+     * Constructor
+     */
+    inline IAXMediaData()
+	: Mutex(true,"IAXTransaction::InMedia"),
+	m_lastOut(0), m_lastIn(0), m_sent(0), m_sentBytes(0),
+	m_recv(0), m_recvBytes(0), m_ooPackets(0), m_ooBytes(0),
+	m_showInNoFmt(true)
+	{}
+
+    /**
+     * Print statistics
+     * @param buf Destination buffer
+     */
+    void print(String& buf);
+
+protected:
+    u_int32_t m_lastOut;                 // Last transmitted mini timestamp
+    u_int32_t m_lastIn;                  // Last received timestamp
+    unsigned int m_sent;                 // Packets sent
+    unsigned int m_sentBytes;            // Bytes sent
+    unsigned int m_recv;                 // Packets received
+    unsigned int m_recvBytes;            // Bytes received
+    unsigned int m_ooPackets;            // Dropped received out of order packets
+    unsigned int m_ooBytes;              // Dropped received out of order bytes
+    bool m_showInNoFmt;                  // Show incoming media arrival without format debug
+};
+
 /**
  * This class holds all the data needded for the management of an IAX2 transaction
  *  which might be a call leg, a register/unregister or a poke one
@@ -1192,105 +1485,128 @@ public:
         { return m_userdata; }
 
     /**
-     * Retrive the local call number
+     * Retrieve the local call number
      * @return 15-bit local call number
      */
     inline u_int16_t localCallNo() const
         { return m_lCallNo; }
 
     /**
-     * Retrive the remote call number
+     * Retrieve the remote call number
      * @return 15-bit remote call number
      */
     inline u_int16_t remoteCallNo() const
         { return m_rCallNo; }
 
     /**
-     * Retrive the remote host+port address
+     * Retrieve the remote host+port address
      * @return A reference to the remote address
      */
     inline const SocketAddr& remoteAddr() const
         { return m_addr; }
 
     /**
-     * Retrive the username
+     * Retrieve the username
      * @return A reference to the username
      */
     inline const String& username()
 	{ return m_username; }
 
     /**
-     * Retrive the calling number
+     * Retrieve the calling number
      * @return A reference to the calling number
      */
     inline const String& callingNo()
 	{ return m_callingNo; }
 
     /**
-     * Retrive the calling name
+     * Retrieve the calling name
      * @return A reference to the calling name
      */
     inline const String& callingName()
 	{ return m_callingName; }
 
     /**
-     * Retrive the called number
+     * Retrieve the called number
      * @return A reference to the called number
      */
     inline const String& calledNo()
 	{ return m_calledNo; }
 
     /**
-     * Retrive the called context
+     * Retrieve the called context
      * @return A reference to the called context
      */
     inline const String& calledContext()
 	{ return m_calledContext; }
 
     /**
-     * Retrive the challenge sent/received during authentication
+     * Retrieve the challenge sent/received during authentication
      * @return A reference to the challenge
      */
     inline const String& challenge()
 	{ return m_challenge; }
 
     /**
-     * Retrive the media format used during initialization
-     * @return The initial media format
+     * Retrieve the media of a given type
+     * @param type Media type to retrieve
+     * @return IAXFormat pointer or 0 for invalid type
      */
-    inline u_int32_t format()
-	{ return m_format; }
+    IAXFormat* getFormat(int type);
 
     /**
-     * Retrive the incoming media format
-     * @return The incoming media format
+     * Retrieve the media data for a given type
+     * @param type Media type to retrieve
+     * @return IAXMediaData pointer or 0 for invalid type
      */
-    inline u_int32_t formatIn()
-	{ return m_formatIn; }
+    IAXMediaData* getData(int type);
 
     /**
-     * Retrive the outgoing media format
-     * @return The outgoing media format
+     * Retrieve the media format used during initialization
+     * @param type Media type to retrieve
+     * @return The initial media format for the given type
      */
-    inline u_int32_t formatOut() const 
-	{ return m_formatOut; }
+    inline u_int32_t format(int type) {
+	    IAXFormat* fmt = getFormat(type);
+	    return fmt ? fmt->format() : 0;
+	}
 
     /**
-     * Retrive the media capability of this transaction
+     * Retrieve the incoming media format
+     * @param type Media type to retrieve
+     * @return The incoming media format for the given type
+     */
+    inline u_int32_t formatIn(int type) {
+	    IAXFormat* fmt = getFormat(type);
+	    return fmt ? fmt->in() : 0;
+	}
+
+    /**
+     * Retrieve the outgoing media format
+     * @param type Media type to retrieve
+     * @return The outgoing media format for the given type
+     */
+    inline u_int32_t formatOut(int type) {
+	    IAXFormat* fmt = getFormat(type);
+	    return fmt ? fmt->out() : 0;
+	}
+
+    /**
+     * Retrieve the media capability of this transaction
      * @return The media capability of this transaction
      */
     inline u_int32_t capability() const
 	{ return m_capability; }
 
     /**
-     * Retrive the expiring time for a register/unregister transaction
+     * Retrieve the expiring time for a register/unregister transaction
      * @return The expiring time for a register/unregister transaction
      */
     inline u_int32_t expire() const
 	{ return m_expire; }
 
     /**
-     * Retrive the authentication data sent/received during authentication
+     * Retrieve the authentication data sent/received during authentication
      * @return A reference to the authentication data
      */
     inline const String& authdata()
@@ -1305,21 +1621,27 @@ public:
     IAXTransaction* processFrame(IAXFrame* frame);
 
     /**
-     * Process received mini frame data
+     * Process received media data
      * @param data Received data
      * @param tStamp Mini frame timestamp
-     * @param voice True if received mini frame inside a Voice full frame
+     * @param type Media type
+     * @param full True if received in a full frame
+     * @param mark Mark flag
      * @return 0
      */
-    IAXTransaction* processMedia(DataBlock& data, u_int32_t tStamp, bool voice = false);
+    IAXTransaction* processMedia(DataBlock& data, u_int32_t tStamp,
+	int type = IAXFormat::Audio, bool full = false, bool mark = false);
 
     /**
      * Send media data to remote peer. Update the outgoing media format if changed
      * @param data Data to send
      * @param format Data format
-     * @return 'this' if successful or 0
+     * @param type Media type
+     * @param mark Mark flag
+     * @return The number of bytes sent
      */
-    IAXTransaction* sendMedia(const DataBlock& data, u_int32_t format);
+    unsigned int sendMedia(const DataBlock& data, u_int32_t format,
+	int type = IAXFormat::Audio, bool mark = false);
 
     /**
      * Get an IAX event from the queue
@@ -1369,11 +1691,12 @@ public:
     /**
      * Send an ACCEPT/REGACK frame to remote peer
      * This method is thread safe
+     * @param expires Optional pointer to expiring time for register transactions
      * @return False if the transaction type is not New and state is NewRemoteInvite or NewRemoteInvite_AuthRep or
      *  if the transaction type is not RegReq and state is NewRemoteInvite or
      *  type is not RegReq/RegRel and state is NewRemoteInvite_AuthRep
      */
-    bool sendAccept();
+    bool sendAccept(unsigned int* expires = 0);
 
     /**
      * Send a HANGUP frame to remote peer
@@ -1449,9 +1772,19 @@ public:
     bool enableTrunking(IAXMetaTrunkFrame* trunkFrame);
 
     /**
-     * Print transaction data on stdin
+     * Process a received call token
+     * This method is thread safe
+     * @param callToken Received call token
      */
-    void print();
+    void processCallToken(const DataBlock& callToken);
+
+    /**
+     * Print transaction data on stdin
+     * @param printStats True to print media statistics
+     * @param printFrames True to print in/out pending frames
+     * @param location Additional location info to be shown in debug
+     */
+    void print(bool printStats = false, bool printFrames = false, const char* location = "status");
 
     /**
      * Standard message sent if unsupported/unknown/none authentication methosd was received
@@ -1498,6 +1831,11 @@ protected:
     IAXTransaction(IAXEngine* engine, Type type, u_int16_t lcallno, const SocketAddr& addr, IAXIEList& ieList,
 	void* data = 0);
 
+    /**
+     * Cleanup
+     */
+    virtual void destroyed();
+
     /**
      * Init data members from an IE list
      * @param ieList IE list to init from
@@ -1534,7 +1872,7 @@ protected:
      * @param createIEList If true create IE list in the generated event
      * @return Pointer to a valid IAXEvent
      */
-    IAXEvent* terminate(u_int8_t evType, bool local, const IAXFullFrame* frame = 0, bool createIEList = true);
+    IAXEvent* terminate(u_int8_t evType, bool local, IAXFullFrame* frame = 0, bool createIEList = true);
 
     /**
      * Wait for ACK to terminate the transaction. No more events will be generated
@@ -1543,7 +1881,7 @@ protected:
      * @param frame Frame to build event from
      * @return Pointer to a valid IAXEvent
      */
-    IAXEvent* waitForTerminate(u_int8_t evType, bool local, const IAXFullFrame* frame);
+    IAXEvent* waitForTerminate(u_int8_t evType, bool local, IAXFullFrame* frame);
 
     /**
      * Constructs an IAXFrameOut frame, send it to remote peer and put it in the transmission list
@@ -1554,8 +1892,21 @@ protected:
      * @param len Frame IE list length
      * @param tStamp Frame timestamp. If 0 the transaction timestamp will be used
      * @param ackOnly Frame's acknoledge only flag
+     * @param mark Frame mark flag
      */
     void postFrame(IAXFrame::Type type, u_int32_t subclass, void* data = 0, u_int16_t len = 0, u_int32_t tStamp = 0,
+	bool ackOnly = false, bool mark = false);
+
+    /**
+     * Constructs an IAXFrameOut frame, send it to remote peer and put it in the transmission list
+     * This method is thread safe
+     * @param type Frame type
+     * @param subclass Frame subclass
+     * @param ies Frame IE list
+     * @param tStamp Frame timestamp. If 0 the transaction timestamp will be used
+     * @param ackOnly Frame's acknoledge only flag
+     */
+    void postFrameIes(IAXFrame::Type type, u_int32_t subclass, IAXIEList* ies, u_int32_t tStamp = 0,
 		bool ackOnly = false);
 
     /**
@@ -1574,7 +1925,7 @@ protected:
      * @param newState The transaction new state
      * @return Pointer to an IAXEvent or 0 (invalid IE list)
      */
-    IAXEvent* createEvent(u_int8_t evType, bool local, const IAXFullFrame* frame, State newState);
+    IAXEvent* createEvent(u_int8_t evType, bool local, IAXFullFrame* frame, State newState);
 
     /**
      * Create an event from a received frame that is a response to a sent frame and
@@ -1748,7 +2099,7 @@ protected:
      * @param delFrame Delete frame flag. If true on exit, a request was found
      * @return A valid IAXEvent or 0
      */
-    IAXEvent* processMidCallControl(const IAXFullFrame* frame, bool& delFrame);
+    IAXEvent* processMidCallControl(IAXFullFrame* frame, bool& delFrame);
 
     /**
      * Process mid call IAX control frames
@@ -1756,7 +2107,7 @@ protected:
      * @param delFrame Delete frame flag. If true on exit, a request was found
      * @return A valid IAXEvent or 0
      */
-    IAXEvent* processMidCallIAXControl(const IAXFullFrame* frame, bool& delFrame);
+    IAXEvent* processMidCallIAXControl(IAXFullFrame* frame, bool& delFrame);
 
     /**
      * Test if frame is a Reject/RegRej frame
@@ -1764,7 +2115,7 @@ protected:
      * @param delFrame Delete frame flag. If true on exit, a request was found
      * @return A valid IAXEvent or 0.
      */
-    IAXEvent* remoteRejectCall(const IAXFullFrame* frame, bool& delFrame);
+    IAXEvent* remoteRejectCall(IAXFullFrame* frame, bool& delFrame);
 
     /**
      * Terminate the transaction if state is Terminating on a remote request
@@ -1774,11 +2125,12 @@ protected:
     IAXEvent* getEventTerminating(u_int64_t time);
 
     /**
-     * Process received Voice frames
-     * @param frame Received voice frame
+     * Process received media full frames
+     * @param frame Received frame
+     * @param type Media type
      * @return 0
      */
-    IAXTransaction* processVoiceFrame(const IAXFullFrame* frame);
+    IAXTransaction* processMediaFrame(const IAXFullFrame* frame, int type);
 
     /**
      * Send all frames from outgoing queue with outbound sequence number starting with seqNo.
@@ -1818,6 +2170,9 @@ protected:
     }
 
 private:
+    void adjustTStamp(u_int32_t& tStamp);
+    void postFrame(IAXFrameOut* frame);
+
     // Params
     bool m_localInitTrans;			// True: local initiated transaction
     bool m_localReqEnd;				// Local client requested terminate
@@ -1833,10 +2188,9 @@ private:
     IAXEngine* m_engine;			// Engine that owns this transaction
     void* m_userdata;				// Arbitrary user data
     u_int32_t m_lastFullFrameOut;		// Last transmitted full frame timestamp
-    u_int16_t m_lastMiniFrameOut;		// Last transmitted mini frame timestamp
-    u_int32_t m_lastMiniFrameIn;		// Last received mini frame timestamp
+    IAXMediaData m_dataAudio;
+    IAXMediaData m_dataVideo;
     u_int16_t m_lastAck;			// Last ack'd received frame's oseqno
-    Mutex m_mutexInMedia;			// Keep received media thread safe
     IAXEvent* m_pendingEvent;			// Pointer to a pending event or 0
     IAXEvent* m_currentEvent;			// Pointer to last generated event or 0
     // Outgoing frames management
@@ -1863,10 +2217,10 @@ private:
     String m_challenge;				// Challenge
     String m_authdata;				// Auth data received with auth reply
     u_int32_t m_expire;				// Registration expiring time
-    u_int32_t m_format;				// Media format used for initial negotiation
-    u_int32_t m_formatIn;			// Incoming media format
-    u_int32_t m_formatOut;			// Outgoing media format
+    IAXFormat m_format;				// Audio format
+    IAXFormat m_formatVideo;			// Video format
     u_int32_t m_capability;			// Media capability of this transaction
+    bool m_callToken;                           // Call token supported/expected
     // Meta trunking
     IAXMetaTrunkFrame* m_trunkFrame;		// Reference to a trunk frame if trunking is enabled for this transaction
 };
@@ -1979,7 +2333,7 @@ public:
      * @return IE list reference
      */
     inline IAXIEList& getList()
-	{ return m_ieList; }
+	{ return *m_ieList; }
 
 protected:
     /**
@@ -2001,7 +2355,7 @@ protected:
      * @param transaction IAX transaction that generated the event
      * @param frame The frame that generated the event
      */
-    IAXEvent(Type type, bool local, bool final, IAXTransaction* transaction, const IAXFullFrame* frame = 0);
+    IAXEvent(Type type, bool local, bool final, IAXTransaction* transaction, IAXFullFrame* frame = 0);
 
 private:
     inline IAXEvent() {}		// Default constructor
@@ -2012,7 +2366,7 @@ private:
     bool m_local;			// If true the event is generated locally, the receiver MUST not respond
     bool m_final;			// Final event flag
     IAXTransaction* m_transaction;	// Transaction that generated this event
-    IAXIEList m_ieList;			// IAXInfoElement list
+    IAXIEList* m_ieList;		// IAXInfoElement list
 };
 
 /**
@@ -2036,10 +2390,12 @@ public:
      * @param capab Media capabilities of this engine
      * @param trunkSendInterval Send trunk meta frame interval
      * @param authRequired Automatically challenge all clients for authentication
+     * @param params Optional extra parameter list
      */
     IAXEngine(const char* iface, int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
 	u_int16_t authTimeout, u_int16_t transTimeout, u_int16_t maxFullFrameDataLen,
-	u_int32_t format, u_int32_t capab, u_int32_t trunkSendInterval, bool authRequired);
+	u_int32_t format, u_int32_t capab, u_int32_t trunkSendInterval, bool authRequired,
+	NamedList* params = 0);
 
     /**
      * Destructor
@@ -2069,8 +2425,11 @@ public:
      * @param transaction IAXTransaction that owns the call leg
      * @param data Media data
      * @param tStamp Media timestamp
+     * @param type Media type
+     * @param mark Mark flag
      */
-    virtual void processMedia(IAXTransaction* transaction, DataBlock& data, u_int32_t tStamp)
+    virtual void processMedia(IAXTransaction* transaction, DataBlock& data, u_int32_t tStamp,
+	int type, bool mark)
 	{}
 
     /**
@@ -2124,10 +2483,11 @@ public:
 
     /**
      * Get the default media format
+     * @param audio True to retrieve default audio format, false for video format
      * @return The default media format
      */
-    inline u_int32_t format() const
-        { return m_format; }
+    inline u_int32_t format(bool audio = true) const
+        { return audio ? m_format : m_formatVideo; }
 
     /**
      * Get the media capability of this engine
@@ -2136,6 +2496,12 @@ public:
     inline u_int32_t capability() const
         { return m_capability; }
 
+    /**
+     * (Re)Initialize the engine
+     * @param params Parameter list
+     */
+    void initialize(const NamedList& params);
+
     /**
      * Read data from socket
      * @param addr Socket to read from
@@ -2147,10 +2513,21 @@ public:
      * @param buf Data to write
      * @param len Data length
      * @param addr Socket to write to
-     * @param frame Optional frame to be printed if debug is DebugAll
+     * @param frame Optional frame to be printed
+     * @param sent Pointer to variable to be filled with the number of bytes sent
      * @return True on success
      */
-    bool writeSocket(const void* buf, int len, const SocketAddr& addr, IAXFullFrame* frame = 0);
+    bool writeSocket(const void* buf, int len, const SocketAddr& addr, IAXFullFrame* frame = 0,
+	unsigned int* sent = 0);
+
+    /**
+     * Write a full frame to socket
+     * @param addr Socket to write to
+     * @param frame Frame to write
+     * @return True on success
+     */
+    inline bool writeSocket(const SocketAddr& addr, IAXFullFrame* frame)
+	{ return !frame || writeSocket(frame->data().data(),frame->data().length(),addr,frame); }
 
     /**
      * Read events
@@ -2178,20 +2555,34 @@ public:
     void keepAlive(SocketAddr& addr);
 
     /**
-     * Process a new format received with a Voice frame
+     * Process a new format received with a full frame
      * @param trans Transaction that received the new format
+     * @param type Media type
      * @param format The received format
      * @return True if accepted
      */
-    virtual bool voiceFormatChanged(IAXTransaction* trans, u_int32_t format)
+    virtual bool mediaFormatChanged(IAXTransaction* trans, int type, u_int32_t format)
 	{ return false; }
 
     /**
-     * Process the initial received format and capability. If accepted on exit will set the transaction format and capability
+     * Check call token on incoming call requests.
+     * This method is called by the engine when processing an incoming call request
+     * @param addr The address from where the call request was received
+     * @param frame Received frame
+     * @return True if accepted, false to ignore the call
+     */
+    virtual bool checkCallToken(const SocketAddr& addr, IAXFullFrame& frame);
+
+    /**
+     * Process the initial received format and capability.
+     * If accepted on exit will set the transaction format and capability
      * @param trans Transaction that received the new format
+     * @param caps Optional codecs to set in transaction before processing
+     * @param type Media type
      * @return True if accepted
      */
-    bool acceptFormatAndCapability(IAXTransaction* trans);
+    bool acceptFormatAndCapability(IAXTransaction* trans, unsigned int* caps = 0,
+	int type = IAXFormat::Audio);
 
     /**
      * Default event handler. event MUST NOT be deleted
@@ -2211,6 +2602,17 @@ public:
      */
     void removeTrunkFrame(IAXMetaTrunkFrame* metaFrame);
 
+    /**
+     * Send a trunk frame
+     * @param metaFrame The trunk meta frame to sent
+     */
+    inline void sendTrunkFrame(IAXMetaTrunkFrame* metaFrame) {
+	    if (!metaFrame)
+		return;
+	    Lock lck(m_mutexTrunk);
+	    metaFrame->send();
+	}
+
     /**
      * Keep calling processTrunkFrames to send trunked media data
      */
@@ -2223,6 +2625,18 @@ public:
     inline Socket& socket()
 	{ return m_socket; }
 
+    /**
+     * Send engine formats
+     * @param caps Capabilities
+     * @param fmtAudio Default audio format
+     * @param fmtVideo Default video format
+     */
+    inline void setFormats(u_int32_t caps, u_int32_t fmtAudio, u_int32_t fmtVideo) {
+	    m_format = fmtAudio;
+	    m_formatVideo = fmtVideo;
+	    m_capability = caps;
+	}
+
     /**
      * Get the MD5 data from a challenge and a password
      * @param md5data Destination String
@@ -2239,6 +2653,25 @@ public:
      */
     static bool isMD5ChallengeCorrect(const String& md5data, const String& challenge, const String& password);
 
+    /**
+     * Build a time signed secret used to authenticate an IP address
+     * @param buf Destination buffer
+     * @param secret Extra secret to add to MD5 sum
+     * @param addr Socket address
+     */
+    static void buildAddrSecret(String& buf, const String& secret,
+	const SocketAddr& addr);
+
+    /**
+     * Decode a secret built using buildAddrSecret()
+     * @param buf Input buffer
+     * @param secret Extra secret to check
+     * @param addr Socket address
+     * @return Secret age, negative if invalid
+     */
+    static int addrSecretAge(const String& buf, const String& secret,
+	const SocketAddr& addr);
+
 protected:
     /**
      * Process all trunk meta frames in the queue
@@ -2301,8 +2734,15 @@ private:
     u_int16_t m_authTimeout;			// Timeout (in seconds) of acknoledged auth frames sent
     u_int32_t m_transTimeout;			// Timeout (in seconds) on remote request of transactions
     						//  belonging to this engine
+    bool m_callToken;                           // Call token required on incoming calls
+    String m_callTokenSecret;                   // Secret used to generate call tokens
+    int m_callTokenAge;                         // Max allowed call token age
+    bool m_showCallTokenFailures;               // Print incoming call token failures to output
+    bool m_rejectMissingCallToken;              // Reject/ignore incoming calls without call token if mandatory
+    bool m_printMsg;                            // Print frame to output
     // Media
     u_int32_t m_format;				// The default media format
+    u_int32_t m_formatVideo;                    // Default video format
     u_int32_t m_capability;			// The media capability
     // Trunking
     Mutex m_mutexTrunk;				// Mutex for trunk operations
