@@ -69,14 +69,14 @@ MGCPPrivateThread::MGCPPrivateThread(MGCPEngine* engine, bool process,
     m_engine(engine),
     m_action(process?Process:Receive)
 {
-    XDebug(m_engine,DebugInfo,"MGCPPrivateThread::MGCPPrivateThread() [%p]",this);
+    DDebug(m_engine,DebugInfo,"MGCPPrivateThread::MGCPPrivateThread() [%p]",this);
     if (m_engine)
 	m_engine->appendThread(this);
 }
 
 MGCPPrivateThread::~MGCPPrivateThread()
 {
-    XDebug(m_engine,DebugInfo,"MGCPPrivateThread::~MGCPPrivateThread() [%p]",this);
+    DDebug(m_engine,DebugInfo,"MGCPPrivateThread::~MGCPPrivateThread() [%p]",this);
     if (m_engine)
 	m_engine->removeThread(this);
 }
@@ -102,6 +102,7 @@ void MGCPPrivateThread::run()
  */
 MGCPEngine::MGCPEngine(bool gateway, const char* name, const NamedList* params)
     : Mutex(true,"MGCPEngine"),
+    m_iterator(m_transactions),
     m_gateway(gateway),
     m_initialized(false),
     m_nextId(1),
@@ -139,26 +140,26 @@ MGCPEngine::~MGCPEngine()
 // Initialize this engine
 void MGCPEngine::initialize(const NamedList& params)
 {
-    int level = params.getIntValue("debuglevel");
+    int level = params.getIntValue(YSTRING("debuglevel"));
     if (level)
 	debugLevel(level);
 
-    m_allowUnkCmd = params.getBoolValue("allow_unknown_cmd",false);
-    int val = params.getIntValue("retrans_interval",TR_RETRANS_INTERVAL);
+    m_allowUnkCmd = params.getBoolValue(YSTRING("allow_unknown_cmd"),false);
+    int val = params.getIntValue(YSTRING("retrans_interval"),TR_RETRANS_INTERVAL);
     m_retransInterval = 1000 * (val < TR_RETRANS_INTERVAL_MIN ? TR_RETRANS_INTERVAL_MIN : val);
-    val = params.getIntValue("retrans_count",TR_RETRANS_COUNT);
+    val = params.getIntValue(YSTRING("retrans_count"),TR_RETRANS_COUNT);
     m_retransCount = (val < TR_RETRANS_COUNT_MIN ? TR_RETRANS_COUNT_MIN : val);
-    val = params.getIntValue("extra_time_to_live",TR_EXTRA_TIME);
+    val = params.getIntValue(YSTRING("extra_time_to_live"),TR_EXTRA_TIME);
     m_extraTime = 1000 * (val < TR_EXTRA_TIME_MIN ? TR_EXTRA_TIME_MIN : val);
 
     if (!m_initialized) {
-	val = params.getIntValue("max_recv_packet",RECV_BUF_LEN);
+	val = params.getIntValue(YSTRING("max_recv_packet"),RECV_BUF_LEN);
 	m_maxRecvPacket = val < RECV_BUF_LEN ? RECV_BUF_LEN : val;
     }
 
-    m_parseParamToLower = params.getBoolValue("lower_case_params",true);
-    m_provisional = params.getBoolValue("send_provisional",true);
-    m_ackRequest = params.getBoolValue("request_ack",true);
+    m_parseParamToLower = params.getBoolValue(YSTRING("lower_case_params"),true);
+    m_provisional = params.getBoolValue(YSTRING("send_provisional"),true);
+    m_ackRequest = params.getBoolValue(YSTRING("request_ack"),true);
 
     // Bind socket if not valid
     if (!m_socket.valid()) {
@@ -335,6 +336,11 @@ bool MGCPEngine::receive(unsigned char* buffer, SocketAddr& addr)
 {
     if (!m_socket.valid())
 	return false;
+    if (Socket::efficientSelect() && m_socket.canSelect()) {
+	bool canRead = false;
+	if (m_socket.select(&canRead,0,0,Thread::idleUsec()) && !canRead)
+	    return false;
+    }
     int len = maxRecvPacket();
     int rd = m_socket.recvFrom(buffer,len,addr);
     if (rd == Socket::socketError()) {
@@ -379,9 +385,9 @@ bool MGCPEngine::receive(unsigned char* buffer, SocketAddr& addr)
 	// Command messages may contain ACK'd incoming transaction's responses
 	// See RFC 3435: 3.2.2.19 and 3.5.1
 	if (msg->isCommand()) {
-	    String s = msg->params.getValue("k");
+	    String s = msg->params.getValue(YSTRING("k"));
 	    if (!(s || m_parseParamToLower))
-		s = msg->params.getValue("K");
+		s = msg->params.getValue(YSTRING("K"));
 	    if (s) {
 		unsigned int len = 0;
 		unsigned int* trList = decodeAck(s,len);
@@ -425,7 +431,7 @@ bool MGCPEngine::receive(unsigned char* buffer, SocketAddr& addr)
 	    new MGCPTransaction(this,msg,false,addr);
 	    continue;
 	}
-	DDebug(this,DebugNote,"Received response %d for unknown transaction %u",
+	Debug(this,DebugNote,"Received response %d for unknown transaction %u",
 	    msg->code(),msg->transactionId());
 	TelEngine::destruct(msg);
     }
@@ -476,14 +482,15 @@ void MGCPEngine::runProcess()
 MGCPEvent* MGCPEngine::getEvent(u_int64_t time)
 {
     lock();
-    ListIterator iter(m_transactions);
     while (true) {
 	if (Thread::check(false))
 	    break;
-	MGCPTransaction* tr = static_cast<MGCPTransaction*>(iter.get());
+	MGCPTransaction* tr = static_cast<MGCPTransaction*>(m_iterator.get());
         // End of iteration? NO: get a reference to the transaction
-	if (!tr)
+	if (!tr) {
+	    m_iterator.assign(m_transactions);
 	    break;
+	}
 	RefPointer<MGCPTransaction> sref = tr;
 	if (!sref)
 	    continue;
@@ -507,8 +514,7 @@ bool MGCPEngine::processEvent(MGCPEvent* event)
     if (!event)
 	return false;
     MGCPTransaction* trans = event->transaction();
-    void* data = trans ? trans->userData() : 0;
-    if (processEvent(trans,event->message(),data)) {
+    if (processEvent(trans,event->message())) {
 	// Get rid of the event, it was handled
 	delete event;
 	return true;
@@ -518,9 +524,9 @@ bool MGCPEngine::processEvent(MGCPEvent* event)
 
 // Process an event generated by a transaction. Descendants must override this
 //  method if they want to process events
-bool MGCPEngine::processEvent(MGCPTransaction* trans, MGCPMessage* msg, void* data)
+bool MGCPEngine::processEvent(MGCPTransaction* trans, MGCPMessage* msg)
 {
-    Debug(this,DebugStub,"MGCPEngine::processEvent(%p,%p,%p)",trans,msg,data);
+    Debug(this,DebugStub,"MGCPEngine::processEvent(%p,%p)",trans,msg);
     return false;
 }
 
@@ -562,16 +568,16 @@ void MGCPEngine::cleanup(bool gracefully, const char* text)
 	return;
 
     // Terminate private threads
-    XDebug(this,DebugAll,"Terminating %u private threads",m_threads.count());
+    Debug(this,DebugAll,"Terminating %u private threads",m_threads.count());
     lock();
     ListIterator iter(m_threads);
     for (GenObject* o = 0; 0 != (o = iter.get());)
 	static_cast<MGCPPrivateThread*>(o)->cancel(!gracefully);
     unlock();
-    XDebug(this,DebugAll,"Waiting for private threads to terminate");
+    DDebug(this,DebugAll,"Waiting for private threads to terminate");
     while (m_threads.skipNull())
 	Thread::yield();
-    XDebug(this,DebugAll,"Private threads terminated");
+    DDebug(this,DebugAll,"Private threads terminated");
 }
 
 // Write data to socket
