@@ -506,18 +506,20 @@ public:
      * @param value Original Endpoint ID to copy
      */
     inline MGCPEndpointId(const MGCPEndpointId& value)
-	: m_port(0)
-	{ set(value.user(),value.host(),value.port()); }
+	: m_id(value.id()), m_endpoint(value.user()),
+	  m_host(value.host()), m_port(value.port())
+	{ }
 
     /**
      * Constructor. Construct this endpoint id
      * @param endpoint The user part of the endpoint's URI
      * @param host The IP address of the endpoint's URI
      * @param port The port used by the endpoint to receive data
+     * @param addPort Add :port at end of id only if port is not zero
      */
-    inline MGCPEndpointId(const char* endpoint, const char* host, int port)
+    inline MGCPEndpointId(const char* endpoint, const char* host, int port, bool addPort = true)
 	: m_port(0)
-	{ set(endpoint,host,port); }
+	{ set(endpoint,host,port,addPort); }
 
     /**
      * Get the full id of the endpoint
@@ -550,17 +552,19 @@ public:
     /**
      * Set the port used by this endpoint
      * @param newPort The new port used by this endpoint
+     * @param addPort Add :port at end of id only if port is not zero
      */
-    inline void port(int newPort)
-	{ set(m_endpoint,m_host,newPort); }
+    inline void port(int newPort, bool addPort = true)
+	{ set(m_endpoint,m_host,newPort,addPort); }
 
     /**
      * Set this endpoint id. Convert it to lower case
      * @param endpoint The user part of the endpoint's URI
      * @param host The IP address of the endpoint's URI
      * @param port The port used by the endpoint to receive data
+     * @param addPort Add :port at end of id only if port is not zero
      */
-    void set(const char* endpoint, const char* host, int port);
+    void set(const char* endpoint, const char* host, int port, bool addPort = true);
 
     /**
      * Set this endpoint id. Convert it to lower case
@@ -601,11 +605,12 @@ public:
      * @param endpoint The endpoint part of the endpoint's id
      * @param host The IP address of this endpoint
      * @param port The port used to send data to this endpoint
+     * @param addPort Add :port at end of id only if port is not zero
      */
-    inline MGCPEpInfo(const char* endpoint, const char* host, int port)
-	: MGCPEndpointId(endpoint,host,port), address(AF_INET) {
-	    address.host(host);
-	    address.port(port);
+    inline MGCPEpInfo(const char* endpoint, const char* host, int port, bool addPort = true)
+	: MGCPEndpointId(endpoint,host,port,addPort),
+	  m_address(AF_INET), m_resolve(true) {
+	    m_address.port(port);
 	}
 
     /**
@@ -616,14 +621,33 @@ public:
 	{ return id(); }
 
     /**
-     * The IP address and port of this endpoint
+     * Retrieve the current address for this endpoint information
+     * @return Address and port of this endpoint info
      */
-    SocketAddr address;
+    inline const SocketAddr& address() const
+	{ return m_address; }
+
+    /**
+     * Retrieve the address for this endpoint information, resolve name if needed
+     * @return Address and port of this endpoint info
+     */
+    const SocketAddr& address();
+
+    /**
+     * Set a new socket address in the endpoint info
+     * @param addr New address and port to set in the endpoint
+     */
+    inline void address(const SocketAddr& addr)
+	{ m_resolve = false; m_address = addr; }
 
     /**
      * An alias name of the remote endpoint, may be empty
      */
     String alias;
+
+private:
+    SocketAddr m_address;
+    bool m_resolve;
 };
 
 /**
@@ -642,8 +666,9 @@ public:
      * @param user The user part of the endpoint's id
      * @param host The host part of the endpoint's id
      * @param port The port part of the endpoint's id
+     * @param addPort Add :port at end of id only if port is not zero
      */
-    MGCPEndpoint(MGCPEngine* engine, const char* user, const char* host, int port);
+    MGCPEndpoint(MGCPEngine* engine, const char* user, const char* host, int port, bool addPort = true);
 
     /**
      * Destructor. Remove itself from engine's list
@@ -672,6 +697,8 @@ public:
      * @param port The port used to send data to this endpoint.
      *  Set to 0 to set it to the default port defined by the protocol and the
      *  opposite of the engine's mode
+     *  A value of -1 uses the default but doesn't add :port at end of ID
+     *  Other negative values use specified port but don't add :port at end
      * @return Valid MGCPEpInfo pointer or 0 if the data wasn't added
      */
     MGCPEpInfo* append(const char* endpoint, const char* host, int port = 0);
@@ -868,7 +895,7 @@ public:
      * @param cmd The command name to check
      * @return True if the given command is known by this engine
      */
-    inline bool knownCommand(const char* cmd)
+    inline bool knownCommand(const String& cmd)
 	{ Lock lock(this); return (m_knownCommands.find(cmd) != 0); }
 
     /**
@@ -980,11 +1007,10 @@ public:
      *  method if they want to process events
      * @param trans Pointer to the transaction that generated the event
      * @param msg MGCP message of the event, may be NULL
-     * @param data User data that is stored in transaction, may be NULL
      * @return True if the event was processed. If the event carry a received
      *  command and it's not processed the transaction will receive an 'unknown command' response
      */
-    virtual bool processEvent(MGCPTransaction* trans, MGCPMessage* msg, void* data);
+    virtual bool processEvent(MGCPTransaction* trans, MGCPMessage* msg);
 
     /**
      * Returns an unprocessed event to this engine to be deleted.
@@ -1014,6 +1040,13 @@ public:
     static inline int defaultPort(bool gateway)
 	{ return gateway ? 2427 : 2727; }
 
+    /**
+     * Handle a transaction that has timed out
+     * @param tr The transaction that has timed out
+     */
+    virtual void timeout(MGCPTransaction* tr)
+	{ }
+
     /**
      * The list of commands defined in RFC 3435
      */
@@ -1065,6 +1098,11 @@ protected:
      */
     ObjList m_transactions;
 
+    /**
+     * The transaction list iterator used to get events
+     */
+    ListIterator m_iterator;
+
 private:
     // Append a private thread to the list
     void appendThread(MGCPPrivateThread* thread);
