@@ -73,7 +73,7 @@ unsigned int RTPBaseIO::ssrcInit()
     if (m_ssrcInit) {
 	m_ssrcInit = false;
 	do {
-	    m_ssrc = ::random();
+	    m_ssrc = Random::random();
 	} while (0 == m_ssrc);
     }
     return m_ssrc;
@@ -164,6 +164,8 @@ void RTPReceiver::rtpData(const void* data, int len)
     }
     if (len < 0)
 	return;
+    if (!len)
+	pc = 0;
 
     // grab some data at the first packet received or resync
     if (m_ssrcInit) {
@@ -173,6 +175,8 @@ void RTPReceiver::rtpData(const void* data, int len)
 	m_seq = seq-1;
 	m_seqCount = 0;
 	m_warn = true;
+	if (m_dejitter)
+	    m_dejitter->clear();
     }
 
     if (ss != m_ssrc) {
@@ -184,23 +188,42 @@ void RTPReceiver::rtpData(const void* data, int len)
 		Debug(DebugWarn,"RTP Received SSRC %08X but expecting %08X [%p]",
 		    ss,m_ssrc,this);
 	    }
+	    m_wrongSSRC++;
 	    return;
 	}
 	// SSRC accepted, sync sequence and resync the timestamp offset
 	m_seq = seq;
 	m_ts = ts - m_tsLast;
 	m_seqCount = 0;
+	if (m_dejitter)
+	    m_dejitter->clear();
 	// drop this packet, next packet will come in correctly
 	return;
     }
 
-    // substraction with overflow
+    u_int32_t rollover = m_rollover;
+    // compare unsigned to detect rollovers
+    if (seq < m_seq)
+	rollover++;
+    u_int64_t seq48 = rollover;
+    seq48 = (seq48 << 16) | seq;
+
+    // if some security data is present authenticate the packet now
+    if (secPtr && !rtpCheckIntegrity((const unsigned char*)data,len + padding + 12,secPtr + m_mkiLen,ss,seq48))
+	return;
+
+    // substraction with overflow to compute sequence difference
     int16_t ds = seq - m_seq;
-    // check if we received duplicate or delayed packet
+    if (ds != 1)
+	m_seqLost++;
+    if (ds == 0)
+	return;
+
+    // check if we received a packet too much out of sequence
     // be much more tolerant when authenticating as we cannot resync
-    if ((ds <= 0) || ((ds > SEQ_DESYNC_COUNT) && !secPtr)) {
+    if ((ds <= -SEQ_DESYNC_COUNT) || ((ds > SEQ_DESYNC_COUNT) && !secPtr)) {
 	m_ioLostPkt++;
-	if (ds && !secPtr) {
+	if (!secPtr) {
 	    // try to resync sequence unless we need to authenticate
 	    if (m_seqCount++) {
 		if (seq == ++m_seqSync) {
@@ -212,6 +235,9 @@ void RTPReceiver::rtpData(const void* data, int len)
 			m_ts = ts - m_tsLast;
 			m_seqCount = 0;
 			m_warn = true;
+			m_syncLost++;
+			if (m_dejitter)
+			    m_dejitter->clear();
 			// drop this packet, next packet will come in correctly
 			return;
 		    }
@@ -222,38 +248,32 @@ void RTPReceiver::rtpData(const void* data, int len)
 	    else
 		m_seqSync = seq;
 	}
-	if (m_warn && ds) {
+	if (m_warn) {
 	    m_warn = false;
 	    Debug(DebugWarn,"RTP received SEQ %u while current is %u [%p]",seq,m_seq,this);
 	}
 	return;
     }
 
-    if (ds > 1)
-	m_ioLostPkt += (ds - 1);
-
-    u_int32_t rollover = m_rollover;
-    // this time compare unsigned to detect rollovers
-    if (seq < m_seq)
-	rollover++;
-    u_int64_t seq48 = rollover;
-    seq48 = (seq48 << 16) | seq;
-
-    // if some security data is present authenticate the packet now
-    if (secPtr && !rtpCheckIntegrity((const unsigned char*)data,len + padding + 12,secPtr + m_mkiLen,ss,seq48))
+    if (!rtpDecipher(const_cast<unsigned char*>(pc),len + padding,secPtr,ss,seq48))
 	return;
 
-    // keep track of the last valid sequence number and timestamp we have seen
-    m_seq = seq;
-    m_rollover = rollover;
     m_tsLast = ts - m_ts;
     m_seqCount = 0;
     m_ioPackets++;
     m_ioOctets += len;
+    // keep track of the last valid sequence number and timestamp we have seen
+    m_seq = seq;
+    m_rollover = rollover;
 
-    if (!len)
-	pc = 0;
-    if (rtpDecipher(const_cast<unsigned char*>(pc),len + padding,secPtr,ss,seq48))
+    if (m_dejitter) {
+	if (!m_dejitter->rtpRecv(marker,typ,m_tsLast,pc,len))
+	    m_ioLostPkt++;
+	return;
+    }
+    if (ds > 1)
+	m_ioLostPkt += (ds - 1);
+    if (ds >= 1)
 	rtpRecv(marker,typ,m_tsLast,pc,len);
 }
 
@@ -270,15 +290,8 @@ bool RTPReceiver::rtpRecv(bool marker, int payload, unsigned int timestamp, cons
     if (payload == silencePayload())
 	return decodeSilence(marker,timestamp,data,len);
     finishEvent(timestamp);
-    if (payload == dataPayload()) {
-#if 0
-// dejitter is broken - don't use it
-	if (m_dejitter)
-	    return m_dejitter->rtpRecvData(marker,timestamp,data,len);
-	else
-#endif
-	    return rtpRecvData(marker,timestamp,data,len);
-    }
+    if (payload == dataPayload())
+	return rtpRecvData(marker,timestamp,data,len);
     return false;
 }
 
@@ -377,14 +390,24 @@ bool RTPReceiver::rtpCheckIntegrity(const unsigned char* data, int len, const vo
 	: true;
 }
 
+void RTPReceiver::stats(NamedList& stat) const
+{
+    if (m_session)
+	stat.setParam("remoteip",m_session->UDPSession::transport()->remoteAddr().host());
+    stat.setParam("lostpkts",String(m_ioLostPkt));
+    stat.setParam("synclost",String(m_syncLost));
+    stat.setParam("wrongssrc",String(m_wrongSSRC));
+    stat.setParam("seqslost",String(m_seqLost));
+}
+
 
 RTPSender::RTPSender(RTPSession* session, bool randomTs)
-    : RTPBaseIO(session), m_evTime(0), m_tsLast(0), m_padding(0)
+    : RTPBaseIO(session), m_evTime(0), m_padding(0)
 {
     if (randomTs) {
-	m_ts = ::random() & ~1;
+	m_ts = Random::random() & ~1;
 	// avoid starting sequence numbers too close to zero
-	m_seq = 2500 + (::random() % 60000);
+	m_seq = 2500 + (Random::random() % 60000);
     }
 }
 
@@ -417,8 +440,8 @@ bool RTPSender::rtpSend(bool marker, int payload, unsigned int timestamp, const
 	}
     }
 
-    DataBlock buf(0,len+padding+m_secLen+12);
-    unsigned char* pc = (unsigned char*)buf.data();
+    m_buffer.resize(len + padding + m_secLen + 12);
+    unsigned char* pc = (unsigned char*)m_buffer.data();
     if (padding)
 	pc[len + padding + 11] = padding;
     *pc++ = byte1;
@@ -438,8 +461,8 @@ bool RTPSender::rtpSend(bool marker, int payload, unsigned int timestamp, const
 	rtpEncipher(pc,len + padding);
     }
     if (m_secLen)
-	rtpAddIntegrity((const unsigned char*)buf.data(),len + padding + 12,pc + (len + padding + m_mkiLen));
-    static_cast<RTPProcessor*>(m_session->UDPSession::transport())->rtpData(buf.data(),buf.length());
+	rtpAddIntegrity((const unsigned char*)m_buffer.data(),len + padding + 12,pc + (len + padding + m_mkiLen));
+    static_cast<RTPProcessor*>(m_session->UDPSession::transport())->rtpData(m_buffer.data(),m_buffer.length());
     return true;
 }
 
@@ -551,6 +574,10 @@ void RTPSender::rtpAddIntegrity(const unsigned char* data, int len, unsigned cha
 	m_secure->rtpAddIntegrity(data,len,authData);
 }
 
+void RTPSender::stats(NamedList& stat) const
+{
+}
+
 
 UDPSession::UDPSession()
     : m_transport(0), m_timeoutTime(0), m_timeoutInterval(0)
@@ -634,8 +661,10 @@ void UDPSession::setTimeout(int interval)
 
 
 RTPSession::RTPSession()
-    : m_direction(FullStop),
-      m_send(0), m_recv(0), m_secure(0)
+    : Mutex(true,"RTPSession"),
+      m_direction(FullStop),
+      m_send(0), m_recv(0), m_secure(0),
+      m_reportTime(0), m_reportInterval(0)
 {
     DDebug(DebugInfo,"RTPSession::RTPSession() [%p]",this);
 }
@@ -668,6 +697,12 @@ void RTPSession::timerTick(const Time& when)
 	else
 	    m_timeoutTime = when + m_timeoutInterval;
     }
+    if (m_reportInterval) {
+	if (when >= m_reportTime) {
+	    m_reportTime = when + m_reportInterval;
+	    sendRtcpReport(when);
+	}
+    }
 }
 
 void RTPSession::rtpData(const void* data, int len)
@@ -685,7 +720,8 @@ void RTPSession::rtcpData(const void* data, int len)
     if ((m_direction & RecvOnly) == 0)
 	return;
     if (m_recv) {
-	m_timeoutTime = 0;
+	if ((m_timeoutTime != INF_TIMEOUT) || m_recv->ssrc())
+	    m_timeoutTime = 0;
 	m_recv->rtcpData(data,len);
     }
 }
@@ -738,6 +774,8 @@ bool RTPSession::checkCipher(const String& name)
 
 void RTPSession::transport(RTPTransport* trans)
 {
+    if (!trans)
+	sendRtcpBye();
     UDPSession::transport(trans);
     if (!m_transport)
 	m_direction = FullStop;
@@ -748,6 +786,7 @@ void RTPSession::sender(RTPSender* send)
     DDebug(DebugInfo,"RTPSession::sender(%p) old=%p [%p]",send,m_send,this);
     if (send == m_send)
 	return;
+    sendRtcpBye();
     RTPSender* tmp = m_send;
     m_send = send;
     if (tmp)
@@ -848,9 +887,112 @@ void RTPSession::getStats(String& stats) const
     }
 }
 
+void RTPSession::setReports(int interval)
+{
+    if (interval > 0 && m_transport && m_transport->rtcpSock()->valid()) {
+	if (interval < 500)
+	    interval = 500;
+	else if (interval > 60000)
+	    interval = 60000;
+	m_reportInterval = interval * (u_int64_t)1000 + (Random::random() % 20000);
+    }
+    else
+	m_reportInterval = 0;
+    m_reportTime = 0;
+}
+
+void RTPSession::getStats(NamedList& stats) const
+{
+    if (m_send)
+	m_send->stats(stats);
+    if (m_recv)
+	m_recv->stats(stats);
+    stats.setParam("wrongsrc",String(m_wrongSrc));
+}
+
+static void store32(unsigned char* buf, unsigned int& len, u_int32_t val)
+{
+    buf[len++] = (unsigned char)(val >> 24);
+    buf[len++] = (unsigned char)(val >> 16);
+    buf[len++] = (unsigned char)(val >> 8);
+    buf[len++] = (unsigned char)(val & 0xff);
+}
+
+void RTPSession::sendRtcpReport(const Time& when)
+{
+    if (!((m_send || m_recv) && m_transport && m_transport->rtcpSock()->valid()))
+	return;
+    unsigned char buf[52];
+    buf[0] = 0x80; // RC=0
+    buf[1] = 0xc9; // RR
+    buf[2] = 0;
+    unsigned int len = 8;
+    if (m_send && m_send->ioPackets()) {
+	// Include a sender report
+	buf[1] = 0xc8; // SR
+	// NTP timestamp
+	store32(buf,len,2208988800 + (when.usec() / 1000000));
+	store32(buf,len,((when.usec() % 1000000) << 32) / 1000000);
+	// RTP timestamp
+	store32(buf,len,m_send->tsLast());
+	// Packet and octet counters
+	store32(buf,len,m_send->ioPackets());
+	store32(buf,len,m_send->ioOctets());
+    }
+    if (m_recv && m_recv->ioPackets()) {
+	// Add a single receiver report
+	buf[0] |= 0x01; // RC=1
+	store32(buf,len,m_recv->ssrc());
+	u_int32_t lost = m_recv->ioPacketsLost();
+	u_int32_t lostf = 0xff & (lost * 255 / (lost + m_recv->ioPackets()));
+	store32(buf,len,(lost & 0xffffff) | (lostf << 24));
+	store32(buf,len,m_recv->fullSeq());
+	// TODO: Compute and store Jitter, LSR and DLSR
+	store32(buf,len,0);
+	store32(buf,len,0);
+	store32(buf,len,0);
+    }
+    // Don't send a RR with no receiver report blocks...
+    if (len <= 8)
+	return;
+    DDebug(DebugInfo,"RTPSession sending RTCP Report [%p]",this);
+    unsigned int lptr = 4;
+    store32(buf,lptr,(m_send ? m_send->ssrcInit() : 0));
+    buf[3] = (len - 1) / 4; // same as ((len + 3) / 4) - 1
+    static_cast<RTPProcessor*>(m_transport)->rtcpData(buf,len);
+}
+
+void RTPSession::sendRtcpBye()
+{
+    if (!(m_send && m_transport && m_transport->rtcpSock()->valid()))
+	return;
+    u_int32_t ssrc = m_send->ssrc();
+    if (!ssrc)
+	return;
+    DDebug(DebugInfo,"RTPSession sending RTCP Bye [%p]",this);
+    // SSRC was initialized if we sent at least one RTP or RTCP packet
+    unsigned char buf[8];
+    buf[0] = 0x81;
+    buf[1] = 0xcb;
+    buf[2] = 0;
+    buf[3] = 1; // len = 2 x 32bit
+    buf[4] = (unsigned char)(ssrc >> 24);
+    buf[5] = (unsigned char)(ssrc >> 16);
+    buf[6] = (unsigned char)(ssrc >> 8);
+    buf[7] = (unsigned char)(0xff & ssrc);
+    static_cast<RTPProcessor*>(m_transport)->rtcpData(buf,8);
+}
+
+void RTPSession::incWrongSrc() 
+{
+    XDebug(DebugAll,"RTPSession::incWrongSrc() [%p]",this);
+    m_wrongSrc++;
+}
+
 
 UDPTLSession::UDPTLSession(u_int16_t maxLen, u_int8_t maxSec)
-    : m_rxSeq(0xffff), m_txSeq(0xffff),
+    : Mutex(true,"UDPTLSession"),
+      m_rxSeq(0xffff), m_txSeq(0xffff),
       m_maxLen(maxLen), m_maxSec(maxSec),
       m_warn(true)
 {
@@ -967,6 +1109,7 @@ bool UDPTLSession::udptlSend(const void* data, int len, u_int16_t seq)
 {
     if (!(UDPSession::transport() && data && len))
 	return false;
+    Lock lck(this);
     int pl = len + 5;
     if ((len > 255) || (pl > m_maxLen)) {
 	Debug(DebugWarn,"UDPTL could not send IFP with len=%d [%p]",len,this);
