@@ -105,6 +105,19 @@ public:
      */
     virtual void getStats(String& stats) const;
 
+    /**
+     * Increase the counter for number of RTP packets received from a wrong source
+     */
+    virtual inline void incWrongSrc()
+	{  }
+
+    /**
+     * Get the number of RTP packets that were received from a wrong source
+     * @return Number of RTP packets received from a wrong source
+     */
+    inline unsigned int wrongSrc()
+	{ return m_wrongSrc; }
+
 protected:
     /**
      * Set a new RTP group for this processor
@@ -118,6 +131,8 @@ protected:
      */
     virtual void timerTick(const Time& when) = 0;
 
+    unsigned int m_wrongSrc;
+
 private:
     RTPGroup* m_group;
 };
@@ -271,6 +286,13 @@ public:
     inline Socket* rtpSock()
 	{ return &m_rtpSock; }
 
+    /**
+     * Get the RTCP socket used by this transport
+     * @return Pointer to the RTCP socket
+     */
+    inline Socket* rtcpSock()
+	{ return &m_rtcpSock; }
+
     /**
      * Drill a hole in a firewall or NAT for the RTP and RTCP sockets
      * @return True if at least a packet was sent for the RTP socket
@@ -307,6 +329,9 @@ private:
     SocketAddr m_localAddr;
     SocketAddr m_remoteAddr;
     SocketAddr m_remoteRTCP;
+    SocketAddr m_remotePref;
+    SocketAddr m_rxAddrRTP;
+    SocketAddr m_rxAddrRTCP;
     bool m_autoRemote;
 };
 
@@ -335,14 +360,20 @@ public:
     /**
      * Process and store one RTP data packet
      * @param marker True if the marker bit is set in data packet
+     * @param payload Payload number
      * @param timestamp Sampling instant of the packet data
      * @param data Pointer to data block to process
      * @param len Length of the data block in bytes
-     * @return True if data was handled
+     * @return True if the data packet was queued
      */
-    virtual bool rtpRecvData(bool marker, unsigned int timestamp,
+    virtual bool rtpRecv(bool marker, int payload, unsigned int timestamp,
 	const void* data, int len);
 
+    /**
+     * Clear the delayed packets queue and all variables
+     */
+    void clear();
+
 protected:
     /**
      * Method called periodically to keep the data flowing
@@ -353,12 +384,13 @@ protected:
 private:
     ObjList m_packets;
     RTPReceiver* m_receiver;
-    unsigned int m_mindelay;
-    unsigned int m_maxdelay;
+    unsigned int m_minDelay;
+    unsigned int m_maxDelay;
     unsigned int m_headStamp;
     unsigned int m_tailStamp;
     u_int64_t m_headTime;
-    u_int64_t m_tailTime;
+    u_int64_t m_sampRate;
+    unsigned char m_fastRate;
 };
 
 /**
@@ -378,7 +410,7 @@ public:
 	  m_ssrcInit(true), m_ssrc(0), m_ts(0),
 	  m_seq(0), m_rollover(0), m_secLen(0), m_mkiLen(0),
 	  m_evTs(0), m_evNum(-1), m_evVol(-1),
-	  m_ioPackets(), m_ioOctets(0),
+	  m_ioPackets(), m_ioOctets(0), m_tsLast(0),
 	  m_dataType(-1), m_eventType(-1), m_silenceType(-1)
 	{ }
 
@@ -490,6 +522,13 @@ public:
     inline u_int32_t ioOctets() const
 	{ return m_ioOctets; }
 
+    /**
+     * Get the timestamp of the last packet as transmitted over the wire
+     * @return Timestamp of last packet sent or received
+     */
+    inline unsigned int tsLast() const
+	{ return m_ts + m_tsLast; }
+
     /**
      * Get the session this object belongs to
      * @return Pointer to RTP session or NULL
@@ -539,6 +578,7 @@ protected:
     int m_evVol;
     u_int32_t m_ioPackets;
     u_int32_t m_ioOctets;
+    unsigned int m_tsLast;
 
 private:
     int m_dataType;
@@ -553,14 +593,16 @@ private:
 class YRTP_API RTPReceiver : public RTPBaseIO
 {
     friend class RTPSession;
+    friend class RTPDejitter;
 public:
     /**
      * Constructor
      */
     inline RTPReceiver(RTPSession* session = 0)
 	: RTPBaseIO(session),
-	  m_ioLostPkt(0), m_dejitter(0), m_tsLast(0),
-	  m_seqSync(0), m_seqCount(0), m_warn(true)
+	  m_ioLostPkt(0), m_dejitter(0),
+	  m_seqSync(0), m_seqCount(0), m_warn(true),
+	  m_seqLost(0), m_wrongSSRC(0), m_syncLost(0)
 	{ }
 
     /**
@@ -644,6 +686,12 @@ public:
     */
     virtual void rtpNewSSRC(u_int32_t newSsrc, bool marker);
 
+    /**
+     * Retrieve the statistical data from this receiver in a NamedList. Reset all the data.
+     * @param stat NamedList to populate with the values for different counters
+     */
+    virtual void stats(NamedList& stat) const;
+
 protected:
     /**
      * Method called periodically to finish lingering events
@@ -685,10 +733,12 @@ private:
     void finishEvent(unsigned int timestamp);
     bool pushEvent(int event, int duration, int volume, unsigned int timestamp);
     RTPDejitter* m_dejitter;
-    unsigned int m_tsLast;
     u_int16_t m_seqSync;
     u_int16_t m_seqCount;
     bool m_warn;
+    unsigned int m_seqLost;
+    unsigned int m_wrongSSRC;
+    unsigned int m_syncLost;
 };
 
 /**
@@ -769,6 +819,12 @@ public:
      */
     bool padding(int chunk);
 
+    /**
+     * Retrieve the statistical data from this receiver in a NamedList. Reset all the data.
+     * @param stat NamedList to populate with the values for different counters
+     */
+    virtual void stats(NamedList& stat) const;
+
 protected:
     /**
      * Method called periodically to send events and buffered data
@@ -796,8 +852,8 @@ protected:
 
 private:
     int m_evTime;
-    unsigned int m_tsLast;
     unsigned char m_padding;
+    DataBlock m_buffer;
     bool sendEventData(unsigned int timestamp);
 };
 
@@ -905,7 +961,7 @@ protected:
  * An unidirectional or bidirectional RTP session
  * @short Full RTP session
  */
-class YRTP_API RTPSession : public UDPSession
+class YRTP_API RTPSession : public UDPSession, public Mutex
 {
 public:
     /**
@@ -1029,7 +1085,7 @@ public:
      */
     inline bool rtpSend(bool marker, int payload, unsigned int timestamp,
 	const void* data, int len)
-	{ return m_send && m_send->rtpSend(marker,payload,timestamp,data,len); }
+	{ Lock lck(this); return m_send && m_send->rtpSend(marker,payload,timestamp,data,len); }
 
     /**
      * Send one RTP data packet
@@ -1041,7 +1097,7 @@ public:
      */
     inline bool rtpSendData(bool marker, unsigned int timestamp,
 	const void* data, int len)
-	{ return m_send && m_send->rtpSendData(marker,timestamp,data,len); }
+	{ Lock lck(this); return m_send && m_send->rtpSendData(marker,timestamp,data,len); }
 
     /**
      * Send one RTP event
@@ -1052,7 +1108,7 @@ public:
      * @return True if data sending was attempted
      */
     inline bool rtpSendEvent(int event, int duration, int volume = 0, unsigned int timestamp = 0)
-	{ return m_send && m_send->rtpSendEvent(event,duration,volume,timestamp); }
+	{ Lock lck(this); return m_send && m_send->rtpSendEvent(event,duration,volume,timestamp); }
 
     /**
      * Send one RTP key event
@@ -1063,7 +1119,14 @@ public:
      * @return True if data sending was attempted
      */
     inline bool rtpSendKey(char key, int duration, int volume = 0, unsigned int timestamp = 0)
-	{ return m_send && m_send->rtpSendKey(key,duration,volume,timestamp); }
+	{ Lock lck(this); return m_send && m_send->rtpSendKey(key,duration,volume,timestamp); }
+
+    /**
+     * Retrieve the number of lost packets in current received
+     * @return Number of packets in sequence gaps
+     */
+    inline u_int32_t ioPacketsLost() const
+	{ return m_recv ? m_recv->ioPacketsLost() : 0; }
 
     /**
      * Get the payload padding size
@@ -1181,7 +1244,7 @@ public:
      * @return True if address set, false if a failure occured
      */
     inline bool localAddr(SocketAddr& addr, bool rtcp = true)
-	{ return m_transport && m_transport->localAddr(addr,rtcp); }
+	{ Lock lck(this); return m_transport && m_transport->localAddr(addr,rtcp); }
 
     /**
      * Get the stored security provider or of the sender
@@ -1196,6 +1259,23 @@ public:
      */
     void security(RTPSecure* secure);
 
+    /**
+     * Set the RTCP report interval
+     * @param interval Average interval between reports in msec, zero to disable
+     */
+    void setReports(int interval);
+
+    /**
+     * Put the collected statistical data 
+     * @param stats NamedList to populate with the data
+     */
+    virtual void getStats(NamedList& stats) const;
+
+    /**
+     * Increase the counter for number of RTP packets received from a wrong source
+     */
+    virtual void incWrongSrc();
+
 protected:
     /**
      * Method called periodically to push any asynchronous data or statistics
@@ -1203,18 +1283,31 @@ protected:
      */
     virtual void timerTick(const Time& when);
 
+    /**
+     * Send a RTCP report
+     * @param when Time to use as base for timestamps
+     */
+    void sendRtcpReport(const Time& when);
+
+    /**
+     * Send a RTCP BYE when the sender is stopped or replaced
+     */
+    void sendRtcpBye();
+
 private:
     Direction m_direction;
     RTPSender* m_send;
     RTPReceiver* m_recv;
     RTPSecure* m_secure;
+    u_int64_t m_reportTime;
+    u_int64_t m_reportInterval;
 };
 
 /**
  * A bidirectional UDPTL session usable for T.38
  * @short UDPTL session
  */
-class YRTP_API UDPTLSession : public UDPSession
+class YRTP_API UDPTLSession : public UDPSession, public Mutex
 {
 public:
     /**
@@ -1228,7 +1321,7 @@ public:
      * @return True if address set, false if a failure occured
      */
     inline bool localAddr(SocketAddr& addr)
-	{ return m_transport && m_transport->localAddr(addr,false); }
+	{ Lock lck(this); return m_transport && m_transport->localAddr(addr,false); }
 
     /**
      * Get the maximum UDPTL packet length
