@@ -52,6 +52,8 @@ const TokenDict SDPParser::s_payloads[] = {
     { "speex",       102 },
     { "speex/16000", 103 },
     { "speex/32000", 104 },
+    { "isac/16000",  105 },
+    { "isac/32000",  106 },
     { "mjpeg",        26 },
     { "h261",         31 },
     { "h263",         34 },
@@ -75,12 +77,14 @@ const TokenDict SDPParser::s_rtpmap[] = {
     { "G729/8000",    18 },
     { "G729A/8000",   18 },
     { "MPA/90000",    14 },
-    { "ILBC/8000",    98 },
+    { "iLBC/8000",    98 },
     { "AMR/8000",     96 },
     { "AMR-WB/16000", 99 },
     { "SPEEX/8000",  102 },
     { "SPEEX/16000", 103 },
     { "SPEEX/32000", 104 },
+    { "iSAC/16000",  105 },
+    { "iSAC/32000",  106 },
     { "JPEG/90000",   26 },
     { "H261/90000",   31 },
     { "H263/90000",   34 },
@@ -92,16 +96,17 @@ const TokenDict SDPParser::s_rtpmap[] = {
 
 // Parse a received SDP body
 ObjList* SDPParser::parse(const MimeSdpBody& sdp, String& addr, ObjList* oldMedia,
-    const String& media)
+    const String& media, bool force)
 {
-    DDebug(DebugAll,"SDPParser::parse(%p,%s,%p,'%s')",&sdp,addr.c_str(),oldMedia,media.safe());
+    DDebug(DebugAll,"SDPParser::parse(%p,%s,%p,'%s',%s)",
+	&sdp,addr.c_str(),oldMedia,media.safe(),String::boolText(force));
     const NamedString* c = sdp.getLine("c");
     if (c) {
 	String tmp(*c);
 	if (tmp.startSkip("IN IP4")) {
 	    tmp.trimBlanks();
 	    // Handle the case media is muted
-	    if (tmp == "0.0.0.0")
+	    if (tmp == YSTRING("0.0.0.0"))
 		tmp.clear();
 	    addr = tmp;
 	}
@@ -134,7 +139,7 @@ ObjList* SDPParser::parse(const MimeSdpBody& sdp, String& addr, ObjList* oldMedi
 	    trans.toLower();
 	    rtp = false;
 	}
-	else {
+	else if (!force) {
 	    Debug(this,DebugWarn,"Unknown SDP transport '%s' for media '%s'",
 		trans.c_str(),type.c_str());
 	    continue;
@@ -191,6 +196,7 @@ ObjList* SDPParser::parse(const MimeSdpBody& sdp, String& addr, ObjList* oldMedi
 			}
 			if (line.startsWith("TELEPHONE-EVENT/")) {
 			    rfc2833 = var;
+			    payload.clear();
 			    continue;
 			}
 			const char* pload = 0;
@@ -245,7 +251,7 @@ ObjList* SDPParser::parse(const MimeSdpBody& sdp, String& addr, ObjList* oldMedi
 		else if ((mode == 30) || (ptime == 30))
 		    payload = "ilbc30";
 		else
-		    payload = m_hacks.getValue("ilbc_default","ilbc30");
+		    payload = m_hacks.getValue(YSTRING("ilbc_default"),"ilbc30");
 	    }
 
 	    if (amrOctet && payload == "amr")
@@ -261,7 +267,7 @@ ObjList* SDPParser::parse(const MimeSdpBody& sdp, String& addr, ObjList* oldMedi
 			mappings << ",";
 		    mappings << payload << "=" << var;
 		}
-		if ((payload == "g729") && m_hacks.getBoolValue("g729_annexb",annexB))
+		if ((payload == "g729") && m_hacks.getBoolValue(YSTRING("g729_annexb"),annexB))
 		    aux << ",g729b";
 	    }
 	}
@@ -276,7 +282,7 @@ ObjList* SDPParser::parse(const MimeSdpBody& sdp, String& addr, ObjList* oldMedi
 	}
 	bool append = false;
 	if (net)
-	    net->update(fmt,port);
+	    net->update(fmt,port,-1,force);
 	else {
 	    net = new SDPMedia(type,trans,fmt,port);
 	    append = true;
@@ -324,11 +330,17 @@ void SDPParser::initialize(const NamedList* codecs, const NamedList* hacks, cons
     }
     DDebug(this,DebugNote,"Default audio codecs: %s",m_audioFormats.c_str());
     m_ignorePort = m_hacks.getBoolValue("ignore_sdp_port",false);
-    m_rfc2833 = true;
+    m_rfc2833 = 101;
     m_secure = false;
     m_sdpForward = false;
     if (general) {
-	m_rfc2833 = general->getBoolValue("rfc2833",m_rfc2833);
+	if (general->getBoolValue("rfc2833",true)) {
+	    m_rfc2833 = general->getIntValue("rfc2833",m_rfc2833);
+	    if (m_rfc2833 < 96 || m_rfc2833 > 127)
+		m_rfc2833 = 101;
+	}
+	else
+	    m_rfc2833 = -1;
 	m_secure = general->getBoolValue("secure",m_secure);
 	m_sdpForward = general->getBoolValue("forward_sdp",m_sdpForward);
     }
