@@ -44,7 +44,8 @@ SDPSession::SDPSession(SDPParser* parser, NamedList& params)
 {
     m_rtpForward = params.getBoolValue("rtp_forward");
     m_secure = params.getBoolValue("secure",parser->m_secure);
-    m_rfc2833 = params.getBoolValue("rfc2833",parser->m_rfc2833);
+    m_rfc2833 = parser->m_rfc2833;
+    setRfc2833(params.getParam("rfc2833"));
 }
 
 SDPSession::~SDPSession()
@@ -94,6 +95,18 @@ void SDPSession::putMedia(NamedList& msg, ObjList* mList, bool putPort)
 	msg.setParam("media",String::boolText(false));
 }
 
+// Update the RFC 2833 availability and payload
+void SDPSession::setRfc2833(const String& value)
+{
+    if (value.toBoolean(true)) {
+	m_rfc2833 = value.toInteger(m_parser->m_rfc2833);
+	if (m_rfc2833 < 96 || m_rfc2833 > 127)
+	    m_rfc2833 = value.toBoolean(false) ? 101 : m_parser->m_rfc2833;
+    }
+    else
+	m_rfc2833 = -1;
+}
+
 // Build and dispatch a chan.rtp message for a given media. Update media on success
 bool SDPSession::dispatchRtp(SDPMedia* media, const char* addr, bool start,
     bool pick, RefObject* context)
@@ -309,6 +322,12 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
     bool defcodecs = m_parser->m_codecs.getBoolValue("default",true);
     for (ObjList* ml = mediaList->skipNull(); ml; ml = ml->skipNext()) {
 	SDPMedia* m = static_cast<SDPMedia*>(ml->get());
+	int rfc2833 = 0;
+	if ((m_rfc2833 >= 0) && m->isAudio()) {
+	    rfc2833 = m->rfc2833().toInteger(m_rfc2833);
+	    if (rfc2833 < 96 || rfc2833 > 127)
+		rfc2833 = 101;
+	}
 	String mline(m->fmtList());
 	ObjList* l = mline.split(',',false);
 	mline = *m;
@@ -319,7 +338,7 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 	int ptime = 0;
 	ObjList* f = l;
 	for (; f; f = f->next()) {
-	    String* s = static_cast<String*>(f->get());
+	    const String* s = static_cast<const String*>(f->get());
 	    if (s) {
 		int mode = 0;
 		if (*s == "ilbc20")
@@ -332,20 +351,69 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 		int defcode = payload;
 		String tmp = *s;
 		tmp << "=";
+		bool found = false;
 		for (ObjList* pl = map; pl; pl = pl->next()) {
-		    String* mapping = static_cast<String*>(pl->get());
+		    const String* mapping = static_cast<const String*>(pl->get());
 		    if (!mapping)
 			continue;
 		    if (mapping->startsWith(tmp)) {
 			payload = -1;
 			tmp = *mapping;
 			tmp >> "=" >> payload;
+			found = true;
 			XDebug(m_parser,DebugAll,"RTP mapped payload %d for '%s' [%p]",
 			    payload,s->c_str(),this);
 			break;
 		    }
+		    String tmp2 = *mapping;
+		    int pload;
+		    tmp2 >> "=" >> pload;
+		    if (payload == pload) {
+			XDebug(m_parser,DebugAll,"RTP conflict for payload %d, allocating new [%p]",
+			    payload,this);
+			payload = -1;
+			u_int32_t bmap = 0;
+			for (ObjList* sl = map; sl; sl = sl->next()) {
+			    mapping = static_cast<const String*>(sl->get());
+			    if (!mapping)
+				continue;
+			    tmp2 = *mapping;
+			    pload = 0;
+			    tmp2 >> "=" >> pload;
+			    if (pload >= 96 && pload < 127)
+				bmap |= 1 << (pload - 96);
+			}
+			// allocate free and non-standard is possible
+			for (pload = 96; pload < 127; pload++) {
+			    if (pload == rfc2833)
+				continue;
+			    if (lookup(pload,SDPParser::s_rtpmap))
+				continue;
+			    if ((bmap & (1 << (pload - 96))) == 0) {
+				payload = pload;
+				break;
+			    }
+			}
+			if (payload >= 0)
+			    break;
+			// none free, allocate from "standard" ones too
+			for (pload = 96; pload < 127; pload++) {
+			    if (pload == rfc2833)
+				continue;
+			    if ((bmap & (1 << (pload - 96))) == 0) {
+				payload = pload;
+				break;
+			    }
+			}
+			break;
+		    }
 		}
 		if (payload >= 0) {
+		    if (!found) {
+			tmp = *s;
+			tmp << "=" << payload;
+			map->append(new String(tmp));
+		    }
 		    if (defcode < 0)
 			defcode = payload;
 		    const char* map = lookup(defcode,SDPParser::s_rtpmap);
@@ -382,10 +450,7 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 	TelEngine::destruct(l);
 	TelEngine::destruct(map);
 
-	if (m_rfc2833 && frm && m->isAudio()) {
-	    int rfc2833 = m->rfc2833().toInteger(-1);
-	    if (rfc2833 < 0)
-		rfc2833 = 101;
+	if (rfc2833 && frm) {
 	    // claim to support telephone events
 	    frm << " " << rfc2833;
 	    String* s = new String;
@@ -547,7 +612,7 @@ void SDPSession::updateFormats(const NamedList& msg, bool changeMedia)
 	if (rtp) {
 	    if (rtp->update(*p))
 		Debug(m_parser,DebugNote,"Formats for '%s' changed to '%s' [%p]",
-		    tmp.c_str(),p->c_str(),this);
+		    tmp.c_str(),rtp->formats().c_str(),this);
 	}
 	else if (*p) {
 	    Debug(m_parser,DebugNote,"Got formats '%s' for absent media '%s' [%p]",
@@ -610,7 +675,7 @@ bool SDPSession::addRtpParams(NamedList& msg, const String& natAddr,
 }
 
 // Reset this object to default values
-void SDPSession::resetSdp()
+void SDPSession::resetSdp(bool all)
 {
     m_mediaStatus = MediaMissing;
     TelEngine::destruct(m_rtpMedia);
@@ -622,8 +687,10 @@ void SDPSession::resetSdp()
     m_sdpSession = 0;
     m_sdpVersion = 0;
     m_host.clear();
-    m_secure = m_parser->secure();
-    m_rfc2833 = m_parser->rfc2833();
+    if (all) {
+	m_secure = m_parser->secure();
+	m_rfc2833 = m_parser->rfc2833();
+    }
 }
 
 // Build a populated chan.rtp message
@@ -666,7 +733,7 @@ Message* SDPSession::buildChanRtp(SDPMedia* media, const char* addr, bool start,
     if (m_secure) {
 	if (media->remoteCrypto()) {
 	    String sdes = media->remoteCrypto();
-	    Regexp r("^\\([0-9]\\+\\) \\+\\([^ ]\\+\\) \\+\\([^ ]\\+\\) *\\(.*\\)$");
+	    static const Regexp r("^\\([0-9]\\+\\) \\+\\([^ ]\\+\\) \\+\\([^ ]\\+\\) *\\(.*\\)$");
 	    if (sdes.matches(r)) {
 		m->addParam("secure",String::boolText(true));
 		m->addParam("crypto_tag",sdes.matchString(1));
