@@ -27,6 +27,11 @@
 
 #include <string.h>
 
+// Maximum wait for a non-critical mutex acquisition
+#ifndef MAX_LOCK_WAIT
+#define MAX_LOCK_WAIT 10000
+#endif
+
 #define MIN_TICK_SLEEP 500
 #define DEF_TICK_SLEEP 5000
 #define MAX_TICK_SLEEP 50000
@@ -54,6 +59,18 @@ using namespace TelEngine;
 static ObjList s_factories;
 static Mutex s_mutex(true,"SignallingFactory");
 
+// Retrieve a value from a list
+// Shift it if upper bits are set and mask is not set
+// Mask it with a given mask
+static inline unsigned char fixValue(const NamedList& list, const String& param,
+    const TokenDict* dict, unsigned char mask, unsigned char upperMask, unsigned char shift)
+{
+    unsigned char val = (unsigned char)list.getIntValue(param,dict,0);
+    if (0 != (val & upperMask) && 0 == (val & mask))
+	val >>= shift;
+    return val & mask;
+}
+
 SignallingFactory::SignallingFactory(bool fallback)
 {
     s_mutex.lock();
@@ -92,21 +109,21 @@ SignallingComponent* SignallingFactory::build(const String& type, const NamedLis
     lock.drop();
     DDebug(DebugInfo,"Factory creating default '%s' named '%s'",type.c_str(),name->c_str());
     // now build some objects we know about
-    if (type == "SS7MTP2")
+    if (type == YSTRING("SS7MTP2"))
 	return new SS7MTP2(*name);
-    else if (type == "SS7MTP3")
+    else if (type == YSTRING("SS7M2PA"))
+	return new SS7M2PA(*name);
+    else if (type == YSTRING("SS7MTP3"))
 	return new SS7MTP3(*name);
-    else if (type == "SS7Router")
+    else if (type == YSTRING("SS7Router"))
 	return new SS7Router(*name);
-    else if (type == "SS7Management")
+    else if (type == YSTRING("SS7Management"))
 	return new SS7Management(*name);
-    else if (type == "SS7Maintenance")
-	return new SS7Maintenance(*name);
-    else if (type == "ISDNQ921")
+    else if (type == YSTRING("ISDNQ921"))
 	return new ISDNQ921(*name,*name);
-    else if (type == "ISDNQ931")
+    else if (type == YSTRING("ISDNQ931"))
 	return new ISDNQ931(*name,*name);
-    else if (type == "ISDNQ931Monitor")
+    else if (type == YSTRING("ISDNQ931Monitor"))
 	return new ISDNQ931Monitor(*name,*name);
     Debug(DebugMild,"Factory could not create '%s' named '%s'",type.c_str(),name->c_str());
     return 0;
@@ -129,11 +146,12 @@ void* SignallingFactory::buildInternal(const String& type, const NamedList* name
 
 
 SignallingComponent::SignallingComponent(const char* name, const NamedList* params)
-    : m_engine(0)
+    : m_engine(0), m_compType("unknown")
 {
     if (params) {
-	name = params->getValue("debugname",name);
-	debugLevel(params->getIntValue("debuglevel",-1));
+	name = params->getValue(YSTRING("debugname"),name);
+	m_compType = params->getValue(YSTRING("type"),m_compType);
+	debugLevel(params->getIntValue(YSTRING("debuglevel"),-1));
     }
     DDebug(engine(),DebugAll,"Component '%s' created [%p]",name,this);
     setName(name);
@@ -171,6 +189,28 @@ bool SignallingComponent::control(NamedList& params)
     return false;
 }
 
+NamedList* SignallingComponent::controlCreate(const char* oper)
+{
+    if (m_name.null())
+	return 0;
+    NamedList* params = new NamedList("chan.control");
+    params->addParam("component",m_name);
+    if (!TelEngine::null(oper))
+	params->addParam("operation",oper);
+    return params;
+}
+
+bool SignallingComponent::controlExecute(NamedList* params)
+{
+    bool ok = false;
+    if (params) {
+	ok = control(*params);
+	TelEngine::destruct(params);
+    }
+    return ok;
+}
+
+
 void SignallingComponent::engine(SignallingEngine* eng)
 {
     if (eng == m_engine)
@@ -215,10 +255,23 @@ unsigned long SignallingComponent::tickSleep(unsigned long usec) const
     return m_engine ? m_engine->tickSleep(usec) : 0;
 }
 
+void SignallingNotifier::notify(NamedList& notifs)
+{
+    DDebug(DebugInfo,"SignallingNotifier::notify() [%p] stub",this);
+}
+
+void SignallingNotifier::cleanup()
+{
+    DDebug(DebugInfo,"SignallingNotifier::cleanup() [%p] stub",this);
+}
+
+
+static SignallingEngine* s_self = 0;
+long SignallingEngine::s_maxLockWait = MAX_LOCK_WAIT;
 
 SignallingEngine::SignallingEngine(const char* name)
     : Mutex(true,"SignallingEngine"),
-      m_thread(0), m_listChanged(true),
+      m_thread(0),
       m_usecSleep(DEF_TICK_SLEEP), m_tickSleep(0)
 {
     debugName(name);
@@ -232,6 +285,8 @@ SignallingEngine::~SignallingEngine()
 	stop();
     }
     lock();
+    if (s_self == this)
+	s_self = 0;
     unsigned int n = m_components.count();
     if (n)
 	Debug(this,DebugNote,"Cleaning up %u components [%p]",n,this);
@@ -239,6 +294,17 @@ SignallingEngine::~SignallingEngine()
     unlock();
 }
 
+SignallingEngine* SignallingEngine::self(bool create)
+{
+    if (create && !s_self) {
+	// if mutex debugging is in force don't limit the lock time
+	if (Lockable::wait())
+	    s_maxLockWait = -1;
+	s_self = new SignallingEngine;
+    }
+    return s_self;
+}
+
 SignallingComponent* SignallingEngine::find(const String& name)
 {
     Lock mylock(this);
@@ -247,6 +313,8 @@ SignallingComponent* SignallingEngine::find(const String& name)
 
 SignallingComponent* SignallingEngine::find(const String& name, const String& type, const SignallingComponent* start)
 {
+    XDebug(this,DebugAll,"Engine finding '%s' of type %s from %p [%p]",
+	name.c_str(),type.c_str(),start,this);
     Lock mylock(this);
     ObjList* l = m_components.skipNull();
     if (start) {
@@ -269,20 +337,24 @@ bool SignallingEngine::find(const SignallingComponent* component)
     if (!component)
 	return false;
     Lock mylock(this);
+    DDebug(this,DebugAll,"Engine finding component @%p [%p]",component,this);
     return m_components.find(component) != 0;
 }
 
-SignallingComponent* SignallingEngine::build(const String& type, const NamedList& params, bool init)
+SignallingComponent* SignallingEngine::build(const String& type, const NamedList& params, bool init, bool ref)
 {
+    XDebug(this,DebugAll,"Engine building '%s' of type %s [%p]",
+	params.c_str(),type.c_str(),this);
     Lock mylock(this);
     SignallingComponent* c = find(params,type);
-    if (c && c->alive()) {
-	DDebug(this,DebugAll,"Engine returning existing component '%s' @%p [%p]",
-	    c->toString().c_str(),c,this);
+    if (c && (ref ? c->ref() : c->alive())) {
+	DDebug(this,DebugAll,"Engine returning existing component '%s' @%p (%d) [%p]",
+	    c->toString().c_str(),c,c->refcount(),this);
 	return c;
     }
     c = SignallingFactory::build(type,&params);
-    XDebug(this,DebugAll,"Created component @%p [%p]",c,this);
+    DDebug(this,DebugAll,"Created new component '%s' @%p [%p]",
+	c->toString().c_str(),c,this);
     insert(c);
     if (init && c)
 	c->initialize(&params);
@@ -337,6 +409,14 @@ bool SignallingEngine::remove(const String& name)
     return true;
 }
 
+void SignallingEngine::notify(SignallingComponent* component, NamedList notifs)
+{
+    if (!(m_notifier && component))
+	return;
+    Debug(this,DebugAll,"Engine [%p] sending notify from '%s' [%p]",this,component->toString().c_str(),component);
+    m_notifier->notify(notifs);
+}
+
 bool SignallingEngine::control(NamedList& params)
 {
     bool ok = false;
@@ -406,25 +486,30 @@ unsigned long SignallingEngine::tickSleep(unsigned long usec)
 
 unsigned long SignallingEngine::timerTick(const Time& when)
 {
+    RefPointer<SignallingComponent> c;
     lock();
     m_tickSleep = m_usecSleep;
-    m_listChanged = false;
-    for (ObjList* l = &m_components; l; l = l->next()) {
-	SignallingComponent* c = static_cast<SignallingComponent*>(l->get());
-	if (c) {
-	    c->timerTick(when);
-	    // if the list was changed (can be only from this thread) we
-	    //  break out and get back later - cheaper than using a ListIterator
-	    if (m_listChanged)
-		break;
-	}
+    ListIterator iter(m_components);
+    while (c = static_cast<SignallingComponent*>(iter.get())) {
+	unlock();
+	c->timerTick(when);
+	c = 0;
+	lock();
     }
-    unsigned long rval = m_listChanged ? 0 : m_tickSleep;
+    unsigned long rval = m_tickSleep;
     m_tickSleep = m_usecSleep;
     unlock();
     return rval;
 }
 
+void SignallingEngine::maxLockWait(long maxWait)
+{
+    if (maxWait < 0)
+	maxWait = -1;
+    else if (maxWait < MIN_TICK_SLEEP)
+	maxWait = MIN_TICK_SLEEP;
+    s_maxLockWait = maxWait;
+}
 
 SignallingThreadPrivate::~SignallingThreadPrivate()
 {
@@ -448,21 +533,39 @@ void SignallingThreadPrivate::run()
 }
 
 
+/*
+ * SignallingTimer
+ */
+// Retrieve a timer interval from a list of parameters
+unsigned int SignallingTimer::getInterval(const NamedList& params, const char* param,
+    unsigned int minVal, unsigned int defVal, unsigned int maxVal, bool allowDisable)
+{
+    unsigned int val = (unsigned int)params.getIntValue(param,defVal);
+    if (!val)
+        return allowDisable ? 0 : minVal;
+    if (val < minVal)
+	return minVal;
+    if (maxVal && val > maxVal)
+	return maxVal;
+    return val;
+}
+
+
 /**
  * SignallingUtils
  */
 
 // Coding standard as defined in Q.931/Q.850
-static TokenDict s_dict_codingStandard[] = {
+static const TokenDict s_dict_codingStandard[] = {
 	{"CCITT",            0x00},
-	{"ISO/IEC",          0x20},
-	{"national",         0x40},
-	{"network specific", 0x50},
+	{"ISO/IEC",          0x01},
+	{"national",         0x02},
+	{"network specific", 0x03},
 	{0,0}
 	};
 
 // Locations as defined in Q.850
-static TokenDict s_dict_location[] = {
+static const TokenDict s_dict_location[] = {
 	{"U",    0x00},                  // User
 	{"LPN",  0x01},                  // Private network serving the local user
 	{"LN",   0x02},                  // Public network serving the local user
@@ -477,22 +580,32 @@ static TokenDict s_dict_location[] = {
 // Q.850 2.2.5. Cause class: Bits 4-6
 // Q.850 Table 1. Cause value: Bits 0-6
 // Defined for CCITT coding standard
-static TokenDict s_dict_causeCCITT[] = {
+static const TokenDict s_dict_causeCCITT[] = {
 	// normal-event class
 	{"normal-event",                   0x00},
 	{"unallocated",                    0x01}, // Unallocated (unassigned) number
 	{"noroute-to-network",             0x02}, // No route to specified transit network
 	{"noroute",                        0x03}, // No route to destination
+	{"send-info-tone",                 0x04}, // Send special information tone
+	{"misdialed-trunk-prefix",         0x05}, // Misdialed trunk prefix
 	{"channel-unacceptable",           0x06}, // Channel unacceptable
 	{"call-delivered",                 0x07}, // Call awarded and being delivered in an established channel
+	{"preemption",                     0x08}, // Preemption
+	{"preemption-circuit-reserved",    0x09}, // Preemption circuit reserved for re-use
+	{"ported-number",                  0x0e}, // QoR: ported number Q.850 Addendum 1 (06/2000)
+	{"excess-digits",                  0x0e}, // Excess digits received, call is proceeding
 	{"normal-clearing",                0x10}, // Normal Clearing
 	{"busy",                           0x11}, // User busy
 	{"noresponse",                     0x12}, // No user responding
 	{"noanswer",                       0x13}, // No answer from user (user alerted)
+	{"offline",                        0x14}, // Subscriber absent
 	{"rejected",                       0x15}, // Call Rejected
 	{"moved",                          0x16}, // Number changed
+	{"redirection",                    0x17}, // Redirection to new destination Q.850 05/98
+	{"rejected-by-feature",            0x18}, // Call rejected due to feature at the destination Q.850 Amendment 1 (07/2001)
+	{"looping",                        0x19}, // Exchange routing error (hop counter) Q.850 05/98
 	{"answered",                       0x1a}, // Non-selected user clearing (answered elsewhere)
-	{"offline",                        0x1b}, // Destination out of order
+	{"out-of-order",                   0x1b}, // Destination out of order
 	{"invalid-number",                 0x1c}, // Invalid number format
 	{"facility-rejected",              0x1d}, // Facility rejected
 	{"status-enquiry-rsp",             0x1e}, // Response to STATUS ENQUIRY
@@ -500,11 +613,17 @@ static TokenDict s_dict_causeCCITT[] = {
 	// resource-unavailable class
 	{"resource-unavailable",           0x20}, // Resource unavailable
 	{"congestion",                     0x22}, // No circuit/channel available
+	{"channel-congestion",             0x22},
 	{"net-out-of-order",               0x26}, // Network out of order
+	{"frame-mode-conn-down",           0x27}, // Permanent frame mode connection out of service
+	{"frame-mode-conn-up",             0x28}, // Permanent frame mode connection operational
+	{"noconn",                         0x29},
 	{"temporary-failure",              0x29}, // Temporary failure
 	{"congestion",                     0x2a}, // Switching equipment congestion
+	{"switch-congestion",              0x2a},
 	{"access-info-discarded",          0x2b}, // Access information discarded
-	{"noconn",                         0x2c}, // Requested channel not available
+	{"channel-unavailable",            0x2c}, // Requested channel not available
+	{"preemption-congestion",          0x2e}, // Precedence call blocked
 	{"noresource",                     0x2f}, // Resource unavailable, unspecified
 	{"service-unavailable",            0x30}, // Service or option not available
 	{"qos-unavailable",                0x31}, // Quality of service unavailable
@@ -514,6 +633,7 @@ static TokenDict s_dict_causeCCITT[] = {
 	{"bearer-cap-not-auth",            0x39}, // Bearer capability not authorized
 	{"bearer-cap-not-available",       0x3a}, // Bearer capability not presently available
 	{"nomedia",                        0x3a},
+	{"invalid-access-info-out",        0x3e}, // Inconsistency in designated outgoing access information and subscriber class
 	{"service-unavailable",            0x3f}, // Service or option not available
 	// service-not-implemented class
 	{"bearer-cap-not-implemented",     0x41}, // Bearer capability not implemented
@@ -528,7 +648,10 @@ static TokenDict s_dict_causeCCITT[] = {
 	{"duplicate-callid",               0x54}, // Call identity in use
 	{"no-call-suspended",              0x55}, // No call suspended
 	{"suspended-call-cleared",         0x56}, // Call having the requested call identity has been cleared
+	{"not-subscribed",                 0x57}, // User not member of CUG
 	{"incompatible-dest",              0x58}, // Incompatible destination
+	{"unknown-group",                  0x5a}, // Non-existent CUG
+	{"invalid-transit-net",            0x5b}, // Invalid transit network selection
 	{"invalid-message",                0x5f}, // Invalid message, unspecified
 	// protocol-error class 
 	{"missing-mandatory-ie",           0x60}, // Mandatory information element is missing
@@ -538,6 +661,8 @@ static TokenDict s_dict_causeCCITT[] = {
 	{"invalid-ie",                     0x64}, // Invalid information element contents
 	{"wrong-state-message",            0x65}, // Message not compatible with call state
 	{"timeout",                        0x66}, // Recovery on timer expiry
+	{"unknown-param-passed-on",        0x67}, // Parameter non-existent or not implemented, passed on
+	{"unknown-param-message-droppped", 0x6e}, // Message with unrecognized parameter, discarded
 	{"protocol-error",                 0x6f}, // Protocol error, unspecified
 	// interworking class
 	{"interworking",                   0x7f}, // Interworking, unspecified
@@ -546,7 +671,7 @@ static TokenDict s_dict_causeCCITT[] = {
 
 // Q.931 4.5.5. Information transfer capability: Bits 0-4
 // Defined for CCITT coding standard
-static TokenDict s_dict_transferCapCCITT[] = {
+static const TokenDict s_dict_transferCapCCITT[] = {
 	{"speech",       0x00},          // Speech
 	{"udi",          0x08},          // Unrestricted digital information
 	{"rdi",          0x09},          // Restricted digital information
@@ -558,15 +683,15 @@ static TokenDict s_dict_transferCapCCITT[] = {
 
 // Q.931 4.5.5. Transfer mode: Bits 5,6
 // Defined for CCITT coding standard
-static TokenDict s_dict_transferModeCCITT[] = {
+static const TokenDict s_dict_transferModeCCITT[] = {
 	{"circuit",      0x00},          // Circuit switch mode
-	{"packet",       0x40},          // Packet mode
+	{"packet",       0x02},          // Packet mode
 	{0,0}
 	};
 
 // Q.931 4.5.5. Transfer rate: Bits 0-4
 // Defined for CCITT coding standard
-static TokenDict s_dict_transferRateCCITT[] = {
+static const TokenDict s_dict_transferRateCCITT[] = {
 	{"packet",        0x00},         // Packet mode only
 	{"64kbit",        0x10},         // 64 kbit/s
 	{"2x64kbit",      0x11},         // 2x64 kbit/s
@@ -579,7 +704,7 @@ static TokenDict s_dict_transferRateCCITT[] = {
 
 // Q.931 4.5.5. User information Layer 1 protocol: Bits 0-4
 // Defined for CCITT coding standard
-static TokenDict s_dict_formatCCITT[] = {
+static const TokenDict s_dict_formatCCITT[] = {
 	{"v110",          0x01},         // Recomendation V.110 and X.30
 	{"mulaw",         0x02},         // Recomendation G.711 mu-law
 	{"alaw",          0x03},         // Recomendation G.711 A-law 
@@ -591,7 +716,7 @@ static TokenDict s_dict_formatCCITT[] = {
 	{0,0}
 	};
 
-TokenDict* SignallingUtils::s_dictCCITT[5] = {
+const TokenDict* SignallingUtils::s_dictCCITT[5] = {
 	s_dict_causeCCITT,
 	s_dict_formatCCITT,
 	s_dict_transferCapCCITT,
@@ -599,16 +724,24 @@ TokenDict* SignallingUtils::s_dictCCITT[5] = {
 	s_dict_transferRateCCITT
 	};
 
-// Check if a list's parameter (comma separated list of flags) has a given flag
-bool SignallingUtils::hasFlag(const NamedList& list, const char* param, const char* flag)
+// Check if a comma separated list of flags has a given flag
+bool SignallingUtils::hasFlag(const String& flags, const char* flag)
 {
-    String s = list.getValue(param);
-    ObjList* obj = s.split(',',false);
+    ObjList* obj = flags.split(',',false);
     bool found = (obj->find(flag) != 0);
     TelEngine::destruct(obj);
     return found;
 }
 
+// Append a flag to a comma separated list of flags
+bool SignallingUtils::appendFlag(String& flags, const char* flag)
+{
+    if (TelEngine::null(flag) || hasFlag(flags,flag))
+	return false;
+    flags.append(flag,",");
+    return true;
+}
+
 // Remove a flag from a comma separated list of flags
 bool SignallingUtils::removeFlag(String& flags, const char* flag)
 {
@@ -624,6 +757,23 @@ bool SignallingUtils::removeFlag(String& flags, const char* flag)
     return (found != 0);
 }
 
+// Check if a list's parameter (comma separated list of flags) has a given flag
+bool SignallingUtils::hasFlag(const NamedList& list, const char* param, const char* flag)
+{
+    const String* s = list.getParam(param);
+    return s && hasFlag(*s,flag);
+}
+
+// Append a flag to a list parameter, create parameter if missing
+bool SignallingUtils::appendFlag(NamedList& list, const char* param, const char* flag)
+{
+    String* s = list.getParam(param);
+    if (s)
+	return appendFlag(*s,flag);
+    list.addParam(param,flag);
+    return true;
+}
+
 // Add string (keyword) if found or integer parameter to a named list
 void SignallingUtils::addKeyword(NamedList& list, const char* param, const TokenDict* tokens, unsigned int val)
 {
@@ -657,6 +807,7 @@ unsigned int SignallingUtils::dumpDataExt(const SignallingComponent* comp, Named
 	Debug(comp,DebugMild,"Utils::dumpDataExt invalid ext bits for %s (len=%u)",param,len);
 	return 0;
     }
+    count++;
     dumpData(comp,list,param,buf,count,sep);
     return count;
 }
@@ -706,7 +857,7 @@ bool SignallingUtils::decodeCause(const SignallingComponent* comp, NamedList& li
     }
     String causeName = prefix;
     // Byte 0: Coding standard (bit 5,6), location (bit 0-3)
-    unsigned char coding = buf[0] & 0x60;
+    unsigned char coding = (buf[0] & 0x60) >> 5;
     addKeyword(list,causeName + ".coding",codings(),coding);
     addKeyword(list,causeName + ".location",locations(),buf[0] & 0x0f);
     unsigned int crt = 1;
@@ -750,10 +901,10 @@ bool SignallingUtils::decodeCaps(const SignallingComponent* comp, NamedList& lis
     String capsName = prefix;
     // Byte 0: Coding standard (bit 5,6), Information transfer capability (bit 0-4)
     // Byte 1: Transfer mode (bit 5,6), Transfer rate (bit 0-4)
-    unsigned char coding = buf[0] & 0x60;
+    unsigned char coding = (buf[0] & 0x60) >> 5;
     addKeyword(list,capsName + ".coding",codings(),coding);
     addKeyword(list,capsName + ".transfercap",dict(2,coding),buf[0] & 0x1f);
-    addKeyword(list,capsName + ".transfermode",dict(3,coding),buf[1] & 0x60);
+    addKeyword(list,capsName + ".transfermode",dict(3,coding),(buf[1] & 0x60) >> 5);
     u_int8_t rate = buf[1] & 0x1f;
     addKeyword(list,capsName + ".transferrate",dict(4,coding),rate);
     // Figure 4.11 Note 1: Next byte is the rate multiplier if the transfer rate is 'multirate' (0x18)
@@ -783,7 +934,7 @@ bool SignallingUtils::decodeCaps(const SignallingComponent* comp, NamedList& lis
 // Encode a comma separated list of flags. Flags can be prefixed with the '-'
 //  character to be reset if previously set
 void SignallingUtils::encodeFlags(const SignallingComponent* comp,
-	int& dest, const String& flags, TokenDict* dict)
+	int& dest, const String& flags, const TokenDict* dict)
 {
     if (flags.null() || !dict)
 	return;
@@ -792,7 +943,7 @@ void SignallingUtils::encodeFlags(const SignallingComponent* comp,
     for (ObjList* o = list->skipNull(); o; o = o->skipNext()) {
 	String* s = static_cast<String*>(o->get());
 	bool set = !s->startSkip("-",false);
-	TokenDict* p = dict;
+	const TokenDict* p = dict;
 	for (; p->token && *s != p->token; p++) ;
 	if (!p->token) {
 	    DDebug(comp,DebugAll,"Utils::encodeFlags '%s' not found",s->c_str());
@@ -808,6 +959,31 @@ void SignallingUtils::encodeFlags(const SignallingComponent* comp,
     TelEngine::destruct(list);
 }
 
+// Encode a comma separated list of signalling flags
+unsigned int SignallingUtils::encodeFlags(const SignallingComponent* comp, const String& flags,
+    const SignallingFlags* dict, const char* paramName)
+{
+    if (!dict)
+	return 0;
+    unsigned int v = 0;
+    ObjList* l = flags.split(',',false);
+    for (ObjList* o = l->skipNull(); o; o = o->skipNext()) {
+	const String* s = static_cast<const String*>(o->get());
+	for (const SignallingFlags* d = dict; d->mask; d++) {
+	    if (*s == d->name) {
+		if (v & d->mask) {
+		    Debug(comp,DebugMild,"Flag %s. %s overwriting bits 0x%x",
+			paramName,d->name,v & d->mask);
+		    v &= d->mask;
+		}
+		v |= d->value;
+	    }
+	}
+    }
+    TelEngine::destruct(l);
+    return v;
+}
+
 // Q.850 2.1
 bool SignallingUtils::encodeCause(const SignallingComponent* comp, DataBlock& buf,
 	const NamedList& params, const char* prefix, bool isup, bool fail)
@@ -815,9 +991,9 @@ bool SignallingUtils::encodeCause(const SignallingComponent* comp, DataBlock& bu
     u_int8_t data[4] = {2,0x80,0x80,0x80};
     String causeName = prefix;
     // Coding standard (0: CCITT) + location. If no location, set it to 0x0a: "BI"
-    unsigned char coding = (unsigned char)params.getIntValue(causeName + ".coding",codings(),0);
+    unsigned char coding = fixValue(params,causeName + ".coding",codings(),0x03,0x60,5);
     unsigned char loc = (unsigned char)params.getIntValue(causeName + ".location",locations(),0x0a);
-    data[1] |= ((coding & 0x03) << 5) | (loc & 0x0f);
+    data[1] |= (coding << 5) | (loc & 0x0f);
     // Recommendation (only for Q.931)
     if (!isup) {
 	unsigned char rec = (unsigned char)params.getIntValue(causeName + ".rec",0,0);
@@ -828,9 +1004,7 @@ bool SignallingUtils::encodeCause(const SignallingComponent* comp, DataBlock& bu
     }
     // Value. Set to normal-clearing if missing for CCITT encoding or
     //  to 0 for other encoding standards
-    unsigned char val = 0;
-    if (!coding)
-	val = (unsigned char)params.getIntValue(causeName,dict(0,0),0x10);
+    unsigned char val = (unsigned char)params.getIntValue(causeName,dict(0,coding),!coding ? 0x10 : 0);
     data[data[0]] |= (val & 0x7f);
     // Diagnostic
     DataBlock diagnostic;
@@ -860,12 +1034,12 @@ bool SignallingUtils::encodeCaps(const SignallingComponent* comp, DataBlock& buf
     String capsName = prefix;
     // Byte 1: Coding standard (bit 5,6), Information transfer capability (bit 0-4)
     // Byte 2: Transfer mode (bit 5,6), Transfer rate (bit 0-4)
-    unsigned char coding = (unsigned char)params.getIntValue(capsName + ".coding",codings(),0);
+    unsigned char coding = fixValue(params,capsName + ".coding",codings(),0x03,0x60,5);
     unsigned char cap = (unsigned char)params.getIntValue(capsName + ".transfercap",dict(2,coding),0);
-    unsigned char mode = (unsigned char)params.getIntValue(capsName + ".transfermode",dict(3,coding),0);
+    unsigned char mode = fixValue(params,capsName + ".transfermode",dict(3,coding),0x03,0x60,5);
     unsigned char rate = (unsigned char)params.getIntValue(capsName + ".transferrate",dict(4,coding),0x10);
-    data[1] |= ((coding << 5) & 0x60) | (cap & 0x1f);
-    data[2] |= ((mode << 5) & 0x60) | (rate & 0x1f);
+    data[1] |= (coding << 5) | (cap & 0x1f);
+    data[2] |= (mode << 5) | (rate & 0x1f);
     if (rate == 0x18) {
 	data[0] = 3;
 	rate = (unsigned char)params.getIntValue(capsName + ".multiplier",0,0);
@@ -959,4 +1133,44 @@ unsigned int* SignallingUtils::parseUIntArray(const String& source,
     return 0;
 }
 
+
+/*
+ * SignallingMessageTimerList
+ */
+// Add a pending operation to the list. Start its timer
+SignallingMessageTimer* SignallingMessageTimerList::add(SignallingMessageTimer* m,
+    const Time& when)
+{
+    if (!m)
+	return 0;
+    m->stop();
+    m->start(when.msec());
+    if (m->global().interval() && !m->global().started())
+	m->global().start(when.msec());
+    ObjList* ins = skipNull();
+    for (; ins; ins = ins->skipNext()) {
+	SignallingMessageTimer* crt = static_cast<SignallingMessageTimer*>(ins->get());
+	if (m->fireTime() < crt->fireTime())
+	    break;
+    }
+    if (!ins)
+	append(m);
+    else
+	ins->insert(m);
+    return m;
+}
+
+// Check if the first operation timed out
+SignallingMessageTimer* SignallingMessageTimerList::timeout(const Time& when)
+{
+    ObjList* o = skipNull();
+    if (!o)
+	return 0;
+    SignallingMessageTimer* m = static_cast<SignallingMessageTimer*>(o->get());
+    if (!(m->timeout(when.msec()) || m->global().timeout(when.msec())))
+	return 0;
+    o->remove(false);
+    return m;
+}
+
 /* vi: set ts=8 sw=4 sts=4 noet: */
