@@ -160,6 +160,46 @@ void SS7Layer2::attach(SS7L2User* l2user)
     l2user->attach(this);
 }
 
+void SS7Layer2::timerTick(const Time& when)
+{
+    SignallingComponent::timerTick(when);
+    if (!m_l2userMutex.lock(SignallingEngine::maxLockWait()))
+	return;
+    RefPointer<SS7L2User> tmp = m_notify ? m_l2user : 0;
+    m_notify = false;
+    m_l2userMutex.unlock();
+    if (tmp) {
+	XDebug(this,DebugAll,"SS7Layer2 notifying user [%p]",this);
+	tmp->notify(this);
+    }
+}
+
+void SS7Layer2::notify()
+{
+    unsigned int wasUp = 0;
+    bool doNotify = false;
+    if (!operational()) {
+	wasUp = upTime();
+	m_lastUp = 0;
+	doNotify = (wasUp != 0);
+    }
+    else if (!m_lastUp) {
+	m_lastUp = Time::secNow();
+	doNotify = true;
+    }
+    m_l2userMutex.lock();
+    m_notify = true;
+    m_l2userMutex.unlock();
+    if (doNotify && engine()) {
+	NamedList params("");
+	params.addParam("from",toString());
+	params.addParam("type","ss7-layer2");
+	params.addParam("operational",String::boolText(operational()));
+	params.addParam("text",statusName());
+	engine()->notify(this,params);
+    }
+}
+
 unsigned int SS7Layer2::status() const
 {
     return ProcessorOutage;
@@ -192,14 +232,14 @@ bool SS7Layer2::control(Operation oper, NamedList* params)
 
 bool SS7Layer2::control(NamedList& params)
 {
-    String* ret = params.getParam("completion");
-    const String* oper = params.getParam("operation");
-    const char* cmp = params.getValue("component");
+    String* ret = params.getParam(YSTRING("completion"));
+    const String* oper = params.getParam(YSTRING("operation"));
+    const char* cmp = params.getValue(YSTRING("component"));
     int cmd = oper ? oper->toInteger(s_dict_control,-1) : -1;
     if (ret) {
 	if (oper && (cmd < 0))
 	    return false;
-	String part = params.getValue("partword");
+	String part = params.getValue(YSTRING("partword"));
 	if (cmp) {
 	    if (toString() != cmp)
 		return false;
@@ -214,9 +254,34 @@ bool SS7Layer2::control(NamedList& params)
     return (cmd >= 0) && control((Operation)cmd,&params);
 }
 
-ObjList* SS7Layer2::recoverMSU()
+bool SS7Layer2::getEmergency(NamedList* params, bool emg) const
 {
-    return 0;
+    if (m_autoEmergency && !emg) {
+	const SS7MTP3* mtp3 = YOBJECT(SS7MTP3,m_l2user);
+	if (mtp3 && !mtp3->linksActive())
+	    emg = true;
+    }
+    if (params)
+	emg = params->getBoolValue(YSTRING("emergency"),emg);
+    return emg;
+}
+
+bool SS7Layer2::inhibit(int setFlags, int clrFlags)
+{
+    int old = m_inhibited;
+    m_inhibited = (m_inhibited | setFlags) & ~clrFlags;
+    if (old != m_inhibited || (setFlags & clrFlags)) {
+	bool cycle = (setFlags & Inactive) && operational();
+	if (cycle)
+	    control(Pause);
+	Debug(this,DebugNote,"Link inhibition changed 0x%02X -> 0x%02X [%p]",
+	    old,m_inhibited,this);
+	if (operational())
+	    notify();
+	if (cycle)
+	    control(Resume);
+    }
+    return true;
 }
 
 
@@ -227,8 +292,9 @@ SS7MTP2::SS7MTP2(const NamedList& params, unsigned int status)
       m_status(status), m_lStatus(OutOfService), m_rStatus(OutOfAlignment),
       m_interval(0), m_resend(0), m_abort(0), m_fillTime(0), m_congestion(false),
       m_bsn(127), m_fsn(127), m_bib(true), m_fib(true),
-      m_lastFsn(128), m_lastBsn(127), m_lastBib(true), m_errors(0),
-      m_resendMs(250), m_abortMs(5000), m_fillIntervalMs(20), m_fillLink(true)
+      m_lastFsn(128), m_lastBsn(127), m_lastBib(true), m_errors(0), m_maxErrors(64),
+      m_resendMs(250), m_abortMs(5000), m_fillIntervalMs(20), m_fillLink(true),
+      m_autostart(false), m_flushMsus(true)
 {
 #ifdef DEBUG
     if (debugAt(DebugAll)) {
@@ -238,8 +304,13 @@ SS7MTP2::SS7MTP2(const NamedList& params, unsigned int status)
 	    &params,statusName(true),this,tmp.c_str());
     }
 #endif
-    m_fillLink = params.getBoolValue("filllink",m_fillLink);
-    setDumper(params.getValue("layer2dump"));
+    m_fillLink = params.getBoolValue(YSTRING("filllink"),m_fillLink);
+    m_maxErrors = params.getIntValue(YSTRING("maxerrors"),64);
+    if (m_maxErrors < 8)
+	m_maxErrors = 8;
+    else if (m_maxErrors > 256)
+	m_maxErrors = 256;
+    setDumper(params.getValue(YSTRING("layer2dump")));
 }
 
 SS7MTP2::~SS7MTP2()
@@ -290,14 +361,24 @@ bool SS7MTP2::initialize(const NamedList* config)
 	config->dump(tmp,"\r\n  ",'\'',true);
     Debug(this,DebugInfo,"SS7MTP2::initialize(%p) [%p]%s",config,this,tmp.c_str());
 #endif
-    if (config)
-	debugLevel(config->getIntValue("debuglevel_mtp2",
-	    config->getIntValue("debuglevel",-1)));
-    bool noStart = true;
+    if (config) {
+	debugLevel(config->getIntValue(YSTRING("debuglevel_mtp2"),
+	    config->getIntValue(YSTRING("debuglevel"),-1)));
+	m_autoEmergency = config->getBoolValue(YSTRING("autoemergency"),true);
+	unsigned int maxErrors = config->getIntValue(YSTRING("maxerrors"),m_maxErrors);
+	if (maxErrors < 8)
+	    m_maxErrors = 8;
+	else if (maxErrors > 256)
+	    m_maxErrors = 256;
+	else
+	    m_maxErrors = maxErrors;
+    }
+    m_autostart = !config || config->getBoolValue(YSTRING("autostart"),true);
+    m_flushMsus = !config || config->getBoolValue(YSTRING("flushmsus"),true);
     if (config && !iface()) {
-	NamedString* name = config->getParam("sig");
+	NamedString* name = config->getParam(YSTRING("sig"));
 	if (!name)
-	    name = config->getParam("basename");
+	    name = config->getParam(YSTRING("basename"));
 	if (name) {
 	    NamedPointer* ptr = YOBJECT(NamedPointer,name);
 	    NamedList* ifConfig = ptr ? YOBJECT(NamedList,ptr->userData()) : 0;
@@ -306,7 +387,7 @@ bool SS7MTP2::initialize(const NamedList* config)
 	    params.addParam("protocol","ss7");
 	    if (ifConfig) {
 		params.copyParams(*ifConfig);
-		int rx = params.getIntValue("rxunderrun");
+		int rx = params.getIntValue(YSTRING("rxunderrun"));
 		if ((rx > 0) && (rx < 25))
 		    params.setParam("rxunderrun","25");
 	    }
@@ -321,43 +402,52 @@ bool SS7MTP2::initialize(const NamedList* config)
 	    if (!(ifc->initialize(ifConfig) && control((Operation)SignallingInterface::Enable,ifConfig)))
 		TelEngine::destruct(SignallingReceiver::attach(0));
 	}
-	noStart = !config->getBoolValue("autostart",true);
     }
-    return iface() && (noStart || control(Resume,const_cast<NamedList*>(config)));
+    return iface() && control(Resume,const_cast<NamedList*>(config));
 }
 
 bool SS7MTP2::control(Operation oper, NamedList* params)
 {
     if (params) {
 	lock();
-	m_fillLink = params->getBoolValue("filllink",m_fillLink);
+	m_fillLink = params->getBoolValue(YSTRING("filllink"),m_fillLink);
+	m_autoEmergency = params->getBoolValue(YSTRING("autoemergency"),m_autoEmergency);
+	m_autostart = params->getBoolValue(YSTRING("autostart"),m_autostart);
+	m_flushMsus = params->getBoolValue(YSTRING("flushmsus"),m_flushMsus);
+	unsigned int maxErrors = params->getIntValue(YSTRING("maxerrors"),m_maxErrors);
+	if (maxErrors < 8)
+	    m_maxErrors = 8;
+	else if (maxErrors > 256)
+	    m_maxErrors = 256;
+	else
+	    m_maxErrors = maxErrors;
 	// The following are for test purposes
-	if (params->getBoolValue("toggle-bib"))
+	if (params->getBoolValue(YSTRING("toggle-bib")))
 	    m_bib = !m_bib;
-	if (params->getBoolValue("toggle-fib"))
+	if (params->getBoolValue(YSTRING("toggle-fib")))
 	    m_fib = !m_fib;
-	int tmp = params->getIntValue("change-fsn");
+	int tmp = params->getIntValue(YSTRING("change-fsn"));
 	if (tmp)
 	    m_fsn = (m_fsn + tmp) & 0x7f;
 	unlock();
-	tmp = params->getIntValue("send-lssu",-1);
+	tmp = params->getIntValue(YSTRING("send-lssu"),-1);
 	if (tmp >= 0)
 	    transmitLSSU(tmp);
-	if (params->getBoolValue("send-fisu"))
+	if (params->getBoolValue(YSTRING("send-fisu")))
 	    transmitFISU();
+	if (params->getBoolValue(YSTRING("simulate-error")))
+	    notify(SignallingInterface::HardwareError);
     }
     switch (oper) {
 	case Pause:
-	    m_status = OutOfService;
-	    abortAlignment();
-	    transmitLSSU();
+	    abortAlignment(false);
 	    return true;
 	case Resume:
-	    if (aligned())
+	    if (aligned() || !m_autostart)
 		return true;
 	    // fall-through
 	case Align:
-	    startAlignment(params && params->getBoolValue("emergency"));
+	    startAlignment(getEmergency(params));
 	    return true;
 	case Status:
 	    return operational();
@@ -371,17 +461,22 @@ bool SS7MTP2::notify(SignallingInterface::Notification event)
     switch (event) {
 	case SignallingInterface::LinkDown:
 	    Debug(this,DebugWarn,"Interface is down - realigning [%p]",this);
-	    abortAlignment();
+	    abortAlignment(m_autostart);
 	    break;
 	case SignallingInterface::LinkUp:
 	    Debug(this,DebugInfo,"Interface is up [%p]",this);
+	    control(Resume);
 	    break;
 	default:
 	    XDebug(this,DebugMild,"Got error %u: %s [%p]",
 		event,lookup(event,SignallingInterface::s_notifName),this);
-	    if (++m_errors >= 4) {
-		Debug(this,DebugWarn,"Got %d errors - realigning [%p]",m_errors,this);
-		abortAlignment();
+	    {
+		unsigned int err = (m_errors += 256) >> 8;
+		if (err >= (operational() ? m_maxErrors :
+		    ((m_rStatus == EmergencyAlignment) ? 1 : 4))) {
+		    Debug(this,DebugWarn,"Got %u errors - realigning [%p]",err,this);
+		    abortAlignment(m_autostart);
+		}
 	    }
     }
     return true;
@@ -389,7 +484,9 @@ bool SS7MTP2::notify(SignallingInterface::Notification event)
 
 void SS7MTP2::timerTick(const Time& when)
 {
-    lock();
+    SS7Layer2::timerTick(when);
+    if (!lock(SignallingEngine::maxLockWait()))
+	return;
     bool tout = m_interval && (when >= m_interval);
     if (tout)
 	m_interval = 0;
@@ -402,20 +499,23 @@ void SS7MTP2::timerTick(const Time& when)
     unlock();
     if (aborting) {
 	Debug(this,DebugWarn,"Timeout for MSU acknowledgement, realigning [%p]",this);
-	abortAlignment();
+	abortAlignment(m_autostart);
 	return;
     }
     if (operational()) {
 	if (tout) {
 	    Debug(this,DebugInfo,"Proving period ended, link operational [%p]",this);
 	    lock();
+	    m_lastSeqRx = -1;
 	    unsigned int q = m_queue.count();
-	    if (q >= 64) {
+	    if (!q)
+		;
+	    else if (m_flushMsus || q >= 64) {
 		// there shouldn't have been that many queued MSUs
 		Debug(this,DebugWarn,"Cleaning %u queued MSUs from proved link! [%p]",q,this);
 		m_queue.clear();
 	    }
-	    else if (q) {
+	    else {
 		Debug(this,DebugNote,"Changing FSN of %u MSUs queued in proved link! [%p]",q,this);
 		// transmit a FISU just before the bunch of MSUs
 		transmitFISU();
@@ -466,18 +566,15 @@ void SS7MTP2::timerTick(const Time& when)
 	}
     }
     else if (tout) {
-	if (m_lStatus == OutOfService) {
-	    switch (m_status) {
-		case NormalAlignment:
-		case EmergencyAlignment:
+	switch (m_lStatus) {
+	    case OutOfService:
+		if (m_status != OutOfService)
 		    setLocalStatus(OutOfAlignment);
-		    break;
-		default:
-		    setLocalStatus(m_status);
-	    }
+		break;
+	    case OutOfAlignment:
+		Debug(this,DebugMild,"Initial alignment timed out, retrying");
+		break;
 	}
-	else if (m_lStatus == OutOfAlignment)
-	    Debug(this,DebugMild,"Initial alignment timed out, retrying");
     }
     if (when >= m_fillTime) {
 	if (operational())
@@ -539,24 +636,29 @@ bool SS7MTP2::transmitMSU(const SS7MSU& msu)
 }
 
 // Remove the MSUs in the queue, the upper layer will move them to another link
-ObjList* SS7MTP2::recoverMSU()
+void SS7MTP2::recoverMSU(int sequence)
 {
-    lock();
-    ObjList* lst = 0;
+    Debug(this,DebugInfo,"Recovering MSUs from sequence %d",sequence);
     for (;;) {
+	lock();
 	DataBlock* pkt = static_cast<DataBlock*>(m_queue.remove(false));
+	unlock();
 	if (!pkt)
 	    break;
-	if (pkt->length() > 3) {
-	    SS7MSU* msu = new SS7MSU(3 + (char*)pkt->data(),pkt->length() - 3);
-	    if (!lst)
-		lst = new ObjList;
-	    lst->append(msu);
+	unsigned char* head = pkt->data(0,4);
+	if (head) {
+	    int seq = head[1] & 0x7f;
+	    if (sequence < 0 || ((seq - sequence) & 0x7f) < 0x3f) {
+		sequence = -1;
+		SS7MSU msu(head + 3,pkt->length() - 3);
+		recoveredMSU(msu);
+	    }
+	    else
+		Debug(this,DebugAll,"Not recovering MSU with seq=%d, requested %d",
+		    seq,sequence);
 	}
 	TelEngine::destruct(pkt);
     }
-    unlock();
-    return lst;
 }
 
 // Decode a received packet into signalling units
@@ -578,6 +680,9 @@ bool SS7MTP2::receivedPacket(const DataBlock& packet)
 	return false;
     }
 
+    // adjust error counter
+    if (m_errors && operational())
+	m_errors--;
     // process LSSU and FISU to detect link status changes
     switch (len) {
 	case 2:
@@ -626,6 +731,9 @@ bool SS7MTP2::receivedPacket(const DataBlock& packet)
 	    m_resend = Time::now();
 	}
 	unqueueAck(bsn);
+	// end proving now if received MSU with correct sequence
+	if (m_interval && (diff == 1))
+	    m_interval = Time::now();
     }
     else {
 	// keep sequence numbers in sync with the remote
@@ -642,7 +750,7 @@ bool SS7MTP2::receivedPacket(const DataBlock& packet)
     // just drop MSUs if not operational or out of sequence
     if (!((diff == 1) && operational()))
 	return false;
-    m_bsn = fsn;
+    m_lastSeqRx = m_bsn = fsn;
     m_fillTime = 0;
     DDebug(this,DebugInfo,"New local bsn=%u/%d fsn=%u/%d [%p]",
 	m_bsn,m_bib,m_fsn,m_fib,this);
@@ -731,7 +839,7 @@ void SS7MTP2::processLSSU(unsigned int status)
     setRemoteStatus(status);
     if (status == Busy) {
 	if (unaligned)
-	    abortAlignment();
+	    abortAlignment(m_autostart);
 	else
 	    m_congestion = true;
 	return;
@@ -741,12 +849,19 @@ void SS7MTP2::processLSSU(unsigned int status)
 	case OutOfAlignment:
 	case NormalAlignment:
 	case EmergencyAlignment:
+	    if (m_lStatus == OutOfService) {
+		if (m_status != OutOfService)
+		    setLocalStatus(OutOfAlignment);
+		break;
+	    }
 	    if (!(unaligned && startProving()))
 		setLocalStatus(m_status);
 	    break;
 	default:
-	    if (!m_interval)
-		abortAlignment();
+	    if (!m_interval) {
+		if (m_status != OutOfService)
+		    abortAlignment(m_autostart);
+	    }
 	    else if (m_lStatus != OutOfService && m_lStatus != OutOfAlignment)
 		m_interval = 0;
     }
@@ -804,29 +919,38 @@ void SS7MTP2::startAlignment(bool emergency)
     else
 	Debug(this,DebugInfo,"Starting %s alignment [%p]",
 	    (emergency ? "emergency" : "normal"),this);
+    m_bsn = m_fsn = 127;
+    m_bib = m_fib = true;
+    if (m_lStatus != OutOfService) {
+	setLocalStatus(OutOfService);
+	unlock();
+	transmitLSSU();
+	lock();
+    }
     m_status = emergency ? EmergencyAlignment : NormalAlignment;
     m_abort = m_resend = 0;
     setLocalStatus(OutOfAlignment);
     m_interval = Time::now() + 5000000;
-    m_fsn = 127;
-    m_fib = true;
     unlock();
     transmitLSSU();
     SS7Layer2::notify();
 }
 
-void SS7MTP2::abortAlignment()
+void SS7MTP2::abortAlignment(bool retry)
 {
     lock();
     DDebug(this,DebugNote,"Aborting alignment [%p]",this);
+    if (!retry)
+	m_status = OutOfService;
     setLocalStatus(OutOfService);
     m_interval = Time::now() + 1000000;
     m_abort = m_resend = 0;
     m_errors = 0;
-    m_fsn = 127;
-    m_fib = true;
+    m_bsn = m_fsn = 127;
+    m_bib = m_fib = true;
     m_fillTime = 0;
     unlock();
+    transmitLSSU();
     SS7Layer2::notify();
 }
 
