@@ -30,11 +30,14 @@
 
 using namespace TelEngine;
 
-TokenDict SignallingCircuit::s_lockNames[] = {
+const TokenDict SignallingCircuit::s_lockNames[] = {
     {"localhw",            LockLocalHWFail},
     {"localmaint",         LockLocalMaint},
+    {"lockinghw",          LockingHWFail},
+    {"lockingmaint",       LockingMaint},
     {"localhwchanged",     LockLocalHWFailChg},
     {"localmaintchanged",  LockLocalMaintChg},
+    {"resetting",          Resetting},
     {"remotehw",           LockRemoteHWFail},
     {"remotemaint",        LockRemoteMaint},
     {"remotehwchanged",    LockRemoteHWFailChg},
@@ -42,6 +45,21 @@ TokenDict SignallingCircuit::s_lockNames[] = {
     {0,0},
 };
 
+const TokenDict SignallingCallControl::s_mediaRequired[] = {
+    { "no", SignallingCallControl::MediaNever },
+    { "false", SignallingCallControl::MediaNever },
+    { "off", SignallingCallControl::MediaNever },
+    { "disable", SignallingCallControl::MediaNever },
+    { "answered", SignallingCallControl::MediaAnswered },
+    { "connected", SignallingCallControl::MediaAnswered },
+    { "ringing", SignallingCallControl::MediaRinging },
+    { "progress", SignallingCallControl::MediaRinging },
+    { "yes", SignallingCallControl::MediaAlways },
+    { "true", SignallingCallControl::MediaAlways },
+    { "on", SignallingCallControl::MediaAlways },
+    { "enable", SignallingCallControl::MediaAlways },
+    { 0, 0 }
+};
 
 /**
  * SignallingCallControl
@@ -49,18 +67,21 @@ TokenDict SignallingCircuit::s_lockNames[] = {
 SignallingCallControl::SignallingCallControl(const NamedList& params,
 	const char* msgPrefix)
     : Mutex(true,"SignallingCallControl"),
+      m_mediaRequired(MediaNever),
       m_verifyEvent(false),
       m_verifyTimer(0),
       m_circuits(0),
       m_strategy(SignallingCircuitGroup::Increment),
       m_exiting(false)
 {
+    // Controller location
+    m_location = params.getValue(YSTRING("location"));
     // Strategy
-    const char* strategy = params.getValue("strategy","increment");
+    const char* strategy = params.getValue(YSTRING("strategy"),"increment");
     m_strategy = SignallingCircuitGroup::str2strategy(strategy);
     String restrict;
     if (m_strategy != SignallingCircuitGroup::Random)
-	restrict = params.getValue("strategy-restrict");
+	restrict = params.getValue(YSTRING("strategy-restrict"));
     if (!restrict.null()) {
 	if (restrict == "odd")
 	    m_strategy |= SignallingCircuitGroup::OnlyOdd;
@@ -73,11 +94,15 @@ SignallingCallControl::SignallingCallControl(const NamedList& params,
     }
 
     // Message prefix
-    m_msgPrefix = params.getValue("message-prefix",msgPrefix);
+    m_msgPrefix = params.getValue(YSTRING("message-prefix"),msgPrefix);
 
     // Verify event timer
     m_verifyTimer.interval(params,"verifyeventinterval",10,120,true,true);
     m_verifyTimer.start();
+
+    // Media Required
+    m_mediaRequired = (MediaRequired)params.getIntValue(YSTRING("needmedia"),
+	s_mediaRequired,m_mediaRequired);
 }
 
 SignallingCallControl::~SignallingCallControl()
@@ -129,8 +154,36 @@ bool SignallingCallControl::reserveCircuit(SignallingCircuit*& cic, const char*
 	}
 	cic = m_circuits->reserve(*list,mandatory,checkLock,s,m_circuits->findRange(range));
     }
-    else
+    else if (range) {
+	const char* nRange = range;
+	switch (nRange[0]) {
+	    case '!':
+		mandatory = true;
+		nRange++;
+		break;
+	    case '?':
+		mandatory = false;
+		nRange++;
+		break;
+	}
+	int num = String(nRange).toInteger();
+	if (num > 0) {
+	    // Specific circuit required
+	    SignallingCircuit* circuit = m_circuits->find(num);
+	    if (circuit && !circuit->locked(checkLock) && circuit->reserve()) {
+		if (circuit->ref())
+		    cic = circuit;
+		else
+		    m_circuits->release(circuit);
+	    }
+	    if (cic || mandatory)
+		return (cic != 0);
+	    DDebug(DebugInfo,"SignallingCallControl. Fallback, circuit %u not available [%p]",num,this);
+	}
 	cic = m_circuits->reserve(checkLock,-1,m_circuits->findRange(range));
+    }
+    else
+	cic = m_circuits->reserve(checkLock,-1);
     return (cic != 0);
 }
 
@@ -160,6 +213,16 @@ bool SignallingCallControl::releaseCircuit(unsigned int code, bool sync)
 SignallingEvent* SignallingCallControl::getEvent(const Time& when)
 {
     lock();
+    // Verify ?
+    if (m_verifyEvent && m_verifyTimer.timeout(when.msec())) {
+	SignallingMessage* msg = new SignallingMessage;
+	SignallingEvent* event = new SignallingEvent(SignallingEvent::Verify,msg,this);
+	buildVerifyEvent(msg->params());
+	TelEngine::destruct(msg);
+	setVerify(true,false,&when);
+	unlock();
+	return event;
+    }
     ListIterator iter(m_calls);
     for (;;) {
 	SignallingCall* call = static_cast<SignallingCall*>(iter.get());
@@ -195,14 +258,6 @@ SignallingEvent* SignallingCallControl::getEvent(const Time& when)
 		return event;
 	}
     }
-    // Verify ?
-    if (m_verifyTimer.timeout(when.msecNow()) && m_verifyEvent) {
-	SignallingMessage* msg = new SignallingMessage;
-	SignallingEvent* event = new SignallingEvent(SignallingEvent::Verify,msg,this);
-	buildVerifyEvent(msg->params());
-	m_verifyTimer.start(when.msecNow());
-	return event;
-    }
     // Terminate if exiting and no more calls
     //TODO: Make sure we raise this event one time only
     if (exiting() && !m_calls.skipNull())
@@ -231,6 +286,19 @@ void SignallingCallControl::removeCall(SignallingCall* call, bool del)
     unlock();
 }
 
+// Set the verify event flag. Restart/fire verify timer
+void SignallingCallControl::setVerify(bool restartTimer, bool fireNow, const Time* time)
+{
+    m_verifyEvent = true;
+    if (!restartTimer)
+	return;
+    m_verifyTimer.stop();
+    if (!fireNow)
+	m_verifyTimer.start(time ? time->msec() : Time::msecNow());
+    else
+	m_verifyTimer.fire();
+}
+
 
 /**
  * SignallingCall
@@ -238,6 +306,7 @@ void SignallingCallControl::removeCall(SignallingCall* call, bool del)
 SignallingCall::SignallingCall(SignallingCallControl* controller, bool outgoing, bool signalOnly)
     : Mutex(true,"SignallingCall"),
     m_lastEvent(0),
+    m_overlap(false),
     m_controller(controller),
     m_outgoing(outgoing),
     m_signalOnly(signalOnly),
@@ -295,7 +364,7 @@ SignallingMessage* SignallingCall::dequeue(bool remove)
 /**
  * SignallingEvent
  */
-TokenDict SignallingEvent::s_types[] = {
+const TokenDict SignallingEvent::s_types[] = {
 	{"Unknown",  Unknown},
 	{"Generic",  Generic},
 	{"NewCall",  NewCall},
@@ -404,7 +473,7 @@ bool SignallingCircuitEvent::sendEvent()
 /**
  * SignallingCircuit
  */
-static TokenDict s_cicTypeDict[] = {
+static const TokenDict s_cicTypeDict[] = {
     {"TDM",     SignallingCircuit::TDM},
     {"RTP",     SignallingCircuit::RTP},
     {"IAX",     SignallingCircuit::IAX},
@@ -413,13 +482,14 @@ static TokenDict s_cicTypeDict[] = {
     {0,0}
 };
 
-static TokenDict s_cicStatusDict[] = {
+static const TokenDict s_cicStatusDict[] = {
     {"Missing",   SignallingCircuit::Missing},
     {"Disabled",  SignallingCircuit::Disabled},
     {"Idle",      SignallingCircuit::Idle},
     {"Reserved",  SignallingCircuit::Reserved},
     {"Starting",  SignallingCircuit::Starting},
     {"Stopping",  SignallingCircuit::Stopping},
+    {"Special",   SignallingCircuit::Special},
     {"Connected", SignallingCircuit::Connected},
     {0,0}
 };
@@ -577,6 +647,17 @@ SignallingCircuitRange::SignallingCircuitRange(const String& rangeStr,
     add(rangeStr);
 }
 
+// Allocate and return an array containing range circuits
+unsigned int* SignallingCircuitRange::copyRange(unsigned int& count) const
+{
+    if (!m_count)
+	return 0;
+    count = m_count;
+    unsigned int* tmp = new unsigned int[count];
+    ::memcpy(tmp,range(),m_range.length());
+    return tmp;
+}
+
 // Add codes to this range from a string
 bool SignallingCircuitRange::add(const String& rangeStr)
 {
@@ -649,7 +730,7 @@ void SignallingCircuitRange::updateLast()
 /**
  * SignallingCircuitGroup
  */
-TokenDict SignallingCircuitGroup::s_strategy[] = {
+const TokenDict SignallingCircuitGroup::s_strategy[] = {
 	{"increment", Increment},
 	{"decrement", Decrement},
 	{"lowest",    Lowest},
@@ -762,7 +843,7 @@ SignallingCircuitSpan* SignallingCircuitGroup::buildSpan(const String& name, uns
 	    : NamedList(name), m_group(group)
 	    { }
 	virtual void* getObject(const String& name) const
-	    { return (name == "SignallingCircuitGroup") ? m_group : NamedList::getObject(name); }
+	    { return (name == YSTRING("SignallingCircuitGroup")) ? m_group : NamedList::getObject(name); }
 	SignallingCircuitGroup* m_group;
     };
 
@@ -805,7 +886,7 @@ void SignallingCircuitGroup::insertRange(const String& range, const char* name,
     if (strategy < 0)
 	strategy = m_range.m_strategy;
     m_ranges.append(new SignallingCircuitRange(range,name,strategy));
-    Debug(this,DebugAll,"Added range %s: %s [%p]",name,range.c_str(),this);
+    Debug(this,DebugNote,"Added range %s: %s [%p]",name,range.c_str(),this);
 }
 
 // Remove a span from list
@@ -852,12 +933,17 @@ bool SignallingCircuitGroup::status(unsigned int cic, SignallingCircuit::Status
     return circuit && circuit->status(newStat,sync);
 }
 
-inline void adjustParity(unsigned int& n, int strategy)
+inline void adjustParity(unsigned int& n, int strategy, bool up)
 {
-    if ((strategy & SignallingCircuitGroup::OnlyEven) && (n & 1))
-	n &= ~1;
-    else if ((strategy & SignallingCircuitGroup::OnlyOdd) && !(n & 1))
-	n |= 1;
+    if (((strategy & SignallingCircuitGroup::OnlyEven) && (n & 1)) ||
+	((strategy & SignallingCircuitGroup::OnlyOdd) && !(n & 1))) {
+	if (up)
+	    n++;
+	else if (n)
+	    n--;
+	else
+	    n = (strategy & SignallingCircuitGroup::OnlyEven) ? 0 : 1;
+    }
 }
 
 // Choose the next circuit code to check, depending on strategy
@@ -870,16 +956,18 @@ unsigned int SignallingCircuitGroup::advance(unsigned int n, int strategy,
 	case Increment:
 	case Lowest:
 	    n += delta;
-	    if (n >= range.m_last)
-		n = delta;
+	    if (n >= range.m_last) {
+		n = 0;
+		adjustParity(n,strategy,true);
+	    }
 	    break;
 	case Decrement:
 	case Highest:
 	    if (n >= delta)
 		n -= delta;
 	    else {
-		n = range.m_last - 1;
-		adjustParity(n,strategy);
+		n = range.m_last;
+		adjustParity(n,strategy,false);
 	    }
 	    break;
 	default:
@@ -902,7 +990,7 @@ SignallingCircuit* SignallingCircuitGroup::reserve(int checkLock, int strategy,
 	return 0;
     if (strategy < 0)
 	strategy = range->m_strategy;
-    int dir = 1;
+    bool up = true;
     unsigned int n = range->m_used;
     // first adjust the last used channel number
     switch (strategy & 0xfff) {
@@ -910,22 +998,25 @@ SignallingCircuit* SignallingCircuitGroup::reserve(int checkLock, int strategy,
 	    n = (n + 1) % range->m_last;
 	    break;
 	case Decrement:
-	    n = (n ? n : range->m_last) - 1;
-	    dir = -1;
+	    if (n < 2)
+		n = range->m_last;
+	    else
+		n--;
+	    up = false;
 	    break;
 	case Lowest:
 	    n = 0;
 	    break;
 	case Highest:
-	    n = range->m_last - 1;
-	    dir = -1;
+	    n = range->m_last;
+	    up = false;
 	    break;
 	default:
 	    while ((range->m_last > 1) && (n == range->m_used))
-		n = ::random() % range->m_last;
+		n = Random::random() % range->m_last;
     }
     // then go to the proper even/odd start circuit
-    adjustParity(n,strategy);
+    adjustParity(n,strategy,up);
     // remember where the scan started
     unsigned int start = n;
     // try at most how many channels we have, halve that if we only scan even or odd
@@ -1137,15 +1228,15 @@ AnalogLine::AnalogLine(AnalogLineGroup* grp, unsigned int cic, const NamedList&
     if (m_type == Recorder)
 	m_type = FXO;
     m_address << m_group->toString() << "/" << m_circuit->code();
-    m_inband = params.getBoolValue("dtmfinband",false);
-    String tmp = params.getValue("echocancel");
+    m_inband = params.getBoolValue(YSTRING("dtmfinband"),false);
+    String tmp = params.getValue(YSTRING("echocancel"));
     if (tmp.isBoolean())
 	m_echocancel = tmp.toBoolean() ? 1 : -1;
-    m_answerOnPolarity = params.getBoolValue("answer-on-polarity",false);
-    m_hangupOnPolarity = params.getBoolValue("hangup-on-polarity",false);
-    m_polarityControl = params.getBoolValue("polaritycontrol",false);
+    m_answerOnPolarity = params.getBoolValue(YSTRING("answer-on-polarity"),false);
+    m_hangupOnPolarity = params.getBoolValue(YSTRING("hangup-on-polarity"),false);
+    m_polarityControl = params.getBoolValue(YSTRING("polaritycontrol"),false);
 
-    m_callSetup = (CallSetupInfo)lookup(params.getValue("callsetup"),csNames(),After);
+    m_callSetup = (CallSetupInfo)lookup(params.getValue(YSTRING("callsetup")),csNames(),After);
 
     m_callSetupTimeout = getValidInt(params,"callsetup-timeout",2000);
     m_noRingTimeout = getValidInt(params,"ring-timeout",10000);
@@ -1155,9 +1246,9 @@ AnalogLine::AnalogLine(AnalogLineGroup* grp, unsigned int cic, const NamedList&
     DDebug(m_group,DebugAll,"AnalogLine() addr=%s type=%s [%p]",
 	address(),lookup(m_type,typeNames()),this);
 
-    if (!params.getBoolValue("out-of-service",false)) {
+    if (!params.getBoolValue(YSTRING("out-of-service"),false)) {
 	resetCircuit();
-	if (params.getBoolValue("connect",true))
+	if (params.getBoolValue(YSTRING("connect"),true))
 	    connect(false);
     }
     else
