@@ -39,7 +39,7 @@ SIPMessage::SIPMessage(const SIPMessage& original)
       body(0), m_ep(0),
       m_valid(original.isValid()), m_answer(original.isAnswer()),
       m_outgoing(original.isOutgoing()), m_ack(original.isACK()),
-      m_cseq(-1)
+      m_cseq(-1), m_flags(original.getFlags())
 {
     DDebug(DebugAll,"SIPMessage::SIPMessage(&%p) [%p]",
 	&original,this);
@@ -68,15 +68,15 @@ SIPMessage::SIPMessage(const SIPMessage& original)
 SIPMessage::SIPMessage(const char* _method, const char* _uri, const char* _version)
     : version(_version), method(_method), uri(_uri), code(0),
       body(0), m_ep(0), m_valid(true),
-      m_answer(false), m_outgoing(true), m_ack(false), m_cseq(-1)
+      m_answer(false), m_outgoing(true), m_ack(false), m_cseq(-1), m_flags(-1)
 {
     DDebug(DebugAll,"SIPMessage::SIPMessage('%s','%s','%s') [%p]",
 	_method,_uri,_version,this);
 }
 
-SIPMessage::SIPMessage(SIPParty* ep, const char* buf, int len)
+SIPMessage::SIPMessage(SIPParty* ep, const char* buf, int len, unsigned int* bodyLen)
     : code(0), body(0), m_ep(ep), m_valid(false),
-      m_answer(false), m_outgoing(false), m_ack(false), m_cseq(-1)
+      m_answer(false), m_outgoing(false), m_ack(false), m_cseq(-1), m_flags(-1)
 {
     DDebug(DebugInfo,"SIPMessage::SIPMessage(%p,%d) [%p]\n------\n%s------",
 	buf,len,this,buf);
@@ -88,13 +88,13 @@ SIPMessage::SIPMessage(SIPParty* ep, const char* buf, int len)
     }
     if (len < 0)
 	len = ::strlen(buf);
-    m_valid = parse(buf,len);
+    m_valid = parse(buf,len,bodyLen);
 }
 
 SIPMessage::SIPMessage(const SIPMessage* message, int _code, const char* _reason)
     : code(_code), body(0),
       m_ep(0), m_valid(false),
-      m_answer(true), m_outgoing(true), m_ack(false), m_cseq(-1)
+      m_answer(true), m_outgoing(true), m_ack(false), m_cseq(-1), m_flags(-1)
 {
     DDebug(DebugAll,"SIPMessage::SIPMessage(%p,%d,'%s') [%p]",
 	message,_code,_reason,this);
@@ -103,6 +103,7 @@ SIPMessage::SIPMessage(const SIPMessage* message, int _code, const char* _reason
     reason = _reason;
     if (!(message && message->isValid()))
 	return;
+    m_flags = message->getFlags();
     m_ep = message->getParty();
     if (m_ep)
 	m_ep->ref();
@@ -121,11 +122,12 @@ SIPMessage::SIPMessage(const SIPMessage* message, int _code, const char* _reason
 SIPMessage::SIPMessage(const SIPMessage* original, const SIPMessage* answer)
     : method("ACK"), code(0),
       body(0), m_ep(0), m_valid(false),
-      m_answer(false), m_outgoing(true), m_ack(true), m_cseq(-1)
+      m_answer(false), m_outgoing(true), m_ack(true), m_cseq(-1), m_flags(-1)
 {
     DDebug(DebugAll,"SIPMessage::SIPMessage(%p,%p) [%p]",original,answer,this);
     if (!(original && original->isValid()))
 	return;
+    m_flags = original->getFlags();
     m_ep = original->getParty();
     if (m_ep)
 	m_ep->ref();
@@ -136,13 +138,16 @@ SIPMessage::SIPMessage(const SIPMessage* original, const SIPMessage* answer)
     if (!hl) {
 	String tmp;
 	tmp << version << "/" << getParty()->getProtoName();
-	tmp << " " << getParty()->getLocalAddr() << ":" << getParty()->getLocalPort();
+	if (getParty()) {
+	    Lock lock(getParty()->mutex());
+	    tmp << " " << getParty()->getLocalAddr() << ":" << getParty()->getLocalPort();
+	}
 	hl = new MimeHeaderLine("Via",tmp);
 	header.append(hl);
     }
     if (answer && (answer->code == 200) && (original->method &= "INVITE")) {
 	String tmp("z9hG4bK");
-	tmp << (int)::random();
+	tmp << (int)Random::random();
 	hl->setParam("branch",tmp);
 	const MimeHeaderLine* co = answer->getHeader("Contact");
 	if (co) {
@@ -181,16 +186,21 @@ SIPMessage::~SIPMessage()
     setBody();
 }
 
-void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domain, const char* dlgTag)
+void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domain, const char* dlgTag, int flags)
 {
-    DDebug(engine,DebugAll,"SIPMessage::complete(%p,'%s','%s','%s')%s%s%s [%p]",
-	engine,user,domain,dlgTag,
+    DDebug(engine,DebugAll,"SIPMessage::complete(%p,'%s','%s','%s',%d)%s%s%s [%p]",
+	engine,user,domain,dlgTag,flags,
 	isACK() ? " ACK" : "",
 	isOutgoing() ? " OUT" : "",
 	isAnswer() ? " ANS" : "",
 	this);
     if (!engine)
 	return;
+    if (-1 == flags)
+	flags = m_flags;
+    if (-1 == flags)
+	flags = engine->flags();
+    m_flags = flags;
 
     // don't complete incoming messages
     if (!isOutgoing())
@@ -203,6 +213,9 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 	    return;
 	}
     }
+    String partyLAddr;
+    int partyLPort = 0;
+    getParty()->getAddr(partyLAddr,partyLPort,true);
 
     // only set the dialog tag on ACK
     if (isACK()) {
@@ -213,27 +226,36 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
     }
 
     if (!domain)
-	domain = getParty()->getLocalAddr();
+	domain = partyLAddr;
 
     MimeHeaderLine* hl = const_cast<MimeHeaderLine*>(getHeader("Via"));
     if (!hl) {
 	String tmp;
 	tmp << version << "/" << getParty()->getProtoName();
-	tmp << " " << getParty()->getLocalAddr() << ":" << getParty()->getLocalPort();
+	tmp << " " << partyLAddr << ":" << partyLPort;
 	hl = new MimeHeaderLine("Via",tmp);
-	if (!(isAnswer() || isACK()))
+	if (isReliable() && 0 == (flags & NoConnReuse))
+	    hl->setParam("alias");
+	if (!((flags & (NotReqRport|RportAfterBranch)) || isAnswer() || isACK()))
 	    hl->setParam("rport");
 	header.append(hl);
     }
     if (!(isAnswer() || hl->getParam("branch"))) {
 	String tmp("z9hG4bK");
-	tmp << (int)::random();
+	tmp << (unsigned int)Random::random();
 	hl->setParam("branch",tmp);
     }
     if (isAnswer()) {
-	hl->setParam("received",getParty()->getPartyAddr());
-	hl->setParam("rport",String(getParty()->getPartyPort()));
+	if (!(flags & NotSetReceived)) {
+	    Lock lock(getParty()->mutex());
+	    hl->setParam("received",getParty()->getPartyAddr());
+	}
+	const String* rport = hl->getParam("rport");
+	if (rport && rport->null() && !(flags & NotSetRport))
+	    const_cast<String&>(*rport) = getParty()->getPartyPort();
     }
+    else if ((flags & RportAfterBranch) && !((flags & NotReqRport) || isACK() || hl->getParam("rport")))
+	hl->setParam("rport");
 
     if (!isAnswer()) {
 	hl = const_cast<MimeHeaderLine*>(getHeader("From"));
@@ -246,7 +268,7 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 	    header.append(hl);
 	}
 	if (!hl->getParam("tag"))
-	    hl->setParam("tag",String((int)::random()));
+	    hl->setParam("tag",String((unsigned int)Random::random()));
     }
 
     hl = const_cast<MimeHeaderLine*>(getHeader("To"));
@@ -261,7 +283,7 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 
     if (!(isAnswer() || getHeader("Call-ID"))) {
 	String tmp;
-	tmp << (int)::random() << "@" << domain;
+	tmp << (unsigned int)Random::random() << "@" << domain;
 	addHeader("Call-ID",tmp);
     }
 
@@ -273,7 +295,7 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
     }
 
     const char* info = isAnswer() ? "Server" : "User-Agent";
-    if (!(getHeader(info) || engine->getUserAgent().null()))
+    if (!((flags & NotAddAgent) || getHeader(info) || engine->getUserAgent().null()))
 	addHeader(info,engine->getUserAgent());
 
     // keep 100 answers short - they are hop to hop anyway
@@ -285,7 +307,7 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 	addHeader("Max-Forwards",tmp);
     }
 
-    if ((method == "INVITE") && !getHeader("Contact")) {
+    if ((method == YSTRING("INVITE")) && !getHeader("Contact")) {
 	// automatically add a contact field to (re)INVITE and its answers
 	String tmp(user);
 	if (!tmp) {
@@ -296,13 +318,13 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 	}
 	if (tmp)
 	    tmp = tmp.uriEscape('@',"+?&") + "@";
-	tmp = "<sip:" + tmp; 
-	tmp << getParty()->getLocalAddr() << ":";
-	tmp << getParty()->getLocalPort() << ">";
+	tmp = "<sip:" + tmp;
+	tmp << partyLAddr << ":";
+	tmp << partyLPort << ">";
 	addHeader("Contact",tmp);
     }
 
-    if (!getHeader("Allow"))
+    if (!((flags & NotAddAllow) || getHeader("Allow")))
 	addHeader("Allow",engine->getAllowed());
 }
 
@@ -357,7 +379,7 @@ bool SIPMessage::parseFirst(String& line)
 	    version = line.matchString(3).toUpper();
 	    DDebug(DebugAll,"got request method='%s' uri='%s' version='%s'",
 		method.c_str(),uri.c_str(),version.c_str());
-	    if (method == "ACK")
+	    if (method == YSTRING("ACK"))
 		m_ack = true;
 	}
 	else {
@@ -368,7 +390,7 @@ bool SIPMessage::parseFirst(String& line)
     return true;
 }
 
-bool SIPMessage::parse(const char* buf, int len)
+bool SIPMessage::parse(const char* buf, int len, unsigned int* bodyLen)
 {
     DDebug(DebugAll,"SIPMessage::parse(%p,%d) [%p]",buf,len,this);
     String* line = 0;
@@ -430,14 +452,42 @@ bool SIPMessage::parse(const char* buf, int len)
 	}
 	line->destruct();
     }
-    if (clen >= 0) {
-	if (clen > len)
-	    Debug("SIPMessage",DebugMild,"Content length is %d but only %d in buffer",clen,len);
-	else if (clen < len) {
-	    DDebug("SIPMessage",DebugInfo,"Got %d garbage bytes after content",len - clen);
-	    len = clen;
+    if (!bodyLen) {
+	if (clen >= 0) {
+	    if (clen > len)
+		Debug("SIPMessage",DebugMild,"Content length is %d but only %d in buffer",clen,len);
+	    else if (clen < len) {
+		DDebug("SIPMessage",DebugInfo,"Got %d garbage bytes after content",len - clen);
+		len = clen;
+	    }
 	}
+	buildBody(buf,len);
     }
+    else
+	*bodyLen = (clen >= 0) ? clen : 0;
+    DDebug(DebugAll,"SIPMessage::parse %d header lines, body %p",
+	header.count(),body);
+    return true;
+}
+
+SIPMessage* SIPMessage::fromParsing(SIPParty* ep, const char* buf, int len, unsigned int* bodyLen)
+{
+    SIPMessage* msg = new SIPMessage(ep,buf,len,bodyLen);
+    if (msg->isValid())
+	return msg;
+    DDebug("SIPMessage",DebugInfo,"Invalid message");
+    msg->destruct();
+    return 0;
+}
+
+// Build message's body. Reset it before
+void SIPMessage::buildBody(const char* buf, int len)
+{
+    TelEngine::destruct(body);
+    if (!buf)
+	return;
+    if (len < 0)
+	len = ::strlen(buf);
     const MimeHeaderLine* cType = getHeader("Content-Type");
     if (cType)
 	body = MimeBody::build(buf,len,*cType);
@@ -455,19 +505,8 @@ bool SIPMessage::parse(const char* buf, int len)
 		body->appendHdr(line);
 	}
     }
-    DDebug(DebugAll,"SIPMessage::parse %d header lines, body %p",
+    DDebug(DebugAll,"SIPMessage::buildBody %d header lines, body %p",
 	header.count(),body);
-    return true;
-}
-
-SIPMessage* SIPMessage::fromParsing(SIPParty* ep, const char* buf, int len)
-{
-    SIPMessage* msg = new SIPMessage(ep,buf,len);
-    if (msg->isValid())
-	return msg;
-    DDebug("SIPMessage",DebugInfo,"Invalid message");
-    msg->destruct();
-    return 0;
 }
 
 const MimeHeaderLine* SIPMessage::getHeader(const char* name) const
@@ -525,21 +564,21 @@ int SIPMessage::countHeaders(const char* name) const
     return res;
 }
 
-const NamedString* SIPMessage::getParam(const char* name, const char* param) const
+const NamedString* SIPMessage::getParam(const char* name, const char* param, bool last) const
 {
-    const MimeHeaderLine* hl = getHeader(name);
+    const MimeHeaderLine* hl = last ? getLastHeader(name) : getHeader(name);
     return hl ? hl->getParam(param) : 0;
 }
 
-const String& SIPMessage::getHeaderValue(const char* name) const
+const String& SIPMessage::getHeaderValue(const char* name, bool last) const
 {
-    const MimeHeaderLine* hl = getHeader(name);
+    const MimeHeaderLine* hl = last ? getLastHeader(name) : getHeader(name);
     return hl ? *static_cast<const String*>(hl) : String::empty();
 }
 
-const String& SIPMessage::getParamValue(const char* name, const char* param) const
+const String& SIPMessage::getParamValue(const char* name, const char* param, bool last) const
 {
-    const NamedString* ns = getParam(name,param);
+    const NamedString* ns = getParam(name,param,last);
     return ns ? *static_cast<const String*>(ns) : String::empty();
 }
 
@@ -600,15 +639,16 @@ void SIPMessage::setParty(SIPParty* ep)
 {
     if (ep == m_ep)
 	return;
-    if (m_ep)
-	m_ep->deref();
+    if (ep && !ep->ref())
+	ep = 0;
+    XDebug(DebugAll,"SIPMessage::setParty(%p) current=%p [%p]",ep,m_ep,this);
+    SIPParty* tmp = m_ep;
     m_ep = ep;
-    if (m_ep)
-	m_ep->ref();
+    TelEngine::destruct(tmp);
 }
 
 MimeAuthLine* SIPMessage::buildAuth(const String& username, const String& password,
-    const String& meth, const String& uri, bool proxy) const
+    const String& meth, const String& uri, bool proxy, SIPEngine* engine) const
 {
     const char* hdr = proxy ? "Proxy-Authenticate" : "WWW-Authenticate";
     const ObjList* l = &header;
@@ -623,8 +663,23 @@ MimeAuthLine* SIPMessage::buildAuth(const String& username, const String& passwo
 	    MimeHeaderLine::delQuotes(realm);
 	    int par = uri.find(';');
 	    String msguri = uri.substr(0,par);
+	    NamedList qop(TelEngine::c_safe(t->getParam("qop")));
+	    if (qop) {
+		MimeHeaderLine::delQuotes(qop);
+		if (qop == YSTRING("auth")) {
+		    String nc("00000001");
+		    if (engine)
+			engine->ncGet(nc);
+		    qop.addParam("nc",nc);
+		    MD5 md5;
+		    md5 << String((unsigned int)Random::random()) << nc << String(Time::secNow());
+		    qop.addParam("cnonce",md5.hexDigest());
+		}
+		else
+		    continue;
+	    }
 	    String response;
-	    SIPEngine::buildAuth(username,realm,password,nonce,meth,msguri,response);
+	    SIPEngine::buildAuth(username,realm,password,nonce,meth,msguri,response,qop);
 	    MimeAuthLine* auth = new MimeAuthLine(proxy ? "Proxy-Authorization" : "Authorization","Digest");
 	    auth->setParam("username",MimeHeaderLine::quote(username));
 	    auth->setParam("realm",MimeHeaderLine::quote(realm));
@@ -633,21 +688,31 @@ MimeAuthLine* SIPMessage::buildAuth(const String& username, const String& passwo
 	    auth->setParam("response",MimeHeaderLine::quote(response));
 	    auth->setParam("algorithm","MD5");
 	    // copy opaque data as-is, only if present
-	    const NamedString* opaque = t->getParam("opaque");
+	    const NamedString* opaque = t->getParam(YSTRING("opaque"));
 	    if (opaque)
 		auth->setParam(opaque->name(),*opaque);
+	    if (qop) {
+		auth->setParam("qop",qop);
+		NamedIterator iter(qop);
+		for (const NamedString* ns = 0; 0 != (ns = iter.get());) {
+		    if (ns->name() == YSTRING("nc"))
+			auth->setParam(ns->name(),*ns);
+		    else
+			auth->setParam(ns->name(),MimeHeaderLine::quote(*ns));
+		}
+	    }
 	    return auth;
 	}
     }
     return 0;
 }
 
-MimeAuthLine* SIPMessage::buildAuth(const SIPMessage& original) const
+MimeAuthLine* SIPMessage::buildAuth(const SIPMessage& original, SIPEngine* engine) const
 {
     if (original.getAuthUsername().null())
 	return 0;
     return buildAuth(original.getAuthUsername(),original.getAuthPassword(),
-	original.method,original.uri,(code == 407));
+	original.method,original.uri,(code == 407),engine);
 }
 
 ObjList* SIPMessage::getRoutes() const
