@@ -59,12 +59,17 @@ class SIPEvent;
 class YSIP_API SIPParty : public RefObject
 {
 public:
-    SIPParty();
-    SIPParty(bool reliable);
+    SIPParty(Mutex* mutex = 0);
+    SIPParty(bool reliable, Mutex* mutex = 0);
     virtual ~SIPParty();
     virtual void transmit(SIPEvent* event) = 0;
     virtual const char* getProtoName() const = 0;
     virtual bool setParty(const URI& uri) = 0;
+    virtual void* getTransport() = 0;
+    void setAddr(const String& addr, int port, bool local);
+    void getAddr(String& addr, int& port, bool local);
+    inline Mutex* mutex()
+	{ return m_mutex; }
     inline const String& getLocalAddr() const
 	{ return m_local; }
     inline const String& getPartyAddr() const
@@ -76,8 +81,8 @@ public:
     inline bool isReliable() const
 	{ return m_reliable; }
 protected:
+    Mutex* m_mutex;
     bool m_reliable;
-    bool m_init;
     String m_local;
     String m_party;
     int m_localPort;
@@ -93,6 +98,20 @@ protected:
 class YSIP_API SIPMessage : public RefObject
 {
 public:
+    /**
+     * Various message flags
+     */
+    enum Flags {
+	Defaults          =      0,
+	NotReqRport       = 0x0001,
+	NotAddAllow       = 0x0002,
+	NotAddAgent       = 0x0004,
+	RportAfterBranch  = 0x0008,
+	NotSetRport       = 0x0010,
+	NotSetReceived    = 0x0020,
+	NoConnReuse       = 0x0040,      // Don't add 'alias' parameter to Via header (reliable only)
+    };
+
     /**
      * Copy constructor
      */
@@ -105,8 +124,14 @@ public:
 
     /**
      * Creates a new SIPMessage from parsing a text buffer.
+     * @param ep Party to set in message
+     * @param buf Buffer to parse
+     * @param len Optional buffer length
+     * @param bodyLen Pointer to body length to be set if the message was received
+     *  on a stream transport. If not 0 the buffer must contain the message
+     *  without its body
      */
-    SIPMessage(SIPParty* ep, const char* buf, int len = -1);
+    SIPMessage(SIPParty* ep, const char* buf, int len = -1, unsigned int* bodyLen = 0);
 
     /**
      * Creates a new SIPMessage as answer to another message.
@@ -125,14 +150,34 @@ public:
 
     /**
      * Construct a new SIP message by parsing a text buffer
+     * @param ep Party to set in message
+     * @param buf Buffer to parse
+     * @param len Optional buffer length
+     * @param bodyLen Pointer to body length to be set if the message was received
+     *  on a stream transport. If not 0 the buffer must contain the message
+     *  without its body
      * @return A pointer to a valid new message or NULL
      */
-    static SIPMessage* fromParsing(SIPParty* ep, const char* buf, int len = -1);
+    static SIPMessage* fromParsing(SIPParty* ep, const char* buf, int len = -1,
+	unsigned int* bodyLen = 0);
+
+    /**
+     * Build message's body. Reset it before.
+     * This method should be called after parsing a partial message (headers only)
+     * @param buf Buffer to parse
+     * @param len Optional buffer length
+     */
+    void buildBody(const char* buf, int len = -1);
 
     /**
      * Complete missing fields with defaults taken from a SIP engine
+     * @param engine Pointer to the SIP engine to use for extra parameters
+     * @param user Username to set in the From header instead of that in rURI
+     * @param domain Domain to use in From instead of the local IP address
+     * @param dlgTag Value of dialog tag parameter to set in To header
+     * @param flags Miscellaneous completion flags, -1 to take them from engine
      */
-    void complete(SIPEngine* engine, const char* user = 0, const char* domain = 0, const char* dlgTag = 0);
+    void complete(SIPEngine* engine, const char* user = 0, const char* domain = 0, const char* dlgTag = 0, int flags = -1);
 
     /**
      * Copy an entire header line (including all parameters) from another message
@@ -200,10 +245,18 @@ public:
 
     /**
      * Get the Command Sequence number from this message
+     * @return Number part of CSEQ in this message
      */
     inline int getCSeq() const
 	{ return m_cseq; }
 
+    /**
+     * Get the last flags used by this message
+     * @return Flags last used, ORed together
+     */
+    inline int getFlags() const
+	{ return m_flags; }
+
     /**
      * Find a header line by name
      * @param name Name of the header to locate
@@ -229,24 +282,27 @@ public:
      * Find a header parameter by name
      * @param name Name of the header to locate
      * @param param Name of the parameter to locate in the tag
+     * @param last Find the last header with that name instead of first
      * @return A pointer to the first matching header line or 0 if not found
      */
-    const NamedString* getParam(const char* name, const char* param) const;
+    const NamedString* getParam(const char* name, const char* param, bool last = false) const;
 
     /**
      * Get a string value (without parameters) from a header line
      * @param name Name of the header to locate
+     * @param last Find the last header with that name instead of first
      * @return The value hold in the header or an empty String
      */
-    const String& getHeaderValue(const char* name) const;
+    const String& getHeaderValue(const char* name, bool last = false) const;
 
     /**
      * Get a string value from a parameter in a header line
      * @param name Name of the header to locate
      * @param param Name of the parameter to locate in the tag
+     * @param last Find the last header with that name instead of first
      * @return The value hold in the parameter or an empty String
      */
-    const String& getParamValue(const char* name, const char* param) const;
+    const String& getParamValue(const char* name, const char* param, bool last = false) const;
 
     /**
      * Append a new header line constructed from name and content
@@ -282,17 +338,19 @@ public:
      * @param meth Method to include in the authorization digest
      * @param uri URI to include in the authorization digest
      * @param proxy Set to true to authenticate to a proxy, false to a server
+     * @param engine Optional engine processing this message
      * @return A new authorization line to be used in a new transaction
      */
     MimeAuthLine* buildAuth(const String& username, const String& password,
-	const String& meth, const String& uri, bool proxy = false) const;
+	const String& meth, const String& uri, bool proxy = false, SIPEngine* engine = 0) const;
 
     /**
      * Construct a new authorization line based on this answer and original message
      * @param original Origianl outgoing message
+     * @param engine Optional engine processing this message
      * @return A new authorization line to be used in a new transaction
      */
-    MimeAuthLine* buildAuth(const SIPMessage& original) const;
+    MimeAuthLine* buildAuth(const SIPMessage& original, SIPEngine* engine = 0) const;
 
     /**
      * Prepare the message for automatic client transaction authentication.
@@ -303,14 +361,14 @@ public:
 	{ m_authUser = username; m_authPass = password; }
 
     /**
-     * Retrive the username to be used for auto authentication
+     * Retrieve the username to be used for auto authentication
      * @return Username for auto authentication
      */
     inline const String& getAuthUsername() const
 	{ return m_authUser; }
 
     /**
-     * Retrive the password to be used for auto authentication
+     * Retrieve the password to be used for auto authentication
      * @return Password for auto authentication
      */
     inline const String& getAuthPassword() const
@@ -380,7 +438,7 @@ public:
     MimeBody* body;
 
 protected:
-    bool parse(const char* buf, int len);
+    bool parse(const char* buf, int len, unsigned int* bodyLen);
     bool parseFirst(String& line);
     SIPParty* m_ep;
     bool m_valid;
@@ -388,6 +446,7 @@ protected:
     bool m_outgoing;
     bool m_ack;
     int m_cseq;
+    int m_flags;
     mutable String m_string;
     mutable DataBlock m_data;
     String m_authUser;
@@ -727,6 +786,12 @@ public:
     inline void setTransmit()
 	{ m_transmit = true; }
 
+    /**
+     * Change transaction status to Cleared
+     * This method is not thread safe
+     */
+    inline void setCleared()
+	{ changeState(Cleared); }
 
     /**
      * Send back an authentication required response
@@ -774,6 +839,12 @@ public:
      */
     virtual SIPEvent* getEvent(bool pendingOnly = false);
 
+    /**
+     * Checks if a response message can be sent
+     * @return True if the transaction can send a response message
+     */
+    bool setResponse() const;
+
     /**
      * Creates and transmits a final response message
      * @param code Response code to send
@@ -788,7 +859,7 @@ public:
     void setResponse(SIPMessage* message);
 
     /**
-     * Retrive the latest response code
+     * Retrieve the latest response code
      * @return Code of most recent response, zero if none is known
      */
     inline int getResponseCode() const
@@ -1028,22 +1099,26 @@ public:
      * @param uri URI of the SIP message that is being authenticated
      * @param response Response computed by the authenticated entity
      * @param message Message that is to be authenticated
+     * @param authLine Extra credentials line to validate
      * @param userData Pointer to an optional object passed from @ref authUser
      * @return True if valid user/password, false if verification failed
      */
-    virtual bool checkUser(const String& username, const String& realm, const String& nonce,
+    virtual bool checkUser(String& username, const String& realm, const String& nonce,
 	const String& method, const String& uri, const String& response,
-	const SIPMessage* message, GenObject* userData);
+	const SIPMessage* message, const MimeHeaderLine* authLine, GenObject* userData);
 
     /**
      * Authenticate a message by other means than user credentials. By default
      *  it calls @ref checkUser with empty user credential fields
      * @param noUser No plausible user credentials were detected so far
+     * @param username User account name
      * @param message Message that is to be authenticated
+     * @param authLine Extra credentials line to validate
      * @param userData Pointer to an optional object passed from @ref authUser
      * @return True if message is authenticated, false if verification failed
      */
-    virtual bool checkAuth(bool noUser, const SIPMessage* message, GenObject* userData);
+    virtual bool checkAuth(bool noUser, String& username, const SIPMessage* message,
+	const MimeHeaderLine* authLine, GenObject* userData);
 
     /**
      * Detect the proper credentials for any user in the engine
@@ -1121,6 +1196,22 @@ public:
      */
     u_int64_t getTimer(char which, bool reliable = false) const;
 
+    /**
+     * Get the number of times to send a SIP request.
+     * This value applies only when retransmission is required
+     * @return The number of times to send a SIP request
+     */
+    inline unsigned int getReqTransCount() const
+	{ return m_reqTransCount; }
+
+    /**
+     * Get the number of times to send a response to a SIP request.
+     * This value applies only when retransmission is required
+     * @return The number of times to send a response to a SIP request
+     */
+    inline unsigned int getRspTransCount() const
+	{ return m_rspTransCount; }
+
     /**
      * Get the default value of the Max-Forwards header for this engine
      * @return The maximum number of hops the request is allowed to pass
@@ -1154,6 +1245,13 @@ public:
     inline void lazyTrying(bool lazy100)
 	{ m_lazyTrying = lazy100; }
 
+    /**
+     * Retrieve various flags for this engine
+     * @return Value of flags ORed together
+     */
+    inline int flags() const
+	{ return m_flags; }
+
     /**
      * Get an authentication nonce
      * @param nonce String reference to fill with the current nonce
@@ -1167,6 +1265,12 @@ public:
      */
     long nonceAge(const String& nonce);
 
+    /**
+     * Get a nonce count
+     * @param nc String reference to fill with new nonce count
+     */
+    void ncGet(String& nc);
+
     /**
      * Build an authentication response
      * @param username User account name
@@ -1176,9 +1280,11 @@ public:
      * @param method Method of the SIP message that is being authenticated
      * @param uri URI of the SIP message that is being authenticated
      * @param response String to store the computed response
+     * @param qop Optional quality of protection type (set in list name) and parameters
      */
     static void buildAuth(const String& username, const String& realm, const String& passwd,
-	const String& nonce, const String& method, const String& uri, String& response);
+	const String& nonce, const String& method, const String& uri, String& response,
+	const NamedList& qop = NamedList::empty());
 
     /**
      * Build an authentication response from already hashed components
@@ -1239,11 +1345,15 @@ protected:
 
     u_int64_t m_t1;
     u_int64_t m_t4;
+    int m_reqTransCount;
+    int m_rspTransCount;
     unsigned int m_maxForwards;
     int m_cseq;
+    int m_flags;
     bool m_lazyTrying;
     String m_userAgent;
     String m_allowed;
+    u_int32_t m_nc;
     String m_nonce;
     String m_nonce_secret;
     u_int32_t m_nonce_time;
