@@ -34,6 +34,7 @@ enum {
     CdrStart,
     CdrCall,
     CdrRoute,
+    CdrProgress,
     CdrRinging,
     CdrAnswer,
     CdrUpdate,
@@ -59,13 +60,20 @@ public:
     virtual bool received(Message &msg);
 };
 
+class CommandHandler : public MessageHandler
+{
+public:
+    CommandHandler() : MessageHandler("engine.command") { }
+    virtual bool received(Message &msg);
+};
+
 // Collects CDR information and emits the call.cdr messages when needed
 class CdrBuilder : public NamedList
 {
 public:
     CdrBuilder(const char *name);
     virtual ~CdrBuilder();
-    void update(int type, u_int64_t val);
+    void update(int type, u_int64_t val, const char* status = 0);
     bool update(const Message& msg, int type, u_int64_t val);
     void emit(const char *operation = 0);
     String getStatus() const;
@@ -238,14 +246,15 @@ void CdrBuilder::emit(const char *operation)
 	t_call = m_start;
     if (!t_call)
 	t_call = m_start = t_hangup;
-    if (!t_ringing)
-	t_ringing = t_call;
+
     if (!t_answer)
 	t_answer = t_hangup;
-
-    if (t_answer > t_hangup)
+    else if (t_answer > t_hangup)
 	t_answer = t_hangup;
-    if (t_ringing > t_answer)
+
+    if (!t_ringing)
+	t_ringing = t_answer;
+    else if (t_ringing > t_answer)
 	t_ringing = t_answer;
 
     if (!operation)
@@ -255,7 +264,7 @@ void CdrBuilder::emit(const char *operation)
     DDebug("cdrbuild",DebugAll,"Emit '%s' for '%s' status '%s'",
 	operation,c_str(),m_status.c_str());
     char buf[64];
-    Message *m = new Message("call.cdr");
+    Message *m = new Message("call.cdr",0,true);
     m->addParam("time",printTime(buf,m_start));
     m->addParam("chan",c_str());
     m->addParam("cdrid",m_cdrId);
@@ -268,10 +277,10 @@ void CdrBuilder::emit(const char *operation)
     m->addParam("status",m_status);
     if (!getValue("external")) {
 	const char* ext = 0;
-	if (m_dir == "incoming")
-	    ext = getValue("caller");
-	else if (m_dir == "outgoing")
-	    ext = getValue("called");
+	if (m_dir == YSTRING("incoming"))
+	    ext = getValue(YSTRING("caller"));
+	else if (m_dir == YSTRING("outgoing"))
+	    ext = getValue(YSTRING("called"));
 	if (ext)
 	    m->setParam("external",ext);
     }
@@ -289,7 +298,8 @@ void CdrBuilder::emit(const char *operation)
 String CdrBuilder::getStatus() const
 {
     String s(m_status);
-    s << "|" << getValue("caller") << "|" << getValue("called");
+    s << "|" << getValue(YSTRING("caller")) << "|" << getValue(YSTRING("called")) <<
+	"|" << getValue(YSTRING("billid"));
     unsigned int sec = 0;
     if (m_start)
 	sec = (Time::now() - m_start + 500000) / 1000000;
@@ -297,7 +307,7 @@ String CdrBuilder::getStatus() const
     return s;
 }
 
-void CdrBuilder::update(int type, u_int64_t val)
+void CdrBuilder::update(int type, u_int64_t val, const char* status)
 {
     switch (type) {
 	case CdrStart:
@@ -307,6 +317,7 @@ void CdrBuilder::update(int type, u_int64_t val)
 	case CdrCall:
 	    m_call = val;
 	    break;
+	case CdrProgress:
 	case CdrRinging:
 	    if (!m_ringing)
 		m_ringing = val;
@@ -319,6 +330,8 @@ void CdrBuilder::update(int type, u_int64_t val)
 	    m_hangup = val;
 	    break;
     }
+    if (!TelEngine::null(status))
+	m_status = status;
 }
 
 bool CdrBuilder::update(const Message& msg, int type, u_int64_t val)
@@ -331,8 +344,8 @@ bool CdrBuilder::update(const Message& msg, int type, u_int64_t val)
 	    clear();
 	else {
 	    // set a reason if none was set or one is explicitely provided
-	    const char* reason = msg.getValue("reason");
-	    if (!(reason || getValue("reason")))
+	    const char* reason = msg.getValue(YSTRING("reason"));
+	    if (!(reason || getValue(YSTRING("reason"))))
 		reason = "CDR dropped";
 	    if (reason)
 		setParam("reason",reason);
@@ -342,7 +355,7 @@ bool CdrBuilder::update(const Message& msg, int type, u_int64_t val)
     }
     // cdrwrite must be consistent over all emitted messages so we read it once
     if (m_first)
-	m_write = msg.getBoolValue("cdrwrite",true);
+	m_write = msg.getBoolValue(YSTRING("cdrwrite"),true);
     unsigned int n = msg.length();
     for (unsigned int i = 0; i < n; i++) {
 	const NamedString* s = msg.getParam(i);
@@ -350,12 +363,12 @@ bool CdrBuilder::update(const Message& msg, int type, u_int64_t val)
 	    continue;
 	if (s->null())
 	    continue;
-	if (s->name() == "status") {
+	if (s->name() == YSTRING("status")) {
 	    m_status = *s;
-	    if ((m_status == "incoming") || (m_status == "outgoing"))
+	    if ((m_status == YSTRING("incoming")) || (m_status == YSTRING("outgoing")))
 		m_dir = m_status;
 	}
-	else if (s->name() == "direction")
+	else if (s->name() == YSTRING("direction"))
 	    m_dir = *s;
 	else {
 	    // search the parameter
@@ -363,7 +376,7 @@ bool CdrBuilder::update(const Message& msg, int type, u_int64_t val)
 	    if (!p)
 		continue;
 	    bool overwrite = p->overwrite();
-	    NamedString* str = getParam(s->name());
+	    String* str = getParam(s->name());
 	    // parameter is not yet stored - store a copy
 	    if (!str)
 		addParam(s->name(),*s);
@@ -390,6 +403,7 @@ CdrBuilder* CdrBuilder::find(String &id)
     return static_cast<CdrBuilder*>(s_cdrs[id]);
 }
 
+
 bool CdrHandler::received(Message &msg)
 {
     Lock lock(s_mutex);
@@ -400,28 +414,23 @@ bool CdrHandler::received(Message &msg)
 	s_cdrs.clear();
 	return false;
     }
+    if ((m_type == CdrProgress) && !msg.getBoolValue(YSTRING("earlymedia"),false))
+	return false;
     bool track = true;
     if (m_type == CdrUpdate) {
-	const String* oper = msg.getParam("operation");
-	if (oper && (*oper != "cdrbuild"))
+	const String* oper = msg.getParam(YSTRING("operation"));
+	if (oper && (*oper != YSTRING("cdrbuild")))
 	    track = false;
     }
-    if (!msg.getBoolValue("cdrtrack",track))
+    if (!msg.getBoolValue(YSTRING("cdrtrack"),track))
 	return false;
-    String id(msg.getValue("id"));
+    String id(msg.getValue(YSTRING("id")));
     if (m_type == CdrDrop) {
 	if (!id.startSkip("cdrbuild/",false))
 	    return false;
     }
-    if (id.null()) {
-	id = msg.getValue("module");
-	id += "/";
-	id += msg.getValue("span");
-	id += "/";
-	id += msg.getValue("channel");
-	if (id == "//")
-	    return false;
-    }
+    if (id.null())
+	return false;
     bool rval = false;
     int type = m_type;
     int level = DebugInfo;
@@ -445,7 +454,7 @@ bool CdrHandler::received(Message &msg)
 			}
 		    }
 		}
-		if ((type != CdrHangup) && !msg.getBoolValue("cdrcreate",true))
+		if ((type != CdrHangup) && !msg.getBoolValue(YSTRING("cdrcreate"),true))
 		    break;
 		b = new CdrBuilder(id);
 		s_cdrs.append(b);
@@ -465,28 +474,29 @@ bool CdrHandler::received(Message &msg)
     else
 	Debug("cdrbuild",level,"Got message '%s' for untracked id '%s'",
 	    msg.c_str(),id.c_str());
-    if ((type == CdrRinging) || (type == CdrAnswer)) {
-	id = msg.getValue("peerid");
+    if ((type == CdrRinging) || (type == CdrProgress) || (type == CdrAnswer)) {
+	id = msg.getValue(YSTRING("peerid"));
 	if (id.null())
-	    id = msg.getValue("targetid");
+	    id = msg.getValue(YSTRING("targetid"));
 	if (id && (b = CdrBuilder::find(id))) {
-	    b->update(type,msg.msgTime().usec());
+	    b->update(type,msg.msgTime().usec(),msg.getValue("status"));
 	    b->emit();
 	}
     }
     return rval;
 };
-		    
+
+
 bool StatusHandler::received(Message &msg)
 {
-    const char *sel = msg.getValue("module");
-    if (sel && ::strcmp(sel,"cdrbuild"))
+    const String* sel = msg.getParam(YSTRING("module"));
+    if (!(TelEngine::null(sel) || (*sel == YSTRING("cdrbuild"))))
 	return false;
-    String st("name=cdrbuild,type=cdr,format=Status|Caller|Called|Duration");
+    String st("name=cdrbuild,type=cdr,format=Status|Caller|Called|BillId|Duration");
     s_mutex.lock();
     expireHungup();
     st << ";cdrs=" << s_cdrs.count() << ",hungup=" << s_hungup.count();
-    if (msg.getBoolValue("details",true)) {
+    if (msg.getBoolValue(YSTRING("details"),true)) {
 	st << ";";
 	ObjList *l = &s_cdrs;
 	bool first = true;
@@ -505,7 +515,20 @@ bool StatusHandler::received(Message &msg)
     msg.retValue() << st << "\r\n";
     return false;
 }
-			
+
+
+bool CommandHandler::received(Message &msg)
+{
+    static const String name("cdrbuild");
+    const String* partial = msg.getParam(YSTRING("partline"));
+    if (!partial || *partial != YSTRING("status"))
+	return false;
+    partial = msg.getParam(YSTRING("partword"));
+    if (TelEngine::null(partial) || name.startsWith(*partial))
+	msg.retValue().append(name,"\t");
+    return false;
+}
+
 
 class CdrBuildPlugin : public Plugin
 {
@@ -518,7 +541,8 @@ private:
 };
 
 CdrBuildPlugin::CdrBuildPlugin()
-    : m_first(true)
+    : Plugin("cdrbuild"),
+      m_first(true)
 {
     Output("Loaded module CdrBuild");
 }
@@ -573,6 +597,7 @@ void CdrBuildPlugin::initialize()
 	Engine::install(new CdrHandler("chan.startup",CdrStart));
 	Engine::install(new CdrHandler("call.route",CdrRoute));
 	Engine::install(new CdrHandler("call.execute",CdrCall));
+	Engine::install(new CdrHandler("call.progress",CdrProgress));
 	Engine::install(new CdrHandler("call.ringing",CdrRinging));
 	Engine::install(new CdrHandler("call.answered",CdrAnswer));
 	Engine::install(new CdrHandler("call.update",CdrUpdate));
@@ -580,6 +605,7 @@ void CdrBuildPlugin::initialize()
 	Engine::install(new CdrHandler("call.drop",CdrDrop));
 	Engine::install(new CdrHandler("engine.halt",EngHalt,150));
 	Engine::install(new StatusHandler);
+	Engine::install(new CommandHandler);
     }
 }
 
