@@ -69,6 +69,7 @@ static const char* s_cmds[] = {
     "start",
     "stop",
     "restart",
+    "execute",
     0
 };
 
@@ -169,7 +170,7 @@ private:
     bool m_accepted;
 };
 
-class MsgHolder : public GenObject
+class MsgHolder : public GenObject, public Semaphore
 {
 public:
     MsgHolder(Message &msg);
@@ -362,6 +363,17 @@ static bool runProgram(const char *script, const char *args)
     return true;
 }
 
+static void adjustPath(String& script)
+{
+    if (script.null() || script.startsWith(Engine::pathSeparator()))
+	return;
+    String tmp = Engine::sharedPath();
+    tmp << Engine::pathSeparator() << "scripts";
+    tmp = s_cfg.getValue("general","scripts_dir",tmp);
+    if (!tmp.endsWith(Engine::pathSeparator()))
+	tmp += Engine::pathSeparator();
+    script = tmp + script;
+}
 
 ExtModSource::ExtModSource(Stream* str, ExtModChan* chan)
     : m_str(str), m_brate(16000), m_total(0), m_chan(chan)
@@ -460,7 +472,8 @@ ExtModChan* ExtModChan::build(const char* file, const char* args, int type)
 
 ExtModChan::ExtModChan(const char* file, const char* args, int type)
     : CallEndpoint("ExtModule"),
-      m_recv(0), m_waitRet(0), m_type(type), m_disconn(false), m_waiting(false)
+      m_recv(0), m_waitRet(0), m_type(type),
+      m_running(false), m_disconn(false), m_waiting(false)
 {
     Debug(DebugAll,"ExtModChan::ExtModChan(%d) [%p]",type,this);
     File* reader = 0;
@@ -500,7 +513,9 @@ ExtModChan::ExtModChan(const char* file, const char* args, int type)
 }
 
 ExtModChan::ExtModChan(ExtModReceiver* recv)
-    : CallEndpoint("ExtModule"), m_recv(recv), m_type(DataNone), m_disconn(false)
+    : CallEndpoint("ExtModule"),
+      m_recv(recv), m_waitRet(0), m_type(DataNone),
+      m_running(false), m_disconn(false), m_waiting(false)
 {
     Debug(DebugAll,"ExtModChan::ExtModChan(%p) [%p]",recv,this);
     s_mutex.lock();
@@ -513,11 +528,13 @@ ExtModChan::~ExtModChan()
     Debugger debug(DebugAll,"ExtModChan::~ExtModChan()"," [%p]",this);
     s_mutex.lock();
     s_chans.remove(this,false);
+    ExtModReceiver* recv = m_recv;
+    m_recv = 0;
     s_mutex.unlock();
     setSource();
     setConsumer();
-    if (m_recv)
-	m_recv->die(false);
+    if (recv)
+	recv->die(false);
 }
 
 void ExtModChan::disconnected(bool final, const char *reason)
@@ -534,8 +551,9 @@ void ExtModChan::disconnected(bool final, const char *reason)
 	    m->addParam("address",m_recv->scriptFile());
 	if (reason)
 	    m->addParam("reason",reason);
-	if (getPeer())
-	    m->addParam("peerid",getPeer()->id());
+	String peerId;
+	if (getPeerId(peerId) && peerId)
+	    m->addParam("peerid",peerId);
 	Engine::enqueue(m);
     }
 }
@@ -546,7 +564,7 @@ MsgHolder::MsgHolder(Message &msg)
 {
     // the address of this object should be unique
     char buf[64];
-    ::sprintf(buf,"%p.%ld",this,random());
+    ::sprintf(buf,"%p.%ld",this,Random::random());
     m_id = buf;
 }
 
@@ -718,7 +736,7 @@ ExtModReceiver::ExtModReceiver(const char* name, Stream* io, ExtModChan* chan, i
 }
 
 ExtModReceiver::~ExtModReceiver()
-{   
+{
     Debug(DebugAll,"ExtModReceiver::~ExtModReceiver() pid=%d [%p]",m_pid,this);
     Lock lock(this);
     // One destruction is plenty enough
@@ -844,6 +862,8 @@ void ExtModReceiver::die(bool clearChan)
 	chan->setRecv(0);
     mylock.drop();
 
+    if (m_role == RoleGlobal)
+	Output("Unloading external module '%s' '%s'",m_script.c_str(),m_args.safe());
     // Give the external script a chance to die gracefully
     closeOut();
     if (m_pid > 1) {
@@ -910,7 +930,7 @@ bool ExtModReceiver::received(Message &msg, int id)
     //  other thread - unfortunately this does not work with all mutexes
     // sorry, Maciek - have to do it work in Windows too :-(
     while (ok) {
-	Thread::yield();
+	h.lock(Thread::idleUsec());
 	lock();
 	ok = (m_waiting.find(&h) != 0);
 	if (ok && tout && (Time::now() > tout)) {
@@ -940,14 +960,7 @@ bool ExtModReceiver::create(const char *script, const char *args)
     HANDLE ext2yate[2];
     HANDLE yate2ext[2];
     int x;
-    if (script[0] != '/') {
-	tmp = Engine::sharedPath();
-	tmp << Engine::pathSeparator() << "scripts";
-	tmp = s_cfg.getValue("general","scripts_dir",tmp);
-	if (!tmp.endsWith(Engine::pathSeparator()))
-	    tmp += Engine::pathSeparator();
-	tmp += script;
-    }
+    adjustPath(tmp);
     script = tmp.c_str();
     if (::pipe(ext2yate)) {
 	Debug(DebugWarn, "Unable to create ext->yate pipe: %s",strerror(errno));
@@ -1000,7 +1013,10 @@ bool ExtModReceiver::create(const char *script, const char *args)
 	// Shit happened. Die as quick and brutal as possible
 	::_exit(1);
     }
-    Debug(DebugInfo,"Launched External Script %s", script);
+    if (m_role == RoleGlobal)
+	Output("Loading external module '%s' '%s'",m_script.c_str(),args);
+    else
+	Debug(DebugInfo,"Launched External Script '%s' '%s'",script,args);
     m_in = new File(ext2yate[0]);
     m_out = new File(yate2ext[1]);
 
@@ -1071,8 +1087,8 @@ void ExtModReceiver::run()
 	    closeIn();
 	    flush();
 	    if (invalid)
-		Debug("ExtModule",DebugWarn,"Never got anything valid from '%s' '%s'",
-		m_script.c_str(),m_args.safe());
+		Debug("ExtModule",DebugWarn,"Never got anything valid from terminated '%s' '%s'",
+		    m_script.c_str(),m_args.safe());
 	    if (m_chan && m_chan->running())
 		Thread::sleep(1);
 	    break;
@@ -1246,6 +1262,7 @@ bool ExtModReceiver::processLine(const char* line)
 		    m_chan->waitMsg(0);
 		    m_chan->waiting(true);
 		}
+		msg->unlock();
 		p->remove(false);
 		return false;
 	    }
@@ -1259,7 +1276,7 @@ bool ExtModReceiver::processLine(const char* line)
 	id >> prio >> ":";
 	String fname;
 	String fvalue;
-	Regexp r("^\\([^:]*\\):\\([^:]*\\):\\?\\(.*\\)");
+	static const Regexp r("^\\([^:]*\\):\\([^:]*\\):\\?\\(.*\\)");
 	if (id.matches(r)) {
 	    // a filter is specified
 	    fname = id.matchString(2);
@@ -1393,6 +1410,26 @@ bool ExtModReceiver::processLine(const char* line)
 		ok = val.null() && param;
 		val = param;
 	    }
+	    else if (id.startsWith("config.")) {
+		ok = val.null();
+		// keep the index in substr in sync with length of "config."
+		val = id.substr(7);
+		int sep = val.find('.');
+		if (sep > 0) {
+		    const NamedString* key = Engine::config().getKey(val.substr(0,sep).trimBlanks(),
+			val.substr(sep+1).trimBlanks());
+		    if (key)
+			val = *key;
+		    else {
+			val.clear();
+			ok = false;
+		    }
+		}
+		else {
+		    ok = (Engine::config().getSection(val) != 0);
+		    val.clear();
+		}
+	    }
 	    else if (id == "runid") {
 		ok = val.null();
 		val = Engine::runId();
@@ -1500,7 +1537,7 @@ bool ExtModHandler::received(Message& msg)
     String dest(msg.getValue("callto"));
     if (dest.null())
 	return false;
-    Regexp r("^external/\\([^/]*\\)/\\([^ ]*\\)\\(.*\\)$");
+    static const Regexp r("^external/\\([^/]*\\)/\\([^ ]*\\)\\(.*\\)$");
     if (!dest.matches(r))
 	return false;
     CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userData());
@@ -1556,7 +1593,7 @@ bool ExtModHandler::received(Message& msg)
     recv->unuse();
     if (ch) {
 	em->waitMsg(0);
-	ch->connect(em);
+	ch->connect(em,msg.getValue("reason"));
 	em->waiting(false);
     }
     em->deref();
@@ -1610,6 +1647,20 @@ bool ExtModCommand::received(Message& msg)
 	if (!restart)
 	    return true;
     }
+    else if (line.startSkip("execute")) {
+	if (line.null())
+	    return false;
+	blank = line.find(' ');
+	String exe = line.substr(0,blank);
+	adjustPath(exe);
+	if (blank >= 0)
+	    line = line.substr(blank+1);
+	else
+	    line.clear();
+	bool ok = runProgram(exe,line);
+	msg.retValue() = ok ? "External exec attempt\r\n" : "External exec failed\r\n";
+	return true;
+    }
     ExtModReceiver *r = ExtModReceiver::build(line.substr(0,blank),
 	(blank >= 0) ? line.substr(blank+1).c_str() : (const char*)0);
     msg.retValue() = r ? "External start attempt\r\n" : "External command failed\r\n";
@@ -1627,6 +1678,22 @@ bool ExtModCommand::complete(const String& partLine, const String& partWord, Str
 	    Module::itemComplete(rval,*list,partWord);
 	return true;
     }
+    else if (partLine == "external restart" || partLine == "external stop") {
+	ObjList mod;
+	s_mutex.lock();
+	ObjList *l = &s_modules;
+	for (; l; l=l->next()) {
+	    ExtModReceiver *r = static_cast<ExtModReceiver *>(l->get());
+	    if (!r)
+		continue;
+	    if (mod.find(r->scriptFile()))
+		continue;
+	    mod.append(new String(r->scriptFile()));
+	}
+	s_mutex.unlock();
+	for (l = mod.skipNull(); l; l = l->skipNext())
+	    Module::itemComplete(rval,l->get()->toString(),partWord);
+    }
     return false;
 }
 
@@ -1729,7 +1796,8 @@ ExtListener* ExtListener::build(const char* name, const NamedList& sect)
 
 
 ExtModulePlugin::ExtModulePlugin()
-    : m_handler(0)
+    : Plugin("extmodule"),
+      m_handler(0)
 {
     Output("Loaded module ExtModule");
 }
