@@ -62,7 +62,7 @@ public:
 	{ return m_name; }
     static MOHSource* getSource(String& name, const NamedList& params);
 private:
-    MOHSource(const String &name, const String &command_line);
+    MOHSource(const String &name, const String &command_line, unsigned int rate = 8000);
     String m_name;
     String m_command_line;
     bool create();
@@ -82,10 +82,7 @@ public:
     MOHChan(String& name, const NamedList& params);
     ~MOHChan();
     virtual void disconnected(bool final, const char *reason);
-    inline const String &id() const
-	{ return m_id; }
 private:
-    String m_id;
     static int s_nextid;
 };
 
@@ -121,11 +118,13 @@ private:
 };
 
 
-MOHSource::MOHSource(const String &name, const String &command_line)
+MOHSource::MOHSource(const String &name, const String &command_line, unsigned int rate)
     : ThreadedSource("slin"),
-      m_name(name), m_command_line(command_line), m_swap(false), m_brate(16000)
+      m_name(name), m_command_line(command_line), m_swap(false), m_brate(2*rate)
 {
-    Debug(DebugAll,"MOHSource::MOHSource(\"%s\", \"%s\") [%p]", name.c_str(), command_line.c_str(), this);
+    Debug(DebugAll,"MOHSource::MOHSource('%s','%s',%u) [%p]",name.c_str(),command_line.c_str(),rate,this);
+    if (rate != 8000)
+	m_format << "/" << rate;
 }
 
 MOHSource::~MOHSource()
@@ -152,6 +151,10 @@ void MOHSource::destroyed()
 MOHSource* MOHSource::getSource(String& name, const NamedList& params)
 {
     String cmd = s_cfg.getValue("mohs", name);
+    unsigned int rate = 8000;
+    // honor the rate only if the command knows about it
+    if ((cmd.find("${rate}") >= 0) || (cmd.find("${rate$") >= 0))
+	rate = params.getIntValue("rate",8000);
     if (params.replaceParams(cmd) > 0) {
 	// command is parametrized, suffix name to account for it
 	name << "-" << cmd.hash();
@@ -166,7 +169,7 @@ MOHSource* MOHSource::getSource(String& name, const NamedList& params)
 	}
     }
     if (cmd) {
-	MOHSource *s = new MOHSource(name,cmd);
+	MOHSource *s = new MOHSource(name,cmd,rate);
 	if (s->start("MOH Source")) {
 	    sources.append(s);
 	    return s;
@@ -240,8 +243,15 @@ void MOHSource::run()
     u_int64_t tpos = Time::now();
     m_time = tpos;
 
+    unsigned int pos = 0;
     while ((r > 0) && looping()) {
-	r = (m_in >= 0) ? ::read(m_in,m_data.data(),m_data.length()) : m_data.length();
+	unsigned int len = m_data.length() - pos;
+	if (m_in >= 0) {
+	    unsigned char* ptr = m_data.data(pos,len);
+	    r = ptr ? ::read(m_in,ptr,len) : 0;
+	}
+	else
+	    r = len;
 
 	if (r < 0) {
 	    if (errno == EINTR) {
@@ -250,8 +260,12 @@ void MOHSource::run()
 	    }
 	    break;
 	}
-	if (r < (int)m_data.length())
-	    m_data.assign(m_data.data(),r);
+	pos += r;
+	if (pos < m_data.length()) {
+	    if (!r)
+		Thread::yield();
+	    continue;
+	}
 	if (m_swap) {
 	    uint16_t* p = (uint16_t*)m_data.data();
 	    for (int i = 0; i < r; i+= 2) {
@@ -265,6 +279,7 @@ void MOHSource::run()
 	    Thread::usleep((unsigned long)dly);
 	}
 	Forward(m_data);
+	pos = 0;
 	tpos += (r*1000000ULL/m_brate);
     }
 }
@@ -277,7 +292,9 @@ MOHChan::MOHChan(String& name, const NamedList& params)
 {
     Debug(DebugAll,"MOHChan::MOHChan(\"%s\") [%p]",name.c_str(),this);
     Lock lock(s_mutex);
-    m_id << "moh/" << s_nextid++;
+    String tmp;
+    tmp << "moh/" << s_nextid++;
+    setId(tmp);
     chans.append(this);
     MOHSource *s = MOHSource::getSource(name,params);
     if (s) {
@@ -290,7 +307,7 @@ MOHChan::MOHChan(String& name, const NamedList& params)
 
 MOHChan::~MOHChan()
 {
-    Debug(DebugAll,"MOHChan::~MOHChan() %s [%p]",m_id.c_str(),this);
+    Debug(DebugAll,"MOHChan::~MOHChan() %s [%p]",id().c_str(),this);
     s_mutex.lock();
     chans.remove(this,false);
     s_mutex.unlock();
@@ -307,7 +324,7 @@ bool MOHHandler::received(Message &msg)
     String dest(msg.getValue("callto"));
     if (dest.null())
 	return false;
-    Regexp r("^moh/\\(.*\\)$");
+    static const Regexp r("^moh/\\(.*\\)$");
     if (!dest.matches(r))
 	return false;
     String name = dest.matchString(1);
@@ -370,7 +387,7 @@ bool AttachHandler::received(Message &msg)
     String src(msg.getValue("source"));
     if (src.null())
 	return false;
-    Regexp r("^moh/\\(.*\\)$");
+    static const Regexp r("^moh/\\(.*\\)$");
     if (!src.matches(r))
 	return false;
     src = src.matchString(1);
@@ -405,7 +422,8 @@ bool StatusHandler::received(Message &msg)
 
 
 MOHPlugin::MOHPlugin()
-    : m_handler(0)
+    : Plugin("moh"),
+      m_handler(0)
 {
     Output("Loaded module MOH");
 }
