@@ -71,7 +71,7 @@ public:
 	const char* called = 0)
 	{ m_caller = caller; m_callerName = callername; m_called = called; }
     // Set the caller, callername and called parameters
-    void copyCall(Message& dest, bool privacy = false);
+    void copyCall(NamedList& dest, bool privacy = false);
     // Fill a string with line status parameters
     void statusParams(String& str);
     // Fill a string with line status detail parameters
@@ -104,7 +104,8 @@ class ModuleGroup : public AnalogLineGroup
 public:
     // Create an FXS/FXO group of analog lines
     inline ModuleGroup(AnalogLine::Type type, const char* name)
-	: AnalogLineGroup(type,name), m_init(false), m_thread(0), m_callEndedPlayTime(0)
+	: AnalogLineGroup(type,name),
+	  m_init(false), m_ringback(false), m_thread(0), m_callEndedPlayTime(0)
 	{ m_prefix << name << "/"; }
     // Create a group of analog lines used to record
     inline ModuleGroup(const char* name, ModuleGroup* fxo)
@@ -120,6 +121,8 @@ public:
 	{ return static_cast<ModuleGroup*>(fxo()); }
     inline const String& prefix()
 	{ return m_prefix; }
+    inline bool ringback() const
+	{ return m_ringback; }
     // Remove all channels associated with this group and stop worker thread
     virtual void destruct();
     // Process an event geberated by a line
@@ -173,11 +176,13 @@ private:
     static const char* lineParams[];
 
     bool m_init;                         // Init flag
+    bool m_ringback;                     // Lines need to provide ringback
     String m_prefix;                     // Line prefix used to complete commands
     AnalogWorkerThread* m_thread;        // The worker thread
     // FXS/FXO group data
     String m_callEndedTarget;            // callto when an FXS line was disconnected
     String m_oooTarget;                  // callto when out-of-order (hook is off for a long time after call ended)
+    String m_lang;                       // Language for played tones
     u_int64_t m_callEndedPlayTime;       // Time to play call ended prompt
     // Recorder group data
     ObjList m_endpoints;                 // Record data endpoints
@@ -282,6 +287,7 @@ protected:
 private:
     ModuleLine* m_line;                  // The analog line associated with this channel
     bool m_hungup;                       // Hang up flag
+    bool m_ringback;                     // Circuit ringback provider flag
     bool m_routeOnSecondRing;            // Delay router if waiting callerid
     RecordTrigger m_recording;           // Recording trigger source
     String m_reason;                     // Hangup reason
@@ -292,6 +298,7 @@ private:
                                          // FXS: send call setup after first ring
     String m_callEndedTarget;            // callto when an FXS line was disconnected
     String m_oooTarget;                  // callto when out-of-order (hook is off for a long time after call ended)
+    String m_lang;                       // Language for played tones
     unsigned int m_polarityCount;        // The number of polarity changes received
     bool m_polarity;                     // The last value we've set for the line polarity
     bool m_privacy;                      // Send caller identity
@@ -678,7 +685,7 @@ void ModuleLine::processNotify(Message& msg)
 }
 
 // Set the caller, callername and called parameters
-void ModuleLine::copyCall(Message& dest, bool privacy)
+void ModuleLine::copyCall(NamedList& dest, bool privacy)
 {
     if (privacy)
 	dest.addParam("callerpres","restricted");
@@ -978,6 +985,8 @@ bool ModuleGroup::initialize(const NamedList& params, const NamedList& defaults,
 	debugLevel(level);
     }
 
+    m_ringback = params.getBoolValue("ringback");
+
     Lock2 lock(this,fxoRec());
     bool ok = true;
     if (!m_init) {
@@ -1097,6 +1106,8 @@ void ModuleGroup::copyData(AnalogChannel* chan)
 	return;
     chan->m_callEndedTarget = m_callEndedTarget;
     chan->m_oooTarget = m_oooTarget;
+    if (!chan->m_lang)
+	chan->m_lang = m_lang;
     chan->m_callEndedTimer.interval(m_callEndedPlayTime);
 }
 
@@ -1233,6 +1244,7 @@ bool ModuleGroup::reload(const NamedList& params, const NamedList& defaults,
 	    defaults.getValue("outoforder-target"));
 	if (!m_oooTarget)
 	    m_oooTarget = "tone/outoforder";
+	m_lang = params.getValue("lang",defaults.getValue("lang"));
 	XDebug(this,DebugAll,"Targets: call-ended='%s' outoforder='%s' [%p]",
 	    m_callEndedTarget.c_str(),m_oooTarget.c_str(),this);
     }
@@ -1310,6 +1322,7 @@ AnalogChannel::AnalogChannel(ModuleLine* line, Message* msg, RecordTrigger recor
     : Channel(&plugin,0,(msg != 0)),
     m_line(line),
     m_hungup(false),
+    m_ringback(false),
     m_routeOnSecondRing(false),
     m_recording(recorder),
     m_callEndedTimer(0),
@@ -1322,12 +1335,16 @@ AnalogChannel::AnalogChannel(ModuleLine* line, Message* msg, RecordTrigger recor
     m_callsetup(AnalogLine::NoCallSetup)
 {
     m_line->userdata(this);
-    if (m_line->moduleGroup())
+    if (m_line->moduleGroup()) {
 	m_line->moduleGroup()->setEndpoint(this,true);
+	m_ringback = m_line->moduleGroup()->ringback();
+    }
 
     // Set caller/called from line
-    if (isOutgoing())
+    if (isOutgoing()) {
+	m_lang = msg->getValue("lang");
 	m_line->setCall(msg->getValue("caller"),msg->getValue("callername"),msg->getValue("called"));
+    }
     else
 	if ((m_line->type() == AnalogLine::FXS) || (recorder == FXO))
 	    m_line->setCall("","","off-hook");
@@ -1372,11 +1389,11 @@ AnalogChannel::AnalogChannel(ModuleLine* line, Message* msg, RecordTrigger recor
     // Startup
     Message* m = message("chan.startup");
     m->setParam("direction",status());
+    if (msg)
+	m->copyParams(*msg,"caller,callername,called,billid,callto,username");
     m_line->copyCall(*m);
-    if (isOutgoing()) {
+    if (isOutgoing())
 	m_targetid = msg->getValue("id");
-	m->addParam("billid",msg->getValue("billid"));
-    }
     Engine::enqueue(m);
 
     // Init call
@@ -1402,7 +1419,15 @@ AnalogChannel::AnalogChannel(ModuleLine* line, Message* msg, RecordTrigger recor
 		m_privacy = getPrivacy(*msg);
 		if (m_callsetup == AnalogLine::Before)
 		    m_line->sendCallSetup(m_privacy);
-		m_line->sendEvent(SignallingCircuitEvent::RingBegin,AnalogLine::Dialing);
+		{
+		    NamedList* params = 0;
+		    NamedList callerId("");
+		    if (m_callsetup != AnalogLine::NoCallSetup) {
+			params = &callerId;
+			m_line->copyCall(callerId,m_privacy);
+		    }
+		    m_line->sendEvent(SignallingCircuitEvent::RingBegin,AnalogLine::Dialing,params);
+		}
 		if (m_callsetup == AnalogLine::After)
 		    m_dialTimer.interval(500);
 		break;
@@ -1484,11 +1509,20 @@ bool AnalogChannel::msgRinging(Message& msg)
 	    m_line->acceptPulseDigit(false);
 	m_line->changeState(AnalogLine::Ringing);
     }
-    if (msg.getBoolValue("earlymedia",getPeer() && getPeer()->getSource())) {
+    bool media = msg.getBoolValue("earlymedia",getPeer() && getPeer()->getSource());
+    if (media) {
 	setAudio(false);
 	if (m_line)
 	    m_line->setCircuitParam("echotrain",msg.getValue("echotrain"));
     }
+    else if (m_ringback && m_line) {
+	// Provide ringback from circuit features if supported
+	NamedList params("ringback");
+	params.addParam("tone","ringback");
+	media = m_line->sendEvent(SignallingCircuitEvent::GenericTone,&params);
+    }
+    if (media)
+	m_ringback = false;
     return true;
 }
 
@@ -1499,10 +1533,9 @@ bool AnalogChannel::msgAnswered(Message& msg)
     if (m_line) {
 	m_line->noRingTimer().stop();
 	m_line->removeCallSetupDetector();
-	if (m_line->type() == AnalogLine::FXS) {
-	    m_line->sendEvent(SignallingCircuitEvent::RingEnd);
+	m_line->sendEvent(SignallingCircuitEvent::RingEnd);
+	if (m_line->type() == AnalogLine::FXS)
 	    polarityControl(true);
-	}
 	else {
 	    m_line->acceptPulseDigit(false);
 	    m_line->sendEvent(SignallingCircuitEvent::OffHook);
@@ -1570,6 +1603,8 @@ bool AnalogChannel::callRouted(Message& msg)
     Channel::callRouted(msg);
     setStatus();
     Lock lock(m_mutex);
+    // Update tones language
+    m_lang = msg.getValue("lang",m_lang);
     // Check if the circuit supports tone detection
     if (!m_line->circuit())
 	return true;
@@ -1591,6 +1626,8 @@ bool AnalogChannel::callRouted(Message& msg)
 void AnalogChannel::callAccept(Message& msg)
 {
     Lock lock(m_mutex);
+    // Update tones language
+    m_lang = msg.getValue("lang",m_lang);
     if (isAnswered())
 	return;
     if (m_line) {
@@ -1598,6 +1635,7 @@ void AnalogChannel::callAccept(Message& msg)
 	    m_line->acceptPulseDigit(false);
 	m_line->changeState(AnalogLine::DialComplete);
     }
+    m_ringback = msg.getBoolValue("ringback",m_ringback);
     setAudio(false);
     setAudio(true);
     Channel::callAccept(msg);
@@ -1607,6 +1645,10 @@ void AnalogChannel::callAccept(Message& msg)
 void AnalogChannel::callRejected(const char* error, const char* reason,
 	const Message* msg)
 {
+    if (msg) {
+	Lock lock(m_mutex);
+	m_lang = msg->getValue("lang",m_lang);
+    }
     setReason(error ? error : reason);
     Channel::callRejected(error,m_reason,msg);
     setStatus();
@@ -1628,7 +1670,7 @@ bool AnalogChannel::disconnect(const char* reason)
 	setReason(reason);
 	setStatus("disconnecting");
     }
-    return Channel::disconnect(m_reason);
+    return Channel::disconnect(m_reason,parameters());
 }
 
 // Hangup call
@@ -1933,6 +1975,7 @@ void AnalogChannel::startRouter(bool first)
 		break;
 	    case AnalogLine::FXS:
 		m->addParam("overlapped","true");
+		m->addParam("lang",m_lang,false);
 		break;
 	    default: ;
 	}
@@ -2000,6 +2043,7 @@ bool AnalogChannel::setAnnouncement(const char* status, const char* callto)
 	return false;
     Message* m = message("call.execute",false,true);
     m->addParam("callto",callto);
+    m->addParam("lang",m_lang,false);
     bool ok = Engine::dispatch(*m);
     TelEngine::destruct(m);
     if (ok) {
@@ -2026,6 +2070,7 @@ void AnalogChannel::outCallAnswered(bool stopDial)
     if (stopDial)
 	m_dialTimer.stop();
     m_answered = true;
+    m_ringback = false;
     setStatus("answered");
     if (m_line) {
 	m_line->changeState(AnalogLine::Answered);
@@ -2151,13 +2196,11 @@ AnalogCallRec::AnalogCallRec(ModuleLine* line, bool fxsCaller, const char* id)
 	m_address = m_line->address();
 
     // Set caller/called
-    ModuleLine* info = m_line;
     if (fxsCaller) {
 	if (m_startOnSecondRing && fxo->callSetup() == AnalogLine::Before)
 	    fxo->setCall(fxo->caller(),"",m_line->called());
 	else
 	    fxo->setCall(s_unk,"",m_line->called());
-	info = fxo;
     }
     else
 	m_line->setCall(s_unk,"",fxo->called());
@@ -2565,9 +2608,9 @@ void AnalogDriver::initialize()
 	general = &dummy;
 
     // Startup
+    setup();
     if (!m_init) {
 	m_init = true;
-	setup();
 	installRelay(Masquerade);
 	installRelay(Halt);
 	installRelay(Progress);
@@ -2731,6 +2774,8 @@ bool AnalogDriver::msgExecute(Message& msg, String& dest)
 	    analogCh->callConnect(msg);
 	    msg.setParam("peerid",analogCh->id());
 	    msg.setParam("targetid",analogCh->id());
+	    if (analogCh->line() && (analogCh->line()->type() == AnalogLine::FXS))
+		Engine::enqueue(analogCh->message("call.ringing",false,true));
         }
     }
     else
@@ -3159,7 +3204,7 @@ void AnalogWorkerThread::run()
 	AnalogLineEvent* event = m_client->getEvent(t);
 	if (!event) {
 	    m_client->checkTimers(t);
-	    Thread::yield(true);
+	    Thread::idle(true);
 	    continue;
 	}
 	ModuleLine* line = static_cast<ModuleLine*>(event->line());
