@@ -80,7 +80,7 @@ public:
     inline u_int8_t retransCounter()
 	{ return m_retransmitted; }
     // Method called to change ack number in case the data need to be retransmissed
-    void refreshAck(u_int8_t ack);
+    bool refreshAck(u_int8_t ack);
 private:
     u_int8_t m_seq;                                        // The data sequence number
     u_int8_t m_retransmitted;                              // Counter increased every time when the data is retransmitted
@@ -115,10 +115,11 @@ public:
     // Socket initialization
     bool initSocket(const NamedList& params);
     // Check if we have any new data in queue
-    void checkData();
+    bool checkData(bool force);
     // Retransmit the data between the retransmission timer sequence number and actual sequence number
     void retransData();
-    void checkAck(DataBlock& data, bool ack = false);
+    void checkAck(const DataBlock& data);
+    bool checkSeq(const DataBlock& data);
     void checkTimeouts(u_int64_t time);
     void sendAck();
     void buildAck(DataBlock& data);
@@ -129,20 +130,20 @@ public:
     bool readData();
     void recvMsg(DataBlock& packet);
     bool sendData(const DataBlock& msg);
-    bool handleSyn(DataBlock& data, bool ack);
+    bool handleSyn(const DataBlock& data, bool ack);
     void removeData(u_int8_t ack);
-    void handleNull(DataBlock& data);
+    void handleEack(const DataBlock& data);
+    void handleData(DataBlock& data);
     void reset();
     bool checkChecksum(DataBlock& data);
     //thread functions
     bool running();
     void kill();
-    void handleEack(DataBlock& data);
     void removeOneData(u_int8_t ack);
     bool startThread(Thread::Priority prio = Thread::Normal);
     void stopThread();
     u_int16_t checksum(u_int16_t len,const u_int8_t* buff);
-    void appendChecksum(DataBlock& data, bool recalculate = false);
+    void appendChecksum(DataBlock& data);
     inline void setThread()
 	{ m_thread = 0;}
     inline bool haveSyn(u_int8_t flags)
@@ -171,6 +172,7 @@ private:
     RudpThread* m_thread;                                  // The socket thread
     Socket *m_socket;                                      // The socket
     ObjList m_msgList;                                     // The messages list
+    int m_lastError;                                       // Last error we have displayed
     // Sequence numbers
     unsigned int m_sequence;                               // The sequence number
     unsigned int m_ackNum;                                 // The sequence number of the last in sequence message received
@@ -219,6 +221,7 @@ public:
 	Q_RESUME_R = 7,                                    // Q_RESUME Response Message used with redundant MGC configuration
 	Q_RESET_I = 8,                                     // Q_RESET Invoke Message used with redundant MGC configuration
 	Q_RESET_R = 9,                                     // Q_RESET Response Message used with redundant MGC configuration
+	Q_RESTART = 12,
 	PDU = 0x8000,                                      // PDU - Non Session Manager message from application
     };
     SessionManager(const String& name,const NamedList& param);
@@ -245,6 +248,8 @@ public:
     void handleSmMessage(u_int32_t smMessageType);
     // Inform all users about session manager status change
     void informUser(bool up);
+    inline bool operational() const
+	{ return Operational == m_state; }
     static inline const char* stateName(State s)
 	{ return lookup((int)s,s_smStates); }
     void changeState(State newState);
@@ -259,6 +264,7 @@ private:
     State m_state;                                         // Session Manager state
     String m_name;                                         // The name of this session
     unsigned int m_upUsers;                                // The number of up users
+    SignallingTimer m_standbyTimer;                        // When should send next Standby
     static const TokenDict s_smStates[];
     static const TokenDict s_types[];
 };
@@ -398,6 +404,7 @@ public:
     virtual bool transmitMSU(const SS7MSU& msu);
     void getStringMessage(String& tmp, DataBlock& data);
     virtual bool operational() const;
+    virtual void configure(bool start);
     static inline const char* stateName(State s)
 	{ return lookup((int)s,s_states); }
     inline u_int16_t channel()
@@ -432,6 +439,7 @@ protected:
     u_int16_t m_bearerId;                                  // Bearer ID NOTE it is set to 0
     SignallingTimer m_confReqTimer;                        // The configuration request timer
     bool m_printMsg;                                       // Flag used to see if we print this layer messages
+    bool m_autostart;                                      // Automatically align on resume
     static const TokenDict s_messages[];
     static const TokenDict s_connectM[];
     static const TokenDict s_errors[];
@@ -500,10 +508,13 @@ DataSequence::~DataSequence()
 {
 }
 
-void DataSequence::refreshAck(u_int8_t acn)
+bool DataSequence::refreshAck(u_int8_t acn)
 {
     u_int8_t* ack = data(3);
-    ack[0] = acn;
+    if (acn == *ack)
+	return false;
+    *ack = acn;
+    return true;
 }
 
 /**
@@ -522,11 +533,13 @@ const TokenDict RudpSocket::s_RudpStates[] = {
 
 RudpSocket::RudpSocket(SessionManager* sm)
     : Mutex(true,"RudpSocket"),
-      m_sm(sm), m_thread(0), m_socket(0), m_sequence(0), m_ackNum(0), m_lastAck(0), m_lastSend(0), m_retTStartSeq(0),
+      m_sm(sm), m_thread(0), m_socket(0), m_lastError(-1),
+      m_sequence(0), m_ackNum(0), m_lastAck(0), m_lastSend(0), m_retTStartSeq(0),
       m_syn(1000), m_cumAckTimer(0), m_nullTimer(0), m_retransTimer(0), m_synTimer(0),
       m_version(-1), m_haveChecksum(false), m_sendSyn(false),
       m_connId(0x208000),
-      m_retransCounter(0), m_maxCumAck(0), m_wrongChecksum(0), m_state(RudpDown)
+      m_retransCounter(0), m_maxCumAck(0), m_queueCount(0), m_wrongChecksum(0),
+      m_state(RudpDown)
 {
 }
 
@@ -557,7 +570,7 @@ void RudpSocket::changeState(RudpState newState)
 // Initialize Parameters ,initialize socket and start thread
 bool RudpSocket::initialize(const NamedList& params)
 {
-    m_sequence = params.getIntValue("rudp_sequence",0xff & ::random());
+    m_sequence = params.getIntValue("rudp_sequence",0xff & Random::random());
     if (!Modulo256::between(m_sequence,0,255)) {
 	Debug(m_sm,DebugNote,"Rudp Sequence value out of bounds set to 0");
 	m_sequence = 0;
@@ -625,53 +638,61 @@ bool RudpSocket::initSocket(const NamedList& params)
 // Call check data and verify if any timer has expired is called from readData()
 void RudpSocket::checkTimeouts(u_int64_t time)
 {
-    checkData();
-    if (m_retransTimer.started() && m_retransTimer.timeout(time)) {
+    if (m_state != RudpUp) {
+	if (m_synTimer.timeout(time)) {
+	    m_synTimer.stop();
+	    sendSyn(false);
+	}
+	return;
+    }
+    checkData(false);
+    if (m_retransTimer.timeout(time)) {
 	m_retransTimer.stop();
 	retransData();
     }
-    if (m_cumAckTimer.started() && m_cumAckTimer.timeout(time)) {
-	sendAck();
-	m_cumAckTimer.stop();
+    if (m_cumAckTimer.timeout(time)) {
+    	if (!checkData(true))
+	    sendAck();
     }
     if (m_nullTimer.timeout(time)) {
-	sendNull();
-	m_nullTimer.stop();
-	m_nullTimer.start();
-    }
-    if (m_synTimer.timeout(time) && m_state != RudpUp) {
-	m_synTimer.stop();
-	sendSyn(false);
+    	if (!checkData(true))
+	    sendNull();
     }
 }
 
 // Verify if exist any new data in the queue and transmit all data 
 // with sequence number > last sent and <= actual sequence number
-void RudpSocket::checkData()
+bool RudpSocket::checkData(bool force)
 {
     Lock mylock(this);
-    if (m_queueCount >= m_msgList.count())
-	return;
+    if (!force && (m_queueCount >= m_msgList.count()))
+	return false;
+    bool sent = false;
     ObjList* obj = m_msgList.skipNull();
     for (; obj; obj = obj->skipNext()) {
 	DataSequence* data = static_cast<DataSequence*>(obj->get());
 	if (data && data->sequence() != m_lastSend && Modulo256::between(data->sequence(),m_lastSend,m_sequence)) {
+	    sent = true;
+	    if (data->refreshAck(m_ackNum) && m_haveChecksum)
+		appendChecksum(static_cast<DataBlock&>(*data));
 	    sendData(static_cast<DataBlock>(*data));
-	    // Stop cumulative acknowledge timer because we send acknowledge segment with data
-	    if (m_cumAckTimer.started())
-		m_cumAckTimer.stop();
-	    // Restart null timer because we have data trafic
-	    m_nullTimer.stop();
-	    m_nullTimer.start();
-	    if (!m_retransTimer.started()) {
-		// start retrasnsmission timer and refresh retransmission sequence number
-		m_retransTimer.start();
-		m_retTStartSeq = data->sequence();
-	    }
 	    m_lastSend = data->sequence();
 	}
     }
     m_queueCount = m_msgList.count();
+    if (sent) {
+	// Stop cumulative acknowledge timer because we send acknowledge segment with data
+	m_cumAckTimer.stop();
+	// Restart null timer because we had data trafic
+	m_nullTimer.stop();
+	m_nullTimer.start();
+	if (!m_retransTimer.started()) {
+	    // start retrasnsmission timer and refresh retransmission sequence number
+	    m_retransTimer.start();
+	    m_retTStartSeq = m_lastSend;
+	}
+    }
+    return sent;
 }
 
 // Method called from checkTimeouts
@@ -688,21 +709,20 @@ void RudpSocket::retransData()
 	DataSequence* data = static_cast<DataSequence*>(obj->get());
 	if (data && Modulo256::between(data->sequence(),m_lastAck,m_sequence)) {
 	    if (data->retransCounter() <= m_retransCounter) {
-		if (m_haveChecksum) {
-		    data->refreshAck(m_ackNum);
-		    appendChecksum(static_cast<DataBlock&>(*data),true);
-		    sendData(static_cast<DataBlock&>(*data));
-		} else
-		    sendData(static_cast<DataBlock&>(*data));
+		XDebug(m_sm,DebugInfo,"Retransmission %u of data with seq %u",
+		    data->retransCounter(),data->sequence());
+		if (data->refreshAck(m_ackNum) && m_haveChecksum)
+		    appendChecksum(static_cast<DataBlock&>(*data));
+		sendData(static_cast<DataBlock&>(*data));
 		data->inc();
 		if (!m_retransTimer.started())
 		     m_retransTimer.start();
 	    } else {
-		Debug(m_sm,DebugNote,"RUDP Layer down, received maximum retransmission counter");
+		Debug(m_sm,DebugNote,"RUDP Layer down, retransmission exceeded for seq %u",data->sequence());
 #ifdef DEBUG
 		String aux;
 		aux.hexify(data->data(),data->length(),' ');
-		Debug(m_sm,DebugInfo,"Retransmission counter received for data: %s ",aux.c_str());
+		Debug(m_sm,DebugInfo,"Retransmission exceeded for data: %s ",aux.c_str());
 #endif
 		m_sm->notify(true);
 		changeState(RudpDown);
@@ -742,6 +762,7 @@ void RudpSocket::buildAck(DataBlock& data)
 // Build and return or send an ack segment
 void RudpSocket::sendAck()
 {
+    m_cumAckTimer.stop();
     if (m_state != RudpUp)
 	return;
     DataBlock data;
@@ -753,11 +774,9 @@ void RudpSocket::sendAck()
     sendData(data);
 }
 
-// Build an null segment and enqueue it
+// Build a null segment and enqueue it
 void RudpSocket::sendNull()
 {
-    if (m_state != RudpUp)
-	return;
     DataBlock data;
     buildAck(data);
     u_int8_t* buf = data.data(0,2);
@@ -765,6 +784,8 @@ void RudpSocket::sendNull()
     if (m_haveChecksum)
 	appendChecksum(data);
     keepData(data,m_sequence);
+    m_nullTimer.stop();
+    m_nullTimer.start();
 }
 
 // helper function to store a 16 bit in big endian format
@@ -802,7 +823,7 @@ void RudpSocket::sendSyn(bool recvSyn)
     m_syn = m_sequence;
     switch (m_version) {
 	case 0:
-	    m_haveChecksum ? buf[1] = 12 : buf[1] = 8;
+	    buf[1] = m_haveChecksum ? 12 : 8;
 	    store32(buf+4,m_connId);
 	    break;
 	case 1:
@@ -811,9 +832,9 @@ void RudpSocket::sendSyn(bool recvSyn)
 	    store16(buf+10,0xce0c); // ???
 	    store32(buf+12,m_connId);
 	    store16(buf+16,0x0180); // MSS?
-	    store16(buf+18,m_retransTimer.interval());
-	    store16(buf+20,m_cumAckTimer.interval());
-	    store16(buf+22,m_nullTimer.interval());
+	    store16(buf+18,(u_int16_t)m_retransTimer.interval());
+	    store16(buf+20,(u_int16_t)m_cumAckTimer.interval());
+	    store16(buf+22,(u_int16_t)m_nullTimer.interval());
 	    store16(buf+24,2000); // Transf. state timeout?
 	    buf[26] = m_retransCounter;
 	    buf[27] = m_maxCumAck;
@@ -828,7 +849,7 @@ void RudpSocket::sendSyn(bool recvSyn)
     DataBlock data;
     data.assign((void*)buf,buf[1]);
     if (m_haveChecksum)
-	appendChecksum(data,false);
+	appendChecksum(data);
     if (sendData(data))
 	changeState(RudpWait);
 }
@@ -856,36 +877,47 @@ bool RudpSocket::readData()
     bool readOk = false,error = false;
     if (!m_socket->select(&readOk,0,&error,1000))
 	return false;
-    if (error)
-	Debug(m_sm,DebugInfo,"Reading data error: %s (%d)",
-	    strerror(m_socket->error()),m_socket->error());
+    if (error) {
+	m_socket->updateError();
+	int err = m_socket->error();
+	if (err && (err != m_lastError)) {
+	    m_lastError = err;
+	    Debug(m_sm,DebugMild,"Selecting error: %s (%d)",strerror(err),err);
+	}
+    }
     if (!readOk || error)
 	return false;
     unsigned char buffer [MAX_BUF_SIZE];
     SocketAddr addr;
     int r = m_socket->recv(buffer,MAX_BUF_SIZE);
 
-    if (r <= 0)
+    if (r < 0) {
+	int err = m_socket->error();
+	if (err && (err != m_lastError)) {
+	    m_lastError = err;
+	    Debug(m_sm,DebugMild,"Reading data error: %s (%d)",strerror(err),err);
+	}
+    }
+    else if (r == 0)
 	return false;
+    m_lastError = -1;
     DataBlock packet(buffer,r);
 #ifdef XDEBUG
     String seen;
     seen.hexify(packet.data(),packet.length(),' ');
     Debug(m_sm,DebugInfo,"Reading data: %s length returned = %d",seen.c_str(), r);
 #endif
-    u_int8_t flag = packet.at(0);
-    if (m_state == RudpDown && !haveSyn((u_int8_t)flag))
+    // Sanity checks for packet and header length
+    if (packet.length() < 4)
 	return false;
-    if (haveSyn((u_int8_t)flag) && m_state == RudpUp)
+    if ((unsigned int)packet.at(1) > packet.length())
+	return false;
+    if (m_state == RudpDown && !haveSyn((u_int8_t)packet.at(0)))
 	return false;
     if (m_haveChecksum && !checkChecksum(packet)) {
 	DDebug(m_sm,DebugMild,"Wrong checksum received");
 	return false;
     }
-    if (m_nullTimer.started())
-	m_nullTimer.stop();
-    if (m_state == RudpUp)
-	m_nullTimer.start();
     recvMsg(packet);
     return true;
 }
@@ -907,9 +939,6 @@ bool RudpSocket::sendData(const DataBlock& msg)
 	    return false;
 	}
 	else {
-	    if (m_nullTimer.started())
-		m_nullTimer.stop();
-	    m_nullTimer.start();
 #ifdef XDEBUG
 	    String seen;
 	    seen.hexify(msg.data(),msg.length(),' ');
@@ -930,16 +959,15 @@ void RudpSocket::sendMSG(const DataBlock& data)
     buildAck(auxdata);
     auxdata += data;
     if (m_haveChecksum)
-	appendChecksum(auxdata,false);
+	appendChecksum(auxdata);
     keepData(auxdata,m_sequence);
 }
 
 // Check received packet and send it for processing
 void RudpSocket::recvMsg(DataBlock& packet)
 {
-    unsigned int aux = m_ackNum;
     u_int8_t flag = packet.at(0);
-    Modulo256::inc(aux);
+    // SYN needs to be handled first
     if (haveSyn(flag)) {
 	if (haveAck(flag)) {
 	    handleSyn(packet,true);
@@ -949,48 +977,50 @@ void RudpSocket::recvMsg(DataBlock& packet)
 	handleSyn(packet,false);
 	return;
     }
-    if (packet.at(2) >= 0 && (u_int8_t)packet.at(2) == aux) {
-	if (!(flag == 0x40 && packet.at(1) >= 0 && packet.length() == (u_int8_t)packet.at(1))) {
-	    // If we received an alone ack segment just let it pass trough
-	    if (!m_cumAckTimer.started())
-		m_cumAckTimer.start();
-	    m_ackNum = aux;
-	}
-    } else  // If the received packet is not next in sequence drop it
-	return;
-
-    if (haveNul(flag)) {
-	if (haveAck(flag))
-	    handleNull(packet);
-	else
-	    Debug(m_sm,DebugNote,"Received NULL segment without ACK flag set");
-	return;
+    bool hasData = false;
+    // Then acknowledge pending packets - ACK and EACK
+    if (haveAck(flag)) {
+	hasData = true;
+	checkAck(packet);
     }
-    if (haveEack(flag)) {
+    if (haveEack(flag))
 	handleEack(packet);
-	return;
+
+    // Check for special packets
+    if (haveNul(flag)) {
+	hasData = false;
+	if (!haveAck(flag))
+	    Debug(m_sm,DebugWarn,"Received NULL segment without ACK flag set");
+	checkSeq(packet);
+	m_cumAckTimer.fire();
     }
-#ifdef DEBUG
-    if (haveReset(flag))
-	Debug(m_sm,DebugNote,"Received RESET segment, ignored");
-    if (haveTcs(flag))
-	Debug(m_sm,DebugNote,"Received TCS segment, ignored");
-#endif
-    if (haveAck(flag))
-	checkAck(packet,true);
+    if (haveReset(flag)) {
+	Debug(m_sm,DebugMild,"Received RESET segment, ignored");
+	hasData = false;
+	checkSeq(packet);
+    }
+    if (haveTcs(flag)) {
+	Debug(m_sm,DebugMild,"Received TCS segment, ignored");
+	hasData = false;
+    }
+    // If we had ACK or EACK only also check for data after header
+    if (hasData)
+	handleData(packet);
 }
 
-bool RudpSocket::handleSyn(DataBlock& data, bool ack)
+bool RudpSocket::handleSyn(const DataBlock& data, bool ack)
 {
     DDebug(m_sm,DebugInfo,"Handling SYN%s with length %u",
 	(ack ? "-ACK" : ""),data.length());
     if (m_version < 0) {
 	switch (data.length()) {
 	    case 12:
+		m_version = 0;
 		m_haveChecksum = true;
-		// fall through
+		break;
 	    case 8:
 		m_version = 0;
+		m_haveChecksum = false;
 		break;
 	    case 30:
 		m_version = 1;
@@ -1021,18 +1051,39 @@ bool RudpSocket::handleSyn(DataBlock& data, bool ack)
     return true;
 }
 
-void RudpSocket::checkAck(DataBlock& data, bool alone)
+// Check acknowledged sequence number
+void RudpSocket::checkAck(const DataBlock& data)
 {
-    if (data.at(3) >= 0 && (u_int8_t)data.at(3) == m_syn) {
+    u_int8_t ack = data.at(3);
+    if (ack == m_syn) {
+	m_nullTimer.stop();
+	m_nullTimer.start();
 	changeState(RudpUp);
 	m_sm->notify(false);
 	m_syn = 1000;
     }
-    removeData(data.at(3));
-    if (alone && data.at(1) >= 0 && data.length() > (u_int8_t)data.at(1)) {
-	data.cut(-data.at(1));
-	m_sm->handleData(data);
+    removeData(ack);
+}
+
+// Check incoming sequence number, increment if matched
+bool RudpSocket::checkSeq(const DataBlock& data)
+{
+    u_int8_t seq = data.at(2);
+    unsigned int exp = m_ackNum;
+    Modulo256::inc(exp);
+    if (seq == exp) {
+	m_ackNum = seq;
+	if (!m_cumAckTimer.started())
+	    m_cumAckTimer.start();
+	return true;
     }
+    // Received packet is not next in sequence - ignore it
+    if (seq != m_ackNum)
+	Debug(m_sm,DebugMild,"Packet out of sequence, expecting %u or %u but got %u",
+	    m_ackNum,exp,seq);
+    else
+	Debug(m_sm,DebugNote,"Received duplicate packet %u",seq);
+    return false;
 }
 
 // Remove ack data
@@ -1041,9 +1092,11 @@ void RudpSocket::removeData(u_int8_t ack)
     if (Modulo256::between(m_retTStartSeq,m_lastAck,m_sequence))
 	m_retransTimer.stop();
     Lock mylock(this);
+    XDebug(m_sm,DebugInfo,"Removing packets in range %u - %u",m_lastAck,ack);
     ListIterator iter(m_msgList);
      while (DataSequence* data = static_cast<DataSequence*>(iter.get())) {
 	if (Modulo256::between(data->sequence(),m_lastAck,ack)) {
+	    XDebug(m_sm,DebugAll,"Removed packet with seq %u",data->sequence());
 	    m_msgList.remove(data,true);
 	    if (m_queueCount > 0)
 		m_queueCount -- ;
@@ -1062,22 +1115,25 @@ void RudpSocket::removeData(u_int8_t ack)
     }
 }
 
-void RudpSocket::handleNull(DataBlock& data)
+void RudpSocket::handleEack(const DataBlock& data)
 {
-    checkAck(data);
-    sendAck();
-    if (m_cumAckTimer.started())
-	m_cumAckTimer.stop();
+    u_int8_t pack = data.at(1) - (m_haveChecksum ? 8 : 4);
+    DDebug(m_sm,DebugNote,"Received EACK for %u packets, last Ack %u",pack,m_lastAck);
+    for (int i = 4; i < pack + 4; i++)
+	removeOneData(data.at(i));
+    if (!m_cumAckTimer.started())
+	m_cumAckTimer.start();
 }
 
-void RudpSocket::handleEack(DataBlock& data)
+void RudpSocket::handleData(DataBlock& data)
 {
-    checkAck(data);
-    DDebug(m_sm,DebugNote,"Received EACK");
-    u_int8_t pack = data.at(1) - m_haveChecksum ? 8 : 4;
-    for (int i = 4;i < pack + 4;i ++)
-	removeOneData(data.at(i));
-    sendAck();
+    int hdr = data.at(1);
+    if (data.length() <= (unsigned int)hdr)
+	return;
+    if (!checkSeq(data))
+	return;
+    data.cut(-hdr);
+    m_sm->handleData(data);
 }
 
 void RudpSocket::removeOneData(u_int8_t ack)
@@ -1087,6 +1143,7 @@ void RudpSocket::removeOneData(u_int8_t ack)
     for (; obj; obj = obj->skipNext()) {
 	DataSequence* data = static_cast<DataSequence*>(obj->get());
 	if (data && data->sequence() == ack) {
+	    XDebug(m_sm,DebugAll,"Removed one packet with seq %u",ack);
 	    m_msgList.remove(data,true);
 	    if (m_queueCount > 0)
 		m_queueCount --;
@@ -1095,6 +1152,7 @@ void RudpSocket::removeOneData(u_int8_t ack)
 	    return;
 	}
     }
+    DDebug(m_sm,DebugInfo,"Not found packet with seq %u",ack);
 }
 
 bool RudpSocket::running()
@@ -1138,7 +1196,7 @@ u_int16_t RudpSocket::checksum(u_int16_t len, const u_int8_t* buff)
 	sum += (((u_int16_t)buff[i]) << 8) + ((i+1 < len) ? buff[i+1] : 0);
     while (sum >> 16)
 	sum = (sum & 0xFFFF) + (sum >> 16);
-    return (m_version == 1) ? sum : ~sum;
+    return ~sum;
 }
 
 bool RudpSocket::checkChecksum(DataBlock& data)
@@ -1146,21 +1204,13 @@ bool RudpSocket::checkChecksum(DataBlock& data)
     u_int8_t* buf = data.data(0,data.length());
     if (!buf)
 	return false;
-    if (haveSyn(buf[0]) && buf[1] == 8) {
-	m_haveChecksum = false;
-	return true;
-    }
     if (!haveEack(buf[0]) && buf[1] == 4)
 	return true;
-    if (haveChecksum(buf[0]))
-	return 0xffff == checksum(data.length(),buf);
-    else
-	return 0xffff == checksum(buf[1],buf);
+    return checksum((haveChecksum(buf[0]) ? data.length() : buf[1]),buf) == 0;
 }
 
-void RudpSocket::appendChecksum(DataBlock& data, bool recalculate)
+void RudpSocket::appendChecksum(DataBlock& data)
 {
-    u_int16_t sum = 0;
     int dataLen = data.length();
     int rudpLen = 0;
     u_int8_t* buf = data.data(0,dataLen);
@@ -1170,12 +1220,8 @@ void RudpSocket::appendChecksum(DataBlock& data, bool recalculate)
     u_int8_t* cks = buf + (rudpLen - 4);
     if (haveSyn(buf[0]) && (m_version == 1))
 	cks = buf+4;
-    if (recalculate)
-	cks[0] = cks[1] = 0;
-    if (haveChecksum(buf[0]))
-	sum = checksum(dataLen,buf);
-    else
-	sum = checksum(rudpLen,buf);
+    cks[0] = cks[1] = 0;
+    u_int16_t sum = checksum((haveChecksum(buf[0]) ? dataLen : rudpLen),buf);
     cks[0] = (u_int8_t)(sum >> 8);
     cks[1] = (u_int8_t)(sum & 0xff);
 }
@@ -1195,8 +1241,10 @@ SessionManager* SessionManager::get(const String& name, const NamedList* params)
     lock.drop();
     if (session && !session->ref())
 	session = 0;
-    if (params && !session)
+    if (params && !session) {
 	session = new SessionManager(name,*params);
+	session->debugChain(&plugin);
+    }
     return session;
 }
 
@@ -1219,13 +1267,15 @@ const TokenDict SessionManager::s_types[] = {
     { "Q_RESUME_R", Q_RESUME_R },
     { "Q_RESET_I", Q_RESET_I },
     { "Q_RESET_R", Q_RESET_R },
+    { "Q_RESTART", Q_RESTART },
     { "PDU", PDU },
     { 0, 0 }
 };
 
 SessionManager::SessionManager(const String& name, const NamedList& param)
     : Mutex(true,"SessionManager"),
-      m_socket(0), m_state(Nonoperational), m_name(name)
+      m_socket(0), m_state(Nonoperational), m_name(name),
+      m_upUsers(0), m_standbyTimer(0)
 {
     debugName(0);
     debugName(m_name.c_str());
@@ -1252,6 +1302,7 @@ SessionManager::~SessionManager()
 
 bool SessionManager::initialize(const NamedList& params)
 {
+    m_standbyTimer.interval(params,"send_standby",100,2500,true);
     m_socket = new RudpSocket(this);
     return m_socket->initialize(params);
 }
@@ -1259,6 +1310,7 @@ bool SessionManager::initialize(const NamedList& params)
 void SessionManager::notify(bool down)
 {
     if (down) {
+	m_standbyTimer.stop();
 	changeState(Nonoperational);
 	informUser(false);
     } else {
@@ -1278,9 +1330,11 @@ void SessionManager::handleData(DataBlock& data)
     if (smMessageType == PDU)
 	handlePDU(data);
     else {
+#ifdef DEBUG
 	String aux;
 	aux.hexify(data.data(),data.length(),' ');
-	DDebug(this,DebugInfo,"Session data dump: %s",aux.c_str());
+	Debug(this,DebugInfo,"Session data dump: %s",aux.c_str());
+#endif
 	handleSmMessage(smMessageType);
     }
 }
@@ -1291,14 +1345,19 @@ void SessionManager::initSession()
     if (!m_socket)
 	return;
     u_int8_t buf[4];
-    // send standby message
     buf[0] = buf[1] = buf[2] = 0;
-    buf[3] = 3;
     DataBlock data((void*)buf,4,false);
-    DDebug(this,DebugInfo,"Session manager sending: Standby");
-    m_socket->sendMSG(data);
+    // Standby messages should not be sent too often
+    if (m_standbyTimer.interval() &&
+	(m_standbyTimer.timeout() || !m_standbyTimer.started())) {
+	m_standbyTimer.start();
+	// send standby message
+	buf[3] = Standby;
+	DDebug(this,DebugInfo,"Session manager sending: Standby");
+	m_socket->sendMSG(data);
+    }
     // send active message
-    buf[3] = 2;
+    buf[3] = Active;
     DDebug(this,DebugInfo,"Session manager sending: Active");
     m_socket->sendMSG(data);
     data.clear(false);
@@ -1409,7 +1468,10 @@ void SessionManager::handleSmMessage(u_int32_t smMessageType)
 	case Q_RESUME_R:
 	case Q_RESET_I:
 	case Q_RESET_R:
-	    Debug(DebugMild,"Received unexpected SM message %s",typeName((Type)smMessageType));
+	    Debug(this,DebugMild,"Received unexpected SM message %s",typeName((Type)smMessageType));
+	    break;
+	case Q_RESTART:
+	    Debug(this,DebugAll,"Received SM message %s",typeName((Type)smMessageType));
 	    break;
 	default:
 	    Debug(this,DebugNote,"Unknown message type = 0x%08X",smMessageType);
@@ -1544,7 +1606,7 @@ SLT::SLT(const String& name, const NamedList& param)
       SessionUser(1),
       m_status(Unconfigured), m_rStatus(OutOfService), m_reqStatus(OutOfService),
       m_messageId(1), m_channelId(0), m_bearerId(0),
-      m_confReqTimer(0), m_printMsg(false)
+      m_confReqTimer(0), m_printMsg(false), m_autostart(false)
 {
 #ifdef DEBUG
     String tmp;
@@ -1553,6 +1615,7 @@ SLT::SLT(const String& name, const NamedList& param)
     Debug(this,DebugInfo,"SLT::SLT('%s',%p) [%p]%s",
 	name.c_str(),&param,this,tmp.c_str());
 #endif
+    setCompType("cisco-slt");
     m_channelId = param.getIntValue("channel",0);
     String sessionName = param.getValue("session","session");
     setName(name);
@@ -1563,8 +1626,13 @@ SLT::SLT(const String& name, const NamedList& param)
     }
     m_confReqTimer.interval(param,"configuration",250,5000,true);
     m_printMsg = param.getBoolValue("printslt",false);
-    if (param.getBoolValue("autostart",true))
+    m_autoEmergency = param.getBoolValue("autoemergency",true);
+    m_autostart = param.getBoolValue("autostart",true);
+    if (m_autostart) {
 	m_reqStatus = NormalAlignment;
+	if (m_session && m_session->operational())
+	    configure(true);
+    }
 }
 
 SLT::~SLT()
@@ -1578,11 +1646,14 @@ void SLT::destroyed()
     m_session->remove(this);
     RefPointer<SessionManager> tmp = m_session;
     m_session = 0;
+    SignallingComponent::destroyed();
 }
 
 unsigned int SLT::status() const
 {
-    return m_status;
+    if (Configured != m_status || OutOfService == m_reqStatus)
+	return OutOfService;
+    return m_rStatus;
 }
 
 void SLT::setStatus(unsigned int status)
@@ -1623,26 +1694,29 @@ void SLT::notify(bool up)
 {
     if (!up)
 	setStatus(Unconfigured);
-    else {
-	sendManagement(Configuration_R);
-	m_confReqTimer.start();
-	setStatus(Waiting);
-   }
+    else
+	configure(true);
 }
 
 bool SLT::control(Operation oper, NamedList* params)
 {
+    if (params) {
+	m_autoEmergency = params->getBoolValue("autoemergency",m_autoEmergency);
+	m_autostart = params->getBoolValue("autostart",m_autostart);
+	m_printMsg = params->getBoolValue("printslt",m_printMsg);
+    }
     switch (oper) {
 	case Pause:
 	    setReqStatus(OutOfService);
 	    sendManagement(Disconnect_R);
 	    return true;
 	case Resume:
-	    if (aligned())
+	    if (aligned() || !m_autostart)
 		return true;
+	    // fall through
 	case Align:
 	    {
-		bool emg = params->getBoolValue("emergency");
+		bool emg = getEmergency(params);
 		setReqStatus(emg ? EmergencyAlignment : NormalAlignment);
 		switch (m_status) {
 		    case Configured:
@@ -1651,9 +1725,7 @@ bool SLT::control(Operation oper, NamedList* params)
 		    case Waiting:
 			break;
 		    default:
-			sendManagement(Configuration_R);
-			m_confReqTimer.start();
-			setStatus(Waiting);
+			configure(true);
 		}
 	    }
 	    return true;
@@ -1674,6 +1746,7 @@ bool SLT::aligned() const
 // if we don't receive it we resend the configuration request
 void SLT::timerTick(const Time& when)
 {
+    SS7Layer2::timerTick(when);
     if (m_confReqTimer.timeout()) {
 	sendManagement(Configuration_R);
 	m_confReqTimer.stop();
@@ -1783,13 +1856,7 @@ void SLT::processManagement(u_int16_t msgType, DataBlock& data)
 {
     switch ((Messages)msgType) {
 	case Configuration_C:
-	    m_confReqTimer.stop();
-	    setStatus(Configured);
-	    SS7Layer2::notify();
-	    DDebug(this,DebugInfo,"requested status = %s",statusName(m_reqStatus,false));
-	    if (m_reqStatus != NormalAlignment && m_reqStatus != EmergencyAlignment)
-		break;
-	    sendConnect(m_reqStatus == NormalAlignment ? Normal : Emergency);
+	    configure(false);
 	    break;
 	case Status_C:
 	case Statistic_C:
@@ -1823,12 +1890,10 @@ void SLT::processSltMessage(u_int16_t msgType, DataBlock& data)
 	    }
 	    break;
 	case Disconnect_C:
-	    setRemoteStatus(OutOfService);
-	    if (m_reqStatus == EmergencyAlignment || m_reqStatus == NormalAlignment)
-		sendConnect(m_reqStatus == NormalAlignment ? Normal : Emergency);
-	    break;
 	case Disconnect_I:
 	    setRemoteStatus(OutOfService);
+	    if (!m_autostart)
+		break;
 	    if (m_reqStatus == EmergencyAlignment || m_reqStatus == NormalAlignment)
 		sendConnect(m_reqStatus == NormalAlignment ? Normal : Emergency);
 	    break;
@@ -1840,7 +1905,7 @@ void SLT::processSltMessage(u_int16_t msgType, DataBlock& data)
 	case Link_State_Controller_I:
 	    processCIndication(data);
 	    break;
-	default:	
+	default:
 	    const char* mes = messageType((Messages)msgType);
 	    if (mes)
 		DDebug(this,DebugWarn,"Received unhandled SLT message: %s",mes);
@@ -1856,9 +1921,10 @@ void SLT::buildHeader(DataBlock& data,bool management)
     for (int i = 0;i < 16;i ++)
 	head[i] = 0;
     head[2] = 0x80;
-    head[5] = 1;
+    head[4] = (u_int8_t)(protocol() >> 8);
+    head[5] = (u_int8_t)(protocol() & 0xff);
     if (!management)
-	head[7] = 1;
+	head[7] = 1; // Message ID
     head[10] = (u_int8_t)(m_channelId >> 8);
     head[11] = (u_int8_t)(m_channelId & 0xff);
     data.append((void*)head,16);
@@ -1871,7 +1937,7 @@ void SLT::sendConnect(unsigned int status)
     DataBlock data;
     buildHeader(data);
     u_int8_t* header = data.data(0,16);
-    header[9] = 0x6;
+    header[9] = Connect_R;
     header[15] = 4;
     u_int8_t det[4];
     det[0] = det[1] = det[2] = 0;
@@ -1890,7 +1956,7 @@ void SLT::sendControllerR(unsigned int linkState)
     DataBlock data;
     buildHeader(data);
     u_int8_t* header = data.data(0,16);
-    header[9] = 0x20;
+    header[9] = Link_State_Controller_R;
     header[15] = 4;
     u_int8_t det[4];
     for (int i = 0;i < 4;i ++)
@@ -1916,7 +1982,7 @@ void SLT::sendManagement(unsigned int message)
 	getStringMessage(tmp,data);
 	Debug(this,DebugInfo,"Sending %s",tmp.c_str());
     }
-    m_session->sendData(data);
+    m_session->sendData(data,(Configuration_R == message));
 }
 
 void SLT::sendDisconnect()
@@ -1924,7 +1990,7 @@ void SLT::sendDisconnect()
     DataBlock data;
     buildHeader(data);
     u_int8_t* header = data.data(0,16);
-    header[9] = 0x0a;
+    header[9] = Disconnect_R;
     if (m_printMsg) {
 	String tmp;
 	getStringMessage(tmp,data);
@@ -1983,7 +2049,7 @@ bool SLT::transmitMSU(const SS7MSU& msu)
     DataBlock data;
     buildHeader(data);
     u_int8_t* header = data.data(0,16);
-    header[9] = 0x10;
+    header[9] = Data_Req;
     header[15] = msu.length();
     data += msu;
     if (m_printMsg) {
@@ -1999,19 +2065,63 @@ bool SLT::operational() const
     return aligned();
 }
 
+void SLT::configure(bool start)
+{
+    if (start && m_confReqTimer.interval()) {
+	sendManagement(Configuration_R);
+	m_confReqTimer.start();
+	setStatus(Waiting);
+	return;
+    }
+    m_confReqTimer.stop();
+    setStatus(Configured);
+    SS7Layer2::notify();
+    DDebug(this,DebugInfo,"requested status = %s",statusName(m_reqStatus,false));
+    if (!m_autostart)
+	return;
+    if (m_reqStatus == NormalAlignment || m_reqStatus == EmergencyAlignment)
+	sendConnect(m_reqStatus == NormalAlignment ? Normal : Emergency);
+}
+
 SignallingComponent* SLT::create(const String& type, const NamedList& name)
 {
     if (type != "SS7Layer2")
 	return 0;
+    const String* module = name.getParam("module");
+    if (module && *module != "ciscosm")
+	return 0;
     Configuration cfg(Engine::configFile("ciscosm"));
     const char* sectName = name.getValue("link",name);
     NamedList* layer = cfg.getSection(sectName);
-    if (!layer)
+    if (module) {
+	if (layer)
+	    layer->clearParams();
+	else {
+	    cfg.createSection(sectName);
+	    layer = cfg.getSection(sectName);
+	}
+	layer->copyParams(name);
+	cfg.save();
+    }
+    if (!layer) {
 	return 0;
-    String confSec = layer->getValue("session","default");
+    }
+    String confSec = layer->getValue("session","session");
+    NamedList recvSec(confSec);
+    recvSec.copySubParams(*layer,confSec + ".");
     NamedList* section = cfg.getSection(confSec);
+    if (recvSec.count()) {
+	if (section)
+	    section->clearParams();
+	else {
+	    cfg.createSection(confSec);
+	    section = cfg.getSection(confSec);
+	}
+	section->copyParams(recvSec);
+	cfg.save();
+    }
     if (!section) {
-	Debug(DebugWarn,"Session '%s' does not exist in configuration",confSec.c_str());
+	Debug(&plugin,DebugConf,"Session '%s' does not exist in configuration",confSec.c_str());
 	return 0;
     }
     layer->copyParams(*section);
