@@ -51,8 +51,10 @@ class DbQuery;
 class DbQueryList;
 class MySqlConn;
 class MyAcct;
+class InitThread;
 
 static ObjList s_conns;
+static unsigned int s_failedConns;
 Mutex s_acctMutex(false,"MySQL::accts");
 
 /**
@@ -74,13 +76,6 @@ public:
     ~MyConn();
 
     void closeConn();
-
-    inline void setInit(bool val = true)
-	{ m_init = val; }
-
-    inline bool isInitialized()
-	{ return m_init; }
-
     void runQueries();
     int queryDbInternal(DbQuery* query);
 
@@ -104,14 +99,43 @@ public:
     ~MyAcct();
 
     bool initDb();
+    int initConns();
     void dropDb();
     inline bool ok() const
 	{ return 0 != m_connections.skipNull(); }
 
     void appendQuery(DbQuery* query);
 
+    void incTotal();
+    void incFailed();
+    void incErrorred();
+    void incQueryTime(u_int64_t with);
+    void lostConn();
+    void resetConn();
+    inline unsigned int total()
+	{ return m_totalQueries; }
+    inline unsigned int failed()
+	{ return m_failedQueries; }
+    inline unsigned int errorred()
+	{ return m_errorQueries; }
+    inline bool hasConn()
+	{ return ((int)(m_poolSize - m_failedConns) > 0 ? true : false); }
+    inline unsigned int queryTime()
+        { return (unsigned int) m_queryTime; } //microseconds
+    inline void setRetryWhen()
+	{ m_retryWhen = Time::msecNow() + m_retryTime * 1000; }
+    inline u_int64_t retryWhen()
+	{ return m_retryWhen; }
+    inline bool shouldRetryInit()
+	{ return m_retryTime && m_connections.count() < (unsigned int)m_poolSize; }
+    inline int poolSize()
+	{ return m_poolSize; }
 private:
     unsigned int m_timeout;
+    // interval at which connection initialization should be tried
+    unsigned int m_retryTime;
+    // when should we try initialization again?
+    u_int64_t m_retryWhen;
 
     String m_host;
     String m_user;
@@ -128,6 +152,14 @@ private:
 
     Semaphore m_queueSem;
     Mutex m_queueMutex;
+
+    // stats counters
+    unsigned int m_totalQueries;
+    unsigned int m_failedQueries;
+    unsigned int m_errorQueries;
+    u_int64_t m_queryTime;
+    unsigned int m_failedConns;
+    Mutex m_incMutex;
 };
 
 /**
@@ -159,6 +191,19 @@ public:
     virtual bool received(Message& msg);
 };
 
+/**
+  * Class InitThread
+  * Running thread for initializing MySQL connections
+  */
+class InitThread : public Thread
+{
+public:
+    InitThread();
+    ~InitThread();
+    virtual void run();
+    virtual void cleanup();
+};
+
 /**
   * Class MyModule
   * The MySQL database module
@@ -168,9 +213,21 @@ class MyModule : public Module
 public:
     MyModule();
     ~MyModule();
+    void statusModule(String& str);
+    virtual bool received(Message& msg, int id);
+    InitThread* m_initThread;
+    inline void startInitThread()
+    { 
+	lock();
+	if (!m_initThread)
+	    (m_initThread = new InitThread())->startup();
+	unlock();
+    }
 protected:
     virtual void initialize();
     virtual void statusParams(String& str);
+    virtual void statusDetail(String& str);
+    virtual void genUpdate(Message& msg);
 private:
     bool m_init;
 };
@@ -218,7 +275,7 @@ MyConn::~MyConn()
 {
     m_conn = 0;
     //closeConn();
-    Debug(&module,DebugInfo,"Database connection '%s' destroyed",c_str());
+    Debug(&module,DebugAll,"Database connection '%s' destroyed",c_str());
 }
 
 void MyConn::closeConn()
@@ -229,7 +286,7 @@ void MyConn::closeConn()
     MYSQL* tmp = m_conn;
     m_conn = 0;
     mysql_close(tmp);
-    if(m_owner)
+    if (m_owner)
 	m_owner->m_connections.remove(this);
     Debug(&module,DebugInfo,"Database connection '%s' closed",c_str());
 }
@@ -244,6 +301,7 @@ void MyConn::runQueries()
 	DbQuery* query = static_cast<DbQuery*>(m_owner->m_queryQueue.remove(false));
 	if (!query)
 	    continue;
+	m_owner->incTotal();
 	mylock.drop();
 
 	DDebug(&module,DebugAll,"Connection '%s' will try to execute '%s'",
@@ -268,14 +326,25 @@ bool MyConn::testDb()
 //  return number of rows, -1 for error
 int MyConn::queryDbInternal(DbQuery* query)
 {
-    if (!testDb())
+    if (!testDb()) {
+ 	m_owner->lostConn();
+ 	m_owner->incFailed();
 	return -1;
+    }
+    m_owner->resetConn();
+    u_int64_t start = Time::now();
 
     if (mysql_real_query(m_conn,query->safe(),query->length())) {
 	Debug(&module,DebugWarn,"Query for '%s' failed: %s",c_str(),mysql_error(m_conn));
+	u_int64_t duration = Time::now() - start;
+	m_owner->incQueryTime(duration);
+	m_owner->incErrorred();
 	return -1;
     }
 
+#ifdef DEBUG
+    u_int64_t inter = Time::now();
+#endif
     int total = 0;
     unsigned int warns = 0;
     unsigned int affected = 0;
@@ -294,9 +363,15 @@ int MyConn::queryDbInternal(DbQuery* query)
 		query->m_msg->setParam("rows",String(rows));
 		Array *a = new Array(cols,rows+1);
 		unsigned int c;
-		// add field names
-		for (c = 0; c < cols; c++)
-		    a->set(new String(fields[c].name),c,0);
+		ObjList** columns = new ObjList*[cols];
+		// get top of columns and add field names
+		for (c = 0; c < cols; c++) {
+		    columns[c] = a->getColumn(c);
+		    if (columns[c])
+			columns[c]->set(new String(fields[c].name));
+		    else
+			Debug(&module,DebugGoOn,"No array for column %u",c);
+		}
 		// and now data row by row
 		for (unsigned int r = 1; r <= rows; r++) {
 		    MYSQL_ROW row = mysql_fetch_row(res);
@@ -304,7 +379,10 @@ int MyConn::queryDbInternal(DbQuery* query)
 			break;
 		    unsigned long* len = mysql_fetch_lengths(res);
 		    for (c = 0; c < cols; c++) {
-			if (!row[c])
+			// advance pointer in each column
+			if (columns[c])
+			    columns[c] = columns[c]->next();
+			if (!(columns[c] && row[c]))
 			    continue;
 			bool binary = false;
 			switch (fields[c].type) {
@@ -319,12 +397,13 @@ int MyConn::queryDbInternal(DbQuery* query)
 			if (binary) {
 			    if (!len)
 				continue;
-			    a->set(new DataBlock(row[c],len[c]),c,r);
+			    columns[c]->set(new DataBlock(row[c],len[c]));
 			}
 			else
-			    a->set(new String(row[c]),c,r);
+			    columns[c]->set(new String(row[c]));
 		    }
 		}
+		delete[] columns;
 		query->m_msg->userData(a);
 		a->deref();
 	    }
@@ -332,6 +411,14 @@ int MyConn::queryDbInternal(DbQuery* query)
 	}
     } while (!mysql_next_result(m_conn));
 
+    u_int64_t finish = Time::now();
+    m_owner->incQueryTime(finish - start);
+#ifdef DEBUG
+    Debug(&module,DebugAll,"Query time for '%s' is %u+%u ms",c_str(),
+	(unsigned int)((inter-start+500)/1000),
+	(unsigned int)((finish-inter+500)/1000));
+#endif
+
     if (query->m_msg) {
 	query->m_msg->setParam("affected",String(affected));
 	if (warns)
@@ -347,7 +434,10 @@ MyAcct::MyAcct(const NamedList* sect)
     : String(*sect),
       Mutex(true,"MySQL::acct"),
       m_queueSem(MAX_CONNECTIONS,"MySQL::queue"),
-      m_queueMutex(false,"MySQL::queue")
+      m_queueMutex(false,"MySQL::queue"),
+      m_totalQueries(0), m_failedQueries(0), m_errorQueries(0),
+      m_queryTime(0), m_failedConns(0),
+      m_incMutex(false,"MySQL::inc")
 {
     int tout = sect->getIntValue("timeout",10000);
     // round to seconds
@@ -371,6 +461,9 @@ MyAcct::MyAcct(const NamedList* sect)
 	m_poolSize = MAX_CONNECTIONS;
     Debug(&module, DebugNote, "For account '%s' connection pool size is %d",
 	c_str(),m_poolSize);
+    
+    m_retryTime = sect->getIntValue("initretry",10); // default value is 10 seconds
+    setRetryWhen(); // set retry interval
 }
 
 MyAcct::~MyAcct()
@@ -381,33 +474,22 @@ MyAcct::~MyAcct()
     dropDb();
 }
 
-// initialize the database connection
-bool MyAcct::initDb()
+int MyAcct::initConns()
 {
-    Lock lock(this);
-    // allow specifying the raw connection string
-    Debug(&module,DebugNote,"Initiating pool of %d connections for '%s'",
-	m_poolSize,c_str());
-
-    m_queueSem.lock();
-
-    int initConns = 0;
-
-    s_libMutex.lock();
-    if (0 == s_libCounter++) {
-	DDebug(&module,DebugAll,"Initializing the MySQL library");
-	mysql_library_init(0,0,0);
-    }
-    s_libMutex.unlock();
-
-    for (int i = 0; i < m_poolSize; i++) {
+    int count = m_connections.count();
+    
+    DDebug(&module,DebugInfo,"MyAcct::initConns() - %d connections initialized already, pool required is of %d connections for '%s'",
+	count,m_poolSize,c_str());
+    // set new retry interval
+    setRetryWhen();
+    
+    for (int i = count; i < m_poolSize; i++) {
 	MyConn* mySqlConn = new MyConn(toString() + "." + String(i), this);
 
 	mySqlConn->m_conn = mysql_init(mySqlConn->m_conn);
 	if (!mySqlConn->m_conn) {
 	    Debug(&module,DebugGoOn,"Could not start connection %d for '%s'",i,c_str());
-	    mysql_thread_end();
-	    continue;
+	    return i;
 	}
 	DDebug(&module,DebugAll,"Connection '%s' for account '%s' was created",mySqlConn->c_str(),c_str());
 
@@ -442,26 +524,51 @@ bool MyAcct::initDb()
 #endif
 	    DbThread* thread = new DbThread(mySqlConn);
 
-	    if (thread->startup()) {
-		mySqlConn->setInit();
+	    if (thread->startup())
 		m_connections.append(mySqlConn);
-		initConns++;
-	    }
 	    else {
 	    	delete thread;
 		TelEngine::destruct(mySqlConn);
 	    }
 	}
+	else {
+	    TelEngine::destruct(mySqlConn);
+	    return i;
+	}
     }
-    if (!initConns) {
-	Debug(&module,DebugWarn,"Could not initiate any connections for account '%s'",
-	    c_str());
-	return false;
-    }
-    if (initConns != m_poolSize)
-	Debug(&module,DebugMild,"Could initiate only %d of %d connections for account '%s'",
-	    initConns,m_poolSize,c_str());
 
+    return m_poolSize;
+}
+
+// initialize the database connection
+bool MyAcct::initDb()
+{
+    Lock lock(this);
+    // allow specifying the raw connection string
+    Debug(&module,DebugNote,"Initiating pool of %d connections for '%s'",
+	m_poolSize,c_str());
+
+    m_queueSem.lock();
+
+    s_libMutex.lock();
+    if (0 == s_libCounter++) {
+	DDebug(&module,DebugAll,"Initializing the MySQL library");
+	mysql_library_init(0,0,0);
+    }
+    s_libMutex.unlock();
+    
+    int initCons = initConns();
+    if (!initCons) {
+	Debug(&module,DebugWarn,"Could not initiate any connections for account '%s', trying again in %d seconds",
+	    c_str(),m_retryTime);
+	module.startInitThread();
+	return true;
+    }
+    if (initCons != m_poolSize) {
+	Debug(&module,DebugMild,"Could initiate only %d of %d connections for account '%s', trying again in %d seconds",
+	    initCons,m_poolSize,c_str(),m_retryTime);
+	module.startInitThread();
+    }
     return true;
 }
 
@@ -487,6 +594,60 @@ void MyAcct::dropDb()
     s_libMutex.unlock();
 }
 
+void MyAcct::incTotal()
+{
+    XDebug(&module,DebugAll,"MyAcct::incTotal() [%p] - currently there have been %d queries",this,m_totalQueries);
+    m_incMutex.lock();
+    m_totalQueries++;
+    m_incMutex.unlock();
+    module.changed();
+}
+
+void MyAcct::incFailed()
+{
+    XDebug(&module,DebugAll,"MyAcct::incfailed() [%p] - currently there have been %d failed queries",this,m_failedQueries);
+    m_incMutex.lock();
+    m_failedQueries++;
+    m_incMutex.unlock();
+    module.changed();
+}
+
+void MyAcct::incErrorred()
+{
+    XDebug(&module,DebugAll,"MyAcct::incErrorred() [%p] - currently there have been %d errorred queries",this,m_errorQueries);
+    m_incMutex.lock();
+    m_errorQueries++;
+    m_incMutex.unlock();
+    module.changed();
+}
+
+void MyAcct::incQueryTime(u_int64_t with)
+{
+    XDebug(&module,DebugAll,"MyAcct::incQueryTime(with=" FMT64 ") [%p]",with,this);
+    m_incMutex.lock();
+    m_queryTime += with;
+    m_incMutex.unlock();
+    module.changed();
+}
+
+void MyAcct::lostConn()
+{
+    DDebug(&module,DebugAll,"MyAcct::lostConn() [%p]",this);
+    m_incMutex.lock();
+    if (m_failedConns < (unsigned int) m_poolSize)
+	m_failedConns++;
+    m_incMutex.unlock();
+    module.changed();
+}
+
+void MyAcct::resetConn()
+{
+    DDebug(&module,DebugAll,"MyAcct::hasConn() [%p]",this);
+    m_incMutex.lock();
+    m_failedConns = 0;
+    m_incMutex.unlock();
+}
+
 void MyAcct::appendQuery(DbQuery* query)
 {
     DDebug(&module, DebugAll, "Account '%s' received a new query %p",c_str(),query);
@@ -502,10 +663,8 @@ void MyAcct::appendQuery(DbQuery* query)
 void DbThread::run()
 {
     mysql_thread_init();
-    if (m_conn->isInitialized()) {
-	m_conn->m_thread = this;
-	m_conn->runQueries();
-    }
+    m_conn->m_thread = this;
+    m_conn->runQueries();
 }
 
 void DbThread::cleanup()
@@ -561,11 +720,63 @@ bool MyHandler::received(Message& msg)
     return true;
 }
 
+/**
+  * InitThread
+  */
+InitThread::InitThread()
+    : Thread("Mysql Init")
+{
+    Debug(&module,DebugAll,"InitThread created [%p]",this);
+}
+
+InitThread::~InitThread()
+{
+    Debug(&module,DebugAll,"InitThread thread terminated [%p]",this);
+    module.lock();
+    module.m_initThread = 0;
+    module.unlock();
+}
+
+void InitThread::run()
+{
+    mysql_thread_init();
+    while(!Engine::exiting()) {
+	Thread::sleep(1,true);
+	bool retryAgain = false;
+	s_acctMutex.lock();
+	for (ObjList* o = s_conns.skipNull(); o; o = o->next()) {
+	    MyAcct* acc = static_cast<MyAcct*>(o->get());
+	    if (acc->shouldRetryInit() && acc->retryWhen() <= Time::msecNow()) {
+		int count = acc->initConns();
+		if (count < acc->poolSize())
+		    Debug(&module,(count ? DebugMild : DebugWarn),"Account '%s' has %d initialized connections out of "
+			" a pool of %d",acc->c_str(),count,acc->poolSize());
+		else
+		    Debug(&module,DebugInfo,"All connections for account '%s' have been initialized, pool size is %d",
+			  acc->c_str(),acc->poolSize());
+	    }
+	    if (acc->shouldRetryInit())
+		retryAgain = true;
+	}
+	s_acctMutex.unlock();
+	if (!retryAgain)
+	    break;
+    }
+}
+
+void InitThread::cleanup()
+{
+    Debug(&module,DebugInfo,"InitThread::cleanup() [%p]",this);
+    mysql_thread_end();
+}
+
 /**
   * MyModule
   */
 MyModule::MyModule()
-    : Module ("mysqldb","database",true),m_init(true)
+    : Module ("mysqldb","database",true),
+      m_initThread(0),
+      m_init(true)
 {
     Output("Loaded module MySQL based on %s",mysql_get_client_info());
 }
@@ -574,11 +785,34 @@ MyModule::~MyModule()
 {
     Output("Unloading module MySQL");
     s_conns.clear();
+    s_failedConns = 0;
+    // Wait for expire thread termination
+    while (m_initThread)
+	Thread::idle();
+}
+
+void MyModule::statusModule(String& str)
+{
+    Module::statusModule(str);
+    str.append("format=Total|Failed|Errors|AvgExecTime",",");
 }
 
 void MyModule::statusParams(String& str)
 {
     str.append("conns=",",") << s_conns.count();
+    str.append("failed=",",") << s_failedConns;
+}
+
+void MyModule::statusDetail(String& str)
+{
+    for (unsigned int i = 0; i < s_conns.count(); i++) {
+	MyAcct* acc = static_cast<MyAcct*>(s_conns[i]);
+	str.append(acc->c_str(),",") << "=" << acc->total() << "|" << acc->failed() << "|" << acc->errorred() << "|";
+	if (acc->total() - acc->failed() > 0)
+	    str << (acc->queryTime() / (acc->total() - acc->failed()) / 1000); //miliseconds
+        else
+	    str << "0";
+    }
 }
 
 void MyModule::initialize()
@@ -588,7 +822,9 @@ void MyModule::initialize()
     Configuration cfg(Engine::configFile("mysqldb"));
     if (m_init)
 	Engine::install(new MyHandler(cfg.getIntValue("general","priority",100)));
+    installRelay(Halt);
     m_init = false;
+    s_failedConns = 0;
     for (unsigned int i = 0; i < cfg.sections(); i++) {
 	NamedList* sec = cfg.getSection(i);
 	if (!sec || (*sec == "general"))
@@ -604,11 +840,39 @@ void MyModule::initialize()
 
 	conn = new MyAcct(sec);
 	s_conns.insert(conn);
-	if (!conn->initDb())
+	if (!conn->initDb()) {
 	    s_conns.remove(conn);
+	    s_failedConns++;
+	}
     }
 }
 
+bool MyModule::received(Message& msg, int id)
+{
+    if (id == Halt) {
+	if (m_initThread)
+	    m_initThread->cancel(true);
+    }
+    return Module::received(msg,id);
+}
+
+void MyModule::genUpdate(Message& msg)
+{
+    Lock lock(this);
+    unsigned int index = 0;
+    for (ObjList* o = s_conns.skipNull(); o; o = o->next()) {
+	MyAcct* acc = static_cast<MyAcct*>(o->get());
+	msg.setParam(String("database.") << index,acc->toString());
+	msg.setParam(String("total.") << index,String(acc->total()));
+	msg.setParam(String("failed.") << index,String(acc->failed()));
+	msg.setParam(String("errorred.") << index,String(acc->errorred()));
+	msg.setParam(String("hasconn.") << index,String::boolText(acc->hasConn()));
+	msg.setParam(String("querytime.") << index,String(acc->queryTime()));
+	index++;
+    }
+    msg.setParam("count",String(index));
+}
+
 }; // anonymous namespace
 
 /* vi: set ts=8 sw=4 sts=4 noet: */
