@@ -33,7 +33,8 @@ class YPBX_API PBXAssist : public ChanAssist
 {
 public:
     inline PBXAssist(ChanAssistList* list, const String& id, bool pass, bool guest)
-	: ChanAssist(list,id), m_last(0), m_pass(pass), m_guest(guest),
+	: ChanAssist(list,id),
+	  m_last(0), m_pass(pass), m_guest(guest), m_first(true),
 	  m_state("new"), m_keep("")
 	{ Debug(list,DebugCall,"Created%s assistant for '%s'",guest ? " guest" : "",id.c_str()); }
     virtual void msgStartup(Message& msg);
@@ -42,6 +43,9 @@ public:
     virtual bool msgDisconnect(Message& msg, const String& reason);
     virtual bool msgTone(Message& msg);
     virtual bool msgOperation(Message& msg, const String& operation);
+    void chanReplaced(const String& initial, const String& final);
+    inline void statusDetail(String& str) const
+	{ str.append(id() + "=" + m_state + "|" + m_tones,","); }
 protected:
     inline const String& state() const
 	{ return m_state; }
@@ -57,6 +61,7 @@ protected:
     u_int64_t m_last;
     bool m_pass;
     bool m_guest;
+    bool m_first;
     String m_tones;
     String m_peer1;
     String m_room;
@@ -76,13 +81,17 @@ private:
     bool operTransfer(Message& msg);
     bool operDoTransfer(Message& msg);
     bool operForTransfer(Message& msg);
+    // post conference operations
+    void postConference(Message& msg, int users, bool created);
+    void postConference(Message& msg, const String& name, int users);
 };
 
 class YPBX_API PBXList : public ChanAssistList
 {
 public:
     enum {
-	Operation = AssistPrivate
+	Operation = AssistPrivate,
+	Replaced,
     };
     inline PBXList()
 	: ChanAssistList("pbxassist")
@@ -91,7 +100,15 @@ public:
     virtual ChanAssist* create(Message& msg, const String& id);
     virtual void initialize();
     virtual void init(int priority);
+    virtual bool received(Message& msg, int id);
     virtual bool received(Message& msg, int id, ChanAssist* assist);
+protected:
+    void statusModule(String& str);
+    void statusParams(String& str);
+    void statusDetail(String& str);
+private:
+    void chanReplaced(const NamedList& params);
+    void chanReplaced(const String& initial, const String& final);
 };
 
 // assist all channels by default?
@@ -133,6 +150,18 @@ static String s_onhold;
 // error beep override source name
 static String s_error;
 
+// Language parameter for tones played by the pbx
+static String s_lang;
+
+// Drop conference on hangup
+static bool s_dropConfHangup = true;
+
+// Make channels own the conference rooms
+static bool s_confOwner = false;
+
+// Conference lonely timeout
+static int s_lonely = 0;
+
 // on-hangup transfer list
 static ObjList s_transList;
 static Mutex s_transMutex(false,"PBXAssist::transfer");
@@ -175,8 +204,10 @@ ChanAssist* PBXList::create(Message& msg, const String& id)
 	    DDebug(this,DebugCall,"Creating assistant for '%s'",id.c_str());
 	    bool guest = false;
 	    if (!s_incoming) {
-		const NamedString* status = msg.getParam("status");
-		if (status && (*status == "incoming"))
+		const NamedString* dir = msg.getParam("direction");
+		if (TelEngine::null(dir))
+		    dir = msg.getParam("status");
+		if (dir && (*dir == "incoming"))
 		    guest = true;
 	    }
 	    return new PBXAssist(this,id,msg.getBoolValue("dtmfpass",s_pass),msg.getBoolValue("pbxguest",guest));
@@ -192,6 +223,7 @@ void PBXList::init(int priority)
     ChanAssistList::init(priority);
     installRelay(Tone,priority);
     Engine::install(new MessageRelay("chan.operation",this,Operation,priority));
+    Engine::install(new MessageRelay("chan.replaced",this,Replaced,priority));
 }
 
 void PBXList::initialize()
@@ -227,6 +259,10 @@ void PBXList::initialize()
     s_retake = s_cfg.getValue("general","retake","###");
     s_onhold = s_cfg.getValue("general","onhold","moh/default");
     s_error = s_cfg.getValue("general","error","tone/outoforder");
+    s_lang = s_cfg.getValue("general","lang");
+    s_dropConfHangup = s_cfg.getBoolValue("general","dropconfhangup",true);
+    s_confOwner = s_cfg.getBoolValue("general","confowner",false);
+    s_lonely = s_cfg.getIntValue("general","lonelytimeout");
     unlock();
     if (s_cfg.getBoolValue("general","enabled",false))
 	ChanAssistList::initialize();
@@ -248,6 +284,93 @@ bool PBXList::received(Message& msg, int id, ChanAssist* assist)
     }
 }
 
+// Handler for all messages
+bool PBXList::received(Message& msg, int id)
+{
+    if (Replaced == id) {
+	chanReplaced(msg);
+	return false;
+    }
+    return ChanAssistList::received(msg,id);
+}
+
+// Handler for chan.replaced
+void PBXList::chanReplaced(const NamedList& params)
+{
+    for (unsigned int i = 0; ; i++) {
+	String id("id");
+	String newId("newid");
+	if (i) {
+	    id << "." << i;
+	    newId << "." << i;
+	}
+	const String* initial = params.getParam(id);
+	if (!initial)
+	    break;
+	const String* final = params.getParam(newId);
+	if (!final)
+	    break;
+	if (*initial && *final)
+	    chanReplaced(*initial,*final);
+    }
+}
+
+// Operate single channel ID replacement
+void PBXList::chanReplaced(const String& initial, const String& final)
+{
+    DDebug(this,DebugAll,"Replacing '%s' with '%s'",initial.c_str(),final.c_str());
+    // replace in transfer list, test both sides
+    s_transMutex.lock();
+    for (ObjList* l = s_transList.skipNull(); l; l = l->skipNext()) {
+	NamedString* n = static_cast<NamedString*>(l->get());
+	if (initial == n->name()) {
+	    Debug(this,DebugInfo,"In transfer '%s' replaced '%s' with '%s'",
+		n->c_str(),initial.c_str(),final.c_str());
+	    const_cast<String&>(n->name()) = final;
+	}
+	if (initial == *n) {
+	    Debug(this,DebugInfo,"In transfer '%s' replaced '%s' with '%s'",
+		n->name().c_str(),initial.c_str(),final.c_str());
+	    *n = final;
+	}
+    }
+    s_transMutex.unlock();
+    // replace in assists
+    lock();
+    ListIterator iter(calls());
+    while (PBXAssist* assist = static_cast<PBXAssist*>(iter.get()))
+	assist->chanReplaced(initial,final);
+    unlock();
+}
+
+void PBXList::statusModule(String& str)
+{
+    Module::statusModule(str);
+    str.append("format=State|Keys",",");
+}
+
+void PBXList::statusParams(String& str)
+{
+    Module::statusParams(str);
+    lock();
+    str.append("assisted="+String(calls().count()),",");
+    unlock();
+    str << ",incoming=" << String::boolText(s_incoming);
+    str << ",dtmfpass=" << String::boolText(s_pass);
+    str << ",dialheld=" << String::boolText(s_dialHeld);
+    str << ",diversion=" << String::boolText(s_divProto);
+}
+
+void PBXList::statusDetail(String& str)
+{
+    Module::statusDetail(str);
+    lock();
+    ListIterator iter(calls());
+    while (const PBXAssist* assist = static_cast<const PBXAssist*>(iter.get()))
+	assist->statusDetail(str);
+    unlock();
+}
+
 
 // Change the state of the PBX assist
 void PBXAssist::setState(const String& newState)
@@ -299,12 +422,13 @@ bool PBXAssist::msgDisconnect(Message& msg, const String& reason)
 	return ChanAssist::msgDisconnect(msg,reason);
 
     if ((reason == "hold") || (reason == "park") || (reason == "intrusion")) {
-	if (s_onhold) {
+	String onhold = m_keep.getValue("onhold",s_onhold);
+	if (onhold) {
 	    Channel* c = static_cast<Channel*>(msg.userObject("Channel"));
 	    if (!c)
 		return false;
 	    Message *m = c->message("call.execute",false,true);
-	    m->addParam("callto",s_onhold);
+	    m->addParam("callto",onhold);
 	    m->addParam("reason",reason);
 	    m->addParam("pbxstate",state());
 	    Engine::enqueue(m);
@@ -415,6 +539,7 @@ bool PBXAssist::msgDisconnect(Message& msg, const String& reason)
 	    return false;
 	Message *m = c->message("call.execute",false,true);
 	m->addParam("callto","tone/dial");
+	m->addParam("lang",m_keep.getValue("pbxlang",s_lang),false);
 	m->addParam("reason","hold");
 	m->addParam("pbxstate",state());
 	Engine::enqueue(m);
@@ -457,6 +582,8 @@ bool PBXAssist::msgTone(Message& msg)
 	return true;
 
     Lock lock(list());
+    bool first = m_first;
+    m_first = false;
     int n = s_cfg.sections();
     for (int i = 0; i < n; i++) {
 	NamedList* sect = s_cfg.getSection(i);
@@ -473,7 +600,7 @@ bool PBXAssist::msgTone(Message& msg)
 	tmp = sect->getValue("pbxstates");
 	if (tmp) {
 	    Regexp st(tmp);
-	    if (!st.matches(state()))
+	    if (!(st.matches(state()) || (first && st.matches("first"))))
 		continue;
 	}
 
@@ -552,6 +679,7 @@ void PBXAssist::putPrompt(const char* source, const char* reason)
     m->addParam("id",id());
     m->addParam("pbxstate",state());
     m->addParam("override",source);
+    m->addParam("lang",m_keep.getValue("pbxlang",s_lang),false);
     m->addParam("single","yes");
     if (reason)
 	m->addParam("reason",reason);
@@ -654,9 +782,22 @@ bool PBXAssist::operSetState(Message& msg, const char* newState)
 	defState();
     else
 	setState(tmp);
-    m_room = msg.getValue("room",m_room);
     setGuest(msg);
     setParams(msg);
+    const String* room = msg.getParam("room");
+    if (room && (*room != m_room)) {
+	m_room = *room;
+	if (m_room && !m_guest && msg.getBoolValue("confowner",
+	    m_keep.getBoolValue("pbxconfowner",s_confOwner))) {
+	    Message m("call.conference");
+	    m.addParam("id",id());
+	    m.addParam("room",m_room);
+	    m.addParam("pbxstate",state());
+	    m.addParam("confowner",String::boolText(true));
+	    copyParams(m,msg);
+	    Engine::dispatch(m);
+	}
+    }
     return true;
 }
 
@@ -688,19 +829,25 @@ bool PBXAssist::operConference(Message& msg)
 	peer.clear();
     cancelTransfer(peer);
     const char* room = msg.getValue("room",m_room);
+    int users = 0;
+    bool created = false;
+    bool owner = msg.getBoolValue("confowner",m_keep.getBoolValue("pbxconfowner",s_confOwner));
 
     if (peer) {
 	Message m("call.conference");
 	m.addParam("id",id());
 	m.userData(c);
-	m.addParam("lonely","yes");
+	m.addParam("lonely",m_keep.getValue("pbxlonelytimeout",String(s_lonely)));
 	if (room)
 	    m.addParam("room",room);
 	m.addParam("pbxstate",state());
+	m.addParam("confowner",String::boolText(owner));
 	copyParams(m,msg);
 
 	if (Engine::dispatch(m) && m.userData()) {
 	    m_room = m.getParam("room");
+	    users = m.getIntValue("users");
+	    created = m.getBoolValue("newroom");
 	    if (m_peer1 && (m_peer1 != peer)) {
 		// take the held party in the conference
 		Message* m2 = new Message("chan.masquerade");
@@ -719,6 +866,8 @@ bool PBXAssist::operConference(Message& msg)
 		Engine::enqueue(m2);
 		// no longer holding it
 		m_peer1.clear();
+		if (users > 0)
+		    users++;
 	    }
 	}
 	else
@@ -735,10 +884,13 @@ bool PBXAssist::operConference(Message& msg)
 	c->complete(m,false);
 	m.addParam("callto",room);
 	m.addParam("pbxstate",state());
+	m.addParam("confowner",String::boolText(owner));
 	copyParams(m,msg);
 	if (!Engine::dispatch(m))
 	    return errorBeep("conference failed");
 	m_room = room;
+	users = m.getIntValue("users");
+	created = m.getBoolValue("newroom");
     }
 
     setState("conference");
@@ -752,6 +904,7 @@ bool PBXAssist::operConference(Message& msg)
 	m->addParam("pbxstate",state());
 	Engine::enqueue(m);
     }
+    postConference(msg,users,created);
     return true;
 }
 
@@ -764,6 +917,8 @@ bool PBXAssist::operSecondCall(Message& msg)
     m.copyParam(m_keep,"caller");
     m.addParam("called",msg.getValue("target"));
     m.addParam("pbxstate",state());
+    m.addParam("pbxoper",msg.getValue("operation"),false);
+    m.addParam("reason",msg.getValue("reason"),false);
     copyParams(m,msg,&m_keep);
     // no error check as handling preroute is optional
     Engine::dispatch(m);
@@ -805,6 +960,7 @@ bool PBXAssist::operOnHold(Message& msg)
 	    m_peer1.clear();
     }
 
+    const char* reason = msg.getValue("reason","hold");
     Message* m;
     if (c2) {
 	m = new Message("chan.operation");
@@ -812,14 +968,15 @@ bool PBXAssist::operOnHold(Message& msg)
 	m->addParam("id",c2->id());
 	m->addParam("state","*");
 	defState();
-	c->connect(c2,"hold");
+	c->connect(c2,reason);
     }
     else {
 	m = new Message("chan.masquerade");
 	m->addParam("id",id());
 	m->addParam("callto","tone/dial");
 	m->addParam("message","call.execute");
-	m->addParam("reason","hold");
+	m->addParam("lang",m_keep.getValue("pbxlang",s_lang),false);
+	m->addParam("reason",reason,false);
 	copyParams(*m,msg);
 	setState("dial");
     }
@@ -840,7 +997,7 @@ bool PBXAssist::operReturnHold(Message& msg)
     cancelTransfer(c1->getPeerId());
     m_peer1.clear();
     defState();
-    c1->connect(c2);
+    c1->connect(c2,msg.getValue("reason","pickup"));
     return true;
 }
 
@@ -852,14 +1009,17 @@ bool PBXAssist::operReturnConf(Message& msg)
     Channel* c = static_cast<Channel*>(msg.userObject("Channel"));
     if (!c)
 	return errorBeep("no channel");
+    bool owner = msg.getBoolValue("confowner",m_keep.getBoolValue("pbxconfowner",s_confOwner));
     Message m("call.execute");
     m.userData(c);
     c->complete(m,false);
     m.addParam("callto",m_room);
     m.addParam("pbxstate",state());
+    m.addParam("confowner",String::boolText(owner));
     copyParams(m,msg);
     if (Engine::dispatch(m)) {
 	setState("conference");
+	postConference(msg,m.getIntValue("users"),m.getBoolValue("newroom"));
 	return true;
     }
     return errorBeep(m.getValue("reason",m.getValue("error","conference failed")));
@@ -870,9 +1030,11 @@ bool PBXAssist::operReturnTone(Message& msg, const char* reason)
 {
     cancelTransfer();
     setState(msg.getValue("state","dial"));
+    reason = msg.getValue("reason",reason);
     Message* m = new Message("chan.masquerade");
     m->addParam("id",id());
     m->addParam("callto","tone/dial");
+    m->addParam("lang",m_keep.getValue("pbxlang",s_lang),false);
     m->addParam("message","call.execute");
     m->addParam("pbxstate",state());
     if (reason)
@@ -914,7 +1076,10 @@ bool PBXAssist::operTransfer(Message& msg)
     // make call appear as from the other party
     m.addParam("caller",m_keep.getValue("called"));
     m.addParam("called",msg.getValue("target"));
+    m.addParam("diverter",m_keep.getValue("caller"),false);
     m.addParam("pbxstate",state());
+    m.addParam("pbxoper",msg.getValue("operation"),false);
+    m.addParam("reason",msg.getValue("reason"),false);
     copyParams(m,msg,&m_keep);
     // no error check as handling preroute is optional
     Engine::dispatch(m);
@@ -950,7 +1115,7 @@ bool PBXAssist::operDoTransfer(Message& msg)
     cancelTransfer();
     setState(msg.getValue("state","hangup"));
     m_peer1.clear();
-    c1->connect(c2);
+    c1->connect(c2,msg.getValue("reason","transfer"));
     return true;
 }
 
@@ -979,6 +1144,8 @@ bool PBXAssist::operForTransfer(Message& msg)
     m.copyParam(m_keep,"caller");
     m.addParam("called",msg.getValue("target"));
     m.addParam("pbxstate",state());
+    m.addParam("pbxoper",msg.getValue("operation"),false);
+    m.addParam("reason",msg.getValue("reason"),false);
     copyParams(m,msg,&m_keep);
     // no error check as handling preroute is optional
     Engine::dispatch(m);
@@ -1008,6 +1175,33 @@ bool PBXAssist::operForTransfer(Message& msg)
     return errorBeep(m.getValue("reason",m.getValue("error","call failed")));
 }
 
+// Execute secondary operation after entering conference
+void PBXAssist::postConference(Message& msg, int users, bool created)
+{
+    if (created) {
+	postConference(msg,"opercreate",users);
+	postConference(msg,"opercreate" + String(users),users);
+    }
+    if (users <= 0)
+	return;
+    postConference(msg,"operusers" + String(users),users);
+}
+
+// Try to execute a single secondary operation
+void PBXAssist::postConference(Message& msg, const String& name, int users)
+{
+    const char* oper = msg.getValue(name,m_keep.getValue(name));
+    if (TelEngine::null(oper))
+	return;
+    Message* m = new Message(msg);
+    m->setParam("operation",oper);
+    m->setParam("pbxstate",state());
+    m->setParam("room",m_room);
+    m->setParam("users",String(users));
+    m->userData(msg.userData());
+    Engine::enqueue(m);
+}
+
 // Hangup handler, do any cleanups needed
 void PBXAssist::msgHangup(Message& msg)
 {
@@ -1030,7 +1224,8 @@ void PBXAssist::msgHangup(Message& msg)
 	l = l->skipNext();
     }
     s_transMutex.unlock();
-    if (m_room && (state() == "conference")) {
+    if (m_room && (state() == "conference") &&
+	m_keep.getBoolValue("pbxdropconfhangup",s_dropConfHangup)) {
 	// hangup the conference if we didn't switch out of it
 	Message* m = new Message("call.drop");
 	m->addParam("id",m_room);
@@ -1049,11 +1244,10 @@ void PBXAssist::msgHangup(Message& msg)
 		m = new Message("call.route");
 		m->addParam("id",m_peer1);
 		m->copyParam(m_keep,"billid");
+		m->addParam("caller",m_keep.getValue("called"),false);
 		m->addParam("called",tmp);
-		tmp = m_keep.getValue("called");
-		if (tmp)
-		    m->addParam("caller",tmp);
 		m->addParam("pbxstate",state());
+		m->addParam("reason","onhold");
 		if (!Engine::dispatch(m) || m->retValue().null() || (m->retValue() == "-") || (m->retValue() == "error"))
 		    TelEngine::destruct(m);
 		else {
@@ -1112,6 +1306,16 @@ void PBXAssist::msgExecute(Message& msg)
     m_keep.copyParam(msg,"divert",'_');
 }
 
+// Adjust any channel ID that has been replaced
+void PBXAssist::chanReplaced(const String& initial, const String& final)
+{
+    if (initial == m_peer1) {
+	Debug(list(),DebugInfo,"For '%s' replacing peer '%s' with '%s'",
+	    id().c_str(),initial.c_str(),final.c_str());
+	m_peer1 = final;
+    }
+}
+
 INIT_PLUGIN(PBXList);
 
 }; // anonymous namespace
