@@ -32,19 +32,34 @@ namespace { // anonymous
 
 static ObjList s_conns;
 Mutex s_conmutex(false,"PgSQL::conn");
+static unsigned int s_failedConns;
 
-class PgConn : public GenObject, public Mutex
+class PgConn : public RefObject, public Mutex
 {
 public:
     PgConn(const NamedList* sect);
     ~PgConn();
     virtual const String& toString() const
 	{ return m_name; }
+    virtual void destroyed();
 
     bool ok();
     int queryDb(const char* query, Message* dest = 0);
     bool initDb(int retry = 0);
 
+    inline unsigned int total()
+	{ return m_totalQueries; }
+    inline unsigned int failed()
+	{ return m_failedQueries; }
+    inline unsigned int errorred()
+	{ return m_errorQueries; }
+    inline unsigned int queryTime()
+        { return (unsigned int) m_queryTime; }
+    inline void setConn(bool conn = true)
+	{ m_hasConn = conn; }
+    inline bool hasConn()
+	{ return m_hasConn; }
+
 private:
     void dropDb();
     bool testDb();
@@ -55,6 +70,13 @@ private:
     int m_retry;
     u_int64_t m_timeout;
     PGconn *m_conn;
+
+    // stat counters
+    unsigned int m_totalQueries;
+    unsigned int m_failedQueries;
+    unsigned int m_errorQueries;
+    u_int64_t m_queryTime;
+    bool m_hasConn;
 };
 
 class PgHandler : public MessageHandler
@@ -73,7 +95,10 @@ public:
     ~PgModule();
 protected:
     virtual void initialize();
+    virtual void statusModule(String& str);
     virtual void statusParams(String& str);
+    virtual void statusDetail(String& str);
+    virtual void genUpdate(Message& msg);
 private:
     bool m_init;
 };
@@ -82,7 +107,9 @@ static PgModule module;
 
 PgConn::PgConn(const NamedList* sect)
     : Mutex(true,"PgConn"),
-      m_name(*sect), m_conn(0)
+      m_name(*sect), m_conn(0),
+      m_totalQueries(0), m_failedQueries(0),
+      m_errorQueries(0), m_queryTime(0), m_hasConn(false)
 {
     m_connection = sect->getValue("connection");
     if (m_connection.null()) {
@@ -108,10 +135,16 @@ PgConn::PgConn(const NamedList* sect)
 }
 
 PgConn::~PgConn()
-{ 
+{
+}
+
+void PgConn::destroyed()
+{
+    s_conmutex.lock();
     s_conns.remove(this,false);
+    s_conmutex.unlock();
     dropDb();
-    Debug(&module,DebugInfo,"Database connection %s closed",m_name.c_str());
+    Debug(&module,DebugInfo,"Database account '%s' destroyed",m_name.c_str());
 }
 
 // initialize the database connection and handler data
@@ -170,12 +203,14 @@ bool PgConn::initDb(int retry)
 // drop the connection
 void PgConn::dropDb()
 {
-    Lock lock(this);
+    if (!m_conn)
+	return;
+    Lock mylock(this);
     if (!m_conn)
 	return;
     PGconn* tmp = m_conn;
     m_conn = 0;
-    lock.drop();
+    mylock.drop();
     PQfinish(tmp);
 }
 
@@ -188,13 +223,14 @@ bool PgConn::testDb()
 // public, thread safe version
 bool PgConn::ok()
 {
-    Lock lock(this);
-    return testDb();
+    Lock mylock(this,m_timeout);
+    return mylock.locked() && testDb();
 }
 
 // try to get up the connection, retry if we have to
 bool PgConn::startDb()
 {
+    setConn(true);
     if (testDb())
 	return true;
     for (int i = 0; i < m_retry; i++) {
@@ -204,6 +240,7 @@ bool PgConn::startDb()
 	if (testDb())
 	    return true;
     }
+    setConn(false);
     return false;
 }
 
@@ -211,7 +248,12 @@ bool PgConn::startDb()
 //  return number of rows, -1 for non-retryable errors and -2 to retry
 int PgConn::queryDbInternal(const char* query, Message* dest)
 {
-    Lock lock(this);
+    Lock mylock(this,m_timeout);
+    if (!mylock.locked()) {
+	Debug(&module,DebugWarn,"Failed to lock '%s' for " FMT64U " usec",
+	    m_name.c_str(),m_timeout);
+	return -1;
+    }
     if (!startDb())
 	// no retry - startDb already tried and failed...
 	return -1;
@@ -267,9 +309,20 @@ int PgConn::queryDbInternal(const char* query, Message* dest)
 			if (dest->getBoolValue("results",true) && !PQbinaryTuples(res)) {
 			    Array *a = new Array(columns,rows+1);
 			    for (int k = 0; k < columns; k++) {
-				String *f= new String(PQfname(res,k));
-				a->set(f,k,0);
+				ObjList* column = a->getColumn(k);
+				if (column)
+				    column->set(new String(PQfname(res,k)));
+				else {
+				    Debug(&module,DebugGoOn,"No array column for %d",k);
+				    continue;
+				}
 				for (int j = 0; j < rows; j++) {
+				    column = column->next();
+				    if (!column) {
+					// Stop now: we won't get the next row
+					Debug(&module,DebugGoOn,"No array row %d in column %d",j + 1,k);
+					break;
+				    }
 				    // skip over NULL values
 				    if (PQgetisnull(res,j,k))
 					continue;
@@ -278,7 +331,7 @@ int PgConn::queryDbInternal(const char* query, Message* dest)
 					v = new DataBlock(PQgetvalue(res,j,k),PQgetlength(res,j,k));
 				    else
 					v = new String(PQgetvalue(res,j,k));
-				    a->set(v,k,j+1);
+				    column->set(v);
 				}
 			    }
 			    dest->userData(a);
@@ -299,11 +352,13 @@ int PgConn::queryDbInternal(const char* query, Message* dest)
 	    default:
 		Debug(&module,DebugWarn,"Query error: %s",PQresultErrorMessage(res));
 		dest->setParam("error",PQresultErrorMessage(res));
+		m_errorQueries++;
+		module.changed();
 	}
 	PQclear(res);
     }
     Debug(&module,DebugWarn,"Query timed out for '%s'",m_name.c_str());
-    dest->setParam("error","query timeout");	    
+    dest->setParam("error","query timeout");
     dropDb();
     return -2;
 }
@@ -321,41 +376,48 @@ int PgConn::queryDb(const char* query, Message* dest)
 	return -1;
     Debug(&module,DebugAll,"Performing query \"%s\" for '%s'",
 	query,m_name.c_str());
+    m_totalQueries++;
+    module.changed();
+    u_int64_t start = Time::now();
     for (int i = 0; i < m_retry; i++) {
 	int res = queryDbInternal(query,dest);
 	if (res > -2) {
-	    if (res < 0)
+	    if (res < 0) {
 		failure(dest);
+		m_failedQueries++;
+		module.changed();
+	    }
 	    // ok or non-retryable error, get out of here
+	    u_int64_t finish = Time::now() - start;
+	    m_queryTime += finish;
+	    module.changed();
 	    return res;
-	} 
+	}
     }
     failure(dest);
     return -2;
 }
 
-
-static PgConn* findDb(String& account)
+static PgConn* findDb(const String& account)
 {
     if (account.null())
 	return 0;
-    ObjList* l = s_conns.find(account);
-    return l ? static_cast<PgConn *>(l->get()): 0;
+    return static_cast<PgConn*>(s_conns[account]);
 }
 
 bool PgHandler::received(Message& msg)
 {
-    String tmp(msg.getValue("account"));
-    if (tmp.null())
+    const String* str = msg.getParam("account");
+    if (TelEngine::null(str))
 	return false;
-    Lock lock(s_conmutex);
-    PgConn* db = findDb(tmp);
+    s_conmutex.lock();
+    RefPointer<PgConn> db = findDb(*str);
+    s_conmutex.unlock();
     if (!db)
 	return false;
-    Lock lo(db);
-    lock.drop();
-    String query(msg.getValue("query"));
-    db->queryDb(query,&msg);
+    str = msg.getParam("query");
+    if (!TelEngine::null(str))
+	db->queryDb(*str,&msg);
     msg.setParam("dbtype","pgsqldb");
     return true;
 }
@@ -372,9 +434,33 @@ PgModule::~PgModule()
     s_conns.clear();
 }
 
+void PgModule::statusModule(String& str)
+{
+    Module::statusModule(str);
+    str.append("format=Total|Failed|Errors|AvgExecTime",",");
+}
+
 void PgModule::statusParams(String& str)
 {
+    s_conmutex.lock();
     str.append("conns=",",") << s_conns.count();
+    str.append("failed=",",") << s_failedConns;
+    s_conmutex.unlock();
+}
+
+void PgModule::statusDetail(String& str)
+{
+    s_conmutex.lock();
+    for (unsigned int i = 0; i < s_conns.count(); i++) {
+	PgConn* conn = static_cast<PgConn*>(s_conns[i]);
+	str.append(conn->toString().c_str(),",") << "=" << conn->total() << "|" << conn->failed()
+			<< "|" << conn->errorred() << "|";
+	if (conn->total() - conn->failed() > 0)
+	    str << (conn->queryTime() / (conn->total() - conn->failed()) / 1000); //miliseconds
+        else
+	    str << "0";
+    }
+    s_conmutex.unlock();
 }
 
 void PgModule::initialize()
@@ -394,11 +480,31 @@ void PgModule::initialize()
 	PgConn* conn = new PgConn(sec);
 	if (sec->getBoolValue("autostart",true))
 	    conn->initDb();
-	s_conns.insert(conn);
+	s_conmutex.lock();
+	if (conn->ok())
+	    s_conns.insert(conn);
+	else
+	    s_failedConns++;
+	s_conmutex.unlock();
     }
 
 }
 
+void PgModule::genUpdate(Message& msg)
+{
+    unsigned int index = 0;
+    for (ObjList* o = s_conns.skipNull(); o; o = o->next()) {
+	PgConn* conn = static_cast<PgConn*>(o->get());
+	msg.setParam(String("database.") << index,conn->toString());
+	msg.setParam(String("total.") << index,String(conn->total()));
+	msg.setParam(String("failed.") << index,String(conn->failed()));
+	msg.setParam(String("errorred.") << index,String(conn->errorred()));
+	msg.setParam(String("hasconn.") << index,String::boolText(conn->hasConn()));
+	msg.setParam(String("querytime.") << index,String(conn->queryTime()));
+	index++;
+    }
+    msg.setParam("count",String(index));
+}
 }; // anonymous namespace
 
 /* vi: set ts=8 sw=4 sts=4 noet: */
