@@ -25,8 +25,6 @@
 
 #include <yatephone.h>
 
-#include <stdlib.h>                     // random()
-
 using namespace TelEngine;
 namespace { // anonymous
 
@@ -258,8 +256,8 @@ void QueuedCall::setDestroy(u_int64_t now)
 // Add prio to a list
 void QueuedCall::addPriority(NamedList& params)
 {
-    m_queuePrio = ::random() % 10;
-    m_callerPrio = ::random() % 10;
+    m_queuePrio = Random::random() % 10;
+    m_callerPrio = Random::random() % 10;
     int prio = s_coefQueuePriority  * m_queuePrio + s_coefCallPriority * m_callerPrio;
     params.addParam("priority",String(prio));
 }
@@ -269,15 +267,18 @@ Message* QueuedCall::buildResNotify(int stat)
 {
     Message* m = new Message("resource.notify");
     m->addParam("module",__plugin.name());
+    if (stat != QueuedCall::Hangup)
+	m->addParam("operation","online");
+    else
+	m->addParam("operation","offline");
     m->addParam("account",m_queue);
     m->addParam("username",m_queue);
     String res;
     if (s_addNodeToResource)
 	res << Engine::nodeName() + "/";
     res << m_channelid;
-    m->addParam("resource",res);
-    m->addParam("subscription",String::boolText(true));
-    m->addParam("status",resNotifStatus(stat));
+    m->addParam("instance",res);
+    m->addParam("show",resNotifStatus(stat));
     return m;
 }
 
