@@ -39,7 +39,12 @@ extern "C" {
 #include <wanpipe_defines.h>
 #include <wanpipe_cfg.h>
 #include <wanpipe.h>
+#ifdef NEW_WANPIPE_API
+#include <aft_core.h>
+#warning "The new Sangoma API support is experimental!"
+#else
 #include <sdla_aft_te1.h>
+#endif
 #else
 #include <linux/if_wanpipe.h>
 #include <linux/if.h>
@@ -54,8 +59,13 @@ extern "C" {
 #include <wanec_iface.h>
 #ifdef HAVE_WANPIPE_HWEC_API
 #include <wanec_iface_api.h>
+#ifdef NEW_WANPIPE_API
+#define WAN_EC_CMD_DTMF_ENABLE WAN_EC_API_CMD_TONE_ENABLE
+#define WAN_EC_CMD_DTMF_DISABLE WAN_EC_API_CMD_TONE_DISABLE
+#else
 #define WAN_EC_CMD_DTMF_ENABLE WAN_EC_API_CMD_DTMF_ENABLE
 #define WAN_EC_CMD_DTMF_DISABLE WAN_EC_API_CMD_DTMF_DISABLE
+#endif
 #ifdef u_buffer_config
 #define HAVE_WANPIPE_HWEC_3310
 #endif
@@ -78,26 +88,29 @@ extern "C" {
 #include <sys/ioctl.h>
 #include <fcntl.h>
 
+#ifdef NEW_WANPIPE_API
+#define WP_HEADER WAN_MAX_HDR_SZ
+#else
 #define WP_HEADER 16
-
-#define WP_RD_ERROR    0
-#define WP_RD_STAMP_LO 1
-#define WP_RD_STAMP_HI 2
-
-#define WP_WR_TYPE     0
-#define WP_WR_FORCE    1
-
-#define WP_ERR_FIFO  0x01
-#define WP_ERR_CRC   0x02
-#define WP_ERR_ABORT 0x04
-// by default ignore ABORT and OVERFLOW conditions unrelated to current packet
-#define WP_ERR_MASK  (0xff & ~(WP_ERR_FIFO|WP_ERR_ABORT))
-
+#define WP_RD_ERROR  0
 #define WP_RPT_REPEAT 0                  // Repeat flag in header
 #define WP_RPT_LEN 1                     // Repeated data length
 #define WP_RPT_DATA 2                    // Repeated data offset in header
+#endif
+
 #define WP_RPT_MAXDATA 8                 // Max repeated data length
 
+#define WP_ERR_FIFO  (1 << WP_FIFO_ERROR_BIT)
+#define WP_ERR_CRC   (1 << WP_CRC_ERROR_BIT)
+#define WP_ERR_ABORT (1 << WP_ABORT_ERROR_BIT)
+#ifdef NEW_WANPIPE_API
+#define WP_ERR_FRAME (1 << WP_FRAME_ERROR_BIT)
+#define WP_ERR_DMA   (1 << WP_DMA_ERROR_BIT)
+#endif
+
+// by default ignore ABORT and OVERFLOW conditions unrelated to current packet
+#define WP_ERR_MASK  (0xff & ~(WP_ERR_FIFO|WP_ERR_ABORT))
+
 #define MAX_PACKET 1200
 
 #define MAX_READ_ERRORS 250              // WpSpan::run(): Display read error message
@@ -263,6 +276,7 @@ private:
     bool m_repeatCapable;                // HW repeat available
     Mutex m_repeatMutex;                 // Lock repeat buffer
     DataBlock m_repeatPacket;            // Packet to repeat
+    bool m_down;
 };
 
 // Read signalling data for WpInterface
@@ -326,6 +340,7 @@ public:
     virtual ~WpCircuit();
     virtual bool status(Status newStat, bool sync = false);
     virtual bool updateFormat(const char* format, int direction);
+    virtual bool setParam(const String& param, const String& value);
     virtual void* getObject(const String& name) const;
     inline bool validSource()
 	{ return 0 != m_sourceValid; }
@@ -343,6 +358,7 @@ private:
     WpConsumer* m_consumerValid;         // Circuit's consumer if reserved, otherwise: 0
     WpSource* m_source;
     WpConsumer* m_consumer;
+    String m_specialMode;
 };
 
 // Wanpipe B-channel group
@@ -350,7 +366,7 @@ class WpSpan : public SignallingCircuitSpan
 {
     friend class WpSpanThread;
 public:
-    WpSpan(const NamedList& params);
+    WpSpan(const NamedList& params, const char* debugname);
     virtual ~WpSpan();
     // Initialize data channel span. Return false on failure
     bool init(const NamedList& config, const NamedList& defaults, NamedList& params);
@@ -394,7 +410,7 @@ private:
     unsigned int m_count;                // Circuit count
     unsigned int m_first;                // First circuit code
     unsigned int m_samples;              // Sample count
-    unsigned char m_noData;              // Value to send when no data
+    unsigned int m_noData;               // Value to send on idle channels
     unsigned int m_buflen;               // Buffer length for sources/consumers
     // Used for data processing
     WpCircuit** m_circuits;              // The circuits belonging to this span
@@ -443,6 +459,26 @@ static bool s_repeatCapable = true;      // Global repeat packet capability
 static WpModule driver;
 
 
+static void sendModuleUpdate(bool& notifStat, int status, const String& device)
+{
+    Message* msg = new Message("module.update");
+    msg->addParam("module",driver.name());
+    msg->addParam("interface",device);
+    if(notifStat && status == SignallingInterface::LinkUp) {
+	msg->addParam("notify","interfaceUp");
+	notifStat = false;
+	Engine::enqueue(msg);
+	return;
+    }
+    if (!notifStat && status == SignallingInterface::LinkDown) {
+	msg->addParam("notify","interfaceDown");
+	notifStat = true;
+	Engine::enqueue(msg);
+	return;
+    }
+    TelEngine::destruct(msg);
+}
+
 /**
  * Fifo
  */
@@ -545,11 +581,16 @@ bool WpSocket::echoCancel(bool enable, unsigned long chanmap)
 	    ecapi.cmd = WAN_EC_CMD_DTMF_ENABLE;
 	    ecapi.verbose = WAN_EC_VERBOSE_EXTRA1;
 	    // event on start of tone, before echo canceller
+#ifdef NEW_WANPIPE_API
+	    ecapi.u_tone_config.type = WAN_EC_TONE_PRESENT;
+	    ecapi.u_tone_config.port_map = WAN_EC_CHANNEL_PORT_SOUT;
+#else
 	    ecapi.u_dtmf_config.type = WAN_EC_TONE_PRESENT;
 #ifdef HAVE_WANPIPE_HWEC_3310
 	    ecapi.u_dtmf_config.port_map = WAN_EC_CHANNEL_PORT_SOUT;
 #else
 	    ecapi.u_dtmf_config.port = WAN_EC_CHANNEL_PORT_SOUT;
+#endif
 #endif
 	}
 	else
@@ -589,7 +630,8 @@ bool WpSocket::dtmfDetect(bool enable)
 {
     bool ok = false;
 
-#ifdef HAVE_WANPIPE_HWEC
+    // TODO: new API has different event handling style, may need total rewrite
+#if defined(HAVE_WANPIPE_HWEC) && !defined(NEW_WANPIPE_API)
     api_tx_hdr_t a;
     ::memset(&a,0,sizeof(api_tx_hdr_t));
     a.wp_api_tx_hdr_event_type = WP_API_EVENT_DTMF;
@@ -732,6 +774,11 @@ bool WpSocket::updateLinkStatus()
 // Create WpInterface or WpSpan
 SignallingComponent* WpInterface::create(const String& type, const NamedList& name)
 {
+    const String* module = name.getParam("module");
+    if (module && *module != "wpcard") {
+	DDebug(&driver,DebugWarn,"We aren't the target for creating %s",type.c_str());
+	return 0;
+    }
     bool interface = false;
     if (type == "SignallingInterface")
 	interface = true;
@@ -743,7 +790,17 @@ SignallingComponent* WpInterface::create(const String& type, const NamedList& na
     Configuration cfg(Engine::configFile("wpcard"));
     const char* sectName = name.getValue((interface ? "sig" : "voice"),name.getValue("basename",name));
     NamedList* config = cfg.getSection(sectName);
-    if (!config) {
+    if (module) {
+	DDebug(&driver,DebugAll,"Replace config params in section %s",c_safe(sectName));
+	if (!config) {
+	    cfg.createSection(sectName);
+	    config = cfg.getSection(sectName);
+	}
+	config->copyParams(name);
+	if(!cfg.save())
+	    DDebug(&driver,DebugAll,"Failed to save configuration in file %s ",cfg.c_str());
+    }
+    else if (!config){
 	DDebug(&driver,DebugAll,"No section '%s' in configuration",c_safe(sectName));
 	return 0;
     }
@@ -765,7 +822,7 @@ SignallingComponent* WpInterface::create(const String& type, const NamedList& na
     }
     NamedList* general = cfg.getSection("general");
     NamedList dummy("general");
-    WpSpan* data = new WpSpan(name);
+    WpSpan* data = new WpSpan(name,sectName);
     if (data->init(*config,general?*general:dummy,(NamedList&)name))
 	return data;
     TelEngine::destruct(data);
@@ -797,8 +854,8 @@ WpInterface::~WpInterface()
 bool WpInterface::init(const NamedList& config, NamedList& params)
 {
     // Set socket card / device
-    m_socket.card(config);
-    const char* sig = config.getValue("siggroup");
+    m_socket.card(!params.null() ? params : config);
+    const char* sig = params.getValue("siggroup",config.getValue("siggroup"));
     if (!(sig && *sig)) {
 	Debug(this,DebugWarn,
 	    "Missing or invalid siggroup='%s' in configuration [%p]",
@@ -832,6 +889,7 @@ bool WpInterface::init(const NamedList& config, NamedList& params)
 	s << " hwrepeatcapable=" << String::boolText(m_repeatCapable);
 	Debug(this,DebugInfo,"D-channel: %s [%p]",s.c_str(),this);
     }
+    m_down = false;
     return true;
 }
 
@@ -869,9 +927,16 @@ bool WpInterface::transmitPacket(const DataBlock& packet, bool repeat, PacketTyp
 	if (m_repeatCapable) {
 	    if (packet.length() <= WP_RPT_MAXDATA) {
 		unsigned char* hdr = (unsigned char*)data.data();
+#ifdef NEW_WANPIPE_API
+		((wp_api_hdr_t*)hdr)->wp_api_tx_hdr_hdlc_rpt_repeat = 1;
+		((wp_api_hdr_t*)hdr)->wp_api_tx_hdr_hdlc_rpt_len = packet.length();
+		::memcpy(((wp_api_hdr_t*)hdr)->wp_api_tx_hdr_hdlc_rpt_data,
+		    packet.data(),packet.length());
+#else
 		hdr[WP_RPT_REPEAT] = 1;
 		hdr[WP_RPT_LEN] = packet.length();
 		::memcpy(hdr+WP_RPT_DATA,packet.data(),packet.length());
+#endif
 	    }
 	    else
 		Debug(this,DebugWarn,"Can't repeat packet (type=%u) with length=%u",
@@ -912,7 +977,11 @@ bool WpInterface::receiveAttempt()
 	XDebug(this,DebugAll,"Received %d bytes packet. Header length is %u [%p]",
 	    r,WP_HEADER + m_overRead,this);
 	r -= (WP_HEADER + m_overRead);
+#ifdef NEW_WANPIPE_API
+	unsigned char err = ((wp_api_hdr_t*)buf)->wp_api_rx_hdr_error_map;
+#else
 	unsigned char err = buf[WP_RD_ERROR];
+#endif
 	if (err != m_lastError) {
 	    m_lastError = err;
 	    if (err) {
@@ -1021,10 +1090,14 @@ bool WpInterface::updateStatus()
 	return false;
     Debug(this,DebugNote,"Link status changed to %s [%p]",
 	lookup(m_socket.status(),s_linkStatus),this);
-    if (m_socket.status() == WpSocket::Connected)
+    if (m_socket.status() == WpSocket::Connected) {
 	notify(LinkUp);
-    else
+	sendModuleUpdate(m_down,LinkUp,m_socket.card());
+    }
+    else {
 	notify(LinkDown);
+	sendModuleUpdate(m_down,LinkDown,m_socket.card());
+    }
     return true;
 }
 
@@ -1185,6 +1258,9 @@ bool WpCircuit::status(Status newStat, bool sync)
 	case Disabled:
 	case Idle:
 	case Reserved:
+	    m_specialMode.clear();
+	    // fall through
+	case Special:
 	case Connected:
 	    break;
 	default: ;
@@ -1206,7 +1282,7 @@ bool WpCircuit::status(Status newStat, bool sync)
     // Enable/disable data transfer
     clearEvents();
     bool enableData = false;
-    if (SignallingCircuit::status() == Connected)
+    if (SignallingCircuit::status() >= Special)
 	enableData = true;
     // Don't put this message for final states
     if (!Engine::exiting())
@@ -1215,6 +1291,17 @@ bool WpCircuit::status(Status newStat, bool sync)
     if (enableData) {
 	m_sourceValid = m_source;
 	m_consumerValid = m_consumer;
+	if (newStat == Special) {
+	    Message m("circuit.special");
+	    m.userData(this);
+	    if (group())
+		m.addParam("group",group()->toString());
+	    if (span())
+		m.addParam("span",span()->toString());
+	    if (m_specialMode)
+		m.addParam("mode",m_specialMode);
+	    return Engine::dispatch(m);
+	}
 	return true;
     }
     // Disable data if not already disabled
@@ -1272,6 +1359,15 @@ bool WpCircuit::updateFormat(const char* format, int direction)
     return consumerChanged && sourceChanged;
 }
 
+bool WpCircuit::setParam(const String& param, const String& value)
+{
+    if (param == "special_mode")
+	m_specialMode = value;
+    else
+	return false;
+    return true;
+}
+
 // Get source or consumer
 void* WpCircuit::getObject(const String& name) const
 {
@@ -1317,8 +1413,8 @@ unsigned char WpSpan::s_bitswap[256] = {
 
 // Initialize B-channel group
 // Create circuits. Start worker thread
-WpSpan::WpSpan(const NamedList& params)
-    : SignallingCircuitSpan(params.getValue("debugname"),
+WpSpan::WpSpan(const NamedList& params, const char* debugname)
+    : SignallingCircuitSpan(params.getValue("debugname",debugname),
 	static_cast<SignallingCircuitGroup*>(params.getObject("SignallingCircuitGroup"))),
     m_socket(m_group),
     m_thread(0),
@@ -1366,7 +1462,7 @@ bool WpSpan::init(const NamedList& config, const NamedList& defaults, NamedList&
 	return false;
     }
     // Set socket card / device
-    m_socket.card(config);
+    m_socket.card(!params.null() ? params : config);
     const char* voice = params.getValue("voicegroup",config.getValue("voicegroup"));
     if (!voice) {
 	Debug(m_group,DebugNote,"WpSpan('%s'). Missing or invalid voice group [%p]",
@@ -1376,10 +1472,13 @@ bool WpSpan::init(const NamedList& config, const NamedList& defaults, NamedList&
     m_socket.device(voice);
     m_canSend = !params.getBoolValue("readonly",config.getBoolValue("readonly",false));
     // Type depending data: channel count, samples, circuit list
-    String type = config.getValue("type");
-    String cics = config.getValue("voicechans");
-    unsigned int offs = config.getIntValue("offset",0);
+    String type = params.getValue("type",config.getValue("type"));
+    String cics = params.getValue("voicechans",config.getValue("voicechans"));
+    unsigned int start = params.getIntValue("offset",config.getIntValue("offset",0));
+    start += params.getIntValue("start");
+    start = config.getIntValue("start",start);
     m_samples = params.getIntValue("samples",config.getIntValue("samples"));
+    int idleValue = 0xd5; // A-Law idle code
     if (type.null())
 	type = "E1";
     if (type == "E1") {
@@ -1391,6 +1490,7 @@ bool WpSpan::init(const NamedList& config, const NamedList& defaults, NamedList&
 	    m_samples = 50;
     }
     else if (type == "T1") {
+	idleValue = 0xff; // mu-Law idle code
 	m_chans = 24;
 	m_increment = 24;
 	if (cics.null())
@@ -1415,7 +1515,7 @@ bool WpSpan::init(const NamedList& config, const NamedList& defaults, NamedList&
 
     // Other data
     m_swap = defaults.getBoolValue("bitswap",true);
-    m_noData = defaults.getIntValue("idlevalue",0xff);
+    m_noData = defaults.getIntValue("idlevalue",idleValue);
     m_buflen = defaults.getIntValue("buflen",160);
     m_swap = params.getBoolValue("bitswap",config.getBoolValue("bitswap",m_swap));
     m_noData = params.getIntValue("idlevalue",config.getIntValue("idlevalue",m_noData));
@@ -1429,7 +1529,7 @@ bool WpSpan::init(const NamedList& config, const NamedList& defaults, NamedList&
     if (!m_buflen)
 	m_buflen = 160;
     // Channels
-    if (!createCircuits(params.getIntValue("start") + offs,cics)) {
+    if (!createCircuits(start,cics)) {
 	Debug(m_group,DebugNote,
 	    "WpSpan('%s'). Failed to create voice chans (voicechans=%s) [%p]",
 	    id().safe(),cics.safe(),this);
@@ -1451,7 +1551,10 @@ bool WpSpan::init(const NamedList& config, const NamedList& defaults, NamedList&
 	s << " device=" << m_socket.device();
 	s << " samples=" << m_samples;
 	s << " bitswap=" << String::boolText(m_swap);
-	s << " idlevalue=" << (unsigned int)m_noData;
+	if (m_noData < 256)
+	    s << " idlevalue=" << m_noData;
+	else
+	    s << " idlevalue=(circuit)";
 	s << " buflen=" << (unsigned int)m_buflen;
 	s << " echocancel=" << String::boolText(m_echoCancel);
 	s << " dtmfdetect=" << String::boolText(m_dtmfDetect);
@@ -1499,7 +1602,7 @@ bool WpSpan::createCircuits(unsigned int delta, const String& cicList)
 	ok = false;
 	break;
     }
-    delete cicCodes;
+    delete[] cicCodes;
     return ok;
 }
 
@@ -1560,18 +1663,27 @@ void WpSpan::run()
 		"WpSpan('%s'). Received %u samples. Expected %u [%p]",
 		id().safe(),samples,m_samples,this);
 	if (m_canSend) {
-	    unsigned char noData = swap(m_noData);
 	    for (unsigned int i = 0; i < m_count; i++) {
-		unsigned char* dat = m_buffer + WP_HEADER + i;
 		WpCircuit* circuit = m_circuits[i];
+		// Forward read data if we have a source
 		WpSource* s = (circuit && circuit->validSource()) ? circuit->source() : 0;
-		WpConsumer* c = (circuit && circuit->validConsumer()) ? circuit->consumer() : 0;
-		Lock lock(c);
-		for (int n = samples; n > 0; n--, dat += m_count) {
-		    // Read each byte from buffer. Prepare buffer for sending
-		    if (s)
+		if (s) {
+		    unsigned char* dat = m_buffer + WP_HEADER + i;
+		    for (int n = samples; n > 0; n--, dat += m_count)
 			s->put(swap(*dat));
-		    *dat = c ? swap(c->get()) : noData;
+		}
+		// Fill send buffer for current circuit
+		unsigned char* dat = m_buffer + WP_HEADER + i;
+		WpConsumer* c = (circuit && circuit->validConsumer()) ? circuit->consumer() : 0;
+		if (c) {
+		    Lock lock(c);
+		    for (int n = samples; n > 0; n--, dat += m_count)
+			*dat = swap(c->get());
+		}
+		else {
+		    unsigned char noData = swap((m_noData < 256 || !circuit) ? m_noData : circuit->code() & 0xff);
+		    for (int n = samples; n > 0; n--, dat += m_count)
+			*dat = noData;
 		}
 	    }
 	    ::memset(m_buffer,0,WP_HEADER);
@@ -1617,7 +1729,11 @@ bool WpSpan::readEvent()
 // Return -1 on error
 int WpSpan::readData()
 {
+#ifdef NEW_WANPIPE_API
+    ((wp_api_hdr_t*)m_buffer)->wp_api_rx_hdr_error_map = 0;
+#else
     m_buffer[WP_RD_ERROR] = 0;
+#endif
     int r = m_socket.recv(m_buffer,m_bufferLen);
     // Check errors
     if (r == -1)
@@ -1627,11 +1743,16 @@ int WpSpan::readData()
 	    id().safe(),r,this);
 	return -1;
     }
-    if (m_buffer[WP_RD_ERROR]) {
+#ifdef NEW_WANPIPE_API
+    unsigned char err = ((wp_api_hdr_t*)m_buffer)->wp_api_rx_hdr_error_map;
+#else
+    unsigned char err = m_buffer[WP_RD_ERROR];
+#endif
+    if (err) {
 	m_readErrors++;
 	if (m_readErrors == MAX_READ_ERRORS) {
-	    Debug(m_group,DebugGoOn,"WpSpan('%s'). Read error %u [%p]",
-		id().safe(),m_buffer[WP_RD_ERROR],this);
+	    Debug(m_group,DebugGoOn,"WpSpan('%s'). Read error 0x%02X [%p]",
+		id().safe(),err,this);
 	    m_readErrors = 0;
 	}
     }
@@ -1644,12 +1765,11 @@ int WpSpan::readData()
 
 bool WpSpan::decodeEvent()
 {
+    // TODO: new API has different event handling style, may need total rewrite
+#if defined(WAN_EC_TONE_PRESENT) && !defined(NEW_WANPIPE_API)
     api_rx_hdr_t* ev = (api_rx_hdr_t*)m_buffer;
-
     SignallingCircuitEvent* e = 0;
     WpCircuit* circuit = 0;
-
-#ifdef WAN_EC_TONE_PRESENT
     switch (ev->event_type) {
 	case WP_API_EVENT_NONE:
 	    return false;
