@@ -3,7 +3,7 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * RADIUS Client functionality for YATE
- * 
+ *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
  * Copyright (C) 2004-2006 Null Team
  *
@@ -38,7 +38,7 @@ namespace { // anonymous
 
 #define RADIUS_MAXLEN 4096
 
-enum 
+enum
 {
     RadChanStart,
     RadChanStop,
@@ -47,7 +47,7 @@ enum
     RadCallExecute,
     RadCallRinging,
     RadUserLogin
-};	
+};
 
 enum
 {
@@ -68,6 +68,7 @@ static Socket s_localSock;
 static bool s_localTime = false;
 static bool s_shortnum = false;
 static bool s_unisocket = false;
+static bool s_printAttr = false;
 static bool s_pb_enabled = false;
 static bool s_pb_parallel = false;
 static bool s_pb_simplify = false;
@@ -84,7 +85,7 @@ enum {
     a_date,	// rfc2865 time
     a_avpair	// special text formatted as name=value
 };
-	
+
 // Structure for building attribute tables
 typedef struct {
     int code;
@@ -215,7 +216,7 @@ static rad_dict cisco_dict[]= {
     { 210, "Cisco-PPP-VJ-Slot-Comp", a_int },
     { 212, "Cisco-PPP-Async-Map", a_int },
     { 217, "Cisco-IP-Pool-Definition", a_int },
-    { 218, "Cisco-Asing-IP-Pool", a_int },
+    { 218, "Cisco-Assign-IP-Pool", a_int },
     { 228, "Cisco-Route-IP", a_int },
     { 233, "Cisco-Link-Compression", a_int },
     { 234, "Cisco-Target-Util", a_int },
@@ -286,7 +287,7 @@ static TokenDict dict_errors[] = {
     { "rejected", 18 }, // Host-Request
     { 0, 0 },
 };
-									
+
 
 // Class to hold one RADIUS attribute
 class RadAttrib : public GenObject
@@ -350,7 +351,7 @@ public:
     bool addAttribute(const char* attrib, unsigned char subType, const char* val, bool emptyOk = false);
     void addAttributes(NamedList& params, NamedList* list);
     bool prepareAttributes(NamedList& params, bool forAcct = true, String* user = 0);
-    bool returnAttributes(NamedList& params, const ObjList* attributes);
+    bool returnAttributes(NamedList& params, const ObjList* attributes, bool ok = true);
     static bool fillRandom(DataBlock& data, int len);
 
 private:
@@ -881,7 +882,7 @@ bool RadiusClient::fillRandom(DataBlock& data, int len)
     unsigned int r = 0;
     while (len--) {
 	while (!r)
-	    r = ::random() % RAND_MAX;
+	    r = Random::random();
 	*dd++ = r & 0xff;
 	r = r >> 8;
     }
@@ -1036,7 +1037,7 @@ int RadiusClient::makeRequest(int port, unsigned char request, unsigned char* re
 	    DDebug(&__plugin,DebugInfo,"Received valid response %u on session %u from %s:%d",
 		recdata[0],sessionId,recvAddr.host().c_str(),recvAddr.port());
 	    return NoError;
-	}	
+	}
     }
     Debug(&__plugin,DebugWarn,"Timeout receiving session %u from server %s:%d",
 	sessionId,sockAddr.host().c_str(),sockAddr.port());
@@ -1059,7 +1060,7 @@ int RadiusClient::doAuthenticate(ObjList* result)
 	Debug(&__plugin,DebugMild,"Server returned %u, assuming Access-Reject",response);
 	return AuthFailed;
     }
-	
+
     Debug(&__plugin,DebugInfo,"Server returned Access-Accept");
     return AuthSuccess;
 }
@@ -1098,7 +1099,7 @@ bool RadiusClient::addAttribute(const char* attrib, const char* val, bool emptyO
 	return true;
     }
     attr->destruct();
-    return false;	
+    return false;
 }
 
 // Add one numeric attribute
@@ -1112,7 +1113,7 @@ bool RadiusClient::addAttribute(const char* attrib, int val)
 	return true;
     }
     attr->destruct();
-    return false;	
+    return false;
 }
 
 // Add one text attribute with subtype
@@ -1128,7 +1129,7 @@ bool RadiusClient::addAttribute(const char* attrib, unsigned char subType, const
 	return true;
     }
     attr->destruct();
-    return false;	
+    return false;
 }
 
 // Copy from parameter list (usually message) to RADIUS attributes
@@ -1168,7 +1169,7 @@ void RadiusClient::addAttributes(NamedList& params, NamedList* list)
 	    continue;
 	String val = *s;
 	params.replaceParams(val);
-	Regexp r("^\\([0-9]\\+\\):\\(.*\\)");
+	static const Regexp r("^\\([0-9]\\+\\):\\(.*\\)");
 	if (key.matches(r)) {
 	    int subType = key.matchString(1).toInteger(-1);
 	    if ((subType >= 0) && (subType <= 255)) {
@@ -1278,7 +1279,7 @@ bool RadiusClient::prepareAttributes(NamedList& params, bool forAcct, String* us
     // remember the name of the NAS section
     m_section = *nasSect;
 
-    Debug(&__plugin,DebugNote,"Using sections [%s] and [%s] for %s",
+    Debug(&__plugin,DebugInfo,"Using sections [%s] and [%s] for %s",
 	m_section.c_str(),servName.c_str(),forAcct ? "accounting" : "authentication");
     addAttribute("User-Name",username);
     addAttribute("Calling-Station-Id",caller);
@@ -1291,19 +1292,24 @@ bool RadiusClient::prepareAttributes(NamedList& params, bool forAcct, String* us
 }
 
 // Copy some attributes back from RADIUS answer to parameter list (message)
-bool RadiusClient::returnAttributes(NamedList& params, const ObjList* attributes)
+bool RadiusClient::returnAttributes(NamedList& params, const ObjList* attributes, bool ok)
 {
     Lock lock(s_cfgMutex);
     NamedList* sect = s_cfg.getSection(m_section);
     if (!sect)
 	return false;
 
+    String attrDump;
     for (; attributes; attributes = attributes->next()) {
 	const RadAttrib* attr = static_cast<const RadAttrib*>(attributes->get());
 	if (!attr)
 	    continue;
-	XDebug(&__plugin,DebugInfo,"Returned attribute %d '%s'",attr->code(),attr->name());
-	String tmp("ret:");
+	if (s_printAttr && __plugin.debugAt(DebugAll)) {
+	    String val;
+	    attr->getString(val);
+	    attrDump << "\r\n  " << attr->name() << "='" << val << "'";
+	}
+	String tmp(ok ? "ret:" : "ret-fail:");
 	tmp += attr->name();
 	String* par = sect->getParam(tmp);
 	if (par && *par) {
@@ -1312,12 +1318,16 @@ bool RadiusClient::returnAttributes(NamedList& params, const ObjList* attributes
 	    params.setParam(*par,tmp);
 	}
     }
+    if (attrDump)
+	Debug(&__plugin,DebugAll,"Returned attributes:%s",attrDump.c_str());
     return true;
 }
 
 
 bool AuthHandler::received(Message& msg)
 {
+    if (!msg.getBoolValue("auth_radius",true))
+	return false;
     String proto = msg.getValue("protocol",msg.getValue("module"));
     if (proto.null())
 	return false;
@@ -1356,15 +1366,29 @@ bool AuthHandler::received(Message& msg)
     if (sep >= 0)
 	address = address.substr(0,sep);
     radclient.addAttribute("h323-remote-address",address);
+    String billid = msg.getValue("billid");
+    if (billid) {
+	// create a Cisco-compatible conference ID
+	MD5 cid(billid);
+	String confid;
+	confid << cid.hexDigest().substr(0,8) << " ";
+	confid << cid.hexDigest().substr(8,8) << " ";
+	confid << cid.hexDigest().substr(16,8) << " ";
+	confid << cid.hexDigest().substr(24,8);
+	confid.toUpper();
+	radclient.addAttribute("h323-conf-id",confid);
+    }
 
     ObjList result;
-    if (radclient.doAuthenticate(&result) != AuthSuccess)
+    if (radclient.doAuthenticate(&result) != AuthSuccess) {
+	radclient.returnAttributes(msg,&result,false);
 	return false;
+    }
     // copy back the username we actually authenticated
     if (user)
 	msg.setParam("username",user);
     // and pick whatever other parameters we want to return
-    radclient.returnAttributes(msg,&result);
+    radclient.returnAttributes(msg,&result,true);
     if (s_pb_enabled)
 	portaBillingRoute(msg,&result);
     // signal we don't return a password
@@ -1402,6 +1426,8 @@ static bool ciscoTime(double t, String& ret)
 
 bool AcctHandler::received(Message& msg)
 {
+    if (!msg.getBoolValue("cdrwrite_radius",true))
+	return false;
     String op = msg.getValue("operation");
     int acctStat = 0;
     if (op == "initialize")
@@ -1514,6 +1540,7 @@ void RadiusModule::initialize()
     s_localTime = s_cfg.getBoolValue("general","local_time",false);
     s_shortnum = s_cfg.getBoolValue("general","short_number",false);
     s_unisocket = s_cfg.getBoolValue("general","single_socket",false);
+    s_printAttr = s_cfg.getBoolValue("general","print_attributes",false);
     s_pb_enabled = s_cfg.getBoolValue("portabill","enabled",false);
     s_pb_parallel = s_cfg.getBoolValue("portabill","parallel",false);
     s_pb_simplify = s_cfg.getBoolValue("portabill","simplify",false);
