@@ -555,6 +555,7 @@ private:
     bool m_sendReadOnly;                 // Print send attempt on readonly interface error
     int m_notify;                        // Notify receiver on channel non idle (0: success. 1: not notified. 2: notified)
     SignallingTimer m_timerRxUnder;      // RX underrun notification
+    bool m_down;			 // Interface status
 };
 
 // Signalling span used to create voice circuits
@@ -625,6 +626,7 @@ protected:
     ZapDevice m_device;                  // The device
     ZapDevice::Type m_type;              // Circuit type
     ZapDevice::Format m_format;          // The data format
+    String m_specialMode;                // Special test mode
     bool m_echoCancel;                   // Echo canceller state
     bool m_crtEchoCancel;                // Current echo canceller state
     unsigned int m_echoTaps;             // Echo cancel taps
@@ -774,6 +776,32 @@ static inline bool getBoolValue(const char* param, const NamedList& config,
     return params.getBoolValue(param,defVal);
 }
 
+static void sendModuleUpdate(const String& notif, const String& device, bool& notifStat, int status = 0)
+{
+    Message* msg = new Message("module.update");
+    msg->addParam("module",plugin.name());
+    msg->addParam("interface",device);
+    msg->addParam("notify",notif);
+    if(notifStat && status == SignallingInterface::LinkUp) {
+	notifStat = false;
+	Engine::enqueue(msg);
+	return;
+    }
+    if (!notifStat && status == SignallingInterface::LinkDown) {
+	notifStat = true;
+	Engine::enqueue(msg);
+	return;
+    }
+    if (notif == "alarm") {
+	if (status == ZapDevice::Yellow)
+	    msg->addParam("notify","RAI");
+	if (status == ZapDevice::Blue)
+	    msg->addParam("notify","AIS");
+	Engine::enqueue(msg);
+	return;
+    }
+    TelEngine::destruct(msg);
+}
 
 /**
  * ZapWorkerClient
@@ -1362,8 +1390,13 @@ bool ZapDevice::checkAlarms()
     m_alarmsText = "";
     if (m_alarms) {
 	for(int i = 0; s_alarms[i].token; i++)
-	    if (m_alarms & s_alarms[i].value)
+	    if (m_alarms & s_alarms[i].value) {
 		m_alarmsText.append(s_alarms[i].token,",");
+		if (s_alarms[i].value == ZapDevice::Yellow || s_alarms[i].value == ZapDevice::Blue) {
+		    bool notifStat = false;
+		    sendModuleUpdate("alarm",zapName(),notifStat,s_alarms[i].value);
+		}
+	    }
 	Debug(m_owner,DebugNote,"%sAlarms changed (%d,'%s') on channel %u [%p]",
 	    m_name.safe(),m_alarms,m_alarmsText.safe(),m_channel,m_owner);
     }
@@ -1424,6 +1457,7 @@ bool ZapDevice::select(unsigned int usec)
 
 int ZapDevice::recv(void* buffer, int len)
 {
+    errno = 0;
     int r = ::read(m_handle,buffer,len);
     if (r >= 0) {
 	m_event = false;
@@ -1444,12 +1478,15 @@ int ZapDevice::recv(void* buffer, int len)
 
 int ZapDevice::send(const void* buffer, int len)
 {
+    errno = 0;
     int w = ::write(m_handle,buffer,len);
     if (w == len) {
 	m_writeError = false;
 	return w;
     }
-    if (!m_writeError) {
+    if (errno == ELAST)
+	m_event = true;
+    else if (!m_writeError) {
 	Debug(m_owner,DebugWarn,
 	    "%sWrite failed on channel %u (sent %d instead of %d). %d: %s [%p]",
 	    m_name.safe(),m_channel,w>=0?w:0,len,errno,::strerror(errno),m_owner);
@@ -1667,10 +1704,23 @@ SignallingComponent* ZapInterface::create(const String& type, const NamedList& n
     else
 	return 0;
 
+    const String* module = name.getParam("module");
+    if (module && *module != "zapcard")
+	return 0;
     Configuration cfg(Engine::configFile("zapcard"));
     const char* sectName = name.getValue((circuit ? "voice" : "sig"),name.getValue("basename",name));
     NamedList* config = cfg.getSection(sectName);
-    if (!config) {
+    if (module) {
+	DDebug(&plugin,DebugAll,"Replace config params in section %s",c_safe(sectName));
+	if (!config) {
+	    cfg.createSection(sectName);
+	    config = cfg.getSection(sectName);
+	}
+	config->copyParams(name);
+	if (!cfg.save())
+	    Debug(&plugin,DebugNote,"Failed to save configuration in file %s",cfg.c_str());
+    }
+    else if (!config){
 	DDebug(&plugin,DebugAll,"No section '%s' in configuration",c_safe(sectName));
 	return 0;
     }
@@ -1774,6 +1824,7 @@ bool ZapInterface::init(ZapDevice::Type type, unsigned int code, unsigned int ch
 	s << " priority=" << Thread::priority(m_priority);
 	Debug(this,DebugInfo,"D-channel: %s [%p]",s.c_str(),this);
     }
+    m_down = false;
     return true;
 }
 
@@ -1927,11 +1978,13 @@ void ZapInterface::checkEvents()
 		Debug(this,DebugNote,"Alarms changed '%s' [%p]",
 		    m_device.alarmsText().safe(),this);
 		notify(LinkDown);
+		sendModuleUpdate("interfaceDown",m_device.zapName(),m_down,LinkDown);
 	    }
 	    else {
 		m_device.resetAlarms();
 		DDebug(this,DebugInfo,"No more alarms [%p]",this);
 		notify(LinkUp);
+		sendModuleUpdate("interfaceUp",m_device.zapName(),m_down,LinkUp);
 	    }
 	    return;
 	case ZapDevice::HdlcAbort:
@@ -2009,7 +2062,7 @@ bool ZapSpan::init(ZapDevice::Type type, unsigned int offset,
     if (!digital)
 	m_increment = chans = count;
     m_increment = config.getIntValue("increment",m_increment);
-    unsigned int start = params.getIntValue("start",0);
+    unsigned int start = config.getIntValue("start",params.getIntValue("start",0));
 
     // Create and insert circuits
     unsigned int added = 0;
@@ -2303,6 +2356,10 @@ bool ZapCircuit::setParam(const String& param, const String& value)
 	    m_crtDtmfDetect = ok;
 	return ok;
     }
+    if (param == "special_mode") {
+	m_specialMode = value;
+	return true;
+    }
     return false;
 }
 
@@ -2401,7 +2458,7 @@ bool ZapCircuit::sendEvent(SignallingCircuitEvent::Type type, NamedList* params)
 // Consume data sent by the consumer
 void ZapCircuit::consume(const DataBlock& data)
 {
-    if (!(SignallingCircuit::status() == Connected && m_canSend && data.length()))
+    if (!(SignallingCircuit::status() >= Special && m_canSend && data.length()))
 	return;
 
     // Copy data in buffer
@@ -2463,6 +2520,7 @@ void ZapCircuit::cleanup(bool release, Status stat, bool stop)
 	return;
     }
     status(stat);
+    m_specialMode.clear();
     m_sourceBuffer.clear();
     m_consBuffer.clear();
     m_consErrors = m_consErrorBytes = m_consTotal = 0;
@@ -2603,6 +2661,7 @@ bool ZapAnalogCircuit::status(Status newStat, bool sync)
 	case Disabled:
 	case Idle:
 	case Reserved:
+	case Special:
 	case Connected:
 	    break;
 	default: ;
@@ -2620,7 +2679,7 @@ bool ZapAnalogCircuit::status(Status newStat, bool sync)
 	DDebug(group(),DebugAll,"ZapCircuit(%u). Changed status to %u [%p]",
 	    code(),newStat,this);
 
-    if (newStat != Connected && m_device.valid())
+    if (newStat < Special && m_device.valid())
 	m_device.flushBuffers();
 
     if (newStat == Reserved) {
@@ -2640,12 +2699,25 @@ bool ZapAnalogCircuit::status(Status newStat, bool sync)
 	}
 	return SignallingCircuit::status() == Reserved;
     }
-    else if (newStat == Connected) {
-	if (m_device.valid())
+    else if (newStat >= Special) {
+	if (m_device.valid()) {
 	    createData();
+	    if (newStat == Special) {
+		Message m("circuit.special");
+		m.userData(this);
+		if (group())
+		    m.addParam("group",group()->toString());
+		if (span())
+		    m.addParam("span",span()->toString());
+		if (m_specialMode)
+		    m.addParam("mode",m_specialMode);
+		if (!Engine::dispatch(m))
+		    cleanup(false,Idle,true);
+	    }
+	}
 	else
 	    cleanup(false,Idle,true);
-	return SignallingCircuit::status() == Connected;
+	return SignallingCircuit::status() == newStat;
     }
     return true;
 }
