@@ -38,6 +38,8 @@ namespace { // anonymous
 #define THRESHOLD2_ABS     1e+06
 // relative square of spectral power from total signal power
 #define THRESHOLD2_REL_FAX  0.95
+// same for continuity test tones
+#define THRESHOLD2_REL_COT  0.90
 // sum of tones (low+high) from total
 #define THRESHOLD2_REL_ALL  0.60
 // each tone from threshold from total
@@ -101,20 +103,22 @@ public:
 	{ return m_name; }
     inline const String& id() const
 	{ return m_id; }
-    void setDivert(const Message& msg);
+    void setFaxDivert(const Message& msg);
     void init();
 private:
     void checkDtmf();
     void checkFax();
+    void checkCont();
     String m_id;
     String m_name;
-    String m_divert;
-    String m_caller;
-    String m_called;
+    String m_faxDivert;
+    String m_faxCaller;
+    String m_faxCalled;
     String m_target;
     String m_dnis;
     Mode m_mode;
     bool m_detFax;
+    bool m_detCont;
     bool m_detDtmf;
     bool m_detDnis;
     char m_dtmfTone;
@@ -122,6 +126,7 @@ private:
     double m_xv[3];
     double m_pwr;
     Tone2PoleFilter m_fax;
+    Tone2PoleFilter m_cont;
     Tone2PoleFilter m_dtmfL[4];
     Tone2PoleFilter m_dtmfH[4];
 };
@@ -170,6 +175,18 @@ static Params2Pole s_paramsCNG =
 static Params2Pole s_paramsCED =
     { 8.587870006e+01, -0.9767113407, -0.1551017476 }; // 2100Hz
 
+// generated continuity test verified detector (2010Hz) filter parameters
+// mkfilter -Bu -Bp -o 1 -a 2.5025000000e-01 2.5225000000e-01
+//  -> 2-pole butterworth bandpass, 2010Hz +-8Hz @ -3dB
+static Params2Pole s_paramsCOTv =
+    { 1.601528486e+02, -0.9875119299, -0.0156100298 }; // 2010Hz
+
+// generated continuity test send detector (1780Hz) filter parameters
+// mkfilter -Bu -Bp -o 1 -a 2.1875000000e-01 2.2625000000e-01
+//  -> 2-pole butterworth bandpass, 1780Hz +-30Hz @ -3dB
+static Params2Pole s_paramsCOTs =
+    { 4.343337207e+01, -0.9539525559, 0.3360345780 }; // 1780Hz
+
 // generated DTMF component filter parameters
 // 2-pole butterworth bandpass, +-1% @ -3dB
 static Params2Pole s_paramsDtmfL[] = {
@@ -209,8 +226,8 @@ void Tone2PoleFilter::update(double xd)
 
 ToneConsumer::ToneConsumer(const String& id, const String& name)
     : m_id(id), m_name(name), m_mode(Mono),
-      m_detFax(true), m_detDtmf(true), m_detDnis(false),
-      m_fax(s_paramsCNG)
+      m_detFax(true), m_detCont(false), m_detDtmf(true), m_detDnis(false),
+      m_fax(s_paramsCNG), m_cont(s_paramsCOTv)
 { 
     Debug(&plugin,DebugAll,"ToneConsumer::ToneConsumer(%s,'%s') [%p]",
 	id.c_str(),name.c_str(),this);
@@ -232,19 +249,25 @@ ToneConsumer::ToneConsumer(const String& id, const String& name)
 	m_format = "2*slin";
     if (tmp && (tmp != "*")) {
 	// individual detection requested
-	m_detFax = m_detDtmf = m_detDnis = false;
+	m_detFax = m_detCont = m_detDtmf = m_detDnis = false;
 	ObjList* k = tmp.split(',',false);
 	for (ObjList* l = k; l; l = l->next()) {
 	    String* s = static_cast<String*>(l->get());
 	    if (!s)
 		continue;
 	    m_detFax = m_detFax || (*s == "fax");
+	    m_detCont = m_detCont || (*s == "cotv");
 	    m_detDtmf = m_detDtmf || (*s == "dtmf");
 	    if (*s == "rfax") {
 		// detection of receiving Fax requested
 		m_fax.assign(s_paramsCED);
 		m_detFax = true;
 	    }
+	    else if (*s == "cots") {
+		// detection of COT Send tone requested
+		m_cont.assign(s_paramsCOTs);
+		m_detCont = true;
+	    }
 	    else if (*s == "callsetup") {
 		// call setup info in the form *ANI*DNIS*
 		m_detDnis = true;
@@ -271,6 +294,7 @@ void ToneConsumer::init()
     m_xv[1] = m_xv[2] = 0.0;
     m_pwr = 0.0;
     m_fax.init();
+    m_cont.init();
     for (int i = 0; i < 4; i++) {
 	m_dtmfL[i].init();
 	m_dtmfH[i].init();
@@ -359,7 +383,7 @@ void ToneConsumer::checkDtmf()
     }
 }
 
-// Check if we detected a Fax CNG tone
+// Check if we detected a Fax CNG or CED tone
 void ToneConsumer::checkFax()
 {
     if (m_fax.value() < m_pwr*THRESHOLD2_REL_FAX)
@@ -377,17 +401,43 @@ void ToneConsumer::checkFax()
     m_detFax = false;
     Message* m = new Message("chan.masquerade");
     m->addParam("id",m_id);
-    if (m_divert) {
+    if (m_faxDivert) {
 	Debug(&plugin,DebugCall,"Diverting call %s to: %s",
-	    m_id.c_str(),m_divert.c_str());
+	    m_id.c_str(),m_faxDivert.c_str());
 	m->addParam("message","call.execute");
-	m->addParam("callto",m_divert);
+	m->addParam("callto",m_faxDivert);
 	m->addParam("reason","fax");
     }
     else {
 	m->addParam("message","call.fax");
 	m->addParam("detected","inband");
     }
+    m->addParam("caller",m_faxCaller,false);
+    m->addParam("called",m_faxCalled,false);
+    Engine::enqueue(m);
+}
+
+// Check if we detected a Continuity Test tone
+void ToneConsumer::checkCont()
+{
+    if (m_cont.value() < m_pwr*THRESHOLD2_REL_COT)
+	return;
+    if (m_cont.value() > m_pwr) {
+	DDebug(&plugin,DebugNote,"Overshoot on %s, signal=%0.2f, total=%0.2f",
+	    m_id.c_str(),m_cont.value(),m_pwr);
+	init();
+	return;
+    }
+    DDebug(&plugin,DebugInfo,"Continuity detected on %s, signal=%0.1f, total=%0.1f",
+	m_id.c_str(),m_cont.value(),m_pwr);
+    // prepare for new detection
+    init();
+    m_detCont = false;
+    Message* m = new Message("chan.masquerade");
+    m->addParam("id",m_id);
+    m->addParam("message","chan.dtmf");
+    m->addParam("text","O");
+    m->addParam("detected","inband");
     Engine::enqueue(m);
 }
 
@@ -429,6 +479,8 @@ unsigned long ToneConsumer::Consume(const DataBlock& data, unsigned long tStamp,
 	// update all active detectors
 	if (m_detFax)
 	    m_fax.update(dx);
+	if (m_detCont)
+	    m_cont.update(dx);
 	if (m_detDtmf || m_detDnis) {
 	    for (int j = 0; j < 4; j++) {
 		m_dtmfL[j].update(dx);
@@ -444,6 +496,8 @@ unsigned long ToneConsumer::Consume(const DataBlock& data, unsigned long tStamp,
 		checkDtmf();
 	    if (m_detFax)
 		checkFax();
+	    if (m_detCont)
+		checkCont();
 	}
 	else {
 	    m_dtmfTone = '\0';
@@ -456,7 +510,7 @@ unsigned long ToneConsumer::Consume(const DataBlock& data, unsigned long tStamp,
 }
 
 // Copy parameters required for automatic fax call diversion
-void ToneConsumer::setDivert(const Message& msg)
+void ToneConsumer::setFaxDivert(const Message& msg)
 {
     m_target = msg.getParam("notify");
     if (m_id.null())
@@ -467,11 +521,11 @@ void ToneConsumer::setDivert(const Message& msg)
     m_detFax = true;
     // if divert is empty or false disable diverting
     if (divert->null() || !divert->toBoolean(true))
-	m_divert.clear();
+	m_faxDivert.clear();
     else {
-	m_divert = *divert;
-	m_caller = msg.getValue("caller",m_caller);
-	m_called = msg.getValue("called",m_called);
+	m_faxDivert = *divert;
+	m_faxCaller = msg.getValue("fax_caller",msg.getValue("caller",m_faxCaller));
+	m_faxCalled = msg.getValue("fax_called",msg.getValue("called",m_faxCalled));
     }
 }
 
@@ -487,26 +541,27 @@ bool AttachHandler::received(Message& msg)
 	snif.clear();
     if (cons.null() && snif.null())
 	return false;
-    CallEndpoint* ch = static_cast<CallEndpoint *>(msg.userObject("CallEndpoint"));
-    DataSource* ds = static_cast<DataSource *>(msg.userObject("DataSource"));
+    CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
+    RefPointer<DataEndpoint> de = static_cast<DataEndpoint*>(msg.userObject("DataEndpoint"));
+    DataSource* ds = static_cast<DataSource*>(msg.userObject("DataSource"));
     if (ch) {
 	if (cons) {
 	    ToneConsumer* c = new ToneConsumer(ch->id(),cons);
-	    c->setDivert(msg);
+	    c->setFaxDivert(msg);
 	    ch->setConsumer(c);
 	    c->deref();
 	}
 	if (snif) {
-	    DataEndpoint* de = ch->setEndpoint();
+	    de = ch->setEndpoint();
 	    // try to reinit sniffer if one already exists
 	    ToneConsumer* c = static_cast<ToneConsumer*>(de->getSniffer(snif));
 	    if (c) {
 		c->init();
-		c->setDivert(msg);
+		c->setFaxDivert(msg);
 	    }
 	    else {
 		c = new ToneConsumer(ch->id(),snif);
-		c->setDivert(msg);
+		c->setFaxDivert(msg);
 		de->addSniffer(c);
 		c->deref();
 	    }
@@ -515,7 +570,7 @@ bool AttachHandler::received(Message& msg)
     }
     else if (ds && cons) {
 	ToneConsumer* c = new ToneConsumer(msg.getValue("id"),cons);
-	c->setDivert(msg);
+	c->setFaxDivert(msg);
 	bool ok = DataTranslator::attachChain(ds,c);
 	if (ok)
 	    msg.userData(c);
@@ -524,6 +579,13 @@ bool AttachHandler::received(Message& msg)
 	c->deref();
 	return ok && msg.getBoolValue("single");
     }
+    else if (de && cons) {
+	ToneConsumer* c = new ToneConsumer(msg.getValue("id"),cons);
+	c->setFaxDivert(msg);
+	de->setConsumer(c);
+	c->deref();
+	return msg.getBoolValue("single");
+    }
     else
 	Debug(&plugin,DebugWarn,"ToneDetector attach request with no call endpoint!");
     return false;
@@ -537,8 +599,8 @@ bool RecordHandler::received(Message& msg)
     String id(msg.getValue("id"));
     if (!src.startsWith("tone/"))
 	return false;
-    DataEndpoint* de = static_cast<DataEndpoint *>(msg.userObject("DataEndpoint"));
     CallEndpoint* ch = static_cast<CallEndpoint *>(msg.userObject("CallEndpoint"));
+    RefPointer<DataEndpoint> de = static_cast<DataEndpoint *>(msg.userObject("DataEndpoint"));
     if (ch) {
 	id = ch->id();
 	if (!de)
@@ -546,7 +608,7 @@ bool RecordHandler::received(Message& msg)
     }
     if (de) {
 	ToneConsumer* c = new ToneConsumer(id,src);
-	c->setDivert(msg);
+	c->setFaxDivert(msg);
 	de->setCallRecord(c);
 	c->deref();
 	return true;
