@@ -45,13 +45,41 @@ static ObjList datas;
 typedef struct {
     int nsamples;
     const short* data;
+    bool repeat;
 } Tone;
 
-typedef struct {
-    const Tone* tone;
-    const char* name;
-    const char* alias;
-} ToneDesc;
+class ToneDesc : public String
+{
+public:
+    ToneDesc(const Tone* tone, const String& name,
+	const String& prefix = String::empty());
+    ~ToneDesc();
+    inline const Tone* tones() const
+	{ return m_tones; }
+    inline bool repeatAll() const
+	{ return m_repeatAll; }
+    // Init this tone description from comma separated list of tone data
+    bool setTones(const String& desc);
+    // Tone name/alias match.
+    // Set name to this object's name if true is returned and alias matches
+    bool isName(String& name) const;
+    // Build tones from a list
+    static void buildTones(const String& name, const NamedList& list);
+private:
+    inline void clearTones() {
+	    if (m_tones && m_ownTones)
+		delete[] m_tones;
+	    m_tones = 0;
+	    m_ownTones = true;
+	    toneListChanged();
+	}
+    // Called when tones list changed to update data
+    void toneListChanged();
+    String m_alias;                      // Tone name alias
+    Tone* m_tones;                       // Tones array. Ends with an invalid one (zero)
+    bool m_ownTones;                     // Clear tones when reset/destroyed
+    bool m_repeatAll;                    // True if all tones repeated
+};
 
 class ToneData : public GenObject
 {
@@ -80,6 +108,11 @@ public:
 	{ return (m_f1 == other.f1()) && (m_f2 == other.f2()); }
     const short* data();
     static ToneData* getData(const char* desc);
+    // Decode a tone description from [!]desc[/duration]
+    // Build a tone data if needded
+    // Return true on success
+    static bool decode(const String& desc, int& samples, const short*& data,
+	bool& repeat);
 private:
     bool parse(const char* desc);
     int m_f1;
@@ -96,8 +129,8 @@ public:
     inline const String& name()
 	{ return m_name; }
     bool startup();
-    static ToneSource* getTone(String& tone);
-    static const ToneDesc* getBlock(String& tone, bool oneShot = false);
+    static ToneSource* getTone(String& tone, const String& prefix);
+    static const ToneDesc* getBlock(String& tone, const String& prefix, bool oneShot = false);
     static Tone* buildCadence(const String& desc);
     static Tone* buildDtmf(const String& dtmf, int len = DTMF_LEN, int gap = DTMF_GAP);
 protected:
@@ -105,10 +138,13 @@ protected:
     virtual bool noChan() const
 	{ return false; }
     virtual void cleanup();
+    void advanceTone(const Tone*& tone);
     static const ToneDesc* getBlock(String& tone, const ToneDesc* table);
+    static const ToneDesc* findToneDesc(String& tone, const String& prefix);
     String m_name;
     const Tone* m_tone;
     int m_repeat;
+    bool m_firstPass;
 private:
     DataBlock m_data;
     unsigned m_brate;
@@ -119,7 +155,7 @@ private:
 class TempSource : public ToneSource
 {
 public:
-    TempSource(String& desc, DataBlock* rawdata);
+    TempSource(String& desc, const String& prefix, DataBlock* rawdata);
     virtual ~TempSource();
 protected:
     virtual bool noChan() const
@@ -132,8 +168,9 @@ private:
 class ToneChan : public Channel
 {
 public:
-    ToneChan(String& tone);
+    ToneChan(String& tone, const String& prefix);
     ~ToneChan();
+    bool attachConsumer(const char* consumer);
 };
 
 class AttachHandler : public MessageHandler
@@ -158,6 +195,10 @@ private:
 };
 
 INIT_PLUGIN(ToneGenDriver);
+static ObjList s_toneDesc;               // List of configured tones
+static ObjList s_defToneDesc;            // List of default tones
+static String s_defLang;                 // Default tone language
+static const String s_default = "itu";
 
 // 421.052Hz (19 samples @ 8kHz) sine wave, pretty close to standard 425Hz
 static const short tone421hz[] = {
@@ -194,45 +235,45 @@ static const short tone1777hz[] = {
     6429, 8659, -3420, -9848,
     0 };
 
-static const Tone t_dial[] = { { 8000, tone421hz }, { 0, 0 } };
+static const Tone t_dial[] = { { 8000, tone421hz, true }, { 0, 0 } };
 
-static const Tone t_busy[] = { { 4000, tone421hz }, { 4000, 0 }, { 0, 0 } };
+static const Tone t_busy[] = { { 4000, tone421hz, true }, { 4000, 0, true }, { 0, 0 } };
 
-static const Tone t_specdial[] = { { 7600, tone421hz }, { 400, 0 }, { 0, 0 } };
+static const Tone t_specdial[] = { { 7600, tone421hz, true }, { 400, 0, true }, { 0, 0 } };
 
-static const Tone t_ring[] = { { 8000, tone421hz }, { 32000, 0 }, { 0, 0 } };
+static const Tone t_ring[] = { { 8000, tone421hz, true }, { 32000, 0, true }, { 0, 0 } };
 
-static const Tone t_congestion[] = { { 2000, tone421hz }, { 2000, 0 }, { 0, 0 } };
+static const Tone t_congestion[] = { { 2000, tone421hz, true }, { 2000, 0, true }, { 0, 0 } };
 
 static const Tone t_outoforder[] = {
-    { 800, tone421hz }, { 800, 0 },
-    { 800, tone421hz }, { 800, 0 },
-    { 800, tone421hz }, { 800, 0 },
-    { 1600, tone421hz }, { 1600, 0 },
+    { 800, tone421hz, true }, { 800, 0, true },
+    { 800, tone421hz, true }, { 800, 0, true },
+    { 800, tone421hz, true }, { 800, 0, true },
+    { 1600, tone421hz, true }, { 1600, 0, true },
     { 0, 0 } };
 
 static const Tone t_callwait[] = {
-    { 160, 0 },
-    { 800, tone421hz }, { 800, 0 }, { 800, tone421hz },
-    { 160, 0 },
+    { 160, 0, true },
+    { 800, tone421hz, true }, { 800, 0, true }, { 800, tone421hz, true },
+    { 160, 0, true },
     { 0, 0 } };
 
 static const Tone t_info[] = {
-    { 2640, tone941hz }, { 240, 0 },
-    { 2640, tone1454hz }, { 240, 0 },
-    { 2640, tone1777hz }, { 8000, 0 },
+    { 2640, tone941hz, true }, { 240, 0, true },
+    { 2640, tone1454hz, true }, { 240, 0, true },
+    { 2640, tone1777hz, true }, { 8000, 0, true },
     { 0, 0 } };
 
-static const Tone t_mwatt[] = { { 8000, tone1000hz }, { 0, 0 } };
+static const Tone t_mwatt[] = { { 8000, tone1000hz, true }, { 0, 0 } };
 
-static const Tone t_silence[] = { { 8000, 0 }, { 0, 0 } };
+static const Tone t_silence[] = { { 8000, 0, true }, { 0, 0 } };
 
-static const Tone t_noise[] = { { 2000, ToneData::getData("noise")->data() }, { 0, 0 } };
+static const Tone t_noise[] = { { 2000, ToneData::getData("noise")->data(), true }, { 0, 0 } };
 
 #define MAKE_DTMF(s) { \
-    { DTMF_GAP, 0 }, \
-    { DTMF_LEN, ToneData::getData(s)->data() }, \
-    { DTMF_GAP, 0 }, \
+    { DTMF_GAP, 0, true }, \
+    { DTMF_LEN, ToneData::getData(s)->data(), true }, \
+    { DTMF_GAP, 0, true }, \
     { 0, 0 } \
 }
 static const Tone t_dtmf[][4] = {
@@ -256,52 +297,37 @@ static const Tone t_dtmf[][4] = {
 #undef MAKE_DTMF
 
 #define MAKE_PROBE(s) { \
-    { 8000, ToneData::getData(s)->data() }, \
+    { 8000, ToneData::getData(s)->data(), true }, \
     { 0, 0 } \
 }
 static const Tone t_probes[][2] = {
     MAKE_PROBE("2000+125"),
     MAKE_PROBE("2000*125"),
     MAKE_PROBE("2000*1000"),
+    MAKE_PROBE("2010"),
+    MAKE_PROBE("1780"),
 };
 #undef MAKE_PROBE
 
-static const ToneDesc s_desc[] = {
-    { t_dial, "dial", "dt" },
-    { t_busy, "busy", "bs" },
-    { t_ring, "ring", "rt" },
-    { t_specdial, "specdial", "sd" },
-    { t_congestion, "congestion", "cg" },
-    { t_outoforder, "outoforder", "oo" },
-    { t_info, "info", "in" },
-    { t_mwatt, "milliwatt", "mw" },
-    { t_silence, "silence", 0 },
-    { t_noise, "noise", "cn" },
-    { t_probes[0], "probe/0", "probe" },
-    { t_probes[1], "probe/1", 0 },
-    { t_probes[2], "probe/2", 0 },
-    { 0, 0, 0 }
-};
-
 static const ToneDesc s_descOne[] = {
-    { t_callwait, "callwaiting", "cw" },
-    { t_dtmf[0], "dtmf/0", "0" },
-    { t_dtmf[1], "dtmf/1", "1" },
-    { t_dtmf[2], "dtmf/2", "2" },
-    { t_dtmf[3], "dtmf/3", "3" },
-    { t_dtmf[4], "dtmf/4", "4" },
-    { t_dtmf[5], "dtmf/5", "5" },
-    { t_dtmf[6], "dtmf/6", "6" },
-    { t_dtmf[7], "dtmf/7", "7" },
-    { t_dtmf[8], "dtmf/8", "8" },
-    { t_dtmf[9], "dtmf/9", "9" },
-    { t_dtmf[10], "dtmf/*", "*" },
-    { t_dtmf[11], "dtmf/#", "#" },
-    { t_dtmf[12], "dtmf/a", "a" },
-    { t_dtmf[13], "dtmf/b", "b" },
-    { t_dtmf[14], "dtmf/c", "c" },
-    { t_dtmf[15], "dtmf/d", "d" },
-    { 0, 0, 0 }
+    ToneDesc(t_callwait,"callwaiting"),
+    ToneDesc(t_dtmf[0],"dtmf/0"),
+    ToneDesc(t_dtmf[1],"dtmf/1"),
+    ToneDesc(t_dtmf[2],"dtmf/2"),
+    ToneDesc(t_dtmf[3],"dtmf/3"),
+    ToneDesc(t_dtmf[4],"dtmf/4"),
+    ToneDesc(t_dtmf[5],"dtmf/5"),
+    ToneDesc(t_dtmf[6],"dtmf/6"),
+    ToneDesc(t_dtmf[7],"dtmf/7"),
+    ToneDesc(t_dtmf[8],"dtmf/8"),
+    ToneDesc(t_dtmf[9],"dtmf/9"),
+    ToneDesc(t_dtmf[10],"dtmf/*"),
+    ToneDesc(t_dtmf[11],"dtmf/#"),
+    ToneDesc(t_dtmf[12],"dtmf/a"),
+    ToneDesc(t_dtmf[13],"dtmf/b"),
+    ToneDesc(t_dtmf[14],"dtmf/c"),
+    ToneDesc(t_dtmf[15],"dtmf/d"),
+    ToneDesc((Tone*)0,"")
 };
 
 // This function is here mainly to avoid 64bit gcc b0rking optimizations
@@ -315,6 +341,156 @@ static unsigned int byteRate(u_int64_t time, unsigned int bytes)
     return (unsigned int)((bytes*(u_int64_t)1000000 + time/2) / time);
 }
 
+// Retrieve the alias associated with a given name
+static const char* getAlias(const String& name)
+{
+#define TONE_GETALIAS(n,a) { if (name == n) return a; }
+    if (!name)
+	return 0;
+    TONE_GETALIAS("dial","dt");
+    TONE_GETALIAS("busy","bs");
+    TONE_GETALIAS("ring","rt");
+    TONE_GETALIAS("specdial","sd");
+    TONE_GETALIAS("congestion","cg");
+    TONE_GETALIAS("outoforder","oo");
+    TONE_GETALIAS("info","in");
+    TONE_GETALIAS("milliwatt","mw");
+    TONE_GETALIAS("silence",0);
+    TONE_GETALIAS("noise","cn");
+    TONE_GETALIAS("probe/0","probe");
+    TONE_GETALIAS("probe/1",0);
+    TONE_GETALIAS("probe/2",0);
+    TONE_GETALIAS("cotv","co1");
+    TONE_GETALIAS("cots","co2");
+    TONE_GETALIAS("callwaiting","cw");
+    TONE_GETALIAS("dtmf/0","0");
+    TONE_GETALIAS("dtmf/1","1");
+    TONE_GETALIAS("dtmf/2","2");
+    TONE_GETALIAS("dtmf/3","3");
+    TONE_GETALIAS("dtmf/4","4");
+    TONE_GETALIAS("dtmf/5","5");
+    TONE_GETALIAS("dtmf/6","6");
+    TONE_GETALIAS("dtmf/7","7");
+    TONE_GETALIAS("dtmf/8","8");
+    TONE_GETALIAS("dtmf/9","9");
+    TONE_GETALIAS("dtmf/*","*");
+    TONE_GETALIAS("dtmf/#","#");
+    TONE_GETALIAS("dtmf/a","a");
+    TONE_GETALIAS("dtmf/b","b");
+    TONE_GETALIAS("dtmf/c","c");
+    TONE_GETALIAS("dtmf/d","d");
+    return 0;
+#undef TONE_GETALIAS
+}
+
+ToneDesc::ToneDesc(const Tone* tone, const String& name, const String& prefix)
+    : String(prefix + name),
+    m_tones((Tone*)tone),
+    m_ownTones(false),
+    m_repeatAll(true)
+{
+    const char* alias = getAlias(name);
+    if (alias)
+	m_alias = prefix + alias;
+    toneListChanged();
+    XDebug(&__plugin,DebugAll,"ToneDesc(%s) [%p]",c_str(),this);
+}
+
+ToneDesc::~ToneDesc()
+{
+    clearTones();
+}
+
+// Init this tone description from comma separated list if tone data
+bool ToneDesc::setTones(const String& desc)
+{
+    Debug(&__plugin,DebugAll,"ToneDesc(%s) initializing from '%s' [%p]",
+	c_str(),desc.c_str(),this);
+    clearTones();
+    ObjList* list = desc.split(',',false);
+    m_tones = new Tone[list->count() + 1];
+    m_ownTones = true;
+    int n = 0;
+    for (ObjList* o = list->skipNull(); o; o = o->skipNext(), n++) {
+	const String& s = o->get()->toString();
+	if (ToneData::decode(s,m_tones[n].nsamples,m_tones[n].data,m_tones[n].repeat))
+	    DDebug(&__plugin,DebugAll,
+		"ToneDesc(%s) added tone '%s' samples=%d data=%p repeat=%d [%p]",
+		c_str(),s.c_str(),m_tones[n].nsamples,m_tones[n].data,
+		m_tones[n].repeat,this);
+	else {
+	    Debug(&__plugin,DebugNote,"ToneDesc(%s) invalid tone description '%s' [%p]",
+		c_str(),s.c_str(),this);
+	    n = -1;
+	    break;
+	}
+    }
+    TelEngine::destruct(list);
+    if (n > 0)
+	// Invalidate the last tone in the list
+	::memset(m_tones + n,0,sizeof(Tone));
+    else
+	clearTones();
+    toneListChanged();
+    return n != 0;
+}
+
+// Tone name/alias equality operator. Set name if true is returned
+bool ToneDesc::isName(String& name) const
+{
+    if (name == *this)
+	return true;
+    if (!m_alias || m_alias != name)
+	return false;
+    name = *this;
+    return true;
+}
+
+// Build tone descriptions from a list
+void ToneDesc::buildTones(const String& name, const NamedList& list)
+{
+    DDebug(&__plugin,DebugAll,"Building tones lang=%s from list=%s",
+	name.c_str(),list.c_str());
+    String prefix;
+    ObjList* target = &s_defToneDesc;
+    if (name && name != s_default) {
+	prefix << name << "/";
+	target = &s_toneDesc;
+    }
+    unsigned int n = list.length();
+    for (unsigned int i = 0; i < n; i++) {
+	NamedString* ns = list.getParam(i);
+	if (TelEngine::null(ns))
+	    continue;
+	ToneDesc* d = new ToneDesc(0,ns->name(),prefix);
+	if (d->setTones(*ns)) {
+	    ObjList* o = target->find(d->toString());
+	    if (!o)
+		target->append(d);
+	    else {
+		Debug(&__plugin,DebugInfo,"Replacing tone '%s' (from list '%s')",
+		    d->toString().c_str(),list.c_str());
+		o->set(d);
+	    }
+	}
+	else
+	    TelEngine::destruct(d);
+    }
+}
+
+// Called when tones list changed to update data
+void ToneDesc::toneListChanged()
+{
+    m_repeatAll = true;
+    if (!m_tones)
+	return;
+    for (Tone* tone = m_tones; tone->nsamples; tone++)
+	if (!tone->repeat) {
+	    m_repeatAll = false;
+	    break;
+	}
+}
+
 
 ToneData::ToneData(const char* desc)
     : m_f1(0), m_f2(0), m_mod(false), m_data(0)
@@ -404,7 +580,7 @@ const short* ToneData::data()
 	    int ofs = 65535 >> (-m_f1);
 	    int max = 2 * ofs + 1;
 	    for (int x = 0; x < len; x++)
-		*tmp++ = (short)((::random() % max) - ofs);
+		*tmp++ = (short)((Random::random() % max) - ofs);
 	}
 	else {
 	    double samp = 2*M_PI/8000;
@@ -441,14 +617,47 @@ ToneData* ToneData::getData(const char* desc)
     return d;
 }
 
+// Decode a tone description from [!]desc[/duration]
+// Build a tone data if needded
+// Return true on success
+bool ToneData::decode(const String& desc, int& samples, const short*& data, bool& repeat)
+{
+    if (!desc)
+	return false;
+    samples = 8000;
+    data = 0;
+    repeat = (desc[0] != '!');
+    int start = repeat ? 0 : 1;
+    int pos = desc.find('/',start);
+    String freq;
+    if (pos > 0) {
+	String dur = desc.substr(pos + 1);
+	int duration = dur.toInteger();
+	if (duration > 0) {
+	    // Round up to a multiple of 20
+	    duration += 19;
+	    samples = duration / 20 * 160;
+	}
+	freq = desc.substr(start,pos - start);
+    }
+    else
+	freq = desc.substr(start);
+    // Silence ?
+    if (freq.toInteger(-1) == 0)
+	return true;
+    ToneData* td = ToneData::getData(freq);
+    if (td)
+	data = td->data();
+    return td != 0;
+}
 
 ToneSource::ToneSource(const ToneDesc* tone)
-    : m_tone(0), m_repeat(tone == 0),
+    : m_tone(0), m_repeat(tone == 0), m_firstPass(true),
       m_data(0,320), m_brate(16000), m_total(0), m_time(0)
 {
     if (tone) {
-	m_tone = tone->tone;
-	m_name = tone->name;
+	m_tone = tone->tones();
+	m_name = *tone;
     }
     Debug(&__plugin,DebugAll,"ToneSource::ToneSource(%p) '%s' [%p]",
 	tone,m_name.c_str(),this);
@@ -478,24 +687,80 @@ void ToneSource::cleanup()
     ThreadedSource::cleanup();
 }
 
+void ToneSource::advanceTone(const Tone*& tone)
+{
+    if (!tone)
+	return;
+    const Tone* start = tone;
+    tone++;
+    while (tone && tone != start) {
+	if (!tone->nsamples) {
+	    if ((m_repeat > 0) && !(--m_repeat))
+		m_tone = 0;
+	    tone = m_tone;
+	    m_firstPass = false;
+	    continue;
+	}
+	if (m_firstPass || tone->repeat)
+	    break;
+	tone++;
+    }
+    if (tone == start && !m_firstPass && !tone->repeat) {
+	m_tone = 0;
+	tone = 0;
+    }
+}
+
 const ToneDesc* ToneSource::getBlock(String& tone, const ToneDesc* table)
 {
-    for (; table->tone; table++) {
-	if (tone == table->name)
+    for (; table->tones(); table++) {
+	if (table->isName(tone))
 	    return table;
-	if (table->alias && (tone == table->alias)) {
-	    tone = table->name;
-	    return table;
-	}
     }
     return 0;
 }
 
-const ToneDesc* ToneSource::getBlock(String& tone, bool oneShot)
+const ToneDesc* ToneSource::findToneDesc(String& tone, const String& prefix)
+{
+    XDebug(&__plugin,DebugAll,"ToneSource::findToneDesc(%s,%s)",
+	tone.c_str(),prefix.c_str());
+    ObjList* target = &s_defToneDesc;
+    if (prefix) {
+	tone = prefix + "/" + tone;
+	target = &s_toneDesc;
+    }
+    for (ObjList* o = target->skipNull(); o; o = o->skipNext()) {
+	const ToneDesc* d = static_cast<ToneDesc*>(o->get());
+	if (d->isName(tone))
+	    return d;
+    }
+    if (prefix)
+	tone.startSkip(prefix + "/",false);
+    return 0;
+}
+
+const ToneDesc* ToneSource::getBlock(String& tone, const String& prefix, bool oneShot)
 {
     if (tone.trimBlanks().toLower().null())
 	return 0;
-    const ToneDesc* d = getBlock(tone,s_desc);
+    XDebug(&__plugin,DebugAll,"ToneSource::getBlock(%s,%s,%u)",
+	tone.c_str(),prefix.c_str(),oneShot);
+    const ToneDesc* d = 0;
+    if (prefix) {
+	if (prefix != s_default)
+	    d = findToneDesc(tone,prefix);
+	else {
+	    // Default tone explicitly required
+	    d = findToneDesc(tone,String::empty());
+	    if (!d && oneShot)
+		d = getBlock(tone,s_descOne);
+	    return d;
+	}
+    }
+    if (!d && s_defLang && s_defLang != prefix)
+	d = findToneDesc(tone,s_defLang);
+    if (!d)
+	d = findToneDesc(tone,String::empty());
     if (d)
 	return d;
     if (oneShot)
@@ -523,6 +788,7 @@ Tone* ToneSource::buildDtmf(const String& dtmf, int len, int gap)
     for (unsigned int i = 0; i < dtmf.length(); i++) {
 	t->nsamples = gap;
 	t->data = 0;
+	t->repeat = true;
 	t++;
 
 	int c = dtmf.at(i);
@@ -538,11 +804,13 @@ Tone* ToneSource::buildDtmf(const String& dtmf, int len, int gap)
 
 	t->nsamples = len;
 	t->data = ((c >= 0) && (c < 16)) ? t_dtmf[c][1].data : 0;
+	t->repeat = true;
 	t++;
     }
 
     t->nsamples = gap;
     t->data = 0;
+    t->repeat = true;
     t++;
     t->nsamples = 0;
     t->data = 0;
@@ -550,11 +818,15 @@ Tone* ToneSource::buildDtmf(const String& dtmf, int len, int gap)
     return tmp;
 }
 
-ToneSource* ToneSource::getTone(String& tone)
+ToneSource* ToneSource::getTone(String& tone, const String& prefix)
 {
-    const ToneDesc* td = ToneSource::getBlock(tone);
+    const ToneDesc* td = ToneSource::getBlock(tone,prefix);
+    bool repeat = !td || td->repeatAll();
+    XDebug(&__plugin,DebugAll,"ToneSource::getTone(%s,%s) found %p '%s' repeatall=%s",
+	tone.c_str(),prefix.c_str(),td,td ? td->c_str() : "",String::boolText(repeat));
     // tone name is now canonical
-    ObjList* l = &tones;
+    // Build a fresh source if the list contains tones not repeated
+    ObjList* l = repeat ? &tones : 0;
     for (; l; l = l->next()) {
 	ToneSource* t = static_cast<ToneSource*>(l->get());
 	if (t && (t->name() == tone) && t->running() && (t->refcount() > 1)) {
@@ -589,12 +861,7 @@ void ToneSource::run()
 		// go to the start of the next tone
 		samp = 0;
 		const Tone *otone = tone;
-		tone++;
-		if (!tone->nsamples) {
-		    if ((m_repeat > 0) && !(--m_repeat))
-			m_tone = 0;
-		    tone = m_tone;
-		}
+		advanceTone(tone);
 		nsam = tone ? tone->nsamples : 32000;
 		if (nsam < 0) {
 		    nsam = -nsam;
@@ -629,12 +896,14 @@ void ToneSource::run()
 }
 
 
-TempSource::TempSource(String& desc, DataBlock* rawdata)
+TempSource::TempSource(String& desc, const String& prefix, DataBlock* rawdata)
     : m_single(0), m_rawdata(rawdata)
 {
-    Debug(&__plugin,DebugAll,"TempSource::TempSource(\"%s\") [%p]",desc.c_str(),this);
+    Debug(&__plugin,DebugAll,"TempSource::TempSource(\"%s\",\"%s\") [%p]",
+	desc.c_str(),prefix.safe(),this);
     if (desc.null())
 	return;
+    m_name = desc;
     if (desc.startSkip("*",false))
 	m_repeat = 0;
     // Build a source used to send raw linear data
@@ -645,18 +914,18 @@ TempSource::TempSource(String& desc, DataBlock* rawdata)
 		desc.c_str(),m_rawdata?m_rawdata->length():0,this);
 	    return;
 	}
-	m_name = "rawdata";
 	m_tone = m_single = (Tone*)::malloc(2*sizeof(Tone));
 	m_single[0].nsamples = m_rawdata->length() / sizeof(short);
 	m_single[0].data = (short*)m_rawdata->data();
+	m_single[0].repeat = true;
 	m_single[1].nsamples = 0;
 	m_single[1].data = 0;
 	return;
     }
     // try first the named tones
-    const ToneDesc* tde = getBlock(desc,true);
+    const ToneDesc* tde = getBlock(desc,prefix,true);
     if (tde) {
-	m_tone = tde->tone;
+	m_tone = tde->tones();
 	return;
     }
     // for performance reason accept an entire string of DTMFs
@@ -670,12 +939,15 @@ TempSource::TempSource(String& desc, DataBlock* rawdata)
 	return;
     }
     // now try to build a single tone
-    ToneData* td = ToneData::getData(desc);
-    if (!td)
+    int samples = 8000;
+    const short* data = 0;
+    bool repeat = true;
+    if (!ToneData::decode(desc,samples,data,repeat))
 	return;
     m_single = (Tone*)::malloc(2*sizeof(Tone));
-    m_single[0].nsamples = 8000;
-    m_single[0].data = td->data();
+    m_single[0].nsamples = samples;
+    m_single[0].data = data;
+    m_single[0].repeat = repeat;
     m_single[1].nsamples = 0;
     m_single[1].data = 0;
     m_tone = m_single;
@@ -692,13 +964,13 @@ TempSource::~TempSource()
 }
 
 
-ToneChan::ToneChan(String& tone)
+ToneChan::ToneChan(String& tone, const String& prefix)
     : Channel(__plugin)
 {
-    Debug(this,DebugAll,"ToneChan::ToneChan(\"%s\") [%p]",tone.c_str(),this);
+    Debug(this,DebugAll,"ToneChan::ToneChan(\"%s\",\"%s\") [%p]",tone.c_str(),prefix.safe(),this);
     // protect the list while the new tone source is added to it
     __plugin.lock();
-    ToneSource* t = ToneSource::getTone(tone);
+    ToneSource* t = ToneSource::getTone(tone,prefix);
     __plugin.unlock();
     if (t) {
 	setSource(t);
@@ -714,6 +986,19 @@ ToneChan::~ToneChan()
     Debug(this,DebugAll,"ToneChan::~ToneChan() %s [%p]",id().c_str(),this);
 }
 
+bool ToneChan::attachConsumer(const char* consumer)
+{
+    if (TelEngine::null(consumer))
+	return false;
+    Message m("chan.attach");
+    m.userData(this);
+    m.addParam("id",id());
+    m.addParam("consumer",consumer);
+    m.addParam("single",String::boolText(true));
+    return Engine::dispatch(m);
+}
+
+
 // Get a data block from a binary parameter of msg
 DataBlock* getRawData(Message& msg)
 {
@@ -763,7 +1048,7 @@ bool AttachHandler::received(Message& msg)
 
     Lock lock(__plugin);
     if (src) {
-	ToneSource* t = ToneSource::getTone(src);
+	ToneSource* t = ToneSource::getTone(src,msg["lang"]);
 	if (t) {
 	    de->setSource(t);
 	    t->deref();
@@ -779,7 +1064,7 @@ bool AttachHandler::received(Message& msg)
 	RefPointer<DataConsumer> c = de->getConsumer();
 	DataEndpoint::commonMutex().unlock();
 	if (c) {
-	    TempSource* t = new TempSource(ovr,getRawData(msg));
+	    TempSource* t = new TempSource(ovr,msg["lang"],getRawData(msg));
 	    if (DataTranslator::attachChain(t,c,true) && t->startup())
 		msg.clearParam("override");
 	    else {
@@ -798,7 +1083,7 @@ bool AttachHandler::received(Message& msg)
 	RefPointer<DataConsumer> c = de->getConsumer();
 	DataEndpoint::commonMutex().unlock();
 	if (c) {
-	    TempSource* t = new TempSource(repl,getRawData(msg));
+	    TempSource* t = new TempSource(repl,msg["lang"],getRawData(msg));
 	    if (DataTranslator::attachChain(t,c,false) && t->startup())
 		msg.clearParam("replace");
 	    else {
@@ -820,8 +1105,9 @@ bool ToneGenDriver::msgExecute(Message& msg, String& dest)
 {
     CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userData());
     if (ch) {
-	ToneChan *tc = new ToneChan(dest);
+	ToneChan *tc = new ToneChan(dest,msg["lang"]);
 	tc->initChan();
+	tc->attachConsumer(msg.getValue("consumer"));
 	if (ch->connect(tc,msg.getValue("reason"))) {
 	    tc->callConnect(msg);
 	    msg.setParam("peerid",tc->id());
@@ -834,25 +1120,26 @@ bool ToneGenDriver::msgExecute(Message& msg, String& dest)
     }
     else {
 	Message m("call.route");
-	m.addParam("module",name());
-	m.addParam("cdrtrack",String::boolText(false));
-	m.copyParam(msg,"called");
-	m.copyParam(msg,"caller");
-	m.copyParam(msg,"callername");
-	String callto(msg.getValue("direct"));
+	m.copyParams(msg,msg[YSTRING("copyparams")]);
+	m.clearParam(YSTRING("callto"));
+	m.clearParam(YSTRING("id"));
+	m.setParam("module",name());
+	m.setParam("cdrtrack",String::boolText(false));
+	m.copyParam(msg,YSTRING("called"));
+	m.copyParam(msg,YSTRING("caller"));
+	m.copyParam(msg,YSTRING("callername"));
+	String callto(msg.getValue(YSTRING("direct")));
 	if (callto.null()) {
-	    const char *targ = msg.getValue("target");
+	    const char *targ = msg.getValue(YSTRING("target"));
 	    if (!targ)
-		targ = msg.getValue("called");
+		targ = msg.getValue(YSTRING("called"));
 	    if (!targ) {
 		Debug(DebugWarn,"Tone outgoing call with no target!");
 		return false;
 	    }
-	    callto = msg.getValue("caller");
-	    if (callto.null())
-		callto << prefix() << dest;
 	    m.setParam("called",targ);
-	    m.setParam("caller",callto);
+	    if (!m.getValue(YSTRING("caller")))
+		m.setParam("caller",prefix() + dest);
 	    if ((!Engine::dispatch(m)) || m.retValue().null() || (m.retValue() == "-")) {
 		Debug(DebugWarn,"Tone outgoing call but no route!");
 		return false;
@@ -862,12 +1149,14 @@ bool ToneGenDriver::msgExecute(Message& msg, String& dest)
 	}
 	m = "call.execute";
 	m.setParam("callto",callto);
-	ToneChan *tc = new ToneChan(dest);
+	ToneChan *tc = new ToneChan(dest,msg["lang"]);
 	tc->initChan();
+	tc->attachConsumer(msg.getValue("consumer"));
 	m.setParam("id",tc->id());
 	m.userData(tc);
 	if (Engine::dispatch(m)) {
 	    msg.setParam("id",tc->id());
+	    msg.copyParam(m,YSTRING("peerid"));
 	    tc->deref();
 	    return true;
 	}
@@ -916,11 +1205,53 @@ void ToneGenDriver::initialize()
     Output("Initializing module ToneGen");
     setup(0,true); // no need to install notifications
     Driver::initialize();
-    if (!m_handler) {
-	m_handler = new AttachHandler;
-	Engine::install(m_handler);
-	installRelay(Halt);
+    if (m_handler)
+	return;
+    // Init default tones
+    s_defToneDesc.append(new ToneDesc(t_dial,"dial"));
+    s_defToneDesc.append(new ToneDesc(t_busy,"busy"));
+    s_defToneDesc.append(new ToneDesc(t_ring,"ring"));
+    s_defToneDesc.append(new ToneDesc(t_specdial,"specdial"));
+    s_defToneDesc.append(new ToneDesc(t_congestion,"congestion"));
+    s_defToneDesc.append(new ToneDesc(t_outoforder,"outoforder"));
+    s_defToneDesc.append(new ToneDesc(t_info,"info"));
+    s_defToneDesc.append(new ToneDesc(t_mwatt,"milliwatt"));
+    s_defToneDesc.append(new ToneDesc(t_silence,"silence"));
+    s_defToneDesc.append(new ToneDesc(t_noise,"noise"));
+    s_defToneDesc.append(new ToneDesc(t_probes[0],"probe/0"));
+    s_defToneDesc.append(new ToneDesc(t_probes[1],"probe/1"));
+    s_defToneDesc.append(new ToneDesc(t_probes[2],"probe/2"));
+    s_defToneDesc.append(new ToneDesc(t_probes[3],"cotv"));
+    s_defToneDesc.append(new ToneDesc(t_probes[4],"cots"));
+    // Init tones from config
+    Configuration cfg(Engine::configFile("tonegen"));
+    s_defLang = cfg.getValue("general","lang");
+    if (s_defLang == s_default)
+	s_defLang.clear();
+    unsigned int n = cfg.sections();
+    for (unsigned int i = 0; i < n; i++) {
+	NamedList* l = cfg.getSection(i);
+	if (!l || *l == "general")
+	    continue;
+	String aliases;
+	if (*l != s_default)
+	    aliases = l->getValue("alias");
+	l->clearParam("alias");
+	ToneDesc::buildTones(*l,*l);
+	if (aliases) {
+	    ObjList* list = aliases.split(',',false);
+	    for (ObjList* o = list->skipNull(); o; o = o->skipNext()) {
+		const String& name = o->get()->toString();
+		if (name != s_default)
+		    ToneDesc::buildTones(name,*l);
+	    }
+	    TelEngine::destruct(list);
+	}
     }
+    // Init module
+    m_handler = new AttachHandler;
+    Engine::install(m_handler);
+    installRelay(Halt);
 }
 
 }; // anonymous namespace
