@@ -94,6 +94,8 @@ class WaveChan : public Channel
 public:
     WaveChan(const String& file, bool record, unsigned maxlen, bool autorepeat, const char* format = 0, const NamedString* param = 0);
     ~WaveChan();
+    bool attachSource(const char* source);
+    bool attachConsumer(const char* consumer);
 };
 
 class Disconnector : public Thread
@@ -767,6 +769,30 @@ WaveChan::~WaveChan()
     Debug(this,DebugAll,"WaveChan::~WaveChan() %s [%p]",id().c_str(),this);
 }
 
+bool WaveChan::attachSource(const char* source)
+{
+    if (TelEngine::null(source))
+	return false;
+    Message m("chan.attach");
+    m.userData(this);
+    m.addParam("id",id());
+    m.addParam("source",source);
+    m.addParam("single",String::boolText(true));
+    return Engine::dispatch(m);
+}
+
+bool WaveChan::attachConsumer(const char* consumer)
+{
+    if (TelEngine::null(consumer))
+	return false;
+    Message m("chan.attach");
+    m.userData(this);
+    m.addParam("id",id());
+    m.addParam("consumer",consumer);
+    m.addParam("single",String::boolText(true));
+    return Engine::dispatch(m);
+}
+
 
 bool AttachHandler::received(Message &msg)
 {
@@ -775,7 +801,7 @@ bool AttachHandler::received(Message &msg)
     if (src.null())
 	more--;
     else {
-	Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
+	static const Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
 	if (src.matches(r)) {
 	    if (src.matchString(1) == "play") {
 		src = src.matchString(2);
@@ -795,7 +821,7 @@ bool AttachHandler::received(Message &msg)
     if (cons.null())
 	more--;
     else {
-	Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
+	static const Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
 	if (cons.matches(r)) {
 	    if (cons.matchString(1) == "record") {
 		cons = cons.matchString(2);
@@ -815,7 +841,7 @@ bool AttachHandler::received(Message &msg)
     if (ovr.null())
 	more--;
     else {
-	Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
+	static const Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
 	if (ovr.matches(r)) {
 	    if (ovr.matchString(1) == "play") {
 		ovr = ovr.matchString(2);
@@ -835,7 +861,7 @@ bool AttachHandler::received(Message &msg)
     if (repl.null())
 	more--;
     else {
-	Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
+	static const Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
 	if (repl.matches(r)) {
 	    if (repl.matchString(1) == "play") {
 		repl = repl.matchString(2);
@@ -944,7 +970,7 @@ bool RecordHandler::received(Message &msg)
     if (c1.null())
 	more--;
     else {
-	Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
+	static const Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
 	if (c1.matches(r)) {
 	    if (c1.matchString(1) == "record") {
 		c1 = c1.matchString(2);
@@ -964,7 +990,7 @@ bool RecordHandler::received(Message &msg)
     if (c2.null())
 	more--;
     else {
-	Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
+	static const Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
 	if (c2.matches(r)) {
 	    if (c2.matchString(1) == "record") {
 		c2 = c2.matchString(2);
@@ -987,7 +1013,7 @@ bool RecordHandler::received(Message &msg)
     unsigned int maxlen = msg.getIntValue("maxlen");
 
     CallEndpoint *ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
-    DataEndpoint *de = static_cast<DataEndpoint*>(msg.userObject("DataEndpoint"));
+    RefPointer<DataEndpoint> de = static_cast<DataEndpoint*>(msg.userObject("DataEndpoint"));
     if (ch && !de)
 	de = ch->setEndpoint();
 
@@ -1020,7 +1046,7 @@ bool RecordHandler::received(Message &msg)
 
 bool WaveFileDriver::msgExecute(Message& msg, String& dest)
 {
-    Regexp r("^\\([^/]*\\)/\\(.*\\)$");
+    static const Regexp r("^\\([^/]*\\)/\\(.*\\)$");
     if (!dest.matches(r))
 	return false;
 
@@ -1040,6 +1066,10 @@ bool WaveFileDriver::msgExecute(Message& msg, String& dest)
 	    dest.matchString(2).c_str());
 	WaveChan *c = new WaveChan(dest.matchString(2),meth,maxlen,msg.getBoolValue("autorepeat"),msg.getValue("format"),msg.getParam("callto"));
 	c->initChan();
+	if (meth)
+	    c->attachSource(msg.getValue("source"));
+	else
+	    c->attachConsumer(msg.getValue("consumer"));
 	if (ch->connect(c,msg.getValue("reason"))) {
 	    c->callConnect(msg);
 	    msg.setParam("peerid",c->id());
@@ -1052,20 +1082,26 @@ bool WaveFileDriver::msgExecute(Message& msg, String& dest)
 	}
     }
     Message m("call.route");
-    m.addParam("module",name());
-    m.addParam("cdrtrack",String::boolText(false));
-    String callto(msg.getValue("direct"));
+    m.copyParams(msg,msg[YSTRING("copyparams")]);
+    m.clearParam(YSTRING("callto"));
+    m.clearParam(YSTRING("id"));
+    m.setParam("module",name());
+    m.setParam("cdrtrack",String::boolText(false));
+    m.copyParam(msg,YSTRING("called"));
+    m.copyParam(msg,YSTRING("caller"));
+    m.copyParam(msg,YSTRING("callername"));
+    String callto(msg.getValue(YSTRING("direct")));
     if (callto.null()) {
-	const char *targ = msg.getValue("target");
+	const char *targ = msg.getValue(YSTRING("target"));
+	if (!targ)
+	    targ = msg.getValue(YSTRING("called"));
 	if (!targ) {
 	    Debug(DebugWarn,"Wave outgoing call with no target!");
 	    return false;
 	}
-	callto = msg.getValue("caller");
-	if (callto.null())
-	    callto << prefix() << dest;
-	m.addParam("called",targ);
-	m.addParam("caller",callto);
+	m.setParam("called",targ);
+	if (!m.getValue(YSTRING("caller")))
+	    m.setParam("caller",prefix() + dest);
 	if (!Engine::dispatch(m)) {
 	    Debug(DebugWarn,"Wave outgoing call but no route!");
 	    return false;
@@ -1074,13 +1110,18 @@ bool WaveFileDriver::msgExecute(Message& msg, String& dest)
 	m.retValue().clear();
     }
     m = "call.execute";
-    m.addParam("callto",callto);
+    m.setParam("callto",callto);
     WaveChan *c = new WaveChan(dest.matchString(2),meth,maxlen,msg.getBoolValue("autorepeat"),msg.getValue("format"));
     c->initChan();
+    if (meth)
+	c->attachSource(msg.getValue("source"));
+    else
+	c->attachConsumer(msg.getValue("consumer"));
     m.setParam("id",c->id());
     m.userData(c);
     if (Engine::dispatch(m)) {
 	msg.setParam("id",c->id());
+	msg.copyParam(m,YSTRING("peerid"));
 	c->deref();
 	return true;
     }
