@@ -60,6 +60,8 @@ static TokenDict dict_payloads[] = {
     { "speex",       102 },
     { "speex/16000", 103 },
     { "speex/32000", 104 },
+    { "isac/16000",  105 },
+    { "isac/32000",  106 },
     { "mjpeg",        26 },
     { "h261",         31 },
     { "h263",         34 },
@@ -92,6 +94,7 @@ static String s_localip;
 static String s_notifyMsg;
 static bool s_autoaddr  = true;
 static bool s_anyssrc   = false;
+static bool s_warnFirst = false;
 static bool s_warnLater = false;
 static bool s_rtcp  = true;
 static bool s_drill = false;
@@ -99,9 +102,12 @@ static bool s_drill = false;
 static Thread::Priority s_priority = Thread::Normal;
 static int s_tos     = 0;
 static int s_sleep   = 5;
+static int s_interval= 0;
 static int s_timeout = 0;
-static int s_minjitter = 0;
-static int s_maxjitter = 0;
+static int s_udptlTimeout = 0;
+
+static int s_minJitter = 0;
+static int s_maxJitter = 0;
 
 class YRTPSource;
 class YRTPConsumer;
@@ -121,6 +127,7 @@ public:
     virtual void* getObject(const String& name) const;
     bool setParams(const char* raddr, Message& msg);
     bool setRemote(const char* raddr, unsigned int rport, const Message& msg);
+    void setFaxDivert(const Message& msg);
     bool sendDTMF(char dtmf, int duration = 0);
     void gotDTMF(char tone);
     void gotFax();
@@ -180,18 +187,25 @@ private:
     String m_media;
     String m_format;
     String m_master;
+    String m_faxDivert;
+    String m_faxCaller;
+    String m_faxCalled;
     String m_host;
     unsigned int m_bufsize;
     unsigned int m_port;
     bool m_audio;
     bool m_valid;
+
+    unsigned int m_noAudio;
+    unsigned int m_lostAudio;
 };
 
 class YRTPSession : public RTPSession
 {
 public:
     inline YRTPSession(YRTPWrapper* wrap)
-	: m_wrap(wrap), m_resync(false), m_anyssrc(false), m_getFax(true)
+	: m_wrap(wrap), m_lastLost(0),
+	  m_resync(false), m_anyssrc(false), m_getFax(true)
 	{ }
     virtual ~YRTPSession();
     virtual bool rtpRecvData(bool marker, unsigned int timestamp,
@@ -210,6 +224,7 @@ protected:
     virtual void timeout(bool initial);
 private:
     YRTPWrapper* m_wrap;
+    u_int32_t m_lastLost;
     bool m_resync;
     bool m_anyssrc;
     bool m_getFax;
@@ -255,7 +270,7 @@ public:
 	{ return m_wrap && m_wrap->valid(); }
     virtual unsigned long Consume(const DataBlock &data, unsigned long tStamp, unsigned long flags);
     inline void setSplitable()
-	{ m_splitable = (m_format == "alaw") || (m_format == "mulaw"); }
+	{ m_splitable = (m_format == YSTRING("alaw")) || (m_format == YSTRING("mulaw")); }
 private:
     YRTPWrapper* m_wrap;
     bool m_splitable;
@@ -326,7 +341,7 @@ public:
     virtual ~CipherHolder()
 	{ TelEngine::destruct(m_cipher); }
     virtual void* getObject(const String& name) const
-	{ return (name == "Cipher*") ? (void*)&m_cipher : RefObject::getObject(name); }
+	{ return (name == YSTRING("Cipher*")) ? (void*)&m_cipher : RefObject::getObject(name); }
     inline Cipher* cipher()
 	{ Cipher* tmp = m_cipher; m_cipher = 0; return tmp; }
 private:
@@ -363,6 +378,10 @@ public:
     virtual bool received(Message& msg, int id);
     virtual void statusParams(String& str);
     virtual void statusDetail(String& str);
+
+    void addStats(const NamedList& stats);
+    void genUpdate(Message& msg);
+
 private:
     bool reflectSetup(Message& msg, const char* id, RTPTransport& rtp, const char* rHost, const char* leg);
     bool reflectStart(Message& msg, const char* id, RTPTransport& rtp, SocketAddr& rAddr);
@@ -371,38 +390,40 @@ private:
     void reflectAnswer(Message& msg, bool ignore);
     void reflectHangup(Message& msg);
     bool m_first;
+    ObjList m_updates;
 };
 
 static YRTPPlugin splugin;
 static ObjList s_calls;
 static ObjList s_mirrors;
 static Mutex s_mutex(false,"YRTPChan");
+static Mutex s_refMutex(false,"YRTPChan::reflect");
 static Mutex s_srcMutex(false,"YRTPChan::source");
 
 
 YRTPWrapper::YRTPWrapper(const char* localip, CallEndpoint* conn, const char* media, RTPSession::Direction direction, Message& msg, bool udptl)
     : m_rtp(0), m_udptl(0), m_dir(direction), m_conn(conn),
       m_source(0), m_consumer(0), m_media(media),
-      m_bufsize(0), m_port(0), m_valid(true)
+      m_bufsize(0), m_port(0), m_valid(true), m_noAudio(0), m_lostAudio(0)
 {
     Debug(&splugin,DebugAll,"YRTPWrapper::YRTPWrapper('%s',%p,'%s',%s,%p,%s) [%p]",
 	localip,conn,media,lookup(direction,dict_yrtp_dir),
 	&msg,String::boolText(udptl),this);
     m_id = udptl ? "udptl/" : "yrtp/";
-    m_id << (unsigned int)::random();
+    m_id << (unsigned int)Random::random();
     if (conn)
 	m_master = conn->id();
-    m_audio = !udptl && (m_media == "audio");
+    m_audio = !udptl && (m_media == YSTRING("audio"));
     s_mutex.lock();
     s_calls.append(this);
     if (udptl) {
-	int md = 0xffff & msg.getIntValue("sdp_T38FaxMaxDatagram");
-	md = msg.getIntValue("t38maxdatagram",md);
+	int md = 0xffff & msg.getIntValue(YSTRING("sdp_T38FaxMaxDatagram"));
+	md = msg.getIntValue(YSTRING("t38maxdatagram"),md);
 	if (md < 96)
 	    md = 250;
-	const String* ec = msg.getParam("sdp_T38FaxUdpEC");
-	int ms = (ec && (*ec == "t38UDPRedundancy")) ? 2 : 0;
-	ms = msg.getIntValue("t38redundancy",ms);
+	const String* ec = msg.getParam(YSTRING("sdp_T38FaxUdpEC"));
+	int ms = (ec && (*ec == YSTRING("t38UDPRedundancy"))) ? 2 : 0;
+	ms = msg.getIntValue(YSTRING("t38redundancy"),ms);
 	if (ms < 0)
 	    ms = 0;
 	else if (ms > 16)
@@ -411,7 +432,7 @@ YRTPWrapper::YRTPWrapper(const char* localip, CallEndpoint* conn, const char* me
 	setupUDPTL(localip,md,ms);
     }
     else
-	setupRTP(localip,msg.getBoolValue("rtcp",s_rtcp));
+	setupRTP(localip,msg.getBoolValue(YSTRING("rtcp"),s_rtcp));
     s_mutex.unlock();
 }
 
@@ -442,13 +463,13 @@ YRTPWrapper::~YRTPWrapper()
 
 void* YRTPWrapper::getObject(const String& name) const
 {
-    if (name == "Socket")
+    if (name == YSTRING("Socket"))
 	return m_rtp ? m_rtp->rtpSock() : 0;
-    if (name == "DataSource")
+    if (name == YSTRING("DataSource"))
 	return m_source;
-    if (name == "DataConsumer")
+    if (name == YSTRING("DataConsumer"))
 	return m_consumer;
-    if (name == "RTPSession")
+    if (name == YSTRING("RTPSession"))
 	return m_rtp;
     return RefObject::getObject(name);
 }
@@ -460,9 +481,9 @@ YRTPWrapper* YRTPWrapper::find(const CallEndpoint* conn, const String& media)
     Lock lock(s_mutex);
     ObjList* l = &s_calls;
     for (; l; l=l->next()) {
-	const YRTPWrapper *p = static_cast<const YRTPWrapper *>(l->get());
+	YRTPWrapper *p = static_cast<YRTPWrapper *>(l->get());
 	if (p && (p->conn() == conn) && (p->media() == media))
-	    return const_cast<YRTPWrapper *>(p);
+	    return p->ref() ? p : 0;
     }
     return 0;
 }
@@ -474,9 +495,9 @@ YRTPWrapper* YRTPWrapper::find(const String& id)
     Lock lock(s_mutex);
     ObjList* l = &s_calls;
     for (; l; l=l->next()) {
-	const YRTPWrapper *p = static_cast<const YRTPWrapper *>(l->get());
+	YRTPWrapper *p = static_cast<YRTPWrapper *>(l->get());
 	if (p && (p->id() == id))
-	    return const_cast<YRTPWrapper *>(p);
+	    return p->ref() ? p : 0;
     }
     return 0;
 }
@@ -535,7 +556,7 @@ bool YRTPWrapper::bindLocal(const char* localip, bool rtcp)
 	return false;
     }
     for (; attempt; attempt--) {
-	int lport = (minport + (::random() % (maxport - minport))) & 0xfffe;
+	int lport = (minport + (Random::random() % (maxport - minport))) & 0xfffe;
 	addr.port(lport);
 	if (m_rtp ? m_rtp->localAddr(addr,rtcp) : m_udptl->localAddr(addr)) {
 	    m_host = addr.host();
@@ -553,11 +574,11 @@ bool YRTPWrapper::bindLocal(const char* localip, bool rtcp)
 bool YRTPWrapper::setParams(const char* rip, Message& msg)
 {
     // start or just setup either RTP or UDPTL
-    int rport = msg.getIntValue("remoteport");
+    int rport = msg.getIntValue(YSTRING("remoteport"));
     if (rip && (rport > 0))
 	return m_udptl ? startUDPTL(rip,rport,msg) : startRTP(rip,rport,msg);
     else
-	return m_udptl ? setupUDPTL(msg) : setupSRTP(msg,msg.getBoolValue("secure"));
+	return m_udptl ? setupUDPTL(msg) : setupSRTP(msg,msg.getBoolValue(YSTRING("secure")));
 }
 
 bool YRTPWrapper::setRemote(const char* raddr, unsigned int rport, const Message& msg)
@@ -565,7 +586,7 @@ bool YRTPWrapper::setRemote(const char* raddr, unsigned int rport, const Message
     if (!session())
 	return false;
     SocketAddr addr(AF_INET);
-    if (!(addr.host(raddr) && addr.port(rport) && session()->remoteAddr(addr,msg.getBoolValue("autoaddr",s_autoaddr)))) {
+    if (!(addr.host(raddr) && addr.port(rport) && session()->remoteAddr(addr,msg.getBoolValue(YSTRING("autoaddr"),s_autoaddr)))) {
 	Debug(&splugin,DebugWarn,"RTP failed to set remote address %s:%d [%p]",raddr,rport,this);
 	return false;
     }
@@ -574,11 +595,15 @@ bool YRTPWrapper::setRemote(const char* raddr, unsigned int rport, const Message
 
 void YRTPWrapper::setTimeout(const Message& msg, int timeOut)
 {
-    const String* param = msg.getParam("timeout");
+    const String* param = msg.getParam(YSTRING("timeout"));
     if (param) {
 	// accept true/false to apply default or disable
-	if (param->isBoolean())
-	    timeOut = param->toBoolean() ? s_timeout : 0;
+	if (param->isBoolean()) {
+	    if (param->toBoolean())
+		timeOut = rtp() ? s_timeout : s_udptlTimeout;
+	    else
+		timeOut = 0;
+	}
 	else
 	    timeOut = param->toInteger(timeOut);
     }
@@ -602,14 +627,14 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, Message& msg)
 	return true;
     }
 
-    String p(msg.getValue("payload"));
+    String p(msg.getValue(YSTRING("payload")));
     if (p.null())
-	p = msg.getValue("format");
+	p = msg.getValue(YSTRING("format"));
     int payload = p.toInteger(dict_payloads,-1);
-    int evpayload = msg.getIntValue("evpayload",101);
-    const char* format = msg.getValue("format");
-    int tos = msg.getIntValue("tos",dict_tos,s_tos);
-    int msec = msg.getIntValue("msleep",s_sleep);
+    int evpayload = msg.getIntValue(YSTRING("evpayload"),101);
+    const char* format = msg.getValue(YSTRING("format"));
+    int tos = msg.getIntValue(YSTRING("tos"),dict_tos,s_tos);
+    int msec = msg.getIntValue(YSTRING("msleep"),s_sleep);
 
     if (!format)
 	format = lookup(payload, dict_payloads);
@@ -634,12 +659,10 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, Message& msg)
     }
 
     Debug(&splugin,DebugInfo,"RTP starting format '%s' payload %d [%p]",format,payload,this);
-    int minJitter = msg.getIntValue("minjitter",s_minjitter);
-    int maxJitter = msg.getIntValue("maxjitter",s_maxjitter);
 
     if (!setRemote(raddr,rport,msg))
 	return false;
-    m_rtp->anySSRC(msg.getBoolValue("anyssrc",s_anyssrc));
+    m_rtp->anySSRC(msg.getBoolValue(YSTRING("anyssrc"),s_anyssrc));
     m_format = format;
     // Change format of source and/or consumer,
     //  reinstall them to rebuild codec chains
@@ -666,15 +689,15 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, Message& msg)
 	    m_consumer->deref();
 	}
     }
-    if (!(m_rtp->initGroup(msec,Thread::priority(msg.getValue("thread"),s_priority)) &&
+    if (!(m_rtp->initGroup(msec,Thread::priority(msg.getValue(YSTRING("thread")),s_priority)) &&
 	 m_rtp->direction(m_dir)))
 	return false;
 
     bool secure = false;
-    const String* sec = msg.getParam("crypto_suite");
+    const String* sec = msg.getParam(YSTRING("crypto_suite"));
     if (sec && *sec) {
 	// separate crypto parameters
-	const String* key = msg.getParam("crypto_key");
+	const String* key = msg.getParam(YSTRING("crypto_key"));
 	if (key && *key) {
 	    if (startSRTP(*sec,*key,0))
 		secure = true;
@@ -692,16 +715,22 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, Message& msg)
     m_rtp->dataPayload(payload);
     m_rtp->eventPayload(evpayload);
     m_rtp->setTOS(tos);
-    m_rtp->padding(msg.getIntValue("padding",s_padding));
-    if (msg.getBoolValue("drillhole",s_drill)) {
+    m_rtp->padding(msg.getIntValue(YSTRING("padding"),s_padding));
+    if (msg.getBoolValue(YSTRING("drillhole"),s_drill)) {
 	bool ok = m_rtp->drillHole();
 	Debug(&splugin,(ok ? DebugInfo : DebugWarn),
 	    "Wrapper %s a hole in firewall/NAT [%p]",
 	    (ok ? "opened" : "failed to open"),this);
     }
     setTimeout(msg,s_timeout);
-//    if (maxJitter > 0)
-//	m_rtp->setDejitter(minJitter*1000,maxJitter*1000);
+    m_rtp->setReports(msg.getIntValue(YSTRING("rtcp_interval"),s_interval));
+    // dejittering is only meaningful for audio
+    if (isAudio()){ 
+	int minJitter = msg.getIntValue(YSTRING("minjitter"),s_minJitter);
+	int maxJitter = msg.getIntValue(YSTRING("maxjitter"),s_maxJitter);
+	if (minJitter >= 0 && maxJitter > 0)
+	    m_rtp->setDejitter(minJitter*1000,maxJitter*1000);
+    }
     m_bufsize = s_bufsize;
     return true;
 }
@@ -714,21 +743,21 @@ bool YRTPWrapper::startUDPTL(const char* raddr, unsigned int rport, Message& msg
 	return false;
     }
 
-    int tos = msg.getIntValue("tos",dict_tos,s_tos);
-    int msec = msg.getIntValue("msleep",s_sleep);
+    int tos = msg.getIntValue(YSTRING("tos"),dict_tos,s_tos);
+    int msec = msg.getIntValue(YSTRING("msleep"),s_sleep);
     if (!setRemote(raddr,rport,msg))
 	return false;
-    if (!m_udptl->initGroup(msec,Thread::priority(msg.getValue("thread"),s_priority)))
+    if (!m_udptl->initGroup(msec,Thread::priority(msg.getValue(YSTRING("thread")),s_priority)))
 	return false;
 
     m_udptl->setTOS(tos);
-    if (msg.getBoolValue("drillhole",s_drill)) {
+    if (msg.getBoolValue(YSTRING("drillhole"),s_drill)) {
 	bool ok = m_udptl->drillHole();
 	Debug(&splugin,(ok ? DebugInfo : DebugWarn),
 	    "Wrapper %s a hole in firewall/NAT [%p]",
 	    (ok ? "opened" : "failed to open"),this);
     }
-    setTimeout(msg,s_timeout);
+    setTimeout(msg,s_udptlTimeout);
     return setupUDPTL(msg);
 }
 
@@ -748,7 +777,7 @@ bool YRTPWrapper::setupSRTP(Message& msg, bool buildMaster)
 	if (srtp)
 	    srtp = new RTPSecure(*srtp);
 	else
-	    srtp = new RTPSecure(msg.getValue("crypto_suite"));
+	    srtp = new RTPSecure(msg.getValue(YSTRING("crypto_suite")));
     }
     else
 	buildMaster = false;
@@ -811,15 +840,30 @@ void YRTPWrapper::gotFax()
 	return;
     Message* m = new Message("chan.masquerade");
     m->addParam("id",m_master);
-    m->addParam("message","call.fax");
-    m->addParam("detected","rfc2833");
+    if (m_faxDivert) {
+	Debug(&splugin,DebugCall,"Diverting call %s to: %s",
+	    m_master.c_str(),m_faxDivert.c_str());
+	m->addParam("message","call.execute");
+	m->addParam("callto",m_faxDivert);
+	m->addParam("reason","fax");
+    }
+    else {
+	m->addParam("message","call.fax");
+	m->addParam("detected","rfc2833");
+    }
+    m->addParam("caller",m_faxCaller,false);
+    m->addParam("called",m_faxCalled,false);
     Engine::enqueue(m);
 }
 
 void YRTPWrapper::timeout(bool initial)
 {
-    if (!(initial || s_warnLater))
+    if (!(initial ? s_warnFirst : s_warnLater))
 	return;
+    if (initial)
+	m_noAudio++;
+    else 
+	m_lostAudio++;
     Debug(&splugin,DebugWarn,"%s timeout in%s%s wrapper [%p]",
 	(initial ? "Initial" : "Later"),
 	(m_master ? " channel " : ""),
@@ -880,8 +924,15 @@ void YRTPWrapper::terminate(Message& msg)
 {
     Debug(&splugin,DebugInfo,"YRTPWrapper::terminate() [%p]",this);
     String stats;
-    if (m_rtp)
+    if (m_rtp) {
+	NamedList nl("");
+	m_rtp->getStats(nl);
+	nl.setParam("noaudio",String(m_noAudio));
+	nl.setParam("lostaudio",String(m_lostAudio));
+    	splugin.addStats(nl);
+    	splugin.changed();
 	m_rtp->getStats(stats);
+    }
     if (m_udptl)
 	m_udptl->getStats(stats);
     if (stats)
@@ -889,6 +940,21 @@ void YRTPWrapper::terminate(Message& msg)
     m_valid = false;
 }
 
+void YRTPWrapper::setFaxDivert(const Message& msg)
+{
+    NamedString* divert = msg.getParam(YSTRING("fax_divert"));
+    if (!divert)
+	return;
+    // if divert is empty or false disable diverting
+    if (divert->null() || !divert->toBoolean(true))
+	m_faxDivert.clear();
+    else {
+	m_faxDivert = *divert;
+	m_faxCaller = msg.getValue(YSTRING("fax_caller"),msg.getValue(YSTRING("caller"),m_faxCaller));
+	m_faxCalled = msg.getValue(YSTRING("fax_called"),msg.getValue(YSTRING("called"),m_faxCalled));
+    }
+}
+
 
 YRTPSession::~YRTPSession()
 {
@@ -913,10 +979,17 @@ bool YRTPSession::rtpRecvData(bool marker, unsigned int timestamp, const void* d
     s_srcMutex.unlock();
     if (!source)
 	return false;
+    unsigned long flags = (marker ? DataNode::DataMark : 0);
+    u_int32_t lost = ioPacketsLost();
+    if (lost != m_lastLost) {
+	if (lost > m_lastLost)
+	    flags |= DataNode::DataMissed;
+	m_lastLost = lost;
+    }
     // the source will not be destroyed until we reset the busy flag
     DataBlock block;
     block.assign((void*)data, len, false);
-    source->Forward(block,timestamp,(marker ? DataNode::DataMark : 0));
+    source->Forward(block,timestamp,flags);
     block.clear(false);
     source->busy(false);
     return true;
@@ -1161,7 +1234,7 @@ void YRTPMonitor::timeout(bool initial)
 {
     if (null(m_id))
 	return;
-    if (!(initial || s_warnLater))
+    if (!(initial ? s_warnFirst : s_warnLater))
 	return;
     Debug(&splugin,DebugWarn,"%s timeout in '%s' reflector [%p]",
 	(initial ? "Initial" : "Later"),m_id->c_str(),this);
@@ -1236,7 +1309,7 @@ YRTPReflector::~YRTPReflector()
 bool AttachHandler::received(Message &msg)
 {
     int more = 2;
-    String src(msg.getValue("source"));
+    String src(msg.getValue(YSTRING("source")));
     if (src.null())
 	more--;
     else {
@@ -1246,7 +1319,7 @@ bool AttachHandler::received(Message &msg)
 	    src = "";
     }
 
-    String cons(msg.getValue("consumer"));
+    String cons(msg.getValue(YSTRING("consumer")));
     if (cons.null())
 	more--;
     else {
@@ -1258,8 +1331,8 @@ bool AttachHandler::received(Message &msg)
     if (src.null() && cons.null())
 	return false;
 
-    const char* media = msg.getValue("media","audio");
-    String rip(msg.getValue("remoteip"));
+    const char* media = msg.getValue(YSTRING("media"),"audio");
+    String rip(msg.getValue(YSTRING("remoteip")));
     CallEndpoint* ch = YOBJECT(CallEndpoint,msg.userData());
     if (!ch) {
 	if (!src.null())
@@ -1269,15 +1342,15 @@ bool AttachHandler::received(Message &msg)
 	return false;
     }
 
-    YRTPWrapper *w = YRTPWrapper::find(ch,media);
+    RefPointer<YRTPWrapper> w = YRTPWrapper::find(ch,media);
     if (!w)
-	w = YRTPWrapper::find(msg.getValue("rtpid"));
+	w = YRTPWrapper::find(msg.getValue(YSTRING("rtpid")));
     if (!w) {
-	String lip(msg.getValue("localip"));
+	String lip(msg.getValue(YSTRING("localip")));
 	if (lip.null())
 	    YRTPWrapper::guessLocal(rip,lip);
 	w = new YRTPWrapper(lip,ch,media,RTPSession::SendRecv,msg);
-	w->setMaster(msg.getValue("id"));
+	w->setMaster(msg.getValue(YSTRING("id")));
 
 	if (!src.null()) {
 	    DataSource* s = w->getSource();
@@ -1290,12 +1363,14 @@ bool AttachHandler::received(Message &msg)
 	    ch->setConsumer(c,media);
 	    c->deref();
 	}
-
-	if (w->deref())
-	    return false;
     }
 
+    w->deref();
+    if (w->refcount() <= 1)
+	return false;
+
     w->setParams(rip,msg);
+    w->setFaxDivert(msg);
     msg.setParam("localip",w->host());
     msg.setParam("localport",String(w->port()));
     msg.setParam("rtpid",w->id());
@@ -1308,7 +1383,7 @@ bool AttachHandler::received(Message &msg)
 bool RtpHandler::received(Message &msg)
 {
     bool udptl = false;
-    String trans = msg.getValue("transport");
+    String trans = msg.getValue(YSTRING("transport"));
     if (trans && !trans.startsWith("RTP/")) {
 	if (trans &= "udptl")
 	    udptl = true;
@@ -1316,20 +1391,20 @@ bool RtpHandler::received(Message &msg)
 	    return false;
     }
     Debug(&splugin,DebugAll,"%s message received",(trans ? trans.c_str() : "No-transport"));
-    bool terminate = msg.getBoolValue("terminate",false);
-    String dir(msg.getValue("direction"));
+    bool terminate = msg.getBoolValue(YSTRING("terminate"),false);
+    String dir(msg.getValue(YSTRING("direction")));
     RTPSession::Direction direction = terminate ? RTPSession::FullStop : RTPSession::SendRecv;
     bool d_recv = false;
     bool d_send = false;
-    if (dir == "bidir") {
+    if (dir == YSTRING("bidir")) {
 	d_recv = true;
 	d_send = true;
     }
-    else if (dir == "receive") {
+    else if (dir == YSTRING("receive")) {
 	d_recv = true;
 	direction = RTPSession::RecvOnly;
     }
-    else if (dir == "send") {
+    else if (dir == YSTRING("send")) {
 	d_send = true;
 	direction = RTPSession::SendOnly;
     }
@@ -1337,16 +1412,18 @@ bool RtpHandler::received(Message &msg)
     CallEndpoint* ch = YOBJECT(CallEndpoint,msg.userData());
     DataEndpoint* de = YOBJECT(DataEndpoint,msg.userData());
     const char* media = udptl ? "image" : "audio";
-    media = msg.getValue("media",(de ? de->name().c_str() : media));
-    YRTPWrapper* w = YRTPWrapper::find(ch,media);
+    media = msg.getValue(YSTRING("media"),(de ? de->name().c_str() : media));
+    RefPointer<YRTPWrapper> w = YRTPWrapper::find(ch,media);
     if (w)
-	Debug(&splugin,DebugAll,"Wrapper %p found by CallEndpoint %p",w,ch);
+	Debug(&splugin,DebugAll,"Wrapper %p found by CallEndpoint %p",(YRTPWrapper*)w,ch);
     else {
-	const char* rid = msg.getValue("rtpid");
+	const char* rid = msg.getValue(YSTRING("rtpid"));
 	w = YRTPWrapper::find(rid);
 	if (w)
-	    Debug(&splugin,DebugAll,"Wrapper %p found by ID '%s'",w,rid);
+	    Debug(&splugin,DebugAll,"Wrapper %p found by ID '%s'",(YRTPWrapper*)w,rid);
     }
+    if (w)
+	w->deref();
     if (terminate) {
 	if (w) {
 	    w->terminate(msg);
@@ -1359,13 +1436,13 @@ bool RtpHandler::received(Message &msg)
 	return false;
     }
 
-    String rip(msg.getValue("remoteip"));
+    String rip(msg.getValue(YSTRING("remoteip")));
 
     if (!w) {
 	// it would be pointless to create an unreferenced wrapper
 	if (!(d_recv || d_send))
 	    return false;
-	String lip(msg.getValue("localip"));
+	String lip(msg.getValue(YSTRING("localip")));
 	if (lip.null())
 	    YRTPWrapper::guessLocal(rip,lip);
 	if (lip.null()) {
@@ -1374,12 +1451,12 @@ bool RtpHandler::received(Message &msg)
 	}
 
 	w = new YRTPWrapper(lip,ch,media,direction,msg,udptl);
-	w->setMaster(msg.getValue("id"));
+	w->setMaster(msg.getValue(YSTRING("id")));
+
+	w->deref();
     }
-    else if (w->valid()) {
-	w->ref();
+    else if (w->valid())
 	w->addDirection(direction);
-    }
     else
 	return false;
 
@@ -1409,15 +1486,16 @@ bool RtpHandler::received(Message &msg)
 	}
     }
 
-    if (w->deref())
+    if (w->refcount() <= 1)
 	return false;
 
     w->setParams(rip,msg);
+    w->setFaxDivert(msg);
     msg.setParam("localip",w->host());
     msg.setParam("localport",String(w->port()));
     msg.setParam("rtpid",w->id());
 
-    if (msg.getBoolValue("getsession",!msg.userData()))
+    if (msg.getBoolValue(YSTRING("getsession"),!msg.userData()))
 	msg.userData(w);
     return true;
 }
@@ -1425,16 +1503,19 @@ bool RtpHandler::received(Message &msg)
 
 bool DTMFHandler::received(Message &msg)
 {
-    String targetid(msg.getValue("targetid"));
+    String targetid(msg.getValue(YSTRING("targetid")));
     if (targetid.null())
 	return false;
-    String text(msg.getValue("text"));
+    String text(msg.getValue(YSTRING("text")));
     if (text.null())
 	return false;
-    YRTPWrapper* wrap = YRTPWrapper::find(targetid);
-    if (wrap && wrap->rtp()) {
+    RefPointer<YRTPWrapper> wrap = YRTPWrapper::find(targetid);
+    if (!wrap)
+	return false;
+    wrap->deref();
+    if (wrap->rtp()) {
 	Debug(&splugin,DebugInfo,"RTP DTMF '%s' targetid '%s'",text.c_str(),targetid.c_str());
-	int duration = msg.getIntValue("duration");
+	int duration = msg.getIntValue(YSTRING("duration"));
 	for (unsigned int i=0;i<text.length();i++)
 	    wrap->sendDTMF(text.at(i),duration);
 	return true;
@@ -1456,12 +1537,33 @@ YRTPPlugin::~YRTPPlugin()
     s_mirrors.clear();
 }
 
+void YRTPPlugin::addStats(const NamedList& stats)
+{
+    DDebug(this,DebugAll,"Add updates");
+    s_mutex.lock();
+    m_updates.append(new NamedList(stats));
+    s_mutex.unlock();
+}
+
+void YRTPPlugin::genUpdate(Message& msg)
+{
+    DDebug(this,DebugAll,"::genUpdate() [%p] updates=%d",this,m_updates.count());
+    Lock l(s_mutex);
+    if (m_updates.count() == 0)
+	return;
+    NamedList* updateParams = static_cast<NamedList*>(m_updates.remove(false));
+    msg.copyParams(*updateParams);
+    TelEngine::destruct(updateParams);
+}
+
 void YRTPPlugin::statusParams(String& str)
 {
     s_mutex.lock();
     str.append("chans=",",") << s_calls.count();
-    str.append("mirrors=",",") << s_mirrors.count();
     s_mutex.unlock();
+    s_refMutex.lock();
+    str.append("mirrors=",",") << s_mirrors.count();
+    s_refMutex.unlock();
 }
 
 void YRTPPlugin::statusDetail(String& str)
@@ -1472,11 +1574,13 @@ void YRTPPlugin::statusDetail(String& str)
 	YRTPWrapper* w = static_cast<YRTPWrapper*>(l->get());
         str.append(w->id(),",") << "=" << w->callId();
     }
+    s_mutex.unlock();
+    s_refMutex.lock();
     for (l = s_mirrors.skipNull(); l; l=l->skipNext()) {
 	YRTPReflector* r = static_cast<YRTPReflector*>(l->get());
         str.append(r->idA(),",") << "=" << r->idB().safe("?");
     }
-    s_mutex.unlock();
+    s_refMutex.unlock();
 }
 
 static Regexp s_reflectMatch(
@@ -1492,7 +1596,7 @@ static Regexp s_reflectMatch(
 bool YRTPPlugin::reflectSetup(Message& msg, const char* id, RTPTransport& rtp,
     const char* rHost, const char* leg)
 {
-    String lip(msg.getValue("rtp_localip"));
+    String lip(msg.getValue(YSTRING("rtp_localip")));
     if (lip.null())
 	YRTPWrapper::guessLocal(rHost,lip);
     SocketAddr addr(AF_INET);
@@ -1502,8 +1606,8 @@ bool YRTPPlugin::reflectSetup(Message& msg, const char* id, RTPTransport& rtp,
 	return false;
     }
 
-    int minport = msg.getIntValue("rtp_minport",s_minport);
-    int maxport = msg.getIntValue("rtp_maxport",s_maxport);
+    int minport = msg.getIntValue(YSTRING("rtp_minport"),s_minport);
+    int maxport = msg.getIntValue(YSTRING("rtp_maxport"),s_maxport);
     int attempt = 10;
     if (minport > maxport) {
 	int tmp = maxport;
@@ -1514,9 +1618,9 @@ bool YRTPPlugin::reflectSetup(Message& msg, const char* id, RTPTransport& rtp,
 	maxport++;
 	attempt = 1;
     }
-    bool rtcp = msg.getBoolValue("rtp_rtcp",s_rtcp);
+    bool rtcp = msg.getBoolValue(YSTRING("rtp_rtcp"),s_rtcp);
     for (;;) {
-	int lport = (minport + (::random() % (maxport - minport))) & 0xfffe;
+	int lport = (minport + (Random::random() % (maxport - minport))) & 0xfffe;
 	addr.port(lport);
 	if (rtp.localAddr(addr,rtcp)) {
 	    Debug(this,DebugInfo,"Reflector %s for '%s' bound to %s:%u%s",
@@ -1535,11 +1639,11 @@ bool YRTPPlugin::reflectSetup(Message& msg, const char* id, RTPTransport& rtp,
 bool YRTPPlugin::reflectStart(Message& msg, const char* id, RTPTransport& rtp,
     SocketAddr& rAddr)
 {
-    if (!rtp.remoteAddr(rAddr,msg.getBoolValue("rtp_autoaddr",s_autoaddr))) {
+    if (!rtp.remoteAddr(rAddr,msg.getBoolValue(YSTRING("rtp_autoaddr"),s_autoaddr))) {
 	Debug(this,DebugWarn,"Could not set remote RTP address for '%s'",id);
 	return false;
     }
-    if (msg.getBoolValue("rtp_drillhole",s_drill)) {
+    if (msg.getBoolValue(YSTRING("rtp_drillhole"),s_drill)) {
 	bool ok = rtp.drillHole();
 	Debug(this,(ok ? DebugInfo : DebugWarn),
 	    "Reflector for '%s' %s a hole in firewall/NAT",
@@ -1561,13 +1665,13 @@ void YRTPPlugin::reflectDrop(YRTPReflector*& refl, Lock& mylock)
 
 void YRTPPlugin::reflectExecute(Message& msg)
 {
-    const String* id = msg.getParam("id");
+    const String* id = msg.getParam(YSTRING("id"));
     if (null(id))
 	return;
-    String* sdp = msg.getParam("sdp_raw");
+    String* sdp = msg.getParam(YSTRING("sdp_raw"));
     if (null(sdp))
 	return;
-    if (!(msg.getBoolValue("rtp_forward",false) && msg.getBoolValue("rtp_reflect",false)))
+    if (!(msg.getBoolValue(YSTRING("rtp_forward"),false) && msg.getBoolValue(YSTRING("rtp_reflect"),false)))
 	return;
     DDebug(this,DebugAll,"YRTPPlugin::reflectExecute() A='%s'",id->c_str());
     // we have a candidate
@@ -1580,8 +1684,8 @@ void YRTPPlugin::reflectExecute(Message& msg)
 	Debug(this,DebugWarn,"Invalid RTP transport address for '%s'",id->c_str());
 	return;
     }
-    const char* aHost = msg.getValue("rtp_addr",ra.host().c_str());
-    const char* bHost = msg.getValue("rtp_remoteip",aHost);
+    const char* aHost = msg.getValue(YSTRING("rtp_addr"),ra.host().c_str());
+    const char* bHost = msg.getValue(YSTRING("rtp_remoteip"),aHost);
     YRTPReflector* r = new YRTPReflector(*id,
 	(sdp->find("a=recvonly") >= 0),(sdp->find("a=sendonly") >= 0));
     if (!(reflectSetup(msg,id->c_str(),r->rtpA(),aHost,"A") &&
@@ -1595,18 +1699,18 @@ void YRTPPlugin::reflectExecute(Message& msg)
     templ << "\\3" << r->rtpB().localAddr().host();
     templ << "\\5" << r->rtpB().localAddr().port() << "\\7";
     *sdp = sdp->replaceMatches(templ);
-    s_mutex.lock();
+    s_refMutex.lock();
     s_mirrors.append(r);
-    s_mutex.unlock();
+    s_refMutex.unlock();
 }
 
 void YRTPPlugin::reflectAnswer(Message& msg, bool ignore)
 {
-    const String* peerid = msg.getParam("peerid");
+    const String* peerid = msg.getParam(YSTRING("peerid"));
     if (null(peerid))
 	return;
     YRTPReflector* r = 0;
-    Lock mylock(s_mutex);
+    Lock mylock(s_refMutex);
     ObjList* l = s_mirrors.skipNull();
     for (; l; l=l->skipNext()) {
 	r = static_cast<YRTPReflector*>(l->get());
@@ -1617,7 +1721,7 @@ void YRTPPlugin::reflectAnswer(Message& msg, bool ignore)
     if (!r)
 	return;
     DDebug(this,DebugAll,"YRTPPlugin::reflectAnswer() A='%s'",peerid->c_str());
-    const String* id = msg.getParam("id");
+    const String* id = msg.getParam(YSTRING("id"));
     if (null(id)) {
 	if (ignore)
 	    return;
@@ -1631,8 +1735,8 @@ void YRTPPlugin::reflectAnswer(Message& msg, bool ignore)
 	reflectDrop(r,mylock);
 	return;
     }
-    String* sdp = msg.getParam("sdp_raw");
-    if (null(sdp) || !msg.getBoolValue("rtp_forward",false)) {
+    String* sdp = msg.getParam(YSTRING("sdp_raw"));
+    if (null(sdp) || !msg.getBoolValue(YSTRING("rtp_forward"),false)) {
 	if (ignore)
 	    return;
 	Debug(this,DebugWarn,"Unable to complete RTP reflection for '%s'",peerid->c_str());
@@ -1669,10 +1773,10 @@ void YRTPPlugin::reflectAnswer(Message& msg, bool ignore)
 
 void YRTPPlugin::reflectHangup(Message& msg)
 {
-    const String* id = msg.getParam("id");
+    const String* id = msg.getParam(YSTRING("id"));
     if (null(id))
 	return;
-    Lock mylock(s_mutex);
+    Lock mylock(s_refMutex);
     ObjList* l = s_mirrors.skipNull();
     for (; l; l=l->skipNext()) {
 	YRTPReflector* r = static_cast<YRTPReflector*>(l->get());
@@ -1729,20 +1833,23 @@ void YRTPPlugin::initialize()
     s_minport = cfg.getIntValue("general","minport",MIN_PORT);
     s_maxport = cfg.getIntValue("general","maxport",MAX_PORT);
     s_bufsize = cfg.getIntValue("general","buffer",BUF_SIZE);
-    s_minjitter = cfg.getIntValue("general","minjitter");
-    s_maxjitter = cfg.getIntValue("general","maxjitter");
+    s_minJitter = cfg.getIntValue("general","minjitter",50);
+    s_maxJitter = cfg.getIntValue("general","maxjitter",Engine::clientMode() ? 120 : 0);
     s_tos = cfg.getIntValue("general","tos",dict_tos);
     s_localip = cfg.getValue("general","localip");
     s_autoaddr = cfg.getBoolValue("general","autoaddr",true);
-    s_anyssrc = cfg.getBoolValue("general","anyssrc",false);
+    s_anyssrc = cfg.getBoolValue("general","anyssrc",true);
     s_padding = cfg.getIntValue("general","padding",0);
     s_rtcp = cfg.getBoolValue("general","rtcp",true);
+    s_interval = cfg.getIntValue("general","rtcp_interval",4500);
     s_drill = cfg.getBoolValue("general","drillhole",Engine::clientMode());
     s_sleep = cfg.getIntValue("general","defsleep",5);
     RTPGroup::setMinSleep(cfg.getIntValue("general","minsleep"));
     s_priority = Thread::priority(cfg.getValue("general","thread"));
     s_timeout = cfg.getIntValue("timeouts","timeout",3000);
+    s_udptlTimeout = cfg.getIntValue("timeouts","udptl_timeout",25000);
     s_notifyMsg = cfg.getValue("timeouts","notifymsg");
+    s_warnFirst = cfg.getBoolValue("timeouts","warnfirst",true);
     s_warnLater = cfg.getBoolValue("timeouts","warnlater",false);
     setup();
     if (m_first) {
