@@ -223,6 +223,15 @@ class Yate
 	_yate_print("%%>setlocal:$name:$value\n");
     }
 
+    /**
+     * Asynchronously retrieve a local module parameter
+     * @param $name Name of the parameter to read
+     */
+    static function GetLocal($name)
+    {
+	Yate::SetLocal($name, "");
+    }
+
     /**
      * Constructor. Creates a new outgoing message
      * @param $name Name of the new message
@@ -243,8 +252,8 @@ class Yate
     }
 
     /**
-     * Retrive the value of a named parameter
-     * @param $key Name of the parameter to retrive
+     * Retrieve the value of a named parameter
+     * @param $key Name of the parameter to retrieve
      * @param $defvalue (optional) Default value to return if parameter is not set
      * @return Value of the $key parameter or $defvalue
      */
