@@ -1,4 +1,4 @@
-<?
+<?php
 
 /* libyateivr.php
  * This file is part of the YATE Project http://YATE.null.ro
@@ -104,6 +104,7 @@ class IVR
     {
 	$this->Debug("::SetState('$state')");
 	$this->curstate = $state;
+	$this->OperTable("state");
     }
 
     /**
@@ -131,6 +132,13 @@ class IVR
 		    array_shift($op);
 		    $this->PlayFile($op);
 		    break;
+		case "play_recstop":
+		    array_shift($op);
+		    $this->PlayRecStop($op);
+		    break;
+		case "recstop":
+		    $this->RecStop($op[1]);
+		    break;
 		case "tone":       // Start playing a tone
 		    $this->PlayTone($op[1]);
 		    break;
@@ -381,6 +389,35 @@ class IVR
 	    $this->PlayNext();
     }
 
+    /**
+     * Stop recording then play a wave file or add it to the queue
+     * @param $file1,$file2,... Path to files to play
+     * @param $clear Optional - true to clear the queue and start playing now
+     */
+    function PlayRecStop()
+    {
+	$args = func_get_args();
+	$this->RecStop();
+	call_user_func_array(array($this, "PlayFile"), $args);
+    }
+
+    /**
+     * Stop recording
+     * @param $maxlen Maximum number of octets that should be transferred. Default NULL
+     */
+    function RecStop($maxlen=null)
+    {
+	$m = new Yate("chan.attach");
+	$m->id = "";
+	$m->SetParam("consumer","wave/record/-");
+	$m->SetParam("single",true);
+	if($maxlen) {
+	    $m->SetParam("maxlen", $maxlen);
+	    $m->SetParam("notify", IVR::ChannelID());
+	}
+	$m->Dispatch();
+    }
+
     /**
      * Start playing a tone, clear the file queue
      * @param $tone Name of the tone to play
@@ -444,7 +481,7 @@ class IVR
     }
 
     /**
-     * Retrive the IVR's channel ID, initialize it if required
+     * Retrieve the IVR's channel ID, initialize it if required
      * @return ID of the IVR call leg
      */
     static function ChannelID()
@@ -471,7 +508,7 @@ class IVR
     }
 
     /**
-     * Retrive the target channel ID
+     * Retrieve the target channel ID
      * @return ID of the call leg that called into the IVR
      */
     static function TargetID()
