@@ -178,6 +178,8 @@ public:
 protected:
     ObjList m_params;                    // Header list of parameters
     char m_separator;                    // Parameter separator
+private:
+    void operator=(const MimeHeaderLine&); // no assignment
 };
 
 /**
@@ -223,6 +225,8 @@ public:
      * @param line Destination string
      */
     virtual void buildLine(String& line) const;
+private:
+    void operator=(const MimeAuthLine&); // no assignment
 };
 
 /**
@@ -234,6 +238,7 @@ public:
  */
 class YATE_API MimeBody : public GenObject
 {
+    YNOCOPY(MimeBody); // no automatic copies please
 public:
     /**
      * Destructor
@@ -432,7 +437,7 @@ public:
      *  A random one will be created if missing. The length will be truncated
      *  to 70 if this value is exceeded
      */
-    MimeMultipartBody(const char* subtype = "mixed", const char* boundary = 0);
+    explicit MimeMultipartBody(const char* subtype = "mixed", const char* boundary = 0);
 
     /**
      * Constructor from block of data
