@@ -30,7 +30,7 @@
 #endif
 
 #include <yateclass.h>
-	
+
 /**
  * Holds all Telephony Engine related classes.
  */
@@ -42,6 +42,7 @@ namespace TelEngine {
  */
 class YATE_API Configuration : public String
 {
+    YNOCOPY(Configuration); // no automatic copies please
 public:
     /**
      * Create an empty configuration
@@ -53,7 +54,7 @@ public:
      * @param filename Name of file to initialize from
      * @param warn True to warn if the configuration could not be loaded
      */
-    Configuration(const char* filename, bool warn = true);
+    explicit Configuration(const char* filename, bool warn = true);
 
     /**
      * Assignment from string operator
@@ -69,14 +70,14 @@ public:
 	{ return m_sections.length(); }
 
     /**
-     * Retrive an entire section
+     * Retrieve an entire section
      * @param index Index of the section
      * @return The section's content or NULL if no such section
      */
     NamedList* getSection(unsigned int index) const;
 
     /**
-     * Retrive an entire section
+     * Retrieve an entire section
      * @param sect Name of the section
      * @return The section's content or NULL if no such section
      */
@@ -91,7 +92,7 @@ public:
     NamedString* getKey(const String& sect, const String& key) const;
 
     /**
-     * Retrive the value of a key in a section.
+     * Retrieve the value of a key in a section.
      * @param sect Name of the section
      * @param key Name of the key in section
      * @param defvalue Default value to return if not found
@@ -100,16 +101,21 @@ public:
     const char* getValue(const String& sect, const String& key, const char* defvalue = 0) const;
 
     /**
-     * Retrive the numeric value of a key in a section.
+     * Retrieve the numeric value of a key in a section.
      * @param sect Name of the section
      * @param key Name of the key in section
      * @param defvalue Default value to return if not found
+     * @param minvalue Minimum value allowed for the parameter
+     * @param maxvalue Maximum value allowed for the parameter
+     * @param clamp Control the out of bound values: true to adjust to the nearest
+     *  bound, false to return the default value
      * @return The number contained in the key or the default
      */
-    int getIntValue(const String& sect, const String& key, int defvalue = 0) const;
+    int getIntValue(const String& sect, const String& key, int defvalue = 0,
+	int minvalue = INT_MIN, int maxvalue = INT_MAX, bool clamp = true) const;
 
     /**
-     * Retrive the numeric value of a key in a section trying first a table lookup.
+     * Retrieve the numeric value of a key in a section trying first a table lookup.
      * @param sect Name of the section
      * @param key Name of the key in section
      * @param tokens A pointer to an array of tokens to try to lookup
@@ -119,7 +125,7 @@ public:
     int getIntValue(const String& sect, const String& key, const TokenDict* tokens, int defvalue = 0) const;
 
     /**
-     * Retrive the floating point value of a key in a section.
+     * Retrieve the floating point value of a key in a section.
      * @param sect Name of the section
      * @param key Name of the key in section
      * @param defvalue Default value to return if not found
@@ -128,7 +134,7 @@ public:
     double getDoubleValue(const String& sect, const String& key, double defvalue = 0.0) const;
 
     /**
-     * Retrive the boolean value of a key in a section.
+     * Retrieve the boolean value of a key in a section.
      * @param sect Name of the section
      * @param key Name of the key in section
      * @param defvalue Default value to return if not found
@@ -145,9 +151,9 @@ public:
     /**
      * Makes sure a section with a given name exists, creates if required
      * @param sect Name of section to check or create
+     * @return The section's content or NULL if no such section
      */
-    inline void createSection(const String& sect)
-	{ if (sect) makeSectHolder(sect); }
+    NamedList* createSection(const String& sect);
 
     /**
      * Deletes a key/value pair
@@ -202,8 +208,6 @@ public:
     bool save() const;
 
 private:
-    Configuration(const Configuration& value); // no copy constructor
-    Configuration& operator=(const Configuration& value); // no assignment please
     ObjList *getSectHolder(const String& sect) const;
     ObjList *makeSectHolder(const String& sect);
     ObjList m_sections;
@@ -225,16 +229,25 @@ public:
      *
      * @param name Name of the message - must not be NULL or empty
      * @param retval Default return value
+     * @param broadcast Broadcast flag, true if handling the mesage must not stop it
      */
-    Message(const char* name, const char* retval = 0);
+    explicit Message(const char* name, const char* retval = 0, bool broadcast = false);
 
     /**
      * Copy constructor.
-     * Note that user data and notification are not copied.
+     * Note that user data and notification are not copied
      * @param original Message we are copying from
      */
     Message(const Message& original);
 
+    /**
+     * Copy constructor that can alter the broadcast flag.
+     * Note that user data and notification are not copied
+     * @param original Message we are copying from
+     * @param broadcast Broadcast flag, true if handling the mesage must not stop it
+     */
+    Message(const Message& original, bool broadcast);
+
     /**
      * Destruct the message and dereferences any user data
      */
@@ -248,21 +261,21 @@ public:
     virtual void* getObject(const String& name) const;
 
     /**
-     * Retrive a reference to the value returned by the message.
+     * Retrieve a reference to the value returned by the message.
      * @return A reference to the value the message will return
      */
     inline String& retValue()
 	{ return m_return; }
 
     /**
-     * Retrive a const reference to the value returned by the message.
+     * Retrieve a const reference to the value returned by the message.
      * @return A reference to the value the message will return
      */
     inline const String& retValue() const
 	{ return m_return; }
 
     /**
-     * Retrive the object associated with the message
+     * Retrieve the object associated with the message
      * @return Pointer to arbitrary user RefObject
      */
     inline RefObject* userData() const
@@ -294,14 +307,21 @@ public:
 	{ m_notify = notify; }
 
     /**
-     * Retrive a reference to the creation time of the message.
+     * Retrieve the broadcast flag
+     * @return True if the message is a broadcast (handling does not stop it)
+     */
+    inline bool broadcast() const
+	{ return m_broadcast; }
+
+    /**
+     * Retrieve a reference to the creation time of the message.
      * @return A reference to the @ref Time when the message was created
      */
     inline Time& msgTime()
 	{ return m_time; }
 
     /**
-     * Retrive a const reference to the creation time of the message.
+     * Retrieve a const reference to the creation time of the message.
      * @return A reference to the @ref Time when the message was created
      */
     inline const Time& msgTime() const
@@ -365,6 +385,7 @@ private:
     Time m_time;
     RefObject* m_data;
     bool m_notify;
+    bool m_broadcast;
     void commonEncode(String& str) const;
     int commonDecode(const char* str, int offs);
 };
@@ -379,13 +400,14 @@ class YATE_API MessageHandler : public String
 {
     friend class MessageDispatcher;
     friend class MessageRelay;
+    YNOCOPY(MessageHandler); // no automatic copies please
 public:
     /**
      * Creates a new message handler.
      * @param name Name of the handled message - may be NULL
      * @param priority Priority of the handler, 0 = top
      */
-    MessageHandler(const char* name, unsigned priority = 100);
+    explicit MessageHandler(const char* name, unsigned priority = 100);
 
     /**
      * Handler destructor.
@@ -412,7 +434,7 @@ public:
 	{ return m_priority; }
 
     /**
-     * Retrive the filter (if installed) associated to this handler
+     * Retrieve the filter (if installed) associated to this handler
      */
     inline const NamedString* filter() const
 	{ return m_filter; }
@@ -437,7 +459,7 @@ public:
      */
     void clearFilter();
 
-protected:    
+protected:
     /**
      * Remove the handler from its dispatcher, remove any installed filter.
      * This method is called internally from destruct and the destructor
@@ -475,6 +497,7 @@ public:
  */
 class YATE_API MessageRelay : public MessageHandler
 {
+    YNOCOPY(MessageRelay); // no automatic copies please
 public:
     /**
      * Creates a new message relay.
@@ -548,6 +571,7 @@ class YATE_API MessagePostHook : public GenObject, public MessageNotifier
  */
 class YATE_API MessageDispatcher : public GenObject, public Mutex
 {
+    YNOCOPY(MessageDispatcher); // no automatic copies please
 public:
     /**
      * Creates a new message dispatcher.
@@ -580,6 +604,8 @@ public:
      * Synchronously dispatch a message to the installed handlers.
      * Handlers matching the message name and filter parameter are called in
      *  their installed order (based on priority) until one returns true.
+     * If the message has the broadcast flag set all matching handlers are
+     *  called and the return value is true if any handler returned true.
      * Note that in some cases when a handler is removed from the list
      *  other handlers with equal priority may be called twice.
      * @param msg The message to dispatch
@@ -655,7 +681,7 @@ private:
  *</pre>
  * @short Plugin support
  */
-class YATE_API Plugin : public GenObject
+class YATE_API Plugin : public GenObject, public DebugEnabler
 {
 public:
     /**
@@ -663,13 +689,7 @@ public:
      * @param name the undecorated name of the library that contains the plugin
      * @param earlyInit True to initialize the plugin early
      */
-    Plugin(const char* name, bool earlyInit = false);
-
-    /**
-     * Creates a new Plugin container.
-     * Alternate constructor which is also the default.
-     */
-    Plugin();
+    explicit Plugin(const char* name, bool earlyInit = false);
 
     /**
      * Destroys the plugin.
@@ -678,6 +698,13 @@ public:
      */
     virtual ~Plugin();
 
+    /**
+     * Get a string representation of this object
+     * @return Name of the plugin
+     */
+    virtual const String& toString() const
+	{ return m_name; }
+
     /**
      * Get a pointer to a derived class given that class name
      * @param name Name of the class we are asking for
@@ -697,6 +724,13 @@ public:
     virtual bool isBusy() const
 	{ return false; }
 
+    /**
+     * Retrieve the name of the plugin
+     * @return The plugin's name as String
+     */
+    inline const String& name() const
+	{ return m_name; }
+
     /**
      * Check if the module is to be initialized early
      * @return True if the module should be initialized before regular ones
@@ -705,6 +739,8 @@ public:
 	{ return m_early; }
 
 private:
+    Plugin(); // no default constructor please
+    String m_name;
     bool m_early;
 };
 
@@ -764,6 +800,11 @@ public:
     static void setChecker(EngineCheck* ptr = 0);
 };
 
+/**
+ * Prototype for engine main loop callback
+ */
+typedef int (*EngineLoop)();
+
 /**
  * This class holds global information about the engine.
  * Note: this is a singleton class.
@@ -774,6 +815,7 @@ class YATE_API Engine
 {
     friend class EnginePrivate;
     friend class EngineCommand;
+    YNOCOPY(Engine); // no automatic copies please
 public:
     /**
      * Running modes - run the engine as Console, Client or Server.
@@ -786,6 +828,13 @@ public:
 	ClientProxy = 4,
     };
 
+    enum CallAccept {
+	Accept = 0,
+	Partial = 1,
+	Congestion = 2,
+	Reject = 3,
+    };
+
     /**
      * Plugin load and initialization modes.
      * Default is LoadLate that initailizes the plugin after others.
@@ -804,11 +853,12 @@ public:
      * @param argv Argument array
      * @param env Environment variables
      * @param mode Mode the engine must run as - Console, Client or Server
+     * @param loop Callback function to the main thread's loop
      * @param fail Fail and return after parsing command line arguments
      * @return Program exit code
      */
     static int main(int argc, const char** argv, const char** env,
-	RunMode mode = Console, bool fail = false);
+	RunMode mode = Console, EngineLoop loop = 0, bool fail = false);
 
     /**
      * Display the help information on console
@@ -817,6 +867,18 @@ public:
      */
     static void help(bool client, bool errout = false);
 
+    /**
+     * Initialize the engine
+     * @return Error code, 0 for success
+     */
+    int engineInit();
+
+    /**
+     * Do engine cleanup
+     * @return Error code, 0 for success
+     */
+    int engineCleanup();
+
     /**
      * Run the engine.
      * @return Error code, 0 for success
@@ -836,6 +898,30 @@ public:
     static RunMode mode()
 	{ return s_mode; }
 
+    /**
+     * Get call accept status
+     * @return Engine's call accept status as enumerated value
+     */
+    inline static CallAccept accept() {
+	return s_accept;
+    }
+
+    /**
+     * Set call accept status
+     * @param ca New call accept status as enumerated value
+     */
+    inline static void setAccept(CallAccept ca) {
+	s_accept = ca;
+    }
+
+    /**
+     * Get call accept states
+     * @return states Pointer to the call accept states TokenDict
+     */
+    inline static const TokenDict* getCallAcceptStates() {
+	return s_callAccept;
+    }
+
     /**
      * Check if the engine is running as telephony client
      * @return True if the engine is running in client mode
@@ -948,6 +1034,13 @@ public:
      */
     static void init();
 
+    /**
+     * Reinitialize one plugin
+     * @param name Name of the plugin to initialize, emplty, "*" or "all" to initialize all
+     * @return True if plugin(s) were reinitialized
+     */
+    static bool init(const String& name);
+
     /**
      * Stop the engine and the entire program
      * @param code Return code of the program
@@ -994,10 +1087,11 @@ public:
      * Convenience function.
      * Enqueue a new parameterless message in the message queue
      * @param name Name of the parameterless message to put in queue
+     * @param broadcast Broadcast flag, true if handling the mesage must not stop it
      * @return True if enqueued, false on error (already queued)
      */
-    inline static bool enqueue(const char* name)
-	{ return (name && *name) ? enqueue(new Message(name)) : false; }
+    inline static bool enqueue(const char* name, bool broadcast = false)
+	{ return name && *name && enqueue(new Message(name,0,broadcast)); }
 
     /**
      * Synchronously dispatch a message to the registered handlers
@@ -1017,9 +1111,10 @@ public:
      * Convenience function.
      * Dispatch a parameterless message to the registered handlers
      * @param name The name of the message to create and dispatch
+     * @param broadcast Broadcast flag, true if handling the mesage must not stop it
      * @return True if one handler accepted it, false if all ignored
      */
-    static bool dispatch(const char* name);
+    static bool dispatch(const char* name, bool broadcast = false);
 
     /**
      * Install or remove a hook to catch messages after being dispatched
@@ -1050,9 +1145,9 @@ public:
 	{ return m_dispatcher.handlerCount(); }
 
     /**
-     * Loads the plugins from an extra plugins directory
+     * Loads the plugins from an extra plugins directory or just an extra plugin
      * @param relPath Path to the extra directory, relative to the main modules
-     * @return True if the directory could at least be opened
+     * @return True if the plugin was loaded or the directory could at least be opened
      */
     bool loadPluginDir(const String& relPath);
 
@@ -1062,6 +1157,19 @@ public:
      */
     static void pluginMode(PluginMode mode);
 
+    /**
+     * Retrive the list of captured events of a specific type
+     * @param type Type of captured events, an empty name returns engine events
+     * @return List containing captured events of specified type, NULL if not found
+     */
+    static const ObjList* events(const String& type);
+
+    /**
+     * Clear the list of captured events of a specific type
+     * @param type Type of captured events, an empty name clear engine events
+     */
+    static void clearEvents(const String& type);
+
 protected:
     /**
      * Destroys the engine and everything. You must not call it directly,
@@ -1090,6 +1198,7 @@ protected:
 
 private:
     Engine();
+    void internalStatisticsStart();
     ObjList m_libs;
     MessageDispatcher m_dispatcher;
     static Engine* s_self;
@@ -1102,6 +1211,8 @@ private:
     static NamedList s_params;
     static int s_haltcode;
     static RunMode s_mode;
+    static CallAccept s_accept;
+    static const TokenDict s_callAccept[];
 };
 
 }; // namespace TelEngine
