@@ -30,7 +30,7 @@
 #endif
 
 #include <yatengine.h>
-	
+
 /**
  * Holds all Telephony Engine related classes.
  */
@@ -121,7 +121,7 @@ struct YATE_API FormatInfo {
     /**
      * Normal constructor
      */
-    inline FormatInfo(const char* _name, int fsize = 0, int ftime = 10000,
+    inline explicit FormatInfo(const char* _name, int fsize = 0, int ftime = 10000,
 	const char* _type = "audio", int srate = 8000, int nchan = 1, bool convert = false)
 	: name(_name), type(_type),
 	  frameSize(fsize), frameTime(ftime),
@@ -153,9 +153,9 @@ struct YATE_API TranslatorCaps {
  */
 class YATE_API FormatRepository
 {
+    YNOCOPY(FormatRepository); // no automatic copies please
 private:
     FormatRepository();
-    FormatRepository& operator=(const FormatRepository&);
 public:
     /**
      * Retrieve a format by name and type
@@ -182,14 +182,14 @@ public:
  * An extension of a String that can parse data formats
  * @short A Data format
  */
-class YATE_API DataFormat : public String
+class YATE_API DataFormat : public NamedList
 {
 public:
     /**
      * Creates a new, empty format string.
      */
     inline DataFormat()
-	: m_parsed(0)
+	: NamedList((const char*)0), m_parsed(0)
 	{ }
 
     /**
@@ -197,7 +197,7 @@ public:
      * @param value Initial value of the format
      */
     inline DataFormat(const char* value)
-	: String(value), m_parsed(0)
+	: NamedList(value), m_parsed(0)
 	{ }
 
     /**
@@ -205,7 +205,7 @@ public:
      * @param value Initial value of the format
      */
     inline DataFormat(const DataFormat& value)
-	: String(value), m_parsed(value.getInfo())
+	: NamedList(value), m_parsed(value.getInfo())
 	{ }
 
     /**
@@ -213,7 +213,15 @@ public:
      * @param value Initial value of the format
      */
     inline DataFormat(const String& value)
-	: String(value), m_parsed(0)
+	: NamedList(value), m_parsed(0)
+	{ }
+
+    /**
+     * Constructor from NamedList reference
+     * @param value Initial value of the format and parameters
+     */
+    inline DataFormat(const NamedList& value)
+	: NamedList(value), m_parsed(0)
 	{ }
 
     /**
@@ -221,31 +229,31 @@ public:
      * @param value Initial value of the format
      */
     inline DataFormat(const String* value)
-	: String(value), m_parsed(0)
+	: NamedList(value ? value->c_str() : (const char*)0), m_parsed(0)
 	{ }
 
     /**
      * Constructor from format information
      * @param format Pointer to existing FormatInfo
      */
-    inline DataFormat(const FormatInfo* format)
-	: String(format ? format->name : (const char*)0), m_parsed(format)
+    inline explicit DataFormat(const FormatInfo* format)
+	: NamedList(format ? format->name : (const char*)0), m_parsed(format)
 	{ }
 
     /**
      * Assignment operator.
      */
     inline DataFormat& operator=(const DataFormat& value)
-	{ String::operator=(value); return *this; }
+	{ NamedList::operator=(value); m_parsed = value.getInfo(); return *this; }
 
     /**
-     * Retrive a pointer to the format information
+     * Retrieve a pointer to the format information
      * @return Pointer to the associated format info or NULL if error
      */
     const FormatInfo* getInfo() const;
 
     /**
-     * Retrive the frame size
+     * Retrieve the frame size
      * @param defValue Default value to return if format is unknown
      * @return Frame size in octets/frame, 0 for non-framed, defValue if unknown
      */
@@ -253,7 +261,7 @@ public:
 	{ return getInfo() ? getInfo()->frameSize : defValue; }
 
     /**
-     * Retrive the frame time
+     * Retrieve the frame time
      * @param defValue Default value to return if format is unknown
      * @return Frame time in microseconds, 0 for variable, defValue if unknown
      */
@@ -261,7 +269,7 @@ public:
 	{ return getInfo() ? getInfo()->frameTime : defValue; }
 
     /**
-     * Retrive the sample rate
+     * Retrieve the sample rate
      * @param defValue Default value to return if format is unknown
      * @return Rate in samples/second (audio) or 1e-6 frames/second (video),
      *  0 for unknown, defValue if unknown format
@@ -270,7 +278,7 @@ public:
 	{ return getInfo() ? getInfo()->sampleRate : defValue; }
 
     /**
-     * Retrive the number of channels
+     * Retrieve the number of channels
      * @param defValue Default value to return if format is unknown
      * @return Number of channels (typically 1), defValue if unknown format
      */
@@ -293,7 +301,7 @@ private:
 class YATE_API DataNode : public RefObject
 {
     friend class DataEndpoint;
-
+    YNOCOPY(DataNode); // no automatic copies please
 public:
     /**
      * Flags associated with the DataBlocks forwarded between nodes
@@ -312,7 +320,7 @@ public:
      * Construct a DataNode
      * @param format Description of the data format, default none
      */
-    inline DataNode(const char* format = 0)
+    inline explicit DataNode(const char* format = 0)
 	: m_format(format), m_timestamp(0)
 	{ }
 
@@ -398,7 +406,7 @@ public:
      * Consumer constructor
      * @param format Name of the data format, default "slin" (Signed Linear)
      */
-    inline DataConsumer(const char* format = "slin")
+    inline explicit DataConsumer(const char* format = "slin")
 	: DataNode(format),
 	  m_source(0), m_override(0),
 	  m_regularTsDelta(0), m_overrideTsDelta(0), m_lastTsTime(0)
@@ -472,13 +480,13 @@ private:
 class YATE_API DataSource : public DataNode, public Mutex
 {
     friend class DataTranslator;
-
+    YNOCOPY(DataSource); // no automatic copies please
 public:
     /**
      * Source constructor
      * @param format Name of the data format, default "slin" (Signed Linear)
      */
-    inline DataSource(const char* format = "slin")
+    inline explicit DataSource(const char* format = "slin")
 	: DataNode(format), Mutex(false,"DataSource"),
 	  m_nextStamp(invalidStamp()), m_translator(0) { }
 
@@ -493,7 +501,7 @@ public:
      * @return Pointer to the requested class or NULL if this object doesn't implement it
      */
     virtual void* getObject(const String& name) const;
-    
+
     /**
      * Check if this data source is still valid
      * @return True if still valid, false if node should be removed
@@ -552,12 +560,14 @@ public:
 
 protected:
     unsigned long m_nextStamp;
-    DataTranslator* m_translator;
     ObjList m_consumers;
 private:
-    inline void setTranslator(DataTranslator* translator)
-	{ m_translator = translator; }
+    inline void setTranslator(DataTranslator* translator) {
+	    Lock mylock(this);
+	    m_translator = translator;
+	}
     bool detachInternal(DataConsumer* consumer);
+    DataTranslator* m_translator;
 };
 
 /**
@@ -603,7 +613,7 @@ protected:
      * Threaded Source constructor
      * @param format Name of the data format, default "slin" (Signed Linear)
      */
-    inline ThreadedSource(const char* format = "slin")
+    inline explicit ThreadedSource(const char* format = "slin")
 	: DataSource(format), m_thread(0)
 	{ }
 
@@ -651,7 +661,7 @@ public:
      * @param sFormat Name of the source format (data received from the consumer)
      * @param source Optional pointer to a DataSource object
      */
-    DataTranslator(const char* sFormat, DataSource* source = 0);
+    explicit DataTranslator(const char* sFormat, DataSource* source = 0);
 
     /**
      * Destroys the translator and its source
@@ -816,11 +826,14 @@ private:
  */
 class YATE_API TranslatorFactory : public GenObject
 {
+    YNOCOPY(TranslatorFactory); // no automatic copies please
 protected:
     /**
      * Constructor - registers the factory in the global list
+     * @param name Static name of the factory, used for debugging
      */
-    inline TranslatorFactory()
+    inline explicit TranslatorFactory(const char* name = 0)
+	: m_name(name ? name : "?")
 	{ DataTranslator::install(this); }
 
 public:
@@ -875,6 +888,16 @@ public:
      * @return Pointer to intermediate format or NULL
      */
     virtual const FormatInfo* intermediate() const;
+
+    /**
+     * Get the name of this factory, useful for debugging purposes
+     * @return Name of the factory as specified in the constructor
+     */
+    virtual const char* name() const
+	{ return m_name; }
+
+private:
+    const char* m_name;
 };
 
 /**
@@ -884,12 +907,12 @@ public:
  */
 class YATE_API DataEndpoint : public RefObject
 {
+    YNOCOPY(DataEndpoint); // no automatic copies please
 public:
-
     /**
      * Creates an empty data endpoint
      */
-    DataEndpoint(CallEndpoint* call = 0, const char* name = "audio");
+    explicit DataEndpoint(CallEndpoint* call = 0, const char* name = "audio");
 
     /**
      * Endpoint destruct notification, clears source and consumer
@@ -1085,7 +1108,7 @@ private:
 class YATE_API CallEndpoint : public RefObject
 {
     friend class DataEndpoint;
-
+    YNOCOPY(CallEndpoint); // no automatic copies please
 private:
     CallEndpoint* m_peer;
     String m_id;
@@ -1292,12 +1315,12 @@ private:
  * Module is a descendent of Plugin specialized in implementing modules
  * @short A Plugin that implements a module
  */
-class YATE_API Module : public Plugin, public Mutex, public MessageReceiver, public DebugEnabler
+class YATE_API Module : public Plugin, public Mutex, public MessageReceiver
 {
+    YNOCOPY(Module); // no automatic copies please
 private:
     bool m_init;
     int m_relays;
-    String m_name;
     String m_type;
     Regexp m_filter;
     u_int64_t m_changed;
@@ -1312,14 +1335,7 @@ public:
     virtual void* getObject(const String& name) const;
 
     /**
-     * Retrive the name of the module
-     * @return The module's name as String
-     */
-    inline const String& name() const
-	{ return m_name; }
-
-    /**
-     * Retrive the type of the module
+     * Retrieve the type of the module
      * @return The module's type as String
      */
     inline const String& type() const
@@ -1332,7 +1348,7 @@ public:
     void changed();
 
     /**
-     * Retrive the global update notification delay
+     * Retrieve the global update notification delay
      * @return Update delay value in seconds
      */
     inline static unsigned int updateDelay()
@@ -1411,6 +1427,14 @@ protected:
      */
     static const char* messageName(int id);
 
+    /**
+     * Find the ID or a specific Relay name
+     * @param name Name of the Relay to search for
+     * @return ID of the Relay, zero if not found
+     */
+    static inline int relayId(const char* name)
+	{ return lookup(name,s_messages); }
+
     /**
      * Constructor
      * @param name Plugin name of this driver
@@ -1434,6 +1458,14 @@ protected:
      */
     void setup();
 
+    /**
+     * Check if a specific relay ID is installed
+     * @param id RelayID to test for
+     * @return True if such a relay is installed
+     */
+    inline bool relayInstalled(int id) const
+	{ return (id & m_relays) != 0; }
+
     /**
      * Install a standard message relay
      * @param id RelayID of the new relay to create
@@ -1581,11 +1613,11 @@ private:
  * A class that holds common channel related features (a.k.a. call leg)
  * @short An abstract communication channel
  */
-class YATE_API Channel : public CallEndpoint, public DebugEnabler
+class YATE_API Channel : public CallEndpoint, public DebugEnabler, public MessageNotifier
 {
     friend class Driver;
     friend class Router;
-
+    YNOCOPY(Channel); // no automatic copies please
 private:
     NamedList m_parameters;
     Driver* m_driver;
@@ -1618,6 +1650,12 @@ public:
      */
     virtual void* getObject(const String& name) const;
 
+    /**
+     * Get the big mutex that serializes access to all disconnect parameter lists
+     * @return A reference to the mutex
+     */
+    static Mutex& paramMutex();
+
     /**
      * Put channel variables into a message
      * @param msg Message to fill in
@@ -1925,6 +1963,20 @@ public:
      */
     void filterDebug(const String& item);
 
+    /**
+     * Get the disconnect parameters list
+     * @return Constant reference to disconnect parameters
+     */
+    inline const NamedList& parameters() const
+	{ return m_parameters; }
+
+    /**
+     * Notification for dispatched messages
+     * @param msg Message that was dispatched
+     * @param handled Result of handling the message
+     */
+    virtual void dispatched(const Message& msg, bool handled);
+
 protected:
     /**
      * Constructor
@@ -1972,6 +2024,13 @@ protected:
      */
     virtual void setDisconnect(const NamedList* params);
 
+    /**
+     * Notification after chan.disconnected handling
+     * @param msg The chan.disconnected message
+     * @param handled True if the message was handled
+     */
+    virtual void endDisconnect(const Message& msg, bool handled);
+
     /**
      * Set a new ID for this channel
      * @param newId New ID to set to this channel
@@ -2042,13 +2101,6 @@ protected:
     inline NamedList& parameters()
 	{ return m_parameters; }
 
-    /**
-     * Get the disconnect parameters list
-     * @return Constant teference to disconnect parameters
-     */
-    inline const NamedList& parameters() const
-	{ return m_parameters; }
-
 private:
     void init();
     Channel(); // no default constructor please
@@ -2086,7 +2138,7 @@ public:
     virtual void* getObject(const String& name) const;
 
     /**
-     * Retrive the prefix that is used as base for all channels
+     * Retrieve the prefix that is used as base for all channels
      * @return The driver's prefix
      */
     inline const String& prefix() const
@@ -2321,6 +2373,7 @@ private:
  */
 class YATE_API Router : public Thread
 {
+    YNOCOPY(Router); // no automatic copies please
 private:
     Driver* m_driver;
     String m_id;
